import { Route, Routes } from '@angular/router';
import { ClassificationResolver } from '@core/resolvers/classification.resolver';
import { CountryResolver } from '@core/resolvers/country.resolver';
import { CurrenciesResolver } from '@core/resolvers/currencies.resolver';
import { CurrencyFormattingResolver } from '@core/resolvers/currency-formatting-resolver';
import { SdgResolver } from '@core/resolvers/sdg.resolver';
import { StatusResolver } from '@core/resolvers/status.resolver';
import { TimeZoneResolver } from '@core/resolvers/time-zone.resolver';
import { ArchiveReasonCodesResolver } from '@features/archive/resolvers/archive-reason-codes.resolver';

const staticResolvers = {
  countries: CountryResolver,
  classifications: ClassificationResolver,
  currencies: CurrenciesResolver,
  currencyFormatting: CurrencyFormattingResolver,
  timezones: TimeZoneResolver,
  statuses: StatusResolver,
  archiveReasonCodes: ArchiveReasonCodesResolver,
  sdgs: SdgResolver
};

const ApplyPortalRoute: Route = {
  path: 'apply',
  loadChildren: () => import('./app/apply-portal-routing.module').then(m => m.ApplyPortalRoutingModule)
};
const ManagerPortalRoute = {
  path: 'management',
  loadChildren: () => import('./app/management-portal-routing.module').then(m => m.ManagementPortalRoutingModule)
};
const PlatformPortalRoute = {
  path: 'platform',
  loadChildren: () => import('./app/platform-portal-routing.module').then(m => m.PlatformPortalRoutingModule)
};
export const routes: Routes = [{
  path: '',
  resolve: staticResolvers,
  children: [ApplyPortalRoute, ManagerPortalRoute, PlatformPortalRoute]
}];

