import { Component, Input, OnInit } from '@angular/core';
import { BaseTranslation } from '@core/typings/translation.typing';
import { defaultCKEditorConfig, TypeSafeFormBuilder, TypeSafeFormGroup } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { YCModalComponent } from '@yourcause/common/modals';
interface TranslationFormGroup {
  translation: string;
}

@Component({
  selector: 'gc-translate-rich-text-modal',
  templateUrl: './translate-rich-text-modal.component.html',
  styleUrls: ['./translate-rich-text-modal.component.scss']
})
export class TranslateRichTextModalComponent extends YCModalComponent<{
  newValue: string;
}> implements OnInit {
  @Input() item: BaseTranslation;
  @Input() modalHeader: string;
  @Input() modalSubHeader: string;
  @Input() isEdit = false;
  @Input() translationValue: string;
  @Input() ckEditorConfig = defaultCKEditorConfig;

  formGroup: TypeSafeFormGroup<TranslationFormGroup>;
  primaryButtonText: string;

  constructor (
    private formBuilder: TypeSafeFormBuilder,
    private i18n: I18nService
  ) {
    super();
  }

  ngOnInit () {
    this.primaryButtonText = this.i18n.translate(
      'common:textClose'
    );
    if (this.isEdit) {
      this.primaryButtonText = this.i18n.translate(
        this.translationValue ?
          'GLOBAL:textUpdate' :
          'common:linkCreate'
      );
      this.formGroup = this.formBuilder.group({
        translation: this.translationValue ||
          this.item.defaultTranslationText
      });
    }
  }

  saveChanges () {
    this.closeModal.emit({
      newValue: this.formGroup.value.translation
    });
  }

  cancel () {
    this.closeModal.emit();
  }
}
