import { Pipe, PipeTransform } from '@angular/core';
import { PaymentStatus } from '@core/typings/status.typing';
import { OrganizationEligibleForGivingStatus } from '@yourcause/common';


@Pipe({ name: 'gcShowVettingAlert' })
export class ShowVettingAlertPipe implements PipeTransform {

  transform (
    eligibleforGivingStatus: OrganizationEligibleForGivingStatus,
    isNomination: boolean,
    isClientProcessing: boolean,
    applicantApplying: boolean,
    paymentStatus: PaymentStatus, // passed only if this alert relates to a payment,
    isForApplicantPortal: boolean
  ): boolean {
    if (isForApplicantPortal) {
      return false;
    }
    if (paymentStatus) {
      const paymentPending = [PaymentStatus.Scheduled, PaymentStatus.Pending].includes(
        paymentStatus
      );
      // If the payment is already processed this message does not apply
      if (!paymentPending) {
        return false;
      }
    }
    if (applicantApplying && !isClientProcessing) {
      return true;
    } else {
      const isEligible = eligibleforGivingStatus === OrganizationEligibleForGivingStatus.ELIGIBLE;

      return !isEligible && !isNomination && !isClientProcessing;
    }
  }
}
