import { Pipe, PipeTransform } from '@angular/core';
import { OrganizationEligibleForGivingStatus } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';

@Pipe({ name: 'gcVettingRequestAlertHoverText' })
export class VettingRequestAlertHoverTextPipe implements PipeTransform {

  constructor (
    private i18n: I18nService
  ) {}
  transform (
    eligibleforGivingStatus: OrganizationEligibleForGivingStatus,
    applicantApplying = false
  ): string {
    let returnText: string;
    if (applicantApplying) {
      returnText = this.i18n.translate(
        'common:textIndividualApplyingIneligibleYcProcess',
        {},
        'Program benefiting individual ineligible for disbursement by YourCause'
      );
    } else {
      switch (eligibleforGivingStatus) {
        default:
        case OrganizationEligibleForGivingStatus.UNKNOWN:
        case OrganizationEligibleForGivingStatus.INELIGBLE:
          returnText = this.i18n.translate(
            'common:textOrganizationIneligibleForDisbursementByYC',
            {},
            'Organization ineligible for disbursement by YourCause'
          );
          break;
        case OrganizationEligibleForGivingStatus.INFO_REQUIRED:
          returnText = this.i18n.translate(
            'common:textOrganizationVettingPendingReview',
            {},
            'Organization vetting pending review'
          );
          break;
        case OrganizationEligibleForGivingStatus.ELIGIBLE:
          break;
        }
    }

      return returnText;
  }
}
