import { Pipe, PipeTransform } from '@angular/core';
import { OrganizationEligibleForGivingStatus } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';

@Pipe({ name: 'gcVettingRequestAlertText' })
export class VettingRequestAlertTextPipe implements PipeTransform {

  constructor (
    private i18n: I18nService
  ) { }

  transform (
    eligibleforGivingStatus: OrganizationEligibleForGivingStatus,
    applicantApplying = false
  ): string {
    let returnText: string;
    if (applicantApplying) {
      returnText = this.i18n.translate(
        'common:textBBGFVettingIndividual',
        {},
        'Programs benefitting individuals are ineligible for disbursement by YourCause. Payments can only be created and processed using client processed budgets and funding sources.'
      );
    } else {
      switch (eligibleforGivingStatus) {
        case OrganizationEligibleForGivingStatus.UNKNOWN:
        case OrganizationEligibleForGivingStatus.INELIGBLE:
        default:
          returnText = this.i18n.translate(
            'GLOBAL:textBBGFVettingRequestDeclined2',
            {},
            'The organization associated with this application is ineligible for disbursement by YourCause. Payments can only be created and processed using client processed budgets and funding sources.'
          );
          break;
        case OrganizationEligibleForGivingStatus.INFO_REQUIRED:
          returnText = this.i18n.translate(
            'GLOBAL:textBBGFVettingRequested2',
            {},
            'An outstanding vetting request has been created for the organization associated with this application. Payments can only be created and processed using client processed budgets and funding sources.'
          );
          break;
        case OrganizationEligibleForGivingStatus.ELIGIBLE:
          break;
        }
    }

      return returnText;
  }
}
