import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, Resolve, Router, RouterStateSnapshot } from '@angular/router';
import { PortalDeterminationService } from '@core/services/portal-determination.service';
import { AuthBehaviors } from '@core/services/token/token-behaviors';
import { TokenService } from '@core/services/token/token.service';

@Injectable({
  providedIn: 'root'
})
export class InitialRedirectResolver implements Resolve<void> {
  constructor (
    private portal: PortalDeterminationService,
    private tokenService: TokenService,
    private router: Router,
    private behaviors: AuthBehaviors
  ) { }

  resolve (next: ActivatedRouteSnapshot, routerState: RouterStateSnapshot) {
    const { routeBase } = this.portal;
    if (routerState.url === `/${routeBase}`) {
      if (!this.tokenService.hasCurrentValidToken()) {
        this.router.navigateByUrl(`/${routeBase}/auth/signin`);
      } else {
        this.router.navigateByUrl(this.behaviors.current.postLoginRedirect);
      }
    }
  }
}
