import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { AddOrgUI } from '@core/typings/ui/add-org.typing';
import { ContactFormGroup } from '@features/add-organization/add-org-modal/add-org-modal.component';
import { AddOrganizationService } from '@features/add-organization/add-organization.service';
import { TypeSafeFormGroup } from '@yourcause/common';
import { Subscription } from 'rxjs';

@Component({
  selector: 'gc-add-org-confirmation',
  templateUrl: './add-org-confirmation.component.html',
  styleUrls: ['./add-org-confirmation.component.scss']
})
export class AddOrgConfirmationComponent implements OnInit, OnDestroy {
  @Input() nonprofitContactGroup: TypeSafeFormGroup<ContactFormGroup>;
  showContactForm = this.addOrgService.get('showContact');
  selectedOrg = this.addOrgService.get('searchResult');
  confirmationText = this.addOrgService.get('confirmationText');
  sub = new Subscription();

  constructor (
    private addOrgService: AddOrganizationService
  ) { }

  ngOnInit () {
    this.addOrgService.setValidation(
      !this.showContactForm || this.nonprofitContactGroup.valid
    );
    this.sub.add(this.nonprofitContactGroup.statusChanges.subscribe(() => {
      this.addOrgService.setValidation(this.nonprofitContactGroup.valid);
    }));
  }

  updateAddOrg <K extends keyof AddOrgUI.AddOrgPayload> (
    attr: K,
    data: AddOrgUI.AddOrgPayload[K]
  ) {
    this.addOrgService.setAddOrgAttr(attr, data);
  }

  ngOnDestroy () {
    this.sub.unsubscribe();
  }

}
