import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { AddOrgUI } from '@core/typings/ui/add-org.typing';
import { DetailsFormGroup } from '@features/add-organization/add-org-modal/add-org-modal.component';
import { AddOrganizationService } from '@features/add-organization/add-organization.service';
import { FileService, TypeSafeFormGroup, YcFile } from '@yourcause/common';
import { Subscription } from 'rxjs';

@Component({
  selector: 'gc-add-org-details',
  templateUrl: './add-org-details.component.html',
  styleUrls: ['./add-org-details.component.scss']
})
export class AddOrgDetailsComponent implements OnInit, OnDestroy {
  @Input() detailsFormGroup: TypeSafeFormGroup<DetailsFormGroup>;

  private sub = new Subscription();

  constructor (
    private addOrgService: AddOrganizationService,
    private fileService: FileService
  ) { }

  ngOnInit () {
    this.addOrgService.setValidation(this.detailsFormGroup.valid);
    this.sub.add(this.detailsFormGroup.statusChanges.subscribe(() => {
      this.addOrgService.setValidation(this.detailsFormGroup.valid);
    }));
  }

  updateAddOrg <K extends keyof AddOrgUI.AddOrgPayload> (
    attr: K,
    data: AddOrgUI.AddOrgPayload[K]
  ) {
    this.addOrgService.setAddOrgAttr(attr, data);
  }

  setImage (image: YcFile) {
    if (image) {
      this.addOrgService.setAddOrgAttr('image', image);
      this.detailsFormGroup.get('image').setValue([image]);
      this.fileService.convertFileToB64(image.file)
        .subscribe((base64: string) => {
          this.addOrgService.setAddOrgAttr('imageUrl', base64);
        });
    } else {
      this.addOrgService.setAddOrgAttr('image', null);
      this.addOrgService.setAddOrgAttr('imageUrl', '');
      this.detailsFormGroup.get('image').setValue([]);
    }
  }

  ngOnDestroy () {
    this.sub.unsubscribe();
  }
}
