import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { ProcessingTypes } from '@core/typings/payment.typing';
import { AddOrgUI } from '@core/typings/ui/add-org.typing';
import { LookupFormGroup } from '@features/add-organization/add-org-modal/add-org-modal.component';
import { AddOrganizationService } from '@features/add-organization/add-organization.service';
import { TypeSafeFormGroup } from '@yourcause/common';
import { Subscription } from 'rxjs';

@Component({
  selector: 'gc-add-org-id-lookup',
  templateUrl: './add-org-id-lookup.component.html',
  styleUrls: ['./add-org-id-lookup.component.scss']
})
export class AddOrgIdLookupComponent implements OnInit, OnDestroy {
  @Input() lookupFormGroup: TypeSafeFormGroup<LookupFormGroup>;
  @Input() disableCountrySelect: boolean;
  @Input() processorType: ProcessingTypes;

  private sub = new Subscription();

  constructor (
    private addOrgService: AddOrganizationService
  ) { }

  get countrySelects$ () {
    if (this.processorType === ProcessingTypes.YourCause) {
      return this.addOrgService.internationalCountrySelects$;
    } else {
      return this.addOrgService.countrySelects$;
    }
  }

  ngOnInit () {
    this.addOrgService.setValidation(this.lookupFormGroup.valid);
    this.sub.add(this.lookupFormGroup.statusChanges.subscribe(() => {
      this.addOrgService.setValidation(this.lookupFormGroup.valid);
    }));
  }

  updateAddOrg <K extends keyof AddOrgUI.AddOrgPayload> (
    attr: K,
    data: AddOrgUI.AddOrgPayload[K]
  ) {
    this.addOrgService.setAddOrgAttr(attr, data);
  }

  ngOnDestroy () {
    this.sub.unsubscribe();
  }
}
