import { Injectable } from '@angular/core';
import { HttpRestService } from '@core/services/http-rest.service';
import { APConfigAPI } from '@core/typings/api/ap-config.typing';

@Injectable({ providedIn: 'root' })
export class APConfigResources {
  constructor (
    private httpRestService: HttpRestService
  ) {}

  fetchAPConfiguration () {
    const endpoint = 'api/manager/accountspayable';

    return this.httpRestService.get(endpoint);
  }

  updateAPConfiguration (payload: APConfigAPI.APReportSettings) {
    const endpoint = 'api/manager/accountsPayable/createAndEdit';

    return this.httpRestService.post(endpoint, payload);
  }

  getOrgsForVendorIdExport (
    payload: APConfigAPI.GetDataForVendorIdExport
  ): Promise<APConfigAPI.OrgDataForVendorIdExport[]> {
    const endpoint = 'api/manager/accountspayable/export/organization';

    return this.httpRestService.post(endpoint, payload);
  }

  getApplicantsForVendorIdExport (
    payload: APConfigAPI.GetDataForVendorIdExport
  ): Promise<APConfigAPI.ApplicantDataForVendorIdExport[]> {
    const endpoint = 'api/manager/accountspayable/export/applicant';

    return this.httpRestService.post(endpoint, payload);
  }

  validateVendorIdImport (
    payload: APConfigAPI.ValidateVendorIdImport
  ) {
    const endpoint = 'api/manager/accountspayable/import/validate';

    return this.httpRestService.post(endpoint, payload);
  }

  updateVendorIds (
    importType: APConfigAPI.VendorIdImportType,
    vendorIds: APConfigAPI.ImportVendorIdModalResponse[]
  ) {
    const endpoint = 'api/manager/accountspayable/import/process';

    return this.httpRestService.post(endpoint, {
      vendorIds,
      importType
    });
  }

}

