import { ChangeDetectorRef, Component, OnDestroy, OnInit } from '@angular/core';
import { AwardService } from '@features/awards/award.service';
import { MyAward } from '@features/awards/typings/award.typing';
import { AutoTableRepository, AutoTableRepositoryFactory, PanelTypes, TopLevelFilter, TypeSafeFormBuilder, TypeSafeFormGroup, ValueComparisonService } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { NotifierService } from '@yourcause/common/notifier';
import { Subscription } from 'rxjs';
interface ApplicantAwardsFormGroup {
  search: string;
}

@Component({
  selector: 'gc-applicant-awards-page',
  templateUrl: './applicant-awards-page.component.html',
  styleUrls: ['./applicant-awards-page.component.scss']
})
export class ApplicantAwardsPageComponent implements OnInit, OnDestroy {
  PanelTypes = PanelTypes;
  sub = new Subscription();
  showApplicationForm = false;
  repository: AutoTableRepository<MyAward>;
  formGroup: TypeSafeFormGroup<ApplicantAwardsFormGroup>;
  awardDesc = '';
  topLevelFilters = [
    new TopLevelFilter(
      'text',
      'awardId',
      '',
      this.i18n.translate(
        'APPLY:textSearchByAwardID', {}, 'Search by award ID'
      )
    )
  ];

  constructor (
    private i18n: I18nService,
    private autoTableFactory: AutoTableRepositoryFactory,
    private notifierService: NotifierService,
    private valueComparisonService: ValueComparisonService,
    private formBuilder: TypeSafeFormBuilder,
    private cdr: ChangeDetectorRef,
    private awardService: AwardService
  ) {
    this.sub.add(
      this.awardService.changesTo$('myAwards')
        .subscribe(() => {
          const rows = this.awardService.get('myAwards');
          this.awardDesc = this.i18n.translate(
            'AWARDS:textYouHaveBeenGrantedNumberAwards',
            {
              number: rows.length
            },
            'You have been granted __number__ awards. See below for details.'
          );
          if (rows) {
            if (!this.repository) {
              this.repository = this.autoTableFactory.create({
                key: 'MY_AWARDS',
                columns: [],
                notifier: this.notifierService,
                valueComparisonService: this.valueComparisonService,
                rows,
                topLevelFilters: this.topLevelFilters,
                formGroup: this.formGroup,
                cdr: this.cdr,
                skipAddToState: true
              });
            } else {
              this.repository.rows = rows;
            }
          }
        })
    );
  }

  ngOnInit () {
    this.formGroup = this.formBuilder.group({
      search: ''
    });
  }


  onSearchChange () {
    this.repository.setTopLevelFilter(
      this.topLevelFilters[0],
      0,
      this.formGroup.value.search
    );
  }

  ngOnDestroy () {
    this.sub.unsubscribe();
  }
}
