import { Component, Input, OnInit } from '@angular/core';
import { TranslationService } from '@core/services/translation.service';
import { FundingSourceTypes } from '@core/typings/budget.typing';
import { AwardService } from '@features/awards/award.service';
import { MyAward } from '@features/awards/typings/award.typing';
import { ModalFactory } from '@yourcause/common/modals';
import { ApplicantPaymentsModalComponent } from '../applicant-payments-modal/applicant-payments-modal.component';

@Component({
  selector: 'gc-award-slat',
  templateUrl: './award-slat.component.html',
  styleUrls: ['./award-slat.component.scss']
})
export class AwardSlatComponent implements OnInit  {
  @Input() award: MyAward;

  portalTranslations = this.translationService.viewTranslations;
  programTranslationMap = this.portalTranslations.Grant_Program;
  AwardTypes = FundingSourceTypes;
  totalUnits = 0;

  constructor (
    private modalFactory: ModalFactory,
    private translationService: TranslationService,
    private awardService: AwardService
  ) { }

  ngOnInit () {
    if (this.award.inKindItems) {
      this.totalUnits = this.award.inKindItems.reduce((acc, award) => {
        return acc + +award.count;
      }, 0);
    }
  }

  paymentModal (award: MyAward) {
    const awardForModal = this.awardService.constructAwards([award])[0];
    this.modalFactory.open(
      ApplicantPaymentsModalComponent,
      {
        award: awardForModal
      }
    );
  }

}
