import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { GCCoreModule } from '@core/_core.module';
import { ApplicationManagerResolver } from '@features/application-manager/resolvers/application-manager.resolver';
import { SystemTagsResolver } from '@features/system-tags/resolvers/system-tags.resolver';
import { ApplicantApplicationsComponent } from './applicant-applications/applicant-applications.component';
import { ApplicantAwardsComponent } from './applicant-awards/applicant-awards.component';
import { ApplicantOrganizationsComponent } from './applicant-organizations/applicant-organizations.component';
import { ApplicantProfileWrapperComponent } from './applicant-profile-wrapper/applicant-profile-wrapper.component';
import { ApplicantProfileResolver } from './applicant-profile.resolver';
import { ApplicantModule } from './applicant.module';

@NgModule({
  imports: [
    GCCoreModule,
    ApplicantModule,
    RouterModule.forChild([{
      path: ':id',
      component: ApplicantProfileWrapperComponent,
      resolve: {
        applicant: ApplicantProfileResolver,
        appManager: ApplicationManagerResolver,
        tags: SystemTagsResolver
      },
      children: [{
        path: 'applications',
        component: ApplicantApplicationsComponent,
        data: {
          breadcrumbLabel: 'Applicant Profile',
          breadcrumb: true,
          pageName: 'Applicant - Applications Tab'
        }
      }, {
        path: 'nominations',
        component: ApplicantApplicationsComponent,
        data: {
          breadcrumbLabel: 'Applicant Profile',
          breadcrumb: true,
          pageName: 'Applicant - Nominations Tab'
        }
      }, {
        path: 'organizations',
        component: ApplicantOrganizationsComponent,
        data: {
          breadcrumbLabel: 'Applicant Profile',
          breadcrumb: true,
          pageName: 'Applicant - Organizations Tab'
        }
      }, {
        path: 'awards',
        component: ApplicantAwardsComponent,
        data: {
          breadcrumbLabel: 'Applicant Profile',
          breadcrumb: true,
          pageName: 'Applicant - Awards Tab'
        }
      }, {
        path: '',
        pathMatch: 'full',
        redirectTo: 'applications'
      }]
    }])
  ]
})
export class ApplicantRoutingModule { }
