import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { ApplicationDetail, ApplicationForUi, ApplicationFromApi, ApplicationFromPaginated, BulkApplicationDetail } from '@core/typings/application.typing';
import { ProcessingTypes } from '@core/typings/payment.typing';
import { ApplicationStatuses } from '@core/typings/status.typing';
import { WorkflowManagerActions } from '@core/typings/workflow.typing';
import { EmployeeSSOFieldsData } from '@features/employee-sso-fields/employee-sso-fields.typing';
import { OrganizationEligibleForGivingStatus } from '@yourcause/common';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { skip, timer } from 'rxjs';
import { ApplicationManagerService } from './application-manager.service';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(ApplicationManagerService, {
  imports: [
    GCMockModule
  ]
})
export class ApplicationManagerServiceSpec implements Spec<ApplicationManagerServiceSpec, ApplicationManagerService> {
  appId = 1;
  applicantId = 2;
  orgId = 3;
  cycleId = 4;
  formId = 5;
  revisionId = 6;
  workflowId = 7;
  workflowLevelId = 8;
  programId = 9;
  applicationFormId = 10;
  textFieldKey = 'textFieldKey';
  textFieldId = 757;
  grantProgramCycle = {
    id: this.cycleId,
    name: 'Cycle name',
    startDate: '',
    endDate: '',
    grantProgramId: this.programId,
    isArchived: false,
    clientOrganizationsProcessingTypeId: ProcessingTypes.YourCause,
    isClientProcessing: false
  };
  applicationFromApi: ApplicationFromApi = {
    reservedFunds: false,
    budgetId: null,
    fundingSourceId: null,
    applicationGuid: 'abc-123',
    applicationId: this.appId,
    applicationStatus: ApplicationStatuses.Approved,
    submittedDate: '',
    applicantId: this.applicantId,
    applicantFirstName: 'Lukie',
    applicantCanReceiveEmails: true,
    applicantLastName: 'Smith',
    applicantEmail: 'lukie@mailinator.com',
    applicantPhoneNumber: '',
    applicationProfileImageUrl: '',
    applicantAddress: '',
    programName: 'Program name',
    programId: this.programId,
    grantProgramTimezone: 'UTC',
    organizationId: this.orgId,
    charityId: 5,
    organizationName: 'Org name',
    orgnizationImageUrl: '',
    organizationIdentification: '12-3415',
    organizationAddress: '',
    isPrivateOrg: false,
    daysInReview: 0,
    isApplicationInClientUserWorkflowLevel: true,
    workflowLevelApplicationReview: null,
    reviewRuleType: null,
    totalWorkflowLevelReviews: 4,
    totalWorkflowLevelUsers: 4,
    majorityReviewed: false,
    currentWorkflowName: 'WFL name',
    currentWorkFlowLevelName: 'WF name',
    parentWorkFlowLevelName: 'Parent WFL name',
    parentWorkFlowLevelId: 1,
    canRouteApplication: true,
    canArchiveAndUnarchiveApplication: true,
    canApproveApplication: true,
    canAwardApplication: true,
    canDeclineApplication: true,
    canArchiveApplication: true,
    canExtendFormDueDate: true,
    canRecommendFunding: true,
    hasNonPaidPayments: true,
    workflowLevelUsersWithoutReviews: [],
    workflowLevelRoutes: [],
    managerWillComplete: true,
    isDraft: true,
    programAllowCollaboration: true,
    programSendEmailsToCollaborators: true,
    amountRequested: 500,
    paymentDesignation: 'payment designation',
    careOf: null,
    createdBy: null,
    lastUpdatedBy: null,
    updatedDate: '',
    createdDate: '',
    nonprofitGuid: 'guid',
    currentWorkFlowLevelId: this.workflowLevelId,
    currentWorkFlowId: this.workflowId,
    isArchived: true,
    nominee: null,
    nominationApplicationId: 6,
    canAccessNominationApplication: true,
    isMasked: true,
    canViewMaskedApplicantInfo: false,
    specialHandlingAddress1: 'Address 1',
    specialHandlingAddress2: 'Address 2',
    specialHandlingCity: 'City',
    specialHandlingCountry: 'US',
    specialHandlingName: 'Special Handling name',
    specialHandlingPostalCode: '32690',
    specialHandlingStateProvinceRegion: 'SC',
    specialHandlingNotes: '',
    specialHandlingFileUrl: null,
    specialHandlingReason: '',
    currencyRequestedAmountEquivalent: 0,
    currencyRequested: 'USD',
    inKindAmountRequested: 0,
    inKindItems: [],
    isProgramArchived: true,
    meetsVettingRequirement: true,
    grantProgramCycle: this.grantProgramCycle,
    hasAwards: false,
    hasPayments: false,
    hasCashAward: false,
    organizationEligibleForGivingStatus: OrganizationEligibleForGivingStatus.ELIGIBLE,
    recommendedFundingAmount: 0,
    orgIsInternational: false,
    applicantIsSSO: false,
    currentWorkflowLevelShowBudgetSummaryInfo: true,
    currentWorkflowLevelAllowUserToViewAwardsAndPayments: true,
    currentWorkflowLevelAllowAward: true
  };
  applicationForUi: ApplicationForUi = {
    ...this.applicationFromApi,
    amountRequestedForEdit: this.applicationFromApi.currencyRequestedAmountEquivalent,
    designation: ''
  };
  application: ApplicationFromPaginated = {
    reservedFunds: false,
    applicationId: this.appId,
    applicantId: this.applicantId,
    applicantFirstName: '',
    applicantLastName: '',
    applicationLastName: '',
    applicantFullName: '',
    applicantEmail: '',
    applicantAddress: '',
    applicantAddressInfo: null,
    applicantPhoneNumber: '',
    applicantProfileImageUrl: '',
    organizationId: this.orgId,
    organizationName: '',
    nonprofitGuid: '',
    organizationIdentification: '',
    organizationImageUrl: '',
    organizationAddressInfo: null,
    orgPhoneNumber: '',
    programId: this.programId,
    programName: '',
    submittedDate: '',
    applicationStatus: ApplicationStatuses.Approved,
    isApplicationArchived: true,
    workflowId: this.workflowId,
    workflowName: '',
    currentWorkflowLevelId: this.workflowLevelId,
    currentWorkflowLevelName: '',
    createdBy: null,
    lastUpdatedBy: null,
    dateCreated: '',
    amountRequested: 500,
    nominatedBy: null,
    nominationApplicationId: null,
    dateAddedToWorkflow: '',
    daysInWfl: 5,
    paymentStats: null,
    canArchive: true,
    canViewMaskedApplicantInfo: true,
    isMasked: true,
    isDraft: true,
    programAllowCollaboration: true,
    currencyRequested: '',
    currencyRequestedAmountEquivalent: 0,
    inKindItemTotalCount: 0,
    inKindAmountRequested: 0,
    isEdVetted: true,
    isArchived: true,
    isProgramArchived: true,
    clientOrganizationsProcessingTypeId: ProcessingTypes.YourCause,
    grantProgramCycle: this.grantProgramCycle,
    budgetId: null,
    fundingSourceId: null,
    careOf: null,
    organizationEligibleForGivingStatus: OrganizationEligibleForGivingStatus.ELIGIBLE,
    recommendedFundingAmount: 0,
    orgIsInternational: false,
    parentOrganizationName: '',
    parentOrganizationGuid: '',
    organizationNPPIsActive: true,
    updatedDate: '',
    grantProgramCycleId: this.cycleId,
    grantProgramCycleName: 'cycle name',
    designation: ''
  };

  @BeforeEach()
  async prepare (service: ApplicationManagerService) {
    service['applicationViewService']['applicationViewResources'].getApplication = async () => {
      return this.applicationFromApi;
    };
    service['employeeSSOFieldsService']['employeeSSOFieldsResources'].getEmployeeSSOFieldsForApp = async () => {
      return {} as EmployeeSSOFieldsData;
    };
    service['applicationManagerResources'].getBulkApplicationDetails = async () => {
      return {
        applicationBulkActionIndicators: [{
          applicationId: this.appId,
          inKindItems: [],
          hasAwards: true,
          hasPayments: true,
          currentWorkflowLevelId: this.workflowLevelId,
          hasCashAward: true,
          hasInKindAward: false,
          programHasCashBudget: true,
          programHasInKindBudget: false,
          workflowLevelRoutes:[]
        } as BulkApplicationDetail]
      };
    };
    service['referenceFieldsService'].getReferenceFieldResponses = async () => {
      return {};
    };
    service['applicationActionService']['workflowService']['workflowResources'].getMyWorkflowManagerRoles = async () => {
      return [{
        workflowId: this.workflowId,
        workflowActions: [
          WorkflowManagerActions.Approve,
          WorkflowManagerActions.NotifyOfStatus
        ]
      }, {
        workflowId: this.workflowId,
        workflowActions: [
          WorkflowManagerActions.Approve,
          WorkflowManagerActions.NotifyOfStatus
        ]
      }];
    };
    service['applicationActionService']['workflowService'].setMyWorkflowManagerRolesMap();
  }

  @TestCase('should be able to get app for view form modal')
  async getApplicationForViewFormModal (service: ApplicationManagerService) {
    const app = await service.getApplicationForViewFormModal(
      this.appId,
      this.applicationFormId,
      [],
      {}
    );
    expect(app.applicationStatus).to.be.equal(this.applicationFromApi.applicationStatus);
    expect(app.isArchived).to.be.equal(this.applicationFromApi.isArchived);
  }

  @TestCase('should be able to set app mangager map')
  setApplicationManagerMap (service: ApplicationManagerService) {
    service.setApplicationManagerMap(
      this.appId,
      'isInWorkflowLevel',
      true
    );
    service.setApplicationManagerMap(
      this.appId,
      'programTimezone',
      'UTC'
    );
    const map = service.applicationManagerMap[this.appId];
    expect(map.isInWorkflowLevel).to.be.true;
    expect(map.programTimezone).to.be.equal('UTC');
  }

  @TestCase('should be able to set permissions for app manager map')
  setPermissionsForApplicationManagerMap (service: ApplicationManagerService) {
    service.setPermissionsForApplicationManagerMap(this.application);
    const map = service.applicationManagerMap[this.application.applicationId];
    expect(map.canApprove).to.be.false;
    expect(map.canNotifyOfStatus).to.be.true;
    expect(map.canManageCollabs).to.be.false;
  }

  @TestCase('should be able to update app manager map')
  updateApplicationManagerMap (service: ApplicationManagerService) {
    const newDetail = {
      isInWorkflowLevel: true,
      forms: [],
      canAccessNominationApplication: true,
      programTimezone: 'UTC'
    } as ApplicationDetail;
    service.updateApplicationManagerMap(this.appId, newDetail);
    const map = service.applicationManagerMap[this.appId];
    expect(map.isInWorkflowLevel).to.be.equal(newDetail.isInWorkflowLevel);
    expect(map.canAccessNominationApplication).to.be.equal(newDetail.canAccessNominationApplication);
    expect(map.programTimezone).to.be.equal(newDetail.programTimezone);
  }

  @TestCase('should be able to get bulk application details map')
  async getBulkApplicationDetailsMap (service: ApplicationManagerService) {
    const map = await service.getBulkApplicationDetailsMap([this.appId]);
    const hasSetDetail = !!map[this.appId];
    expect(hasSetDetail).to.be.true;
  }

  @TestCase('should be able to get all apps with awards')
  async getAllAppsWithAwards (service: ApplicationManagerService) {
    let result = await service.getAllAppsWithAwards([this.appId]);
    const hasMyAppWithAward = result.includes(this.appId);
    expect(hasMyAppWithAward).to.be.true;
    // Set has awards to false
    service['applicationManagerResources'].getBulkApplicationDetails = async () => {
      return {
        applicationBulkActionIndicators: [{
          applicationId: this.appId,
          inKindItems: [],
          hasAwards: false,
          hasPayments: true,
          currentWorkflowLevelId: this.workflowLevelId,
          hasCashAward: true,
          hasInKindAward: false,
          programHasCashBudget: true,
          programHasInKindBudget: false,
          workflowLevelRoutes:[]
        } as BulkApplicationDetail]
      };
    };
    result = await service.getAllAppsWithAwards([this.appId]);
    const hasNoAppWithAward = result.length === 0;
    expect(hasNoAppWithAward).to.be.true;
  }

  @TestCase('should be able to get top level filters for app manager')
  getTopLevelFiltersForApplicationManager (service: ApplicationManagerService) {
    const filters = service.getTopLevelFiltersForApplicationManager(
      []
    );
    const hasSearchByDropdown = filters.some((filter) => {
      return filter.filterType === 'searchByDropdown' &&
        filter.column === 'applicationId';
    });
    const hasAppTypeFilter = filters.some((filter) => {
      return filter.filterType === 'typeaheadSingleEquals' &&
        filter.column === 'isArchived';
    });
    const hasAppStatusFilter = filters.some((filter) => {
      return filter.filterType === 'checkboxDropdown' &&
        filter.column === 'applicationStatus';
    });
    const hasDateFilter = filters.some((filter) => {
      return filter.filterType === 'relativeDate' &&
        filter.column === 'updatedDate';
    });
    expect(hasSearchByDropdown).to.be.true;
    expect(hasAppTypeFilter).to.be.true;
    expect(hasAppStatusFilter).to.be.true;
    expect(hasDateFilter).to.be.true;
  }

  @TestCase('should be able to get relative date filter for apps')
  getRelativeDateFilterForApplications (service: ApplicationManagerService) {
    const result = service.getRelativeDateFilterForApplications();
    expect(result.filterType).to.be.equal('relativeDate');
    expect(result.column).to.be.equal('updatedDate');
  }

  @TestCase('should be able to get are manager options visible')
  async areAppManagerOptionsVisible (service: ApplicationManagerService) {
    const appManagerMap = {
      [this.appId]: {
        isInWorkflowLevel: true,
        forms: [],
        canAccessNominationApplication: true,
        programTimezone: 'UTC',
        canApprove: true, // has one permission so should be visible
        canDecline: false,
        canArchiveUnarchive: false,
        canAwardPay: false,
        canDeleteDraft: false,
        canNotifyOfStatus: false,
        canRoute: false,
        canUpdateCycle: false,
        canUpdateStatus: false,
        canViewComms: false,
        canManageCollabs: false,
        canSendMailMerge: false
      } as ApplicationDetail
    };
    let visible = service.areAppManagerOptionsVisible(
      appManagerMap,
      this.application,
      false
    );
    // remove only permission
    expect(visible).to.be.true;
    appManagerMap[this.appId].canApprove = false;
    visible = service.areAppManagerOptionsVisible(
      appManagerMap,
      this.application,
      false
    );
    expect(visible).to.be.false;
  }

  @TestCase('should be able to trigger reset app manager table')
  async triggerResetAppManagerTable (service: ApplicationManagerService) {
    let triggered = false;
    service.changesTo$('triggerResetAppManagerTable').pipe(skip(1)).subscribe(() => {
      triggered = true;
    });
    service.triggerResetAppManagerTable();
    await timer(1000).toPromise();

    expect(triggered).to.be.true;
  }
}

