import { Component } from '@angular/core';
import { Router } from '@angular/router';
import { SpinnerService } from '@core/services/spinner.service';
import { BudgetService } from '@features/budgets/budget.service';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { ProgramService } from '@features/programs/program.service';
import { ALL_SKIP_FILTER, ArrayHelpersService, DEFAULT_DOUBLE_VALUE, TableDataFactory, TopLevelFilter } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { ModalFactory } from '@yourcause/common/modals';
import { BudgetAssignmentsService } from '../budget-assignments.service';
import { BudgetAssignmentApplicationHistory } from '../budget-assignments.typing';
import { UpdateBudgetAssignmentModalComponent } from '../update-budget-assignment/update-budget-assignment.component';

@Component({
  selector: 'gc-budget-assignment-history-page',
  styleUrls: ['./budget-assignment-history-page.component.scss'],
  templateUrl: './budget-assignment-history-page.component.html'
})
export class BudgetAssignmentHistoryPageComponent {
  defaultCurrency = this.clientSettingsService.defaultCurrency;
  tableDataFactory: TableDataFactory<BudgetAssignmentApplicationHistory> = this.budgetAssignmentService.historyDataFactory;

  options = [{
    display: this.i18n.translate('CONFIG:textAllRules', {}, 'All rules'),
    value: DEFAULT_DOUBLE_VALUE,
    dependentOptions: [{
      display: '',
      value: ALL_SKIP_FILTER
    }]
  }, {
    display: this.i18n.translate('common:lblBudget', {}, 'Budget'),
    value: 'assignedBudgetId',
    dependentOptions: [{
      display: this.i18n.translate('CONFIG:textAllBudgets', {}, 'All budgets'),
      value: ALL_SKIP_FILTER
    }].concat(this.arrayHelper.sort(
      this.budgetService.get('allBudgetOptions').map(budget => {
        return {
          display: budget.label,
          value: budget.value
        };
      }), 'display'))
  }, {
    display: this.i18n.translate('common:lblProgram', {}, 'Program'),
    value: 'programId',
    dependentOptions: [{
      display: this.i18n.translate('common:textAllPrograms', {}, 'All programs'),
      value: ALL_SKIP_FILTER
    }].concat(this.arrayHelper.sort(
      this.programService.allPublishedActivePrograms.map(program => ({
        display: program.grantProgramName,
        value: program.grantProgramId
      })), 'display'))
  }];

  topLevelFilters = [
    new TopLevelFilter(
      'doubleDropdownEquals',
      'ruleProgramFormWorkflowFilter',
      null,
      null,
      {
        selectOptions: this.options
      },
      this.i18n.translate(
        'GLOBAL:textSearchRulesByBudgetOrProgram',
        {},
        'Search rules by budget or program'
      ),
      null
    )
  ];

  constructor (
    private programService: ProgramService,
    private arrayHelper: ArrayHelpersService,
    private budgetService: BudgetService,
    private budgetAssignmentService: BudgetAssignmentsService,
    private i18n: I18nService,
    private clientSettingsService: ClientSettingsService,
    private router: Router,
    private spinner: SpinnerService,
    private modalFactory: ModalFactory
  ) { }


  async updateBudget (
    record: BudgetAssignmentApplicationHistory
  ) {
    const result = await this.modalFactory.open(
      UpdateBudgetAssignmentModalComponent,
      {
        programName: record.programName,
        programId: record.programId,
        cycleName: record.cycleName,
        cycleId: record.cycleId,
        currentBudgetName: record.assignedBudgetName,
        currentBudgetId: record.assignedBudgetId,
        currentFundingSourceName: record.assignedFundingSourceName,
        currentFundingSourceId: record.assignedFundingSourceId,
        amountRequested: record.amountRequested,
        applicationIds: [record.applicationId],
        isCurrentlyReserved: record.reservedFunds
      }
    );

    if (result) {
      this.spinner.startSpinner();
      await this.budgetAssignmentService.updateApplicationBudgetFundingSource([record.applicationId], result.budgetId, result.fundingSourceId, result.reserveFunds);
      this.spinner.stopSpinner();
    }
  }

  handleNpoSelect (
    record: BudgetAssignmentApplicationHistory
  ) {
    this.router.navigate([
      `/management/nonprofit/${record.nonprofitGuid || record.organizationId}/profile`
    ]);
  }
}
