import { Component } from '@angular/core';
import { PanelTypes, Tab } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { ModalFactory } from '@yourcause/common/modals';
import { BudgetImportModalComponent } from '../budget-import-modal/budget-import-modal.component';
import { BudgetService } from '../budget.service';

@Component({
  selector: 'gc-budgets-wrapper',
  templateUrl: './budgets-wrapper.component.html',
  styleUrls: ['./budgets-wrapper.component.scss']
})
export class BudgetsWrapperComponent {
  PanelTypes = PanelTypes;
  tabs: Tab[] = [{
    link: '/management/program-setup/budgets/open',
    labelKey: 'BUDGET:textOpenBudgets',
    label: 'Open Budgets'
  }, {
    link: '/management/program-setup/budgets/closed',
    labelKey: 'BUDGET:textClosedBudgets',
    label: 'Closed Budgets'
  }, {
    link: '/management/program-setup/budgets/assignments',
    labelKey: 'BUDGET:textBudgetAssignments',
    label: 'Budget Assignments'
  }, {
    link: '/management/program-setup/budgets/assignment-history',
    labelKey: 'CONFIG:hdrAssignmentHistory',
    label: 'Assignment History'
  }];
  createBudgetText = this.i18n.translate(
    'BUDGET:textCreateBudget',
    {},
    'Create budget'
  );

  createBudgetAssignmentText = this.i18n.translate(
    'BUDGET:textCreateBudgetAssignment',
    {},
    'Create budget assignment'
  );

  constructor (
    private i18n: I18nService,
    private modalFactory: ModalFactory,
    private budgetService: BudgetService
  ) {}

  async openImportBudgetsModal () {
    const result = await this.modalFactory.open(
      BudgetImportModalComponent,
      {}
    );
    if (result) {
      await this.budgetService.handleBudgetImport(result);
    }
  }
}
