import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { CharityBucket } from '@core/typings/charity-buckets.typings';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { CharityBucketsResources } from './charity-buckets.resources';
import { CharityBucketsService } from './charity-buckets.service';


@Injectable({ providedIn: 'root' })
@DescribeAngularService(CharityBucketsService, {
  imports: [
    GCMockModule
  ]
})
export class CharityBucketsServiceSpec implements Spec<CharityBucketsServiceSpec, CharityBucketsService> {
  bucketId = '1';
  charityBuckets: CharityBucket[] = [{
    name: 'Test Bucket',
    bucketId: this.bucketId,
    charityCount: {
      numberOfCharities: 500,
      asOfDate: new Date().toString()
    }
  }];

  constructor (
    private charityBucketsResources: CharityBucketsResources
  ) { }

  @BeforeEach()
  mock (service: CharityBucketsService) {
    service['charityBucketsResources'] = this.charityBucketsResources;
    this.charityBucketsResources.getCharityBuckets = async () => {
      return this.charityBuckets;
    };
  }

  @TestCase('should be able get and set charityBuckets')
  async setCharityBuckets (
    service: CharityBucketsService
  ) {
    await service.getCharityBuckets();
    const buckets = service.charityBuckets;
    expect(buckets.length).to.be.equal(this.charityBuckets.length);
    expect(buckets[0].bucketId).to.be.equal(this.bucketId);
  }

}
