import { Injectable } from '@angular/core';
import { HttpRestService } from '@core/services/http-rest.service';
import { ClientIntAuthorities, ClientSettings, CurrencySetting, SelectedLangsFromApi } from '@core/typings/api/admin-client.typing';
import { ClientBrandingFromApi, ColorSchemePayload, SaveBrandingForApi, UpdateClientCurrencyPayload } from '@core/typings/branding.typing';

@Injectable({ providedIn: 'root' })
export class ClientSettingsResources {

  constructor (
    private  httpRestService: HttpRestService
  ) { }

  getClientSettings (): Promise<ClientSettings> {
    const endpoint = 'api/client/settings/';

    return this.httpRestService.get(endpoint);
  }

  getSelectedLanguages (): Promise<SelectedLangsFromApi[]> {
    const endpoint = 'api/client/languages';

    return this.httpRestService.get(endpoint);
  }

  getSelectedCurrencies (): Promise<CurrencySetting[]> {
    const endpoint = 'api/client/currencies';

    return this.httpRestService.get(endpoint);
  }

  getRegistrationAuthoritiees (countryCode: string): Promise<ClientIntAuthorities> {
    const endpoint = `/api/lookup/GetRegistrationAuthorities?countryCode=${countryCode}`;

    return this.httpRestService.get(endpoint);
  }

  updateClientLanguages (cultures: string[]) {
    const endpoint = 'api/client/languages/update';

    return this.httpRestService.post(endpoint, {
      cultures
    });
  }

  updateClientCurrencies (payload: UpdateClientCurrencyPayload) {
    const endpoint = 'api/client/currencies/update';

    return this.httpRestService.post(endpoint, payload);
  }

  getBranding (): Promise<ClientBrandingFromApi> {
    const endpoint = 'api/client/branding';

    return this.httpRestService.get(endpoint);
  }

  saveBranding (payload: SaveBrandingForApi) {
    return this.httpRestService.post('api/client/branding', payload);
  }

  saveColorScheme (colorSchemes: ColorSchemePayload) {
    const endpoint = 'api/client/ClientColorScheme';

    return this.httpRestService.post(endpoint, colorSchemes);
  }

  getClientSettingsForApplicant (clientID: number): Promise<ClientSettings> {
    return this.httpRestService.get(`api/portal/programs/clientSettings/${clientID}`);
  }

  getAvailableApplicantCurrencies (programId: number): Promise<CurrencySetting[]> {
    const endpoint = `api/portal/programs/clientCurrencies/${programId}`;

    return this.httpRestService.get(endpoint);
  }
}

