import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { AfterEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect, spy } from 'chai';
import { FormDownloadService } from './form-download.service';
import { Form } from './form.typing';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(FormDownloadService, {
  imports: [
    GCMockModule
  ]
})
export class FormDownloadServiceSpec implements Spec<FormDownloadServiceSpec, FormDownloadService> {

  @AfterEach()
  cleanup () {
    spy.restore();
  }

  @TestCase('should be able to handle form download - pass')
  async handleFormDownload (service: FormDownloadService) {
    spy.on(service['pdfService'], 'generateFormPdf', async () => {
      return 'url';
    });
    spy.on(service['fileService'], 'downloadUrlAs', async () => {});
    const form = {} as Form;
    await service.handleFormDownload('2', form, []);
    expect(service['pdfService']['generateFormPdf']).to.have.been.called.once;
    expect(service['fileService']['downloadUrlAs']).to.have.been.called.once;
  }

  @TestCase('should be able to handle form download - fail')
  async handleFormDownloadFail (service: FormDownloadService) {
    spy.on(service['logger'], 'error');
    spy.on(service['notifier'], 'error');
    spy.on(service['pdfService'], 'generateFormPdf', async () => {
      /* eslint-disable no-throw-literal */
      throw {
        error: {
          message: 'Error'
        }
      };
    });
    const form = {} as Form;
    await service.handleFormDownload('2', form, []);
    expect(service['logger']['error']).to.have.been.called.once;
    expect(service['notifier']['error']).to.have.been.called.once;
  }

}
