import { Injectable } from '@angular/core';
import { PDFService } from '@core/services/pdf.service';
import { Form, FormComponentsByTab } from '@features/configure-forms/form.typing';
import { FileService } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { LogService } from '@yourcause/common/logging';
import { NotifierService } from '@yourcause/common/notifier';

@Injectable({ providedIn: 'root' })
export class FormDownloadService {

  constructor (
    private pdfService: PDFService,
    private i18n: I18nService,
    private notifier: NotifierService,
    private logger: LogService,
    private fileService: FileService
  ) { }

  /**
   * Handles downloading a custom form (non application related)
   *
   * @param revisionVersion: Revision version
   * @param form: Form to download
   * @param formComponentsByTab: Array of form components  by tab
   */
  async handleFormDownload (
    revisionVersion: string,
    form: Form,
    formComponentsByTab: FormComponentsByTab[]
  ) {
    try {
      const downloadUrl = await this.pdfService.generateFormPdf({
        form,
        revisionVersion,
        formComponentsByTab
      });
      if (downloadUrl) {
        const formText = this.i18n.translate(
          'GLOBAL:textForm',
          {},
          'Form'
        );
        this.fileService.downloadUrlAs(downloadUrl, formText + '.pdf');
      }
    } catch (e) {
      this.logger.error(e);
      this.notifier.error(this.i18n.translate(
        'common:textErrorDownloadPDF',
        {},
        'There was an error downloading the PDF'
      ));
    }
  }
}
