import { Injectable } from '@angular/core';
import { SimpleStringMap } from '@yourcause/common';
import moment from 'moment';
import { SharedDownloadsResources } from './shared-downloads.resources';

@Injectable({ providedIn: 'root' })
export class SharedDownloadsService {
  constructor (
    readonly sharedDownloadsResources: SharedDownloadsResources
  ) { }
  async evaluateDownloadUrl ({ downloadLink, userGeneratedReportID, expirationParam, fileName }: SimpleStringMap<string>) {

    const expiration = moment.utc(expirationParam);
    // if there is no userGeneratedReportId, this means it's a payment proc report and the user should
    // be able to download
    const fileExists = userGeneratedReportID ? await this.checkLinkValidity(+userGeneratedReportID) : true;

    return {
      downloadLink,
      expiration,
      fileName,
      userGeneratedReportID,
      fileExists
    };
  }
  async checkLinkValidity (id: number) {
    const response = await this.sharedDownloadsResources.checkFileExistsForReport(id);

    return response;
  }
}
