import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { EmployeeSSOFieldsResources } from './employee-sso-fields.resources';
import { EmployeeSSOFieldsService } from './employee-sso-fields.service';
import { EmployeeSSOFieldFromApi, EmployeeSSOFields, EmployeeSSOFieldsData } from './employee-sso-fields.typing';


@Injectable({ providedIn: 'root' })
@DescribeAngularService(EmployeeSSOFieldsService, {
  imports: [
    GCMockModule
  ]
})
export class EmployeeSSOFieldsServiceSpec implements Spec<EmployeeSSOFieldsServiceSpec, EmployeeSSOFieldsService> {
  employeeSSOFieldsFromApi: EmployeeSSOFieldFromApi[] = [{
    name: 'Region',
    inUse: true,
    column: EmployeeSSOFields.Region
  }, {
    name: 'Division',
    inUse: false,
    column: EmployeeSSOFields.Division
  }];
  employeeSSOFieldData: EmployeeSSOFieldsData[] = [{
    region: 'Region',
    employeeTypeCode: 'Employee Type Code',
    reportGroupName: 'Report Group Name',
    division: 'Division',
    departmentName: 'Department',
    customChar1: 'Custom 1',
    customChar2: 'Custom 2',
    customChar3: 'Custom 3',
    customChar4: 'Custom 4',
    customChar5: 'Custom 5',
    customChar6: 'Custom 6',
    customChar7: 'Custom 7',
    customChar8: 'Custom 8',
    customChar9: 'Custom 9',
    customChar10: 'Custom 10',
    externalEmployeeId2: 'External Employee ID 2',
    fullTimeCodeId: 'Full Time Code ID',
    hireDate: 'Hire Date',
    jobCodeId: 'Job Code ID',
    siteCodeId: 'Site Code ID',
    externalEmployeeId: 'External employee ID'
  }];

  constructor (
    private employeeSSOFieldsResources: EmployeeSSOFieldsResources
  ) { }

  @BeforeEach()
  mock (service: EmployeeSSOFieldsService) {
    service['employeeSSOFieldsResources'] = this.employeeSSOFieldsResources;
    this.employeeSSOFieldsResources.getEmployeeSSOFields = async () => {
      return this.employeeSSOFieldsFromApi;
    };
    this.employeeSSOFieldsResources.getEmployeeSSOFieldData = async () => {
      return this.employeeSSOFieldData;
    };
  }

  @TestCase('should be able get and set employeeSSOFields')
  async setEmployeeSSOFields (
    service: EmployeeSSOFieldsService
  ) {
    await service.setEmployeeSSOFields();
    const fields = service.employeeSSOFields;
    expect(fields.length).to.be.equal(2);
  }

  @TestCase('should be able adapt employeeSSOFields with columnName and attr')
  async adaptEmployeeSSOFields (
    service: EmployeeSSOFieldsService
  ) {
    await service.setEmployeeSSOFields();
    const fields = service.employeeSSOFields;
    const regionColumn = fields.find((col) => {
      return col.column === EmployeeSSOFields.Region;
    });
    expect(regionColumn.inUse).to.be.equal(true);
    expect(regionColumn.attr).to.be.equal('region');
  }

  @TestCase('should be able get and set setEmployeeSSOFieldData')
  async setEmployeeSSOFieldData (
    service: EmployeeSSOFieldsService
  ) {
    await service.setEmployeeSSOFieldData();
    const data = service.employeeSSOFieldData;
    expect(data.length).to.be.equal(1);
    expect(data[0].region).to.be.equal('Region');
  }

  @TestCase('should be able get and set setEmployeeColumnsForAutomation')
  async setEmployeeColumnsForAutomation (
    service: EmployeeSSOFieldsService
  ) {
    await service.setEmployeeSSOFields();
    service.setEmployeeColumnsForAutomation();
    const fields = service.get('employeeColumnsForAutomation');
    expect(fields.length).to.be.equal(1);
    expect(fields[0].key).to.be.equal('region');
  }

  @TestCase('should be able get and set setApplicantColumnsForReporting')
  async setApplicantColumnsForReporting (
    service: EmployeeSSOFieldsService
  ) {
    await service.setEmployeeSSOFields();
    service.setApplicantColumnsForReporting();
    const fields = service.get('applicantColumnsForReporting');
    expect(fields.length).to.be.equal(1);
    expect(fields[0].column).to.be.equal('region');
  }


}
