import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, Resolve } from '@angular/router';
import { AdHocReportingDefinitions } from '@features/reporting/services/ad-hoc-reporting-definitions.service';
import { WorkflowLevelAutomationService } from '@features/workflow-automation/workflow-level-automation.service';
import { EmployeeSSOFieldsService } from '../employee-sso-fields.service';

@Injectable({ providedIn: 'root' })
export class EmployeeSSOFieldsConfigurationResolver implements Resolve<boolean> {

  constructor (
    private employeeSSOFieldsService: EmployeeSSOFieldsService,
    private workflowLevelAutomationService: WorkflowLevelAutomationService,
    private adHocReportingDefinitions: AdHocReportingDefinitions
  ) { }

  async resolve (next: ActivatedRouteSnapshot) {
    await this.employeeSSOFieldsService.setEmployeeSSOFields();
    if (next.data.isWorkflowAutomation) {
      this.workflowLevelAutomationService.resetWorkflowObjects(
        next.data.isAdvancement
      );
    }
    if (next.data.isReporting) {
      this.adHocReportingDefinitions.setApplicantColumnsForReporting();
    }

    return true;
  }
}
