import { Component, OnInit } from '@angular/core';
import { SpinnerService } from '@core/services/spinner.service';
import { ArrayHelpersService, AutoTableRepositoryFactory, TypeSafeFormBuilder, TypeSafeFormGroup, TypeToken } from '@yourcause/common';
import { AnalyticsService, EventType } from '@yourcause/common/analytics';
import { YCModalComponent } from '@yourcause/common/modals';
import { EmployeeSSOFieldsService } from '../employee-sso-fields.service';
import { EmployeeSSOField, EmployeeSSOFields } from '../employee-sso-fields.typing';
interface SsoGroup {
  [x: string]: string|boolean;
}

@Component({
  selector: 'gc-update-employee-sso-fields-configuration-modal',
  templateUrl: './update-employee-sso-fields-configuration-modal.component.html',
  styleUrls: ['./update-employee-sso-fields-configuration-modal.component.scss']
})
export class UpdateEmployeeSSOFieldsConfigurationModalComponent extends YCModalComponent<void> implements OnInit {
  formGroup: TypeSafeFormGroup<SsoGroup>;
  employeeSSOFields = this.employeeSSOFieldsService.employeeSSOFields;
  $string = new TypeToken<string>();

  constructor (
    private employeeSSOFieldsService: EmployeeSSOFieldsService,
    private formBuilder: TypeSafeFormBuilder,
    private spinnerService: SpinnerService,
    private autoTableRepositoryFactory: AutoTableRepositoryFactory,
    private arrayHelper: ArrayHelpersService,
    private analyticsService: AnalyticsService
  ) {
    super();
  }

  ngOnInit () {
    const formGroupMap: {
      [x: string]: string|boolean;
    } = {};
    const fields = this.arrayHelper.sort(
      this.employeeSSOFields,
      'column'
    );
    fields.forEach((column) => {
      formGroupMap[column.column] = column.name;
      formGroupMap[(column.column) + '-InUse'] = column.inUse;
    });
    this.formGroup = this.formBuilder.group<SsoGroup>(formGroupMap);
  }

  onCancel () {
    this.closeModal.emit();
  }

  async onSubmit () {
    const payloadMap: {
      [x: string]: EmployeeSSOField;
    } = {};
    this.spinnerService.startSpinner();
    Object.keys(this.formGroup.controls).forEach((control) => {
      let columnType: EmployeeSSOFields;
      if (control.includes('InUse')) {
        columnType = +control.split('-')[0];
        payloadMap[columnType] = {
          ...payloadMap[columnType] || {} as EmployeeSSOField,
          inUse: this.formGroup.get(control).value as boolean,
          column: columnType
        };
      } else {
        columnType = +control;
        payloadMap[columnType] = {
          ...payloadMap[columnType] || {} as EmployeeSSOField,
          name: this.formGroup.get(control).value as string,
          column: columnType
        };
      }
    });
    const payload = Object.keys(payloadMap).map((key) => {
      return payloadMap[key];
    });
    const success = await this.employeeSSOFieldsService.updateEmployeeSSOFields(
      payload
    );
    if (success) {
      await this.employeeSSOFieldsService.resetEmployeeSSOFields();
      const repo = this.autoTableRepositoryFactory.getRepository(
        'CUSTOM_COLUMNS'
      );
      if (repo) {
        repo.reset();
      }
    }
    this.spinnerService.stopSpinner();
    this.closeModal.emit();
    this.analyticsService.emitEvent({
      eventName: 'Update SSO fields config save',
      eventType: EventType.Click,
      extras: null
    });
  }

}
