import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { ClientSettings } from '@core/typings/api/admin-client.typing';
import { ReferenceFieldAPI } from '@core/typings/api/reference-fields.typing';
import { ReferenceFieldsUI } from '@core/typings/ui/reference-fields.typing';
import { BuilderConfigGroupType, FormAudience, FormDefinitionComponent, FormDefinitionForUi, FormTypes } from '@features/configure-forms/form.typing';
import { EmployeeSSOFieldFromApi, EmployeeSSOFields } from '@features/employee-sso-fields/employee-sso-fields.typing';
import { FormulaEvaluationType, FormulaStepValueType } from '@features/formula-builder/formula-builder.typing';
import { ConditionalLogicResultType, EvaluationType } from '@features/logic-builder/logic-builder.typing';
import { REF_COMPONENT_TYPE_PREFIX } from '@features/reference-fields/services/reference-fields.service';
import { AfterEach, BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect, spy } from 'chai';
import { BucketComp } from '../../form-builder.typing';
import { FormBuilderService } from './form-builder.service';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(FormBuilderService, {
  imports: [GCMockModule]
})
export class FormBuilderServiceSpec implements Spec<FormBuilderServiceSpec, FormBuilderService> {
  formId = 1324;
  textFieldKey = 'textField';
  refComponent = {
    key: this.textFieldKey,
    label: 'Text field',
    type: `referenceFields-${this.textFieldKey}`
  } as FormDefinitionComponent;
  picklistKey = 'picklistKey';
  currentDef: FormDefinitionForUi[] = [{
    tabName: 'First Page',
    components: [
      this.refComponent,
      {
        type: `referenceFields-${this.picklistKey}`,
        key: this.picklistKey
      } as FormDefinitionComponent
    ],
    uniqueId: '1',
    index: 0,
    logic: null
  }];
  expectedResult = 2352.23;
  tabs: FormDefinitionForUi[] = [{
    tabName: 'tab1',
    logic: null,
    uniqueId: '1222',
    components: [{
      type: `referenceFields-${this.textFieldKey}`,
      key: this.textFieldKey,
      conditionalValue: [
        {
          evaluationType: 0,
          useAnd: false,
          identifier: '0c2b80d02da645fda27ce41625d483c7d315ee955ada45d29c41e51085983bb6',
          conditions: [
            {
              sourceColumn: [
                'referenceFields',
                this.textFieldKey
              ],
              comparison: 'eq',
              value: true,
              useAnd: false,
              identifier: '8d85af50598a4487b559e06eb7079d9d48ad2f8052d94068b8748012c3873b8a'
            } as any
          ],
          result: this.expectedResult,
          resultType: ConditionalLogicResultType.STATIC_VALUE
        }
      ],
      validate: {
        required: true
      }
    }, {
      type: 'amountRequested',
      key: 'amountRequested',
      validate: {
        required: true
      },
      formula: {
        property: 'referenceFields.budgetAmountTotal',
        step: {
          type: FormulaEvaluationType.Add,
          values: [
            {
              value: 'referenceFields.amount2',
              type: FormulaStepValueType.ParentValue
            },
            {
              value: 'referenceFields.budgetAmountQ2',
              type: FormulaStepValueType.ParentValue
            },
            {
              type: FormulaStepValueType.ParentValue,
              value: 'referenceFields.budgetAmountQ3'
            },
            {
              type: FormulaStepValueType.ParentValue,
              value: 'referenceFields.budgetAmountQ4'
            }
          ]
        }
      }
    }],
    index: 0
  }, {
    tabName: 'tab2',
    logic: null,
    uniqueId: '1353464',
    components: [{
      type: 'inKindItems',
      key: 'inKindItems',
      validate: {
        required: false
      },
      conditionalLogic: {
        evaluationType: EvaluationType.ConditionallyTrue,
        useAnd: false,
        identifier: 'cfb9fded1d714d4bbe6575809b56527a4ffdb95f00ec4327aa352169a59d7b33',
        conditions: [
          {
            sourceColumn: [
              'application',
              'amountRequested'
            ],
            comparison: 'gt',
            value: 500,
            useAnd: false,
            identifier: '11901a36f65b4e86a86d2aca074ea7bda9ece5a4923c4a64af5b839eca6dbf59'
          }
        ]
      } as any
    }],
    index: 1
  }];
  layoutCompTypes = [
    'columns',
    'fieldset',
    'panel',
    'table',
    'well'
  ];
  applicantTextField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Text',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.TextField,
    key: 'textKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 1,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  applicantTextArea: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Text Area',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.TextArea,
    key: 'textAreaKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 2,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  applicantNumber: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Number',
    description: 'desc',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Number,
    key: 'numberKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 3,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  managerTextField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Text',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.TextField,
    key: 'textFieldManagerKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.MANAGER,
    referenceFieldId: 1,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  checkboxField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Checkbox',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Checkbox,
    key: 'checkboxKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 1756,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  dateField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Date',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Date,
    key: 'dateKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.MANAGER,
    referenceFieldId: 23531,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  currencyField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Currency',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Currency,
    key: 'currencyKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.MANAGER,
    referenceFieldId: 23531,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  managerSingleResponseTextField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Text',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.TextField,
    key: this.managerTextField.key,
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.MANAGER,
    referenceFieldId: 1,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  fileUploadFieldForTable: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'File Upload For Table 1',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.FileUpload,
    key: 'fileUploadTableKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 235237,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: true,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  applicantFile: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'File Upload',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.FileUpload,
    key: 'fileUploadKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 435,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  tableField1: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Table 1',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Table,
    key: 'tableKey1',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 12421,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  tableField2: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Table 2',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Table,
    key: 'tableKey2',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 325,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  picklistField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    isEncrypted: false,
    isMasked: false,
    aggregateType: null,
    referenceFieldId: 23523,
    parentReferenceFieldId: null,
    formCount: 0,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    updateDate: '',
    customDataTableGuid: 'guidaasdf',
    name: 'Custom data table',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.CustomDataTable,
    key: 'dataTable1Key',
    supportsMultiple: true,
    categoryId: null,
    formAudience: FormAudience.APPLICANT,
    isSingleResponse: true,
    tableAllowsImport: false,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  subsetFieldPercent: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    isEncrypted: false,
    isMasked: false,
    aggregateType: null,
    referenceFieldId: 7964,
    parentReferenceFieldId: null,
    formCount: 1,
    usedOnReports: true,
    createdBy: '',
    createDate: '',
    updateDate: '',
    updatedBy: '',
    customDataTableGuid: '',
    name: 'Subset field - percent',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Subset,
    key: 'subsetKey',
    supportsMultiple: false,
    categoryId: null,
    formAudience: FormAudience.MANAGER,
    isSingleResponse: true,
    tableAllowsImport: false,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: 32095,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: ReferenceFieldAPI.DataSetCollectionType.Percent
  };
  referenceFields: ReferenceFieldAPI.ReferenceFieldDisplayModel[] = [
    this.applicantTextField,
    this.applicantTextArea,
    this.applicantNumber,
    this.managerTextField,
    this.managerSingleResponseTextField,
    this.tableField1,
    this.tableField2,
    this.picklistField,
    this.checkboxField,
    this.currencyField,
    this.dateField,
    this.applicantFile,
    this.fileUploadFieldForTable,
    this.subsetFieldPercent
  ];
  applicantStandardCompTypes = [
    'amountRequested',
    'inKindItems',
    'designation',
    'specialHandling',
    'careOf'
  ];
  employeeSSOFieldsFromApi: EmployeeSSOFieldFromApi[] = [{
    name: 'Region',
    inUse: true,
    column: EmployeeSSOFields.Region
  }, {
    name: 'Division',
    inUse: false,
    column: EmployeeSSOFields.Division
  }];

  @BeforeEach()
  async prep (service: FormBuilderService) {
    service['referenceFieldService'].setAllReferenceFields(this.referenceFields);
    service['clientSettingsService']['set']('clientSettings', {
      specialHandling: true,
      paymentDesignations: true
    } as ClientSettings);
    service['employeeSSOFieldsService']['employeeSSOFieldsResources'].getEmployeeSSOFields = async () => {
      return this.employeeSSOFieldsFromApi;
    };
    await service['employeeSSOFieldsService'].setEmployeeSSOFields();
  }

  @AfterEach()
  restore () {
    spy.restore();
  }

  @TestCase('should be able to set current form builder definition')
  setCurrentFormBuilderDefinition (service: FormBuilderService) {
    service.setCurrentFormBuilderDefinition(this.currentDef);
    expect(service.currentFormBuilderDefinition).to.deep.equal(this.currentDef);
  }

  @TestCase('should be able to set current form builder audience')
  setCurrentFormBuilderFormAudience (service: FormBuilderService) {
    service.setCurrentFormBuilderFormAudience(FormAudience.MANAGER);
    expect(service.currentFormBuilderFormAudience).to.be.equal(FormAudience.MANAGER);
    service.setCurrentFormBuilderFormAudience(FormAudience.APPLICANT);
    expect(service.currentFormBuilderFormAudience).to.be.equal(FormAudience.APPLICANT);
  }

  @TestCase('should be able to set current form builder index')
  setCurrentFormBuilderFormIndex (service: FormBuilderService) {
    service.setCurrentFormBuilderIndex(3);
    expect(service.currentFormBuilderIndex).to.be.equal(3);
    service.setCurrentFormBuilderIndex(0);
    expect(service.currentFormBuilderIndex).to.be.equal(0);
  }

  @TestCase('should be able to set current form builder form id')
  setCurrentFormBuilderFormId (service: FormBuilderService) {
    service.setCurrentFormBuilderFormId(3);
    expect(service.currentFormBuilderFormId).to.be.equal(3);
    service.setCurrentFormBuilderFormId(this.formId);
    expect(service.currentFormBuilderFormId).to.be.equal(this.formId);
  }

  @TestCase('should be able to set in form builder')
  setInFormBuilder (service: FormBuilderService) {
    service.setInFormBuilder(true);
    expect(service.inFormBuilder).to.be.equal(true);
    service.setInFormBuilder(false);
    expect(service.inFormBuilder).to.be.equal(false);
  }

  @TestCase('should be able to set open quick add modal')
  setOpenQuickAddModal (service: FormBuilderService) {
    service.setOpenQuickAddModal(true);
    expect(service.openQuickAddModal).to.be.equal(true);
    service.setOpenQuickAddModal(false);
    expect(service.openQuickAddModal).to.be.equal(false);
  }

  @TestCase('should be able to get component to view or edit')
  componentToViewOrEdit (service: FormBuilderService) {
    const componentToSet = {
      isEdit: true,
      compType: 'amountRequested',
      compKey: 'amountRequested'
    };
    service.setComponentToViewOrEdit(componentToSet);
    expect(service.componentToViewOrEdit).to.deep.equal(componentToSet);
  }

  @TestCase('should be able to get component tab index map')
  getComponentTabIndexMap (service: FormBuilderService) {
    const map = service.getComponentTabIndexMap(this.tabs);
    const correctLength = Object.keys(map).length === 3; // 3 components
    const correctTabIndexZero = map.amountRequested.tabIndex === 0;
    const correctTabIndexOne = map.inKindItems.tabIndex === 1;
    const correctRefKey = map[this.textFieldKey].refKey === this.textFieldKey;
    expect(correctLength).to.be.true;
    expect(correctTabIndexZero).to.be.true;
    expect(correctTabIndexOne).to.be.true;
    expect(correctRefKey).to.be.true;
  }

  @TestCase('should be able to get builder groups')
  getBuilderGroups (service: FormBuilderService) {
    const groups = service.getBuilderGroups();
    expect(groups[BuilderConfigGroupType.Layout]).to.be.equal('Layout');
    expect(groups[BuilderConfigGroupType.Capture]).to.be.equal('Capture');
    expect(groups[BuilderConfigGroupType.Display]).to.be.equal('Display');
    expect(groups[BuilderConfigGroupType.OnForm]).to.be.equal('Currently on form');
  }

  @TestCase('should be able to get filter number to append')
  getFilterNumberToAppend (service: FormBuilderService) {
    let searchTerm = '';
    const numberOfResults = 5;
    let value = service.getFilterNumberToAppend(searchTerm, numberOfResults);
    // When no search term, we don't append
    expect(!value).to.be.true;
    searchTerm = 'name';
    value = service.getFilterNumberToAppend(searchTerm, numberOfResults);
    expect(value).to.be.equal(` (${numberOfResults})`);
  }

  @TestCase('should be able to get panels by type - capture')
  getPanelsByTypeCapture (service: FormBuilderService) {
    spy.on(service, 'getReferenceFieldsPanel');
    spy.on(service, 'getStandardPanel');
    service.getPanelsByType(
      BuilderConfigGroupType.Capture,
      FormTypes.REQUEST,
      FormAudience.APPLICANT,
      ''
    );
    expect(service['getReferenceFieldsPanel']).to.have.been.called.once;
    expect(service['getStandardPanel']).to.have.been.called.once;
  }

  @TestCase('should be able to get panels by type - display')
  getPanelsByTypeDisplay (service: FormBuilderService) {
    spy.on(service, 'getReferenceFieldsPanel');
    spy.on(service, 'getSsoPanel');
    spy.on(service, 'getReportPanel');
    spy.on(service, 'getCustomContentPanel');
    service.getPanelsByType(
      BuilderConfigGroupType.Display,
      FormTypes.REVIEW,
      FormAudience.MANAGER,
      ''
    );
    expect(service['getReferenceFieldsPanel']).to.have.been.called.once;
    expect(service['getSsoPanel']).to.have.been.called.once;
    expect(service['getReportPanel']).to.have.been.called.once;
    expect(service['getCustomContentPanel']).to.have.been.called.once;
  }

  @TestCase('should be able to get panels by type - layout')
  getPanelsByTypeLayout (service: FormBuilderService) {
    spy.on(service, 'getLayoutPanel');
    service.getPanelsByType(
      BuilderConfigGroupType.Layout,
      FormTypes.REVIEW,
      FormAudience.MANAGER,
      ''
    );
    expect(service['getLayoutPanel']).to.have.been.called.once;
  }

  @TestCase('should be able to get panels by type - on form')
  getPanelsByTypeOnForm (service: FormBuilderService) {
    const onFormComps = [{
      type: `${REF_COMPONENT_TYPE_PREFIX}${this.applicantTextArea.key}`
    }, {
      type: 'amountRequested'
    }];
    spy.on(service['componentHelper'], 'getAllComponents', () => {
      return onFormComps;
    });
    spy.on(service, 'getOnFormPanel');
    service.getPanelsByType(
      BuilderConfigGroupType.OnForm,
      FormTypes.REVIEW,
      FormAudience.MANAGER,
      ''
    );
    expect(service['getOnFormPanel']).to.have.been.called.once;
  }

  @TestCase('should be able to get toolbox panel sections')
  getToolboxPanelSections (service: FormBuilderService) {
    const onFormComps = [{
      type: `${REF_COMPONENT_TYPE_PREFIX}${this.applicantTextArea.key}`
    }, {
      type: 'amountRequested'
    }];
    spy.on(service['componentHelper'], 'getAllComponents', () => {
      return onFormComps;
    });
    spy.on(service, 'getBuilderGroups');
    spy.on(service, 'getPanelsByType');
    service.getToolboxPanelSections(FormTypes.APPROVAL, FormAudience.MANAGER, '');
    expect(service['getBuilderGroups']).to.have.been.called.once;
    const groups = service.getBuilderGroups();
    const groupLength = Object.keys(groups).length;
    expect(service['getPanelsByType']).to.have.been.called.exactly(groupLength);
  }

  @TestCase('should be able to get layout panel - no search')
  getLayoutPanel (service: FormBuilderService) {
    spy.on(service, 'getLayoutComponents');
    const searchTerm = '';
    const panel = service.getLayoutPanel(FormAudience.APPLICANT, searchTerm);
    expect(service['getLayoutComponents']).to.have.been.called.once;
    const hasAllLayoutComps = panel.context.length === this.layoutCompTypes.length &&
      panel.context.some((comp) => {
        return this.layoutCompTypes.includes(comp.type);
      });
    expect(hasAllLayoutComps).to.be.true;
  }

  @TestCase('should be able to get layout panel - with search')
  getLayoutPanelWithSearch (service: FormBuilderService) {
    spy.on(service, 'getLayoutComponents');
    const searchTerm = 'WELLS';
    const panel = service.getLayoutPanel(FormAudience.MANAGER, searchTerm);
    expect(service['getLayoutComponents']).to.have.been.called.once;
    // No reults based on filter, so panel is not returned
    expect(!panel).to.be.true;
  }

  @TestCase('should be able to get layout components')
  getLayoutComponents (service: FormBuilderService) {
    const searchTerm = '';
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    spy.on(service['valueComparisonService'], 'compareTerms');
    const components = service.getLayoutComponents(FormAudience.APPLICANT, searchTerm);
    const hasAllLayoutComps = components.length === this.layoutCompTypes.length &&
      components.some((comp) => {
        return this.layoutCompTypes.includes(comp.type);
      });
    expect(hasAllLayoutComps).to.be.true;
    expect(service['getFieldIcon']).to.have.been.called.exactly(components.length);
    expect(service['getFieldTooltip']).to.have.been.called.exactly(components.length);
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.exactly(components.length);
  }

  @TestCase('should be able to get reference fields panel - capture applicant')
  getReferenceFieldsPanelCaptureApplicant (service: FormBuilderService) {
    spy.on(service, 'getReferenceFieldComponents');
    spy.on(service, 'getFilterNumberToAppend');
    const panel = service.getReferenceFieldsPanel(true, FormTypes.REQUEST, FormAudience.APPLICANT, '');
    expect(service['getReferenceFieldComponents']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.have.been.called.once;
    expect(panel.name).to.be.equal('Applicant responses');
  }

  @TestCase('should be able to get reference fields panel - display manager')
  getReferenceFieldsPanelDisplayManager (service: FormBuilderService) {
    spy.on(service, 'getReferenceFieldComponents');
    spy.on(service, 'getFilterNumberToAppend');
    const panel = service.getReferenceFieldsPanel(false, FormTypes.REVIEW, FormAudience.MANAGER, '');
    expect(service['getReferenceFieldComponents']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.have.been.called.once;
    expect(panel.name).to.be.equal('Applicant responses');
  }

  @TestCase('should be able to get reference fields panel - no results')
  getReferenceFieldsPanelNoResults (service: FormBuilderService) {
    spy.on(service, 'getReferenceFieldComponents');
    spy.on(service, 'getFilterNumberToAppend');
    const panel = service.getReferenceFieldsPanel(false, FormTypes.REVIEW, FormAudience.MANAGER, 'asdfasdf');
    expect(service['getReferenceFieldComponents']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.not.have.been.called;
    expect(!panel).to.be.true;
  }

  @TestCase('should be able to get reference fields components')
  getReferenceFieldComponents (service: FormBuilderService) {
    const isCapture = true;
    const formType = FormTypes.REVIEW;
    const audience = FormAudience.MANAGER;
    const searchTerm = '';
    const filteredFields = service.getFilteredFieldsForBuilder(isCapture, formType, audience, searchTerm);
    spy.on(service, 'getReferenceFieldBucketComp');
    service.getReferenceFieldComponents(isCapture, formType, audience, searchTerm);
    expect(service['getReferenceFieldBucketComp']).to.have.been.called.exactly(filteredFields.length);
  }

  @TestCase('should be able to get reference field bucket comp')
  getReferenceFieldBucketComp (service: FormBuilderService) {
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    spy.on(service, 'componentIsAvailableToAdd');
    const bucketComp = service.getReferenceFieldBucketComp(this.applicantTextArea);
    expect(service['getFieldIcon']).to.have.been.called.once;
    expect(service['getFieldTooltip']).to.have.been.called.once;
    expect(service['componentIsAvailableToAdd']).to.have.been.called.once;
    expect(bucketComp.type).to.be.equal(`${REF_COMPONENT_TYPE_PREFIX}${this.applicantTextArea.key}`);
    expect(bucketComp.isReferenceField).to.be.true;
  }

  @TestCase('should be able to get filtered fields for builder')
  getFilteredFieldsForBuilder (service: FormBuilderService) {
    spy.on(service['valueComparisonService'], 'compareTerms');
    const fields = service.getFilteredFieldsForBuilder(true, FormTypes.ROUTING, FormAudience.APPLICANT, '');
    const shouldPass = [
      this.applicantTextField.key,
      this.applicantTextArea.key,
      this.applicantNumber.key,
      this.picklistField.key,
      this.checkboxField.key
    ];
    expect(fields.length).to.be.equal(shouldPass.length);
    const hasCorrectFields = fields.some((field) => {
      return shouldPass.includes(field.key);
    });
    expect(hasCorrectFields).to.be.true;
    const numOfFields = this.referenceFields.length;
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.exactly(numOfFields);
  }

  @TestCase('should be able to get standard panel - no search')
  getStandardPanel (service: FormBuilderService) {
    spy.on(service, 'getStandardComponents');
    spy.on(service, 'getFilterNumberToAppend');
    const panel = service.getStandardPanel(FormTypes.REQUEST, FormAudience.APPLICANT, '');
    expect(service['getStandardComponents']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.have.been.called.once;
    expect(panel.name).to.be.equal('Standard components');
  }

  @TestCase('should be able to get standard panel - with search')
  getStandardPanelWithSearch (service: FormBuilderService) {
    spy.on(service, 'getStandardComponents');
    spy.on(service, 'getFilterNumberToAppend');
    const panel = service.getStandardPanel(FormTypes.REVIEW, FormAudience.MANAGER, 'asdfasdf');
    expect(service['getStandardComponents']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.not.have.been.called;
    expect(!panel).to.be.true;
  }

  @TestCase('should be able to get standard components - applicant')
  getStandardComponentsApplicant (service: FormBuilderService) {
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    spy.on(service, 'componentIsAvailableToAdd');
    spy.on(service['valueComparisonService'], 'compareTerms');
    const comps = service.getStandardComponents(FormTypes.REQUEST, FormAudience.APPLICANT, [], '');
    expect(comps.length).to.be.equal(this.applicantStandardCompTypes.length);
    const hasCorrectComps = comps.some((comp) => {
      return this.applicantStandardCompTypes.includes(comp.type);
    });
    expect(hasCorrectComps).to.be.true;
    expect(service['getFieldIcon']).to.have.been.called.exactly(this.applicantStandardCompTypes.length);
    expect(service['getFieldTooltip']).to.have.been.called.exactly(this.applicantStandardCompTypes.length);
    expect(service['componentIsAvailableToAdd']).to.have.been.called.exactly(this.applicantStandardCompTypes.length);
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.exactly(this.applicantStandardCompTypes.length);
  }

  @TestCase('should be able to get standard components - manager')
  getStandardComponentsManager (service: FormBuilderService) {
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    spy.on(service, 'componentIsAvailableToAdd');
    spy.on(service['valueComparisonService'], 'compareTerms');
    const comps = service.getStandardComponents(FormTypes.REVIEW, FormAudience.MANAGER, [], '');
    const managerCompTypes = this.applicantStandardCompTypes.concat(
      ['decision', 'reviewerRecommendedFundingAmount']
    );
    expect(comps.length).to.be.equal(managerCompTypes.length);
    const hasCorrectComps = comps.some((comp) => {
      return managerCompTypes.includes(comp.type);
    });
    expect(hasCorrectComps).to.be.true;
    expect(service['getFieldIcon']).to.have.been.called.exactly(managerCompTypes.length);
    expect(service['getFieldTooltip']).to.have.been.called.exactly(managerCompTypes.length);
    expect(service['componentIsAvailableToAdd']).to.have.been.called.exactly(managerCompTypes.length);
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.exactly(managerCompTypes.length);
  }

  @TestCase('should be able to get comp is available to add - false')
  componentIsAvailableToAddFalse (service: FormBuilderService) {
    spy.on(service['componentHelper'], 'getAllComponents', () => {
      return [{
        type: 'designation'
      }, {
        type: 'amountRequested'
      }];
    });
    const isAvailable = service.componentIsAvailableToAdd('amountRequested', []);
    // this already lives on the form, so not available
    expect(isAvailable).to.be.false;
  }

  @TestCase('should be able to get comp is available to add - true')
  componentIsAvailableToAddTrue (service: FormBuilderService) {
    spy.on(service['componentHelper'], 'getAllComponents', () => {
      return [{
        type: 'designation'
      }];
    });
    let isAvailable = service.componentIsAvailableToAdd('amountRequested', []);
    // this does not already live on the form, so it's available
    expect(isAvailable).to.be.true;
    // But, if we pass it in componentsToAdd, it becomes unavailable
    isAvailable = service.componentIsAvailableToAdd('amountRequested', ['amountRequested']);
    expect(isAvailable).to.be.false;
  }

  @TestCase('should be able to get sso panel')
  getSsoPanel (service: FormBuilderService) {
    // with no SSO client setting, null is returned
    let panel = service.getSsoPanel(FormAudience.APPLICANT, '');
    expect(!panel).to.be.true;
    service['clientSettingsService']['set']('clientSettings', {
      ...service['clientSettingsService']['clientSettings'],
      hasSSO: true
    });
    spy.on(service, 'getSsoComponents');
    spy.on(service, 'getFilterNumberToAppend');
    panel = service.getSsoPanel(FormAudience.MANAGER, '');
    expect(service['getSsoComponents']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.have.been.called.once;
    expect(panel.name).to.be.equal('Employee HR data');
    expect(panel.context.length).to.be.equal(this.employeeSSOFieldsFromApi.length);
  }

  @TestCase('should be able to get sso components')
  getSsoComponents (service: FormBuilderService) {
    const ssoFieldAttrs = service['employeeSSOFieldsService']['employeeSSOFields'].map((field) => {
      return field.attr;
    });
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    spy.on(service, 'componentIsAvailableToAdd');
    spy.on(service['valueComparisonService'], 'compareTerms');
    const comps = service.getSsoComponents(FormAudience.APPLICANT, '');
    expect(ssoFieldAttrs.length).to.be.equal(comps.length);
    const hasCorrectFields = comps.some((comp) => {
      return ssoFieldAttrs.includes(comp.key);
    });
    expect(hasCorrectFields).to.be.true;
    expect(service['getFieldIcon']).to.have.been.called.exactly(ssoFieldAttrs.length);
    expect(service['getFieldTooltip']).to.have.been.called.exactly(ssoFieldAttrs.length);
    expect(service['componentIsAvailableToAdd']).to.have.been.called.exactly(ssoFieldAttrs.length);
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.exactly(ssoFieldAttrs.length);
  }

  @TestCase('should be able to get report panel')
  getReportPanel (service: FormBuilderService) {
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    spy.on(service['valueComparisonService'], 'compareTerms');
    spy.on(service, 'getFilterNumberToAppend');
    let panel = service.getReportPanel(FormTypes.ROUTING, FormAudience.APPLICANT, '');
    // routing does not support report fields
    expect(!panel).to.be.true;
    panel = service.getReportPanel(FormTypes.REQUEST, FormAudience.APPLICANT, '');
    // Report bucket is always represented by one item ("Report field")
    expect(panel.context.length).to.be.equal(1);
    expect(panel.name).to.be.equal('Report components');
    const reportField = panel.context[0];
    expect(reportField.type).to.be.equal('reportField');
    expect(service['getFieldIcon']).to.have.been.called.once;
    expect(service['getFieldTooltip']).to.have.been.called.once;
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.have.been.called.once;
  }

  @TestCase('should be able to get custom content panel - no search')
  getCustomContentPanel (service: FormBuilderService) {
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    spy.on(service['valueComparisonService'], 'compareTerms');
    spy.on(service, 'getFilterNumberToAppend');
    const panel = service.getCustomContentPanel(FormAudience.APPLICANT, '');
    expect(panel.name).to.be.equal('Custom text');
    expect(service['getFieldIcon']).to.have.been.called.once;
    expect(service['getFieldTooltip']).to.have.been.called.once;
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.have.been.called.once;
  }

  @TestCase('should be able to get custom content panel - with search')
  getCustomContentPanelWithSearch (service: FormBuilderService) {
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    spy.on(service['valueComparisonService'], 'compareTerms');
    spy.on(service, 'getFilterNumberToAppend');
    const panel = service.getCustomContentPanel(FormAudience.APPLICANT, 'asdfa');
    expect(!panel).to.be.true;
    expect(service['getFieldIcon']).to.not.have.been.called;
    expect(service['getFieldTooltip']).to.not.have.been.called;
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.once;
    expect(service['getFilterNumberToAppend']).to.not.have.been.called;
  }

  @TestCase('should be able to get on form panel - none')
  getOnFormPanelNone (service: FormBuilderService) {
    spy.on(service, 'getOnFormComponents');
    spy.on(service['componentHelper'], 'getAllComponents', () => {
      return [];
    });
    const panel = service.getOnFormPanel('');
    expect(service['getOnFormComponents']).to.have.been.called.once;
    // No comps are on the form
    expect(!panel).to.be.true;
  }

  @TestCase('should be able to get on form panel')
  getOnFormPanel (service: FormBuilderService) {
    spy.on(service, 'getOnFormComponents');
    const onFormComps = [{
      type: `${REF_COMPONENT_TYPE_PREFIX}${this.applicantTextArea.key}`
    }, {
      type: 'amountRequested'
    }];
    spy.on(service['componentHelper'], 'getAllComponents', () => {
      return onFormComps;
    });
    const panel = service.getOnFormPanel('');
    expect(service['getOnFormComponents']).to.have.been.called.once;
    // No comps are on the form
    expect(panel.context.length).to.be.equal(onFormComps.length);
    const hasCorrectComps = onFormComps.some((comp) => {
      const onFormTypes = onFormComps.map((_comp) => _comp.type);

      return onFormTypes.includes(comp.type);
    });
    expect(hasCorrectComps).to.be.true;
  }

  @TestCase('should be able to get on form components')
  getOnFormComponents (service: FormBuilderService) {
    spy.on(service['valueComparisonService'], 'compareTerms');
    spy.on(service, 'getFieldIcon');
    spy.on(service, 'getFieldTooltip');
    const onFormComps = [{
      type: `${REF_COMPONENT_TYPE_PREFIX}${this.applicantTextArea.key}`
    }, {
      type: 'amountRequested'
    }];
    spy.on(service['componentHelper'], 'getAllComponents', () => {
      return onFormComps;
    });
    const comps = service.getOnFormComponents('');
    const hasMyComps = comps.length === onFormComps.length &&
      comps.some((comp) => {
        const onFormCompTypes = onFormComps.map((_comp) => _comp.type);

        return onFormCompTypes.includes(comp.type);
      });
    expect(hasMyComps).to.be.true;
    expect(service['valueComparisonService']['compareTerms']).to.have.been.called.exactly(onFormComps.length);
    expect(service['getFieldIcon']).to.have.been.called.exactly(onFormComps.length);
    expect(service['getFieldTooltip']).to.have.been.called.exactly(onFormComps.length);
    spy.on(service, 'setComponentToViewOrEdit');
    const firstComp = comps[0];
    const editAction = firstComp.actions[0];
    const viewAction = firstComp.actions[1];
    editAction.onClick(firstComp);
    viewAction.onClick(firstComp);
    expect(service['setComponentToViewOrEdit']).to.have.been.called.twice;
  }

  @TestCase('should be able to get field tooltip')
  getFieldTooltip (service: FormBuilderService) {
    spy.on(service['referenceFieldService'], 'getReferenceFieldFromCompType');
    spy.on(service['referenceFieldService'], 'getFieldTypeTranslatedWithIcon');
    const type = `${REF_COMPONENT_TYPE_PREFIX}${this.applicantNumber.key}`;
    const tooltip = service.getFieldTooltip(type);
    expect(service['referenceFieldService']['getReferenceFieldFromCompType']).to.have.been.called.once;
    expect(service['referenceFieldService']['getFieldTypeTranslatedWithIcon']).to.have.been.called.once;
    const hasCorrectDetails = tooltip.includes(this.applicantNumber.name) &&
      tooltip.includes(this.applicantNumber.description) &&
      tooltip.includes(this.applicantNumber.key);
    expect(hasCorrectDetails).to.be.true;
  }

  @TestCase('should be able to get field icon - none')
  getFieldIcon (service: FormBuilderService) {
    const icon = service.getFieldIcon('');
    expect(!icon).to.be.true;
  }

  @TestCase('should be able to add new field to form')
  async addNewFieldToForm (service: FormBuilderService) {
    service.setCurrentFormBuilderDefinition(this.currentDef);
    service.setCurrentFormBuilderFormAudience(FormAudience.APPLICANT);
    service.setCurrentFormBuilderIndex(0);
    service.setCurrentFormBuilderFormId(this.formId);
    spy.on(service['referenceFieldService'], 'handleCreateOrUpdateField', () => {
      return {};
    });
    spy.on(service, 'getReferenceFieldBucketComp', () => {});
    spy.on(service, 'generateComponent', () => {});
    spy.on(service, 'addNewComponent', () => {
      return {};
    });
    spy.on(service['componentHelper'], 'insertFormComponent', () => {});
    const modalResponse: ReferenceFieldsUI.ModalReturn = {
      field: this.picklistField,
      tableFields: [],
      isSecondarySave: true
    };
    await service.addNewFieldToForm(modalResponse);
    expect(service['referenceFieldService']['handleCreateOrUpdateField']).to.have.been.called.once;
    expect(service['getReferenceFieldBucketComp']).to.have.been.called.once;
    expect(service['generateComponent']).to.have.been.called.once;
    expect(service['addNewComponent']).to.have.been.called.once;
    expect(service['componentHelper']['insertFormComponent']).to.have.been.called.once;
  }


  @TestCase('should be able to add new field to form - fail')
  async addNewFieldToFormFail (service: FormBuilderService) {
    spy.on(service['logger'], 'error');
    spy.on(service['notifier'], 'error');
    spy.on(service['referenceFieldService'], 'handleCreateOrUpdateField', () => {
      // eslint-disable-next-line no-throw-literal
      throw {
        error: {
          message: 'Error'
        }
      };
    });
    const modalResponse: ReferenceFieldsUI.ModalReturn = {
      field: this.picklistField,
      tableFields: [],
      isSecondarySave: false
    };
    await service.addNewFieldToForm(modalResponse);
    expect(service['logger']['error']).to.have.been.called.once;
    expect(service['notifier']['error']).to.have.been.called.once;
  }

  @TestCase('should be able to add new component')
  async addNewComponent (service: FormBuilderService) {
    spy.on(service['formHelperService'], 'getValueFromComponent');
    spy.on(service, 'editComponentModal', () => {});
    const newComp = {
      type: `${REF_COMPONENT_TYPE_PREFIX}${this.currencyField.key}`
    } as FormDefinitionComponent;
    await service.addNewComponent(newComp, 1, this.currentDef, false);
    expect(service['formHelperService']['getValueFromComponent']).to.have.been.called.once;
    expect(service['editComponentModal']).to.have.been.called.once;
  }

  @TestCase('should be able to open edit component modal')
  async openEditComponentModal (service: FormBuilderService) {
    spy.on(service['modalFactory'], 'open', () => {});
    const comp = {
      type: `${REF_COMPONENT_TYPE_PREFIX}${this.currencyField.key}`
    } as FormDefinitionComponent;
    await service.editComponentModal(comp, 1, this.currentDef);
    expect(service['modalFactory']['open']).to.have.been.called.once;
  }

  @TestCase('should be able to generate component - ref field')
  generateComponentRefField (service: FormBuilderService) {
    spy.on(service['componentHelper'], 'guessKey');
    const bucketComp = service.getReferenceFieldBucketComp(this.applicantNumber);
    const comp = service.generateComponent(this.currentDef, bucketComp);
    expect(service['componentHelper']['guessKey']).to.have.been.called.once;
    expect(comp.key).to.be.equal(this.applicantNumber.key);
  }

  @TestCase('should be able to generate component - well')
  generateComponentWell (service: FormBuilderService) {
    spy.on(service['componentHelper'], 'guessKey');
    const bucketComp = {
      key: 'well',
      label: '',
      name: 'Well',
      type: 'well',
      icon: '',
      tooltip: '',
      fieldAudience: FormAudience.APPLICANT,
      isReferenceField: false
    };
    const comp = service.generateComponent(this.currentDef, bucketComp);
    expect(service['componentHelper']['guessKey']).to.have.been.called.once;
    expect(comp.key).to.be.equal(bucketComp.key);
    expect(comp.input).to.be.false;
    expect(comp.components).to.deep.equal([]);
  }

  @TestCase('should be able to generate component - columns')
  generateComponentColumns (service: FormBuilderService) {
    spy.on(service['componentHelper'], 'guessKey');
    const bucketComp = {
      key: 'columns',
      label: 'Columns',
      name: 'Columns',
      type: 'columns',
      fieldAudience: FormAudience.APPLICANT,
      icon: '',
      tooltip: '',
      isReferenceField: false
    };
    const comp = service.generateComponent(this.currentDef, bucketComp);
    expect(service['componentHelper']['guessKey']).to.have.been.called.once;
    expect(comp.key).to.be.equal(bucketComp.key);
    const hasColumns = comp.columns.length === 2;
    expect(hasColumns).to.be.true;
    expect(comp.input).to.be.false;
  }

  @TestCase('should be able to generate component - table')
  generateComponentTable (service: FormBuilderService) {
    spy.on(service['componentHelper'], 'guessKey');
    spy.on(service, 'getEmptyGridRows');
    const bucketComp = {
      key: 'table',
      label: 'Table',
      name: 'Table',
      type: 'table',
      fieldAudience: FormAudience.APPLICANT,
      icon: '',
      tooltip: '',
      isReferenceField: false
    };
    const comp = service.generateComponent(this.currentDef, bucketComp);
    expect(service['componentHelper']['guessKey']).to.have.been.called.once;
    expect(comp.key).to.be.equal(bucketComp.key);
    expect(service['getEmptyGridRows']).to.have.been.called.once;
    expect(comp.input).to.be.false;
  }

  @TestCase('should be able to generate component - content')
  generateComponentContent (service: FormBuilderService) {
    spy.on(service['componentHelper'], 'guessKey');
    spy.on(service, 'getEmptyGridRows');
    const bucketComp = {
      key: 'content',
      label: 'Content',
      name: 'Content',
      type: 'content',
      fieldAudience: FormAudience.APPLICANT,
      icon: '',
      tooltip: '',
      isReferenceField: false
    };
    const comp = service.generateComponent(this.currentDef, bucketComp);
    expect(service['componentHelper']['guessKey']).to.have.been.called.once;
    expect(comp.key).to.be.equal(bucketComp.key);
    expect(comp.input).to.be.false;
  }

  @TestCase('should be able to get has visible components')
  hasVisibleComponents (service: FormBuilderService) {
    const comps = [{
      hidden: true
    } as BucketComp, {
      hidden: false
    } as BucketComp];
    let hasVisible = service.hasVisibleComponents(comps);
    expect(hasVisible).to.be.true;
    const compsWithNoVisible = [{
      hidden: true
    } as BucketComp, {
      hidden: true
    } as BucketComp];
    hasVisible = service.hasVisibleComponents(compsWithNoVisible);
    expect(hasVisible).to.be.false;
  }

  @TestCase('should be able to get hidden panel obj')
  getHiddenPanelObj (service: FormBuilderService) {
    const layoutObj = service.getHiddenPanelObj(BuilderConfigGroupType.Layout, []);
    const noName = !layoutObj.name;
    expect(noName).to.be.true;
    expect(layoutObj.uniqueId).to.be.equal(`${BuilderConfigGroupType.Layout}`);
  }
}
