import { Component, EventEmitter, Input, OnChanges, Output, SimpleChanges } from '@angular/core';
import { AbstractControl, FormGroup, ValidationErrors } from '@angular/forms';
import { I18nService } from '@yourcause/common/i18n';


@Component({
  selector: 'gc-formio-error-summary',
  templateUrl: './gc-formio-error-summary.component.html',
  styleUrls: ['./gc-formio-error-summary.component.scss']
})
export class GcFormioErrorSummaryComponent implements OnChanges {
  @Input() tabFormGroup: FormGroup;
  @Input() forceUpdate = false;
  @Input() hiddenCompKeys: string[];
  @Output() onHasErrorsChange = new EventEmitter<boolean>();

  errorMessages: string[] = [];

  constructor (
    private i18n: I18nService
  ) { }

  ngOnChanges (changes: SimpleChanges) {
    if (
      this.tabFormGroup &&
      changes.tabFormGroup || (changes.forceUpdate && this.forceUpdate)
    ) {
      this.errorMessages = [];
      Object.keys(this.tabFormGroup.controls).forEach((key) => {
        if (
          !this.hiddenCompKeys?.includes(key)
        ) {
          const group = this.tabFormGroup.controls[key] as FormGroup;
          if (group?.errors) {
            this.findControlErrors(group, true);
          }
          Object.keys(group?.controls).forEach((controlKey) => {
            const control = group.controls[controlKey];
            if (control.errors) {
              this.findControlErrors(control, false);
            }
          });
        }
      });

      this.onHasErrorsChange.emit(this.errorMessages.length > 0);
    }
  }

  findControlErrors (
    control: AbstractControl,
    isGroup: boolean
  ) {
    Object.keys(control.errors).forEach((errorKey) => {
      const error = control.errors[errorKey];
      if (isGroup && error instanceof Object) {
        // Group errors are nested further
        Object.keys(error).forEach((groupErrorKey) => {
          this.handlePushingErrors(error[groupErrorKey]);
        });
      } else {
        this.handlePushingErrors(error);
      }
    });
  }

  handlePushingErrors (error: ValidationErrors) {
    if (error?.errorMessage) {
      this.errorMessages.push(error.errorMessage);
    } else if (error?.i18nKey) {
      this.errorMessages.push(this.i18n.translate(
        error.i18nKey,
        {},
        error.defaultValue
      ));
    }
  }
}
