import { Component, Input, OnInit, Type } from '@angular/core';
import { SpinnerService } from '@core/services/spinner.service';
import { ReferenceFieldsUI } from '@core/typings/ui/reference-fields.typing';
import { CustomDataTablesService } from '@features/custom-data-tables/custom-data-table.service';
import { ReferenceFieldsService } from '@features/reference-fields/services/reference-fields.service';
import { TableDataDownloadFormat, TypeaheadSelectOption } from '@yourcause/common';
import { YCModalComponent } from '@yourcause/common/modals';
import { PopoverDirective } from 'ngx-bootstrap/popover';

@Component({
  selector: 'gc-import-table-rows-modal',
  templateUrl: './import-table-rows-modal.component.html',
  styleUrls: ['./import-table-rows-modal.component.scss']
})
export class ImportTableRowsModalComponent extends YCModalComponent<boolean> implements OnInit {
  @Input() applicationId: number;
  @Input() applicationFormId: number;
  @Input() tableLabel: string;
  @Input() tableId: number;
  @Input() formId: number;
  @Input() rowsForTable: ReferenceFieldsUI.TableResponseRowForUiMapped[];

  ValidationClass: Type<any>;
  cdtItemsMap: Record<number, TypeaheadSelectOption[]> = {};
  importValid: boolean;
  saving = false;
  csvRows: Record<string, any>[] = [];
  currentPopover: {
    instance?: PopoverDirective;
  } = {};

  constructor (
    private referenceFieldService: ReferenceFieldsService,
    private customDataTableService: CustomDataTablesService,
    private spinnerService: SpinnerService
  ) {
    super();
  }

  ngOnInit () {
    this.cdtItemsMap = this.customDataTableService.getCdtItemsMapForRow(
      this.referenceFieldService.tableColumnsMap[this.tableId]
    );
    this.ValidationClass = this.referenceFieldService.getValidationClassForTableRowImport(
      this.tableId,
      this.cdtItemsMap
    );
  }

  downloadTemplate () {
    this.referenceFieldService.returnTableRowImportData(
      this.tableId,
      this.rowsForTable,
      true,
      TableDataDownloadFormat.CSV
    );
  }

  onContentsChange (rows: Record<string, any>[]) {
    this.csvRows = rows;
  }

  handleUpdateValidation (valid: boolean) {
    this.importValid = valid;
  }

  setCurrentPopover (
    popover: PopoverDirective
  ) {
    if (this.currentPopover.instance) {
      this.currentPopover.instance.hide();
    }

    this.currentPopover.instance = popover;

    setTimeout(() => {
      popover.show();
    });
  }

  async onImport () {
    this.spinnerService.startSpinner();
    this.saving = true;
    const passed = await this.referenceFieldService.handleImportTableRows(
      this.applicationId,
      this.applicationFormId,
      this.tableId,
      this.formId,
      this.csvRows,
      this.cdtItemsMap
    );
    this.saving = false;
    this.spinnerService.stopSpinner();
    if (passed) {
      this.closeModal.emit(true);
    }
  }
}
