import { Component, OnInit, Optional } from '@angular/core';
import { FormGroupDirective } from '@angular/forms';
import { BaseApplication } from '@core/typings/application.typing';
import { DATE_SSO_FIELDS } from '@features/employee-sso-fields/employee-sso-fields.typing';
import { FormBuilderService } from '@features/formio/form-builder/services/form-builder/form-builder.service';
import { GcFormRendererComponent } from '@features/formio/form-renderer/gc-form-renderer/gc-form-renderer.component';
import { ComponentHelperService } from '@features/formio/services/component-helper/component-helper.service';
import { TypeSafeFormBuilder } from '@yourcause/common';
import moment from 'moment';
import { BaseFormioComponent } from '../../base/base.component';

@Component({
  selector: 'gc-employee-sso-field',
  templateUrl: './gc-employee-sso-field.component.html',
  styleUrls: ['./gc-employee-sso-field.component.scss']
})
export class GcEmployeeSSOFieldComponent extends BaseFormioComponent<string> implements OnInit {

  constructor (
    formBuilder: TypeSafeFormBuilder,
    formBuilderService: FormBuilderService,
    componentHelper: ComponentHelperService,
    @Optional() formGroupDir: FormGroupDirective,
    @Optional() renderer: GcFormRendererComponent<BaseApplication>
  ) {
    super(renderer, formGroupDir, formBuilder, formBuilderService, componentHelper);
  }

  async ngOnInit () {
    super.ngOnInit();
    const attr = this.extractFieldAttr();
    const isDate = DATE_SSO_FIELDS.includes(attr);
    let formVal = this.data;
    if (isDate) {
      if (this.data && !Array.isArray(this.data)) {
        formVal = moment(this.data).utc().format('ll');
      }
    }
    this.setFormGroup(formVal || '', []);
  }


  extractFieldAttr () {
    return this.comp.type.split('-')[1];
  }
}
