import { Component, OnInit, Optional } from '@angular/core';
import { FormGroupDirective, ValidatorFn } from '@angular/forms';
import { CurrencyService } from '@core/services/currency.service';
import { ValidatorsService } from '@core/services/validators.service';
import { BaseApplication } from '@core/typings/application.typing';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { FormBuilderService } from '@features/formio/form-builder/services/form-builder/form-builder.service';
import { GcFormRendererComponent } from '@features/formio/form-renderer/gc-form-renderer/gc-form-renderer.component';
import { ComponentHelperService } from '@features/formio/services/component-helper/component-helper.service';
import { TypeSafeFormBuilder } from '@yourcause/common';
import { CurrencyValue } from '@yourcause/common/masking';
import { isEqual } from 'lodash';
import { BaseFormioComponent } from '../../base/base.component';

@Component({
  selector: 'gc-recommended-funding-amount',
  templateUrl: './gc-recommended-funding-amount.component.html',
  styleUrls: ['./gc-recommended-funding-amount.component.scss']
})
export class GcRecommendedFundingAmountComponent extends BaseFormioComponent<CurrencyValue> implements OnInit {
  defaultCurrency = this.clientSettingsService.defaultCurrency;
  formattingData = this.currencyService.formattingData;

  constructor (
    private clientSettingsService: ClientSettingsService,
    private validatorService: ValidatorsService,
    public formBuilder: TypeSafeFormBuilder,
    formBuilderService: FormBuilderService,
    public componentHelper: ComponentHelperService,
    private currencyService: CurrencyService,
    @Optional() formGroupDir: FormGroupDirective,
    @Optional() renderer: GcFormRendererComponent<BaseApplication>
  ) {
    super(renderer, formGroupDir, formBuilder, formBuilderService, componentHelper);
  }

  ngOnInit () {
    super.ngOnInit();
    let validators: ValidatorFn[] = [];
    if (!this.inFormBuilder) {
      validators = this.validatorService.getValidatorsForSimpleComponent(
        this.comp,
        true,
        false,
        false,
        this.translations
      );
    }
    this.setFormGroup(this.data, validators);
  }

  patchValue () {
    if (
      this.afterInit &&
      !isEqual(this.data, this.control.value)
    ) {
      this.amountChanged();
    }
  }

  onAmountChange (amount = this.data.amountForControl) {
    this.control.markAsDirty();
    this.amountChanged(amount);
  }

  amountChanged (amount = this.data.amountForControl) {
    this.onValueChange.emit({
      value: {
        ...this.data,
        amountForControl: amount
      },
      updateFormGroup: true
    });
  }
}
