import { Component, Input, OnChanges, OnInit, Optional } from '@angular/core';
import { FormGroupDirective } from '@angular/forms';
import { PortalDeterminationService } from '@core/services/portal-determination.service';
import { BaseApplication } from '@core/typings/application.typing';
import { AdHocReportingUI } from '@core/typings/ui/ad-hoc-reporting.typing';
import { ReportFieldDataOptions } from '@features/configure-forms/form.typing';
import { FormBuilderService } from '@features/formio/form-builder/services/form-builder/form-builder.service';
import { GcFormRendererComponent } from '@features/formio/form-renderer/gc-form-renderer/gc-form-renderer.component';
import { ComponentHelperService } from '@features/formio/services/component-helper/component-helper.service';
import { ReportFieldService } from '@features/formio/services/report-field/report-field.service';
import { TypeSafeFormBuilder } from '@yourcause/common';
import { BaseFormioComponent } from '../../base/base.component';

@Component({
  selector: 'gc-report-field',
  templateUrl: './gc-report-field.component.html',
  styleUrls: ['./gc-report-field.component.scss']
})
export class AdHocReportFieldComponent extends BaseFormioComponent<void> implements OnInit, OnChanges {
  @Input() reportFieldDataOptions: ReportFieldDataOptions;
  @Input() displayType: AdHocReportingUI.DisplayTypes;

  row: AdHocReportingUI.ReportResponseRow;
  DisplayTypes = AdHocReportingUI.DisplayTypes;
  column: AdHocReportingUI.ColumnImplementation;
  value: any;

  constructor (
    private portalDeterminationService: PortalDeterminationService,
    private reportFieldService: ReportFieldService,
    formBuilder: TypeSafeFormBuilder,
    formBuilderService: FormBuilderService,
    public componentHelper: ComponentHelperService,
    @Optional() formGroupDir: FormGroupDirective,
    @Optional() renderer: GcFormRendererComponent<BaseApplication>
  ) {
    super(renderer, formGroupDir, formBuilder, formBuilderService, componentHelper);
  }

  get portal () {
    return this.portalDeterminationService.isManager ? 'manager' : 'portal';
  }

  get applicationId (): number  {
    return this.parentFields?.applicationId || null;
  }

  get reportFieldResponse (): AdHocReportingUI.ReportResponseRow {
    return this.parentFields ? this.parentFields.reportFieldResponse : null;
  }

  ngOnInit () {
    super.ngOnInit();
    this.setValue();
    this.setFormGroup(this.value, []);
  }

  setValue () {
    this.value = this.reportFieldResponse ?
    this.reportFieldService.getReportFieldValue(
      this.reportFieldDataOptions,
      this.reportFieldResponse
    ) :
    null;
  }

  ngOnChanges () {
    this.setValue();
  }
}
