import { Injectable } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { ClientSettings, CurrencySetting } from '@core/typings/api/admin-client.typing';
import { ReferenceFieldAPI } from '@core/typings/api/reference-fields.typing';
import { ApplicantFormForUI, BaseApplication } from '@core/typings/application.typing';
import { UserTypes } from '@core/typings/client-user.typing';
import { FormInfoForPDF, TableCsvForPdf } from '@core/typings/pdf.typing';
import { ProgramApplicantType } from '@core/typings/program.typing';
import { AdHocReportingUI } from '@core/typings/ui/ad-hoc-reporting.typing';
import { ReferenceFieldsUI } from '@core/typings/ui/reference-fields.typing';
import { DownloadFormVisibility } from '@features/configure-forms/download-form-modal/download-form-modal.component';
import { BaseApplicationForLogic, Form, FormAudience, FormDecisionTypes, FormDefinitionComponent, FormDefinitionForUi, FormStatuses, FormTypes, ReportFieldDataOptions } from '@features/configure-forms/form.typing';
import { KeyValue } from '@features/custom-data-tables/custom-data-tables.typing';
import { EmployeeSSOFieldsData } from '@features/employee-sso-fields/employee-sso-fields.typing';
import { LogicForColumn, LogicState } from '@features/logic-builder/logic-builder.typing';
import { YcFile } from '@yourcause/common';
import { CurrencyRadioOptions } from '@yourcause/common/masking';
import { AfterEach, BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect, spy } from 'chai';
import { cloneDeep } from 'lodash';
import { BehaviorSubject } from 'rxjs';
import { EMPLOYEE_SSO_TYPE_PREFIX, REF_COMPONENT_TYPE_PREFIX } from '../component-helper/component-helper.service';
import { FormHelperService } from './form-helper.service';


@Injectable({ providedIn: 'root' })
@DescribeAngularService(FormHelperService, {
  imports: [
    GCMockModule
  ]
})
export class FormHelperServiceSpec implements Spec<FormHelperServiceSpec, FormHelperService> {
  tableComponent: FormDefinitionComponent = {
    key: 'table',
    type: 'table',
    rows: [
      [{
        components: [{
          key: 'content',
          type: 'content'
        }]
      }],
      [{
        components: [{
          key: 'refField1',
          type: 'referenceFields-text'
        }, {
          key: 'columns',
          type: 'columns',
          columns: [{
            key: 'column1',
            type: 'columns',
            components: [{
              key: 'refField2',
              type: 'referenceFields-select'
            }]
          }]
        }]
      }],
      [{
        components: [{
          key: 'refField3',
          type: 'referenceFields-radio'
        }]
      }]
    ] as FormDefinitionComponent[][]
  };
  form: Form = {
    title: '',
    name: '',
    path: '',
    id: null,
    description: '',
    createdDate: '',
    updatedDate: '',
    formDefinition: [{
      index: 0,
      logic: null,
      tabName: '',
      components: [
        this.tableComponent
      ]
    }],
    formType: null,
    version: null,
    canModify: true,
    availableForTranslation: true,
    defaultLanguageId: '',
    externalApiRequestIds: [],
    referenceFieldIds: [],
    requireSignature: false,
    signatureDescription: ''
  };
  cdtGuid2 = 'guid2';
  applicantTextField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Text',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.TextField,
    key: 'textKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 1,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  refComponent: FormDefinitionComponent = {
    key: 'refField',
    type: `${REF_COMPONENT_TYPE_PREFIX}${this.applicantTextField.key}`
  };
  applicantTextArea: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Text Area',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.TextArea,
    key: 'textAreaKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 2,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  applicantNumber: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Number',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Number,
    key: 'numberKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 3,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  managerTextField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Text',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.TextField,
    key: 'textFieldManagerKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.MANAGER,
    referenceFieldId: 1,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  checkboxField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Checkbox',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Checkbox,
    key: 'checkboxKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 1756,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  dateField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Date',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Date,
    key: 'dateKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.MANAGER,
    referenceFieldId: 23531,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  currencyField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Currency',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Currency,
    key: 'currencyKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.MANAGER,
    referenceFieldId: 23531,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  managerSingleResponseTextField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Text',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.TextField,
    key: this.managerTextField.key,
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.MANAGER,
    referenceFieldId: 1,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  fileUploadFieldForTable: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'File Upload For Table 1',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.FileUpload,
    key: 'fileUploadTableKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 235237,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: true,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  applicantFile: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'File Upload',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.FileUpload,
    key: 'fileUploadKey',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 435,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  tableField1: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Table 1',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Table,
    key: 'tableKey1',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 12421,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };

  tableField2: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    customDataTableGuid: '',
    name: 'Table 2',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Table,
    key: 'tableKey2',
    supportsMultiple: false,
    categoryId: 0,
    formAudience: FormAudience.APPLICANT,
    referenceFieldId: 325,
    formCount: 1,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    aggregateType: null,
    updateDate: '',
    parentReferenceFieldId: null,
    isEncrypted: false,
    isMasked: false,
    tableAllowsImport: false,
    isSingleResponse: true,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  picklistField: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    isEncrypted: false,
    isMasked: false,
    aggregateType: null,
    referenceFieldId: 23523,
    parentReferenceFieldId: null,
    formCount: 0,
    usedOnReports: false,
    createdBy: '',
    createDate: '',
    updatedBy: '',
    updateDate: '',
    customDataTableGuid: this.cdtGuid2,
    name: 'Custom data table',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.CustomDataTable,
    key: 'dataTable1Key',
    supportsMultiple: true,
    categoryId: null,
    formAudience: FormAudience.APPLICANT,
    isSingleResponse: true,
    tableAllowsImport: false,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: null,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: null
  };
  subsetFieldPercent: ReferenceFieldAPI.ReferenceFieldDisplayModel = {
    isEncrypted: false,
    isMasked: false,
    aggregateType: null,
    referenceFieldId: 7964,
    parentReferenceFieldId: null,
    formCount: 1,
    usedOnReports: true,
    createdBy: '',
    createDate: '',
    updateDate: '',
    updatedBy: '',
    customDataTableGuid: '',
    name: 'Subset field - percent',
    description: '',
    type: ReferenceFieldsUI.ReferenceFieldTypes.Subset,
    key: 'subsetKey',
    supportsMultiple: false,
    categoryId: null,
    formAudience: FormAudience.MANAGER,
    isSingleResponse: true,
    tableAllowsImport: false,
    isTableField: false,
    aggregateTableReferenceFieldId: null,
    referenceFieldTableId: 32095,
    standardComponentIsPublished: true,
    isStandardProductField: false,
    subsetCollectionType: ReferenceFieldAPI.DataSetCollectionType.Percent
  };
  referenceFields: ReferenceFieldAPI.ReferenceFieldDisplayModel[] = [
    this.applicantTextField,
    this.applicantTextArea,
    this.applicantNumber,
    this.managerTextField,
    this.managerSingleResponseTextField,
    this.tableField1,
    this.tableField2,
    this.picklistField,
    this.checkboxField,
    this.currencyField,
    this.dateField,
    this.applicantFile,
    this.fileUploadFieldForTable,
    this.subsetFieldPercent
  ];

  requiredComp1 = {
    key: this.applicantTextField.key,
    type: `referenceFields-${this.applicantTextField.key}`,
    validate: {
      required: true
    }
  } as FormDefinitionComponent;
  requiredAmountRequestedComp = {
    key: 'amountRequested',
    type: 'amountRequested',
    validate: {
      required: true
    }
  } as FormDefinitionComponent;
  requiredDesignationComp = {
    key: 'designation',
    type: 'designation',
    validate: {
      required: true
    }
  } as FormDefinitionComponent;
  unrequiredComp2 = {
    key: this.applicantNumber.key,
    type: `referenceFields-${this.applicantNumber.key}`,
    validate: {
      required: false
    }
  } as FormDefinitionComponent;
  requiredComp3 = {
    key: this.applicantTextArea.key,
    type: `referenceFields-${this.applicantTextArea.key}`,
    validate: {
      required: true
    }
  } as FormDefinitionComponent;
  secondTabName = 'Two';
  sampleFormDef: FormDefinitionForUi[] = [{
    logic: null,
    tabName: 'One',
    index: 0,
    components: [
      this.requiredComp1,
      this.unrequiredComp2
    ]
  }, {
    logic: null,
    index: 1,
    tabName: this.secondTabName,
    components: [
      this.requiredComp3
    ]
  }];
  cdtOptions: KeyValue[] = [{
    key: '1',
    sortOrder: 1,
    value: 'Steelers',
    parentKeys: [],
    parentKeysString: '',
    inUse: true
  }, {
    key: '2',
    sortOrder: 2,
    value: 'Cowboys',
    parentKeys: [],
    parentKeysString: '',
    inUse: true
  }];
  selectedCurrencies: CurrencySetting[] = [{
    code: 'EUR',
    displayName: 'EUR',
    default: false
  }, {
    code: 'USD',
    displayName: 'USD',
    default: true
  }];
  revisionId = 23563;
  applicantForms: ApplicantFormForUI[] = [{
    applicationFormId: 123,
    canApproveApplicaitonForm: true,
    canDeclineApplicationForm: true,
    canRequestRevision: true,
    formComments: [{
      workflowLevelId: 1,
      workflowLevelName: '',
      firstName: '',
      lastName: '',
      profileImageUrl: '',
      notes: '',
      createdDate: ''
    }],
    formData: {},
    formDefinition: [{
      logic: null,
      index: 0,
      tabName: '',
      components: [{
        key: this.applicantTextField.key,
        type: `referenceFields-${this.applicantTextField.key}`
      }]
    }],
    formDescription: '',
    formId: 5446,
    formName: '',
    formRevisionId: this.revisionId,
    formStatus: FormStatuses.Sent,
    formSubmittedOn: '',
    createdBy: '',
    submittedBy: '',
    formTypeId: FormTypes.REQUEST,
    myCommentInCurrentLevel: '',
    statusUpdatedBy: '',
    statusUpdatedByClientUser: null,
    statusUpdatedDate: '',
    totalCommentsInCurrentWorkflow: 1,
    isDefault: true,
    portalFormAvailabilityInfo: null,
    updatedBy: '',
    updatedDate: '',
    impersonatedBy: '',
    userType: UserTypes.APPLICANT,
    dueDate: '03/05/2020',
    isOverdue: true,
    formResponse: null,
    name: '',
    programApplicantType: ProgramApplicantType.ORGS,
    createImpersonatedBy: '',
    isDraft: false,
    requireSignature: false,
    signatureDescription: ''
  }];
  formDefinition: FormDefinitionForUi[] = [{
    tabName: 'One',
    index: 0,
    uniqueId: '353',
    logic: null,
    components: [{
      type: `referenceFields-${this.tableField1.key}`
    } as FormDefinitionComponent,
    {
      type: 'referenceFields-otherField'
    } as FormDefinitionComponent]
  }, {
    tabName: 'Two',
    index: 0,
    uniqueId: '353',
    logic: null,
    components: [{
      type: `referenceFields-${this.tableField2.key}`
    } as FormDefinitionComponent]
  }];
  formDefWithFilesAndTables = {
    components: [{
      key: 'subsetKey',
      type: `referenceFields-${this.subsetFieldPercent.key}`
    }, {
      key: 'tableKey',
      type: `referenceFields-${this.tableField1.key}`
    }, {
      key: 'fileKey',
      type: `referenceFields-${this.applicantFile.key}`
    } as FormDefinitionComponent, {
      key: 'text',
      type: `referenceFields-${this.applicantTextField.key}`
    }, {
      key: 'date',
      type: `referenceFields-${this.dateField.key}`
    }, {
      key: 'number',
      type: `referenceFields-${this.applicantNumber.key}`
    }, {
      key: 'key',
      type: `referenceFields-${this.picklistField.key}`
    }, {
      key: 'inKind',
      type: 'inKindItems',
      items: 'item1,item2,item4'
    }]
  } as FormDefinitionForUi;
  specialHandlingFileId = 123;
  fileUploadId = 344;
  fileUploadForTableId = 4564;
  refResponseMap: ReferenceFieldsUI.RefResponseMap = {
    [this.applicantFile.key]: [
      new YcFile(
        'file name',
        null,
        `https://yourcausegrantsqa.com/apply/download-file?applicationId=160574&applicationFormId=157406&fileId=${this.fileUploadId}&fileName=Screen%20Shot%202021-07-15%20at%2010.09.05%20AM.png`,
        this.fileUploadId
      )
    ],
    [this.subsetFieldPercent.key]: [],
    [this.tableField1.key]: [{
      rowId: 1,
      columns: [{
        referenceFieldId: this.fileUploadFieldForTable.referenceFieldId,
        referenceFieldKey: this.fileUploadFieldForTable.key,
        value: [
          new YcFile(
            'file name',
            null,
            `https://yourcausegrantsqa.com/apply/download-file?applicationId=160574&applicationFormId=157406&fileId=${this.fileUploadForTableId}&fileName=Screen%20Shot%202021-07-15%20at%2010.09.05%20AM.png`,
            this.fileUploadForTableId
          )
        ],
        dateValue: null,
        currencyValue: null,
        numericValue: null,
        file: null,
        files: [],
        applicationFormId: 1,
        applicationId: 2
      }]
    }]
  };
  formInfo = {
    formDefinition: [this.formDefWithFilesAndTables],
    specialHandling: {
      address1: '123 Main',
      city: 'Charleston',
      country: 'US',
      state: 'SC',
      postalCode: '29406',
      fileUrl: `https://yourcausegrantsqa.com/management/download-file?applicationId=174451&applicationFormId=171729&fileId=${this.specialHandlingFileId}&fileName=Screen%20Shot%202022-06-13%20at%2010.47.48%20AM.png`
    },
    referenceFields: this.refResponseMap
  } as FormInfoForPDF;

  @BeforeEach()
  async mock (service: FormHelperService) {
    service['referenceFieldService']['timezoneService'].setTimeZones();
    service['referenceFieldService'].setAllReferenceFields(
      this.referenceFields
    );
    service['customDataTableService']['set'](
      'customDataTableOptionsMap',
      {
        [this.cdtGuid2]: this.cdtOptions
      }
    );
    service['clientSettingsService']['set']('clientSettings', {} as ClientSettings);
    service['clientSettingsService'].setAllCurrencies();
    service['clientSettingsService']['clientSettingsResources'].getAvailableApplicantCurrencies = async () => {
      return this.selectedCurrencies;
    };
    service['clientSettingsService']['clientSettingsResources'].getSelectedCurrencies = async () => {
      return this.selectedCurrencies;
    };
    service['clientSettingsService']['userService'].setLastSelectedCurrency('INR');
    await service['clientSettingsService'].setCurrencySettings();
    await service['clientSettingsService'].setAvailableApplicantCurrencies(1);
    service['referenceFieldService']['referenceFieldsResources']['getTableFields'] = async () => {
      return [{
        referenceFieldId: this.fileUploadFieldForTable.referenceFieldId,
        referenceFieldName: 'File upload',
        label: '',
        isRequired: true,
        showInTable: true,
        columnOrder: 1,
        aggregateColumnReferenceFieldId: null
      }, {
        referenceFieldId: this.picklistField.referenceFieldId,
        referenceFieldName: 'Picklist',
        label: '',
        isRequired: true,
        showInTable: true,
        columnOrder: 1,
        aggregateColumnReferenceFieldId: null
      }, {
        referenceFieldId: this.applicantNumber.referenceFieldId,
        referenceFieldName: 'Number',
        label: '',
        isRequired: true,
        showInTable: true,
        columnOrder: 1,
        aggregateColumnReferenceFieldId: null
      }, {
        referenceFieldId: this.applicantTextArea.referenceFieldId,
        referenceFieldName: 'Text area',
        label: '',
        isRequired: true,
        showInTable: true,
        columnOrder: 1,
        aggregateColumnReferenceFieldId: null
      }, {
        referenceFieldId: this.applicantTextField.referenceFieldId,
        referenceFieldName: 'Text field',
        label: '',
        isRequired: true,
        showInTable: true,
        columnOrder: 1,
        aggregateColumnReferenceFieldId: null
      }, {
        referenceFieldId: this.managerTextField.referenceFieldId,
        referenceFieldName: 'Text',
        label: '',
        isRequired: true,
        showInTable: true,
        columnOrder: 1,
        aggregateColumnReferenceFieldId: null
      }];
    };
    service['referenceFieldService']['referenceFieldsResources']['getSubsetFields'] = async () => {
      return [];
    };
    await service['referenceFieldService'].setColumnsForTable(this.tableField1.referenceFieldId);
  }

  @AfterEach()
  cleanup () {
    spy.restore();
  }

  @TestCase('should be able to check if we already have a file')
  checkIfAlreadyHaveFile (service: FormHelperService) {
    const thisFile = {
      applicationId: '1',
      applicationFormId: '2',
      fileId: '3',
      fileName: 'File exists'
    };
    const foundFiles = [{
      applicationId: '1',
      applicationFormId: '2',
      fileId: '3',
      fileName: 'File exists'
    }, {
      applicationId: '2',
      applicationFormId: '3',
      fileId: '4',
      fileName: 'File does not exist'
    }];
    let alreadyHave = service.checkIfAlreadyHaveFile(
      thisFile,
      foundFiles
    );
    expect(alreadyHave).to.be.true;
    alreadyHave = service.checkIfAlreadyHaveFile(
      {
        ...thisFile,
        fileId: '5'
      },
      foundFiles
    );
    expect(alreadyHave).to.be.false;
  }

  @TestCase('should indicate a GM field cannot be saved on an applicant form')
  ensureGMFieldCannotBeSavedOnApplicantForm (service: FormHelperService) {
    const saveDisabled = service.getSaveIsDisabled(
      this.managerSingleResponseTextField,
      false
    );

    expect(saveDisabled).to.be.true;
  }

  @TestCase('should indicate a GM field can be saved on a GM form')
  ensureGMFieldCanBeSavedOnGMForm (service: FormHelperService) {
    const saveDisabled = service.getSaveIsDisabled(
      this.managerSingleResponseTextField,
      true
    );

    expect(saveDisabled).to.be.false;
  }

  @TestCase('should indicate an applicant field can be saved on an applicant form')
  ensureApplicantFieldCanBeSavedOnApplicantForm (service: FormHelperService) {
    const saveDisabled = service.getSaveIsDisabled(
      this.applicantTextField,
      false
    );

    expect(saveDisabled).to.be.false;
  }

  @TestCase('should indicate an applicant field cannot be saved on a GM form')
  ensureApplicantFieldCannotBeSavedOnGMForm (service: FormHelperService) {
    const saveDisabled = service.getSaveIsDisabled(
      this.applicantTextField,
      true
    );

    expect(saveDisabled).to.be.true;
  }

  @TestCase('get formio changes for copy app')
  getFormioChangesForCopyApp (service: FormHelperService) {
    const refResponseMap: ReferenceFieldsUI.RefResponseMap = {
      textfield: 'Text field answer',
      textarea: 'Text area answer'
    };
    const changes = service.getFormioChangesForCopyApp(refResponseMap);
    expect(changes.length).to.be.equal(
      Object.keys(refResponseMap).length
    );
    const allAreRefFields = changes.every((change) => {
      return change.isReferenceField;
    });
    expect(allAreRefFields).to.be.true;
  }

  @TestCase('should be able to get required ref field keys')
  getRequiredReferenceFieldKeys (service: FormHelperService) {
    const requiredRefKeys = service.getRequiredReferenceFieldKeys(
      null,
      [{
        tabName: 'One',
        uniqueId: '',
        index: 0,
        logic: null,
        components: [{
          key: 'Hidden required ref key',
          validate: {
            required: true
          },
          type: `referenceFields-${this.managerTextField.key}`,
          hiddenFromParent: true
        }]
      }, {
        tabName: 'Two',
        uniqueId: '',
        index: 0,
        logic: null,
        components: [{
          key: 'non required comp'
        }, {
          key: 'Visible required ref comp',
          validate: {
            required: true
          },
          type: `referenceFields-${this.applicantNumber.key}`
        }]
      }]
    );
    requiredRefKeys;
    expect(requiredRefKeys.length).to.be.equal(1);
    const foundCorrectField = requiredRefKeys[0] === this.applicantNumber.key;
    expect(foundCorrectField).to.be.true;
  }

  @TestCase('should be able to get required standard fields')
  getRequiredStandardFields (service: FormHelperService) {
    const result = service.getRequiredStandardFields(
      [{
        tabName: 'One',
        uniqueId: '',
        index: 0,
        logic: null,
        components: [{
          key: 'Amount requested - required but hidden',
          validate: {
            required: true
          },
          type: 'amountRequested',
          hiddenFromParent: true
        }]
      }, {
        tabName: 'Two',
        uniqueId: '',
        index: 0,
        logic: null,
        components: [{
          key: 'non required comp'
        }, {
          key: 'Required and visible designation',
          validate: {
            required: true
          },
          type: 'designation'
        }]
      }]
    );
    expect(result.amountRequestedRequired).to.be.false;
    expect(result.paymentDesignationRequired).to.be.true;
    expect(result.careOfRequired).to.be.false;
  }

  @TestCase('should be able to get required standard fields - all required not hidden')
  getRequiredStandardFieldsAllRequired (service: FormHelperService) {
    const result = service.getRequiredStandardFields(
      [{
        tabName: 'One',
        uniqueId: '',
        index: 0,
        logic: null,
        components: [{
          key: 'Amount requested - required',
          validate: {
            required: true
          },
          type: 'amountRequested'
        }]
      }, {
        tabName: 'Two',
        uniqueId: '',
        index: 0,
        logic: null,
        components: [{
          key: 'non required comp'
        }, {
          key: 'Required and visible decision',
          validate: {
            required: true
          },
          type: 'decision'
        }, {
          key: 'Required and visible care of',
          validate: {
            required: true
          },
          type: 'careOf'
        }, {
          key: 'Required and visible recommended funding',
          validate: {
            required: true
          },
          type: 'reviewerRecommendedFundingAmount'
        }]
      }]
    );
    expect(result.amountRequestedRequired).to.be.true;
    expect(result.decisionRequired).to.be.true;
    expect(result.careOfRequired).to.be.true;
    expect(result.reviewerRecommendedFundingAmountRequired).to.be.true;
  }

  @TestCase('should be able to get table ref ids from form definition')
  getTableRefIdsFromFormDefinition (service: FormHelperService) {
    const tableIds = service.getTableAndSubsetIdsFromFormDefinition([this.formDefinition]);
    expect(tableIds.length).to.be.equal(2);
    const hasBothIds = tableIds.some((id) => {
      return id === this.tableField1.referenceFieldId ||
        id === this.tableField2.referenceFieldId;
    });
    expect(hasBothIds).to.be.true;
  }

  @TestCase('should be able to get decision options with recused')
  getDecisionOptionsWithRecused (service: FormHelperService) {
    const options = service.getDecisionOptions(true, '', '', '');
    expect(options.length).to.be.equal(3);
    const hasCorrectOptions = options.every((opt) => {
      return [
        FormDecisionTypes.Approve,
        FormDecisionTypes.Decline,
        FormDecisionTypes.Recused
      ].includes(opt.value);
    });
    expect(hasCorrectOptions).to.be.true;
  }

  @TestCase('should be able to get decision options without recused')
  getDecisionOptionsWithoutRecused (service: FormHelperService) {
    const options = service.getDecisionOptions(false, '', '', '');
    expect(options.length).to.be.equal(2);
    const hasCorrectOptions = options.every((opt) => {
      return [
        FormDecisionTypes.Approve,
        FormDecisionTypes.Decline
      ].includes(opt.value);
    });
    expect(hasCorrectOptions).to.be.true;
  }

  @TestCase('should be able to get logic value format type for ref field')
  getLogicValueFormatTypeForRefField (service: FormHelperService) {
    let formatType = service.getLogicValueFormatType(
      `${REF_COMPONENT_TYPE_PREFIX}${this.applicantTextArea.key}`
    );
    expect(formatType).to.be.equal('text');
    formatType = service.getLogicValueFormatType(
      `${REF_COMPONENT_TYPE_PREFIX}${this.picklistField.key}`
    );
    expect(formatType).to.be.equal('select');
    formatType = service.getLogicValueFormatType(
      `${REF_COMPONENT_TYPE_PREFIX}${this.checkboxField.key}`
    );
    expect(formatType).to.be.equal('checkbox');
    formatType = service.getLogicValueFormatType(
      `${REF_COMPONENT_TYPE_PREFIX}${this.dateField.key}`
    );
    expect(formatType).to.be.equal('date');
    formatType = service.getLogicValueFormatType(
      `${REF_COMPONENT_TYPE_PREFIX}${this.applicantNumber.key}`
    );
    expect(formatType).to.be.equal('number');
    formatType = service.getLogicValueFormatType(
      `${REF_COMPONENT_TYPE_PREFIX}${this.currencyField.key}`
    );
    expect(formatType).to.be.equal('currency');
  }

  @TestCase('should be able to get logic value format type for non ref field')
  getLogicValueFormatTypeForNonRefField (service: FormHelperService) {
    let formatType = service.getLogicValueFormatType(
      'designation'
    );
    expect(formatType).to.be.equal('text');
    formatType = service.getLogicValueFormatType(
      'amountRequested'
    );
    expect(formatType).to.be.equal('currency');
    formatType = service.getLogicValueFormatType(
      'decision'
    );
    expect(formatType).to.be.equal('select');
  }

  @TestCase('should be able to get options and format value type')
  async getOptionsAndFormatValueType (service: FormHelperService) {
    const result = await service.getOptionsAndFormatValueType(
      {
        type: `referenceFields-${this.picklistField.key}`
      } as FormDefinitionComponent
    );
    const hasCorrectOptions = result.options.length === this.cdtOptions.length &&
      result.options.every((opt) => {
        return [
          'Steelers',
          'Cowboys'
        ].includes(opt.label);
      });
    expect(hasCorrectOptions).to.be.true;
    expect(result.logicValueFormatType).to.be.equal('select');
  }

  @TestCase('should be able to get options and format value type - decision')
  async getOptionsAndFormatValueTypeDecision (service: FormHelperService) {
    const component = {
      type: 'decision',
      allowRecused: true,
      recuseValue: 'Recuse',
      truthyValue: 'Approve',
      falsyValue: 'Decline'
    } as FormDefinitionComponent;
    const result = await service.getOptionsAndFormatValueType(
      component
    );
    const hasCorrectOptions = result.options.length === 3 &&
      result.options.every((opt) => {
        return [
          component.recuseValue,
          component.truthyValue,
          component.falsyValue
        ].includes(opt.label);
      });
    expect(hasCorrectOptions).to.be.true;
    expect(result.logicValueFormatType).to.be.equal('select');
  }

  @TestCase('should be able to get value from component - from parent fields')
  getValueFromComponentFromParentFields (service: FormHelperService) {
    const answer = 'some answer';
    const value = service.getValueFromComponent(
      this.refComponent,
      {
        referenceFields: {
          [this.applicantTextField.key]: answer
        }
      },
      false
    );
    expect(value).to.be.equal(answer);
  }

  @TestCase('should be able to get value from component - blank')
  getValueFromComponentBlank (service: FormHelperService) {
    const value = service.getValueFromComponent(
      {
        type: `${REF_COMPONENT_TYPE_PREFIX}${this.picklistField.key}`,
        key: this.picklistField.key
      },
      {
        referenceFields: {}
      },
      false
    );
    expect(value).to.deep.equal([]);
  }

  @TestCase('should be able to get value from component - employee sso')
  getValueFromComponentEmployeeSso (service: FormHelperService) {
    const region = 'SC';
    const value = service.getValueFromComponent(
      {
        type: `${EMPLOYEE_SSO_TYPE_PREFIX}-region`,
        key: 'employeeField'
      },
      {
        employeeInfo: {
          region
        } as EmployeeSSOFieldsData
      },
      false
    );
    expect(value).to.deep.equal(region);
  }

  @TestCase('should be able to get value from component - report field')
  getValueFromComponentReportField (service: FormHelperService) {
    const reportFieldDataOptions: ReportFieldDataOptions = {
      reportFieldObject: 'application',
      reportFieldDisplay: 'approvedDate'
    };
    const approvedDate = '05/23/2021';
    const value = service.getValueFromComponent(
      {
        type: 'reportField',
        key: 'reportField',
        reportFieldDataOptions
      },
      {
        reportFieldResponse: {
          application: {
            id: 2,
            approvedDate
          } as any,
          form: {
            applicationFormId: 12
          } as any,
          referenceFields: []
        } as AdHocReportingUI.ReportResponseRow
      },
      false
    );
    expect(value).to.be.equal('May 23, 2021');
  }

  @TestCase('should be able to get value from component - amount requested')
  getValueFromComponentAmountRequested (service: FormHelperService) {
    spy.on(service, 'getValueForCurrency', () => {});

    service.getValueFromComponent(
      {
        type: 'amountRequested',
        key: 'amount',
        useCustomCurrency: CurrencyRadioOptions.USE_ANY_CURRENCY
      },
      {
        currencyRequested: 'CAD',
        amountRequested: 300,
        currencyRequestedAmountEquivalent: 200
      },
      false
    );
    expect(service['getValueForCurrency']).to.have.been.called.once;
  }

  @TestCase('should be able to get value from component - ref currency')
  getValueFromComponentRefCurrency (service: FormHelperService) {
    const currencyVal = {
      amountEquivalent: 1,
      amountForControl: 2,
      currency: 'CAD',
      amountInDefaultCurrency: 3
    };

    const value = service.getValueFromComponent(
      {
        type: `referenceFields-${this.currencyField.key}`,
        key: 'amount',
        useCustomCurrency: CurrencyRadioOptions.USE_ANY_CURRENCY
      },
      {
        referenceFields: {
          [this.currencyField.key]: currencyVal
        }
      },
      true,
      true
    );
    expect(value).to.be.equal(currencyVal.amountForControl);
  }

  @TestCase('should be able to get value from component - amount requested from edit')
  getValueFromComponentAmountRequestedFromEdit (service: FormHelperService) {
    spy.on(service, 'getValueForCurrency', () => {});
    const amountRequestedForEdit = 800;

    const value = service.getValueFromComponent(
      {
        type: 'amountRequested',
        key: 'amount',
        useCustomCurrency: CurrencyRadioOptions.USE_ANY_CURRENCY
      },
      {
        currencyRequested: 'CAD',
        amountRequested: 300,
        amountRequestedForEdit,
        currencyRequestedAmountEquivalent: 200
      },
      false,
      true
    );
    expect(service['getValueForCurrency']).to.not.have.been.called;
    expect(value).to.be.equal(amountRequestedForEdit);
  }

  @TestCase('should be able to get value from component - recommended funding')
  getValueFromComponentRecommendedFunding (service: FormHelperService) {
    spy.on(service, 'getValueForCurrency');

    service.getValueFromComponent(
      {
        type: 'reviewerRecommendedFundingAmount',
        key: 'reviewerRecommendedFundingAmount'
      },
      {
        reviewerRecommendedFundingAmount: 300
      },
      false,
      true
    );
    expect(service['getValueForCurrency']).to.have.been.called.once;
  }

  @TestCase('should be able to get value from component - decision')
  getValueFromComponentDecision (service: FormHelperService) {
    const value = service.getValueFromComponent(
      {
        type: 'decision',
        key: 'decision'
      },
      {
        decision: FormDecisionTypes.Approve
      },
      false
    );
    expect(value).to.be.equal(FormDecisionTypes.Approve);
  }

  @TestCase('should be able to get value from component - layout comp')
  getValueFromComponentLayout (service: FormHelperService) {
    const value = service.getValueFromComponent(
      {
        type: 'well',
        key: 'well'
      },
      {},
      false
    );
    const hasNoValue = !value;
    expect(hasNoValue).to.be.equal(true);
  }

  @TestCase('should be able to get value for currency - force default false')
  getValueForCurrencyForceDefaultFalse (service: FormHelperService) {
    const amountRequested = 500;
    const equivalent = 400;
    const currency = 'EUR';
    const forceDefault = false;

    const value = service.getValueForCurrency(
      amountRequested,
      equivalent,
      currency,
      forceDefault,
      CurrencyRadioOptions.USE_ANY_CURRENCY,
      '',
      ''
    );
    const hasCorrectAmountForEdit = value.amountForControl === equivalent;
    expect(hasCorrectAmountForEdit).to.be.equal(true);
  }

  @TestCase('should be able to get value for currency - force default true')
  getValueForCurrencyForceDefaultTrue (service: FormHelperService) {
    const amountRequested = 500;
    const equivalent = 400;
    const currency = 'USD';
    const forceDefault = true;

    const value = service.getValueForCurrency(
      amountRequested,
      equivalent,
      currency,
      forceDefault,
      CurrencyRadioOptions.USE_ANY_CURRENCY,
      '',
      ''
    );
    const hasCorrectAmountForEdit = value.amountForControl === amountRequested;
    expect(hasCorrectAmountForEdit).to.be.equal(true);
  }

  @TestCase('should be able to get value for currency - use default')
  getValueForCurrencyUseDefault (service: FormHelperService) {
    const value = service.getValueForCurrency(
      null,
      null,
      null,
      false,
      CurrencyRadioOptions.USE_ANY_CURRENCY,
      '',
      '12'
    );
    const hasCorrectAmountForEdit = value.amountForControl === 12;
    expect(hasCorrectAmountForEdit).to.be.equal(true);
  }

  @TestCase('should be able to get should force default currency')
  shouldForceDefaultCurrencyInAmountRequested (service: FormHelperService) {
    let isManagerForm = false;
    let isManagerEditingApplicantForm = false;
    let result = service.shouldForceDefaultCurrencyInAmountRequested(isManagerForm, isManagerEditingApplicantForm);
    expect(result).to.be.false;
    isManagerEditingApplicantForm = true;
    result = service.shouldForceDefaultCurrencyInAmountRequested(isManagerForm, isManagerEditingApplicantForm);
    expect(result).to.be.true;
    isManagerEditingApplicantForm = false;
    isManagerForm = true;
    result = service.shouldForceDefaultCurrencyInAmountRequested(isManagerForm, isManagerEditingApplicantForm);
    expect(result).to.be.true;
  }

  @TestCase('should be able to get can toggle currency')
  canToggleCurrency (service: FormHelperService) {
    service['clientSettingsService']['set']('defaultCurrency', 'USD');
    let readOnly = true;
    let isForSetValue = false;
    let canToggle = service.canToggleCurrency(
      'EUR',
      readOnly,
      isForSetValue
    );
    // can toggle because it's read only and currency is different
    expect(canToggle).to.be.true;
    readOnly = false;
    canToggle = service.canToggleCurrency(
      'EUR',
      readOnly,
      isForSetValue
    );
    // cant toggle because read only is fale
    expect(canToggle).to.be.false;
    readOnly = true;
    isForSetValue = true;
    canToggle = service.canToggleCurrency(
      'EUR',
      readOnly,
      isForSetValue
    );
    // cant toggle because it's for set value
    expect(canToggle).to.be.false;
  }


  @TestCase('should be able to get file IDs from form')
  getFileIDsFromForm (service: FormHelperService) {
    const formDef: FormDefinitionForUi[] = [{
      tabName: 'First tab',
      logic: null,
      index: 0,
      uniqueId: '',
      components: [{
        type: `referenceFields-${this.applicantFile.key}`
      } as FormDefinitionComponent]
    }, {
      tabName: 'Second tab',
      logic: null,
      index: 0,
      uniqueId: '',
      components: [{
        type: 'specialHandling'
      } as FormDefinitionComponent]
    }, {
      tabName: 'Third tab',
      logic: null,
      index: 0,
      uniqueId: '',
      components: [{
        type: `referenceFields-${this.tableField1.key}`
      } as FormDefinitionComponent]
    }];

    const fileIds = service.getFileIDsFromForm(
      formDef,
      `https://yourcausegrantsqa.com/apply/download-file?applicationId=160574&applicationFormId=157406&fileId=${this.specialHandlingFileId}&fileName=Screen%20Shot%202021-07-15%20at%2010.09.05%20AM.png`,
      this.refResponseMap
    );

    expect(fileIds.includes(this.specialHandlingFileId)).to.be.true;
    expect(fileIds.includes(this.fileUploadId)).to.be.true;
    expect(fileIds.includes(this.fileUploadForTableId)).to.be.true;
  }

  @TestCase('should be able to get default form')
  getDefaultForm (service: FormHelperService) {
    const result = service.getDefaultForm(this.applicantForms);
    expect(result.formId).to.be.equal(this.applicantForms[0].formId);
  }

  @TestCase('should be able to extract files and tables for pdf')
  extractFilesAndTablesForPdf (service: FormHelperService) {
    const result = service.extractFilesAndTablesForPdf([this.formInfo]);
    expect(result.fileUploads.length).to.be.equal(3);
    const hasAllMyFiles = result.fileUploads.every((file) => {
      return [
        this.specialHandlingFileId,
        this.fileUploadId,
        this.fileUploadForTableId
      ].includes(+file.fileId);
    });
    expect(hasAllMyFiles).to.be.true;
    const hasMyCsv = result.tableCsvs.length === 1 &&
      result.tableCsvs[0].fileName === `${this.tableField1.name}.csv`;
    expect(hasMyCsv).to.be.true;
  }

  @TestCase('should be able to add to file name map when already have')
  addToFileNameMap (service: FormHelperService) {
    const fileName = 'File name';
    const fileNameMap: Record<string, number> = {
      [fileName]: 532
    };
    service.addToFileNameMap(true, 'File name', fileNameMap);
    // already have, so it should not add
    expect(Object.keys(fileNameMap).length).to.be.equal(1);
    expect(fileNameMap[fileName]).to.be.equal(532);
  }

  @TestCase('should be able to check if already have table - true')
  checkIfAlreadyHaveTableTrue (service: FormHelperService) {
    const thisTable: TableCsvForPdf = {
      csv: 'csv',
      referenceFieldId: this.tableField1.referenceFieldId,
      fileName: 'Table 1',
      rows: []
    };
    const allTables: TableCsvForPdf[] = [{
      csv: 'csv',
      referenceFieldId: this.tableField1.referenceFieldId,
      fileName: 'Table 1',
      rows: []
    }, {
      csv: 'csv',
      referenceFieldId: this.tableField2.referenceFieldId,
      fileName: 'Table 2',
      rows: []
    }];
    const alreadyHave = service.checkIfAlreadyHaveTable(thisTable, allTables);
    expect(alreadyHave).to.be.true;
  }

  @TestCase('should be able to check if already have table - false')
  checkIfAlreadyHaveTableFalse (service: FormHelperService) {
    const thisTable: TableCsvForPdf = {
      csv: 'csv',
      referenceFieldId: 1421,
      fileName: 'Table 3',
      rows: []
    };
    const allTables: TableCsvForPdf[] = [{
      csv: 'csv',
      referenceFieldId: this.tableField1.referenceFieldId,
      fileName: 'Table 1',
      rows: []
    }, {
      csv: 'csv',
      referenceFieldId: this.tableField2.referenceFieldId,
      fileName: 'Table 2',
      rows: []
    }];
    const alreadyHave = service.checkIfAlreadyHaveTable(thisTable, allTables);
    expect(alreadyHave).to.be.false;
  }

  @TestCase('should be able to get pdf components for only visible')
  getPdfComponentsForOnlyVisible (service: FormHelperService) {
    const sourceMap = new Map<string, LogicForColumn<BaseApplicationForLogic, boolean>>();
    const dependentMap = new Map<string, LogicForColumn<BaseApplicationForLogic, boolean>[]>();
    const logicState = {
      result$: new BehaviorSubject(true),
      dependentMap,
      sourceMap,
      previousResult: {} as BaseApplicationForLogic
    } as LogicState<BaseApplicationForLogic, boolean>;
    const numberOfComps = this.formDefWithFilesAndTables.components.length;
    const adapted = cloneDeep(this.formDefWithFilesAndTables);
    adapted.components[0].isHidden = true;
    adapted.components[4].isHidden = true;
    const numberOfHidden = 2;
    const result = service.getPdfComponentsForOnlyVisible(logicState, [adapted]);
    const components = result[0].components;
    expect(components.length).to.be.equal(numberOfComps - numberOfHidden);
    const doesNotIncludeHidden = components.every((comp) => {
      return !comp.isHidden;
    });
    expect(doesNotIncludeHidden).to.be.true;
  }

  @TestCase('should be able to map components for pdf - inUse picklist option logic')
  mapComponentsForPdf (service: FormHelperService) {
    const opts = [
      ...this.cdtOptions,
      {
        key: '3',
        sortOrder: 3,
        value: 'Dolphins',
        parentKeys: [],
        parentKeysString: '',
        inUse: false
      },
      {
        key: '4',
        sortOrder: 4,
        value: 'Panthers',
        parentKeys: [],
        parentKeysString: '',
        inUse: false
      }
    ];
    service['customDataTableService']['set'](
      'customDataTableOptionsMap',
      {
        [this.cdtGuid2]: opts
      }
    );
    // Make sure the inactive options get filtered out
    const picklistComp: FormDefinitionComponent = {
      key: 'picklistComp',
      type: `${REF_COMPONENT_TYPE_PREFIX}${this.picklistField.key}`
    };
    const mapped = service.mapComponentsForPdf([picklistComp]);
    const mappedComp = mapped[0];
    const options = mappedComp.referenceField.options.map((opt) => opt.label);
    const inUseOptions = opts.filter((opt) => {
      return opt.inUse;
    }).map((opt) => opt.value);
    expect(options).to.deep.equal(inUseOptions);
  }

  @TestCase('should be able to get pdf components for all')
  getPdfComponentsForAll (service: FormHelperService) {
    const numberOfComps = this.formDefWithFilesAndTables.components.length;
    const adapted = cloneDeep(this.formDefWithFilesAndTables);
    adapted.components[0].isHidden = true;
    adapted.components[4].isHidden = true;
    const result = service.getPdfComponentsForAll([adapted], true);
    const components = result[0].components;
    // Should return all regardless of isHidden
    expect(components.length).to.be.equal(numberOfComps);
  }

  @TestCase('should be able to prepare in kind form')
  async prepareInKindForForm (service: FormHelperService) {
    spy.on(service['inKindService'], 'getItemsById', async () => {
      return [{
        name: 'Item 1',
        identification: '134'
      }, {
        name: 'Item 2',
        identification: '135'
      }, {
        name: 'Item 4',
        identification: '464'
      }];
    });
    await service.prepareInKindForForm([this.formDefWithFilesAndTables]);
    expect(service['inKindService']['getItemsById']).to.have.been.called.once;
  }

  @TestCase('should be able to prepare in kind form - no fetch')
  async prepareInKindForFormNoFetch (service: FormHelperService) {
    spy.on(service['inKindService'], 'getItemsById');
    await service.prepareInKindForForm(this.formDefinition);
    expect(service['inKindService']['getItemsById']).to.not.have.been.called;
  }


  @TestCase('should be able to prepare in kind form - no fetch - has comp but no items')
  async prepareInKindForFormNoFetch2ndScenario (service: FormHelperService) {
    const formDef = {
      components: [{
        type: 'inKindItems',
        items: []
      }]
    } as FormDefinitionForUi;
    spy.on(service['inKindService'], 'getItemsById');
    await service.prepareInKindForForm([formDef]);
    expect(service['inKindService']['getItemsById']).to.not.have.been.called;
  }

  @TestCase('should be able to prepare components for render form')
  async prepareComponentsForRenderForm (service: FormHelperService) {
    spy.on(service, 'prepareTablesAndSubsetsFromComponents', async () => {});
    spy.on(service, 'prepareCdtsAndSubsetsFromComponents', async () => {});
    await service.prepareComponentsForRenderForm([this.formDefinition], [1, 2]);
    expect(service['prepareTablesAndSubsetsFromComponents']).to.have.been.called.once;
    expect(service['prepareCdtsAndSubsetsFromComponents']).to.have.been.called.once;
  }

  @TestCase('should be able to prepare cdts and subsets from components')
  async prepareCdtsAndSubsetsFromComponents (service: FormHelperService) {
    spy.on(service['customDataTableService'], 'getAllCdtOptionsPerForm', async () => {});
    spy.on(service['referenceFieldService'], 'setDataPointsForSubset', async () => {});
    const allComps = service['componentHelper'].getAllComponents([this.formDefWithFilesAndTables]);
    const additionalGuids = ['guid32r23'];
    const formIds = [235];
    await service.prepareCdtsAndSubsetsFromComponents(allComps, additionalGuids, formIds, 1);
    expect(service['customDataTableService']['getAllCdtOptionsPerForm']).to.have.been.called.once;
    expect(service['referenceFieldService']['setDataPointsForSubset']).to.have.been.called.once;
  }

  @TestCase('should be able to prepare tables and subsets from components')
  async prepareTablesAndSubsetsFromComponents (service: FormHelperService) {
    spy.on(service['referenceFieldService'], 'setAllTableAndSubsetColumnsOnForm');
    const allComps = service['componentHelper'].getAllComponents([this.formDefWithFilesAndTables]);
    const cdtsToFetch = await service.prepareTablesAndSubsetsFromComponents(allComps);
    expect(service['referenceFieldService']['setAllTableAndSubsetColumnsOnForm']).to.have.been.called.once;
    expect(cdtsToFetch).to.deep.equal([this.cdtGuid2]);
  }

  @TestCase('should be able to get applicable components for pdf - all')
  getApplicableComponentsForPdf (service: FormHelperService) {
    const result = service.getApplicableComponentsForPdf([this.formDefWithFilesAndTables], this.refResponseMap);
    expect(result.length).to.be.equal(this.formDefWithFilesAndTables.components.length);
  }

  @TestCase('should be able to get applicable components for pdf - some hidden')
  getApplicableComponentsForPdfSomeHidden (service: FormHelperService) {
    const formDef = cloneDeep(this.formDefWithFilesAndTables);
    formDef.components[0].isHidden = true;
    formDef.components[6].isHidden = true;
    const result = service.getApplicableComponentsForPdf([formDef], this.refResponseMap);
    expect(result.length).to.be.equal(this.formDefWithFilesAndTables.components.length - 2);
    const doesNotIncludeHidden = result.every((comp) => {
      return ![
        formDef.components[0].key,
        formDef.components[6].key
      ].includes(comp.key);
    });
    expect(doesNotIncludeHidden).to.be.true;
  }

  @TestCase('should be able to check form group validity - valid')
  checkFormGroupValidityValid (service: FormHelperService) {
    const formGroup = new FormGroup({
      [this.checkboxField.key]: new FormControl(true)
    });
    const valid = service.checkFormGroupValidity(formGroup, this.formDefinition[0]);
    expect(valid).to.be.true;
  }

  @TestCase('should be able to check form group validity - invalid')
  checkFormGroupValidityInvalid (service: FormHelperService) {
    const formDef = {
      components: [{
        type: `referenceFields-${this.checkboxField.key}`,
        key: 'checkbox'
      }]
    } as FormDefinitionForUi;
    const formGroup = new FormGroup({
      [formDef.components[0].key]: new FormControl(null, Validators.required)
    });
    const valid = service.checkFormGroupValidity(formGroup, formDef);
    expect(valid).to.be.false;
  }

  @TestCase('should be able to check form group validity - hidden')
  checkFormGroupValidityHidden (service: FormHelperService) {
    const formDef = {
      components: [{
        type: `referenceFields-${this.checkboxField.key}`,
        key: 'checkbox',
        hiddenFromParent: true
      }]
    } as FormDefinitionForUi;
    const formGroup = new FormGroup({
      [formDef.components[0].key]: new FormControl(null, Validators.required)
    });
    const valid = service.checkFormGroupValidity(formGroup, formDef);
    expect(valid).to.be.true;
  }

  @TestCase('should be able to generate data for form definition')
  generateDataForFormDefinition (service: FormHelperService) {
    spy.on(service, 'getValueFromComponent');
    const app: Partial<BaseApplication> = {
      amountRequested: 400,
      amountRequestedForEdit: 500,
      designation: 'designation',
      careOf: 'care of',
      referenceFields: {}
    };
    const formDef = [{
      index: 0,
      components: [{
        type: 'amountRequested',
        key: 'amountRequested'
      }, {
        type: 'designation',
        key: 'designation'
      }, {
        type: 'careOf',
        key: 'careOf'
      }]
    } as FormDefinitionForUi, {
      components: [{
        type: `referenceFields-${this.tableField1.key}`,
        key: 'tableKey'
      }, {
        type: 'columns',
        columns: [{}]
      }]
    } as FormDefinitionForUi];
    const data = service.generateDataForFormDefinition(app, formDef);
    // Should fire for each component except layout comps (4 times)
    expect(service['getValueFromComponent']).to.have.been.called.exactly(4);
    expect(data.amountRequested).to.be.equal(app.amountRequestedForEdit);
    expect(data.designation).to.be.equal(app.designation);
  }

  @TestCase('should be able to get ref field from form key')
  getRefFieldFromFormKey (service: FormHelperService) {
    const key = 'key';
    const formDef = [{
      components: [{
        key,
        type: `referenceFields-${this.tableField1.key}`
      }, {
        key: 'key2'
      }]
    } as FormDefinitionForUi];
    const field = service.getRefFieldFromFormKey(key, formDef);
    expect(field).to.deep.equal(this.tableField1);
  }

  @TestCase('should be able to get ref field from form key - not found')
  getRefFieldFromFormKeyNotFound (service: FormHelperService) {
    const key = 'key';
    const formDef = [{
      components: [{
        key: 'key3',
        type: 'amountRequested'
      }, {
        key: 'key2'
      }]
    } as FormDefinitionForUi];
    const field = service.getRefFieldFromFormKey(key, formDef);
    expect(!field).to.be.true;
  }

  @TestCase('should be able to get form components by tab - all')
  getFormComponentsByTabAll (service: FormHelperService) {
    const sourceMap = new Map<string, LogicForColumn<BaseApplicationForLogic, boolean>>();
    const dependentMap = new Map<string, LogicForColumn<BaseApplicationForLogic, boolean>[]>();
    const logicState = {
      result$: new BehaviorSubject(true),
      dependentMap,
      sourceMap,
      previousResult: {} as BaseApplicationForLogic
    } as LogicState<BaseApplicationForLogic, boolean>;
    spy.on(service, 'getPdfComponentsForAll');
    spy.on(service, 'getPdfComponentsForOnlyVisible');
    service.getFormComponentsByTab(
      DownloadFormVisibility.SHOW_ALL,
      this.form,
      logicState
    );
    expect(service['getPdfComponentsForAll']).to.have.been.called.once;
    expect(service['getPdfComponentsForOnlyVisible']).to.not.have.been.called;
  }

  @TestCase('should be able to get form components by tab - visible')
  getFormComponentsByTabVisible (service: FormHelperService) {
    const sourceMap = new Map<string, LogicForColumn<BaseApplicationForLogic, boolean>>();
    const dependentMap = new Map<string, LogicForColumn<BaseApplicationForLogic, boolean>[]>();
    const logicState = {
      result$: new BehaviorSubject(true),
      dependentMap,
      sourceMap,
      previousResult: {} as BaseApplicationForLogic
    } as LogicState<BaseApplicationForLogic, boolean>;
    spy.on(service, 'getPdfComponentsForAll');
    spy.on(service, 'getPdfComponentsForOnlyVisible');
    service.getFormComponentsByTab(
      DownloadFormVisibility.SHOW_CONDITIONAL,
      this.form,
      logicState
    );
    expect(service['getPdfComponentsForAll']).to.not.have.been.called;
    expect(service['getPdfComponentsForOnlyVisible']).to.have.been.called.once;
  }

  @TestCase('should be able to determine whether to continue initializing form')
  determineWhetherToContinueInitializingForm (service: FormHelperService) {
    const initializing = service.shouldInitializeNextTabOnFormChange(
      true,
      1,
      1
    );
    expect(initializing).to.be.true;
    const initializing2 = service.shouldInitializeNextTabOnFormChange(
      false,
      1,
      1
    );
    expect(initializing2).to.be.false;
    const initializing3 = service.shouldInitializeNextTabOnFormChange(
      true,
      1,
      2
    );
    expect(initializing3).to.be.false;
    expect(initializing2).to.be.false;
    const initializing4 = service.shouldInitializeNextTabOnFormChange(
      false,
      1,
      2
    );
    expect(initializing4).to.be.false;
  }
}

