import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { AdHocReportingAPI } from '@core/typings/api/ad-hoc-reporting.typing';
import { AdHocReportingUI } from '@core/typings/ui/ad-hoc-reporting.typing';
import { ReportFieldDataOptions } from '@features/configure-forms/form.typing';
import { AdHocReportingMappingService } from '@features/reporting/services/ad-hoc-reporting-mapping.service';
import { AdHocReportingService } from '@features/reporting/services/ad-hoc-reporting.service';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { ReportFieldService } from './report-field.service';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(ReportFieldService, {
  imports: [
    GCMockModule
  ]
})
export class ReportFieldServiceSpec implements Spec<ReportFieldServiceSpec, ReportFieldService> {
  approvedDateReportFieldDataOptions: ReportFieldDataOptions = {
    reportFieldObject: 'application',
    reportFieldDisplay: 'approvedDate'
  };
  endDateReportFieldDataOptions: ReportFieldDataOptions = {
    reportFieldObject: 'cycle',
    reportFieldDisplay: 'endDate'
  };
  row: any = {
    application: {
      approvedDate: '3/11/2011'
    },
    cycle: {
      endDate: '3/11/2011'
    }
  };
  value = '3/11/2011';
  buckets: AdHocReportingUI.ColumnBucket[] = [{
    i18nKey: 'string',
    display: '',
    property: 'application',
    columns: [{
      definition: {
        column: 'approvedDate',
        type: 'text',
        i18nKey: 'common:textApprovedDate',
        format: 'value',
        display: 'approvedDate'
      },
      sortType: AdHocReportingAPI.SortTypes.NoSort,
      filterColumn: {
        columnName: '',
        filters: []
      },
      visibleInReport: false,
      columnNameOverride: ''
    }],
    allColumns: []
  }, {
    i18nKey: 'string',
    display: '',
    property: 'cycle',
    columns: [{
      definition: {
        column: 'endDate',
        type: 'text',
        i18nKey: 'common:textEndDate',
        format: 'value',
        display: 'endDate'
      },
      sortType: AdHocReportingAPI.SortTypes.NoSort,
      filterColumn: {
        columnName: '',
        filters: []
      },
      visibleInReport: false,
      columnNameOverride: ''
    }],
    allColumns: []
  }];

  constructor (
    private adHocReportingService: AdHocReportingService,
    private adHocReportingMapper: AdHocReportingMappingService
  ) { }

  @BeforeEach()
  async mock (service: ReportFieldService) {
    service['adHocReportingService'] = this.adHocReportingService;
    service['adHocReportingMapper'] = this.adHocReportingMapper;

    this.adHocReportingService.getBuckets = () => {
      return this.buckets;
    };
    // the method below is overwritten in order to track data
    this.adHocReportingMapper.getValueForColumnFromRow = (
      columnDef: AdHocReportingUI.ColumnDefinition,
      row: any
    ): {value: string; columnDef: AdHocReportingUI.ColumnDefinition; row: any} => {
        return {
          value: this.value,
          columnDef,
          row
      };
    };
  }

  @TestCase('should be able to get report field values')
  getReportFieldValuesTest (service: ReportFieldService) {
    const approvedDateResponse = service.getReportFieldValue(
      this.approvedDateReportFieldDataOptions,
      this.row as any
    );
    const endDateResponse = service.getReportFieldValue(
      this.endDateReportFieldDataOptions,
      this.row as any
    );
    expect(approvedDateResponse.value).to.equal('3/11/2011');
    expect(endDateResponse.value).to.equal('3/11/2011');
    expect(approvedDateResponse.columnDef.column).to.equal(this.approvedDateReportFieldDataOptions.reportFieldDisplay);
    expect(endDateResponse.columnDef.column).to.equal(this.endDateReportFieldDataOptions.reportFieldDisplay);
  }

}

