import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { BudgetFundingSourceAudit } from '@core/typings/budget.typing';
import { BudgetResources } from '@features/budgets/budget.resources';
import { BudgetService } from '@features/budgets/budget.service';
import { DebounceFactory, PaginationOptions, TableDataFactory, TopLevelFilter } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';

@Component({
  selector: 'gc-funding-source-audit-trail-table',
  templateUrl: './funding-source-audit-trail-table.component.html',
  styleUrls: ['./funding-source-audit-trail-table.component.scss']
})
export class FundingSourceAuditTrailTableComponent implements OnInit {
  id = this.activatedRoute.snapshot.parent.params.id;
  tableKey: string;
  topLevelFilters: TopLevelFilter[] = [
    new TopLevelFilter(
      'text',
      'createdByUserName',
      '',
      this.i18n.translate(
        'GLOBAL:textSearchByUser',
        {},
        'Search by user'
      ),
      undefined,
      undefined
    )
  ];
  tableDataFactory: TableDataFactory<BudgetFundingSourceAudit>;

  constructor (
    private budgetResources: BudgetResources,
    private activatedRoute: ActivatedRoute,
    private budgetService: BudgetService,
    private i18n: I18nService
  ) { }

  ngOnInit () {
    this.tableKey = 'FS_AUDIT_TRAIL_' + this.id;
    this.tableDataFactory = DebounceFactory.createSimple(
      async (options: PaginationOptions<BudgetFundingSourceAudit>) => {
        const result = await this.budgetResources.getFundingSourceAuditTrail(
          this.id,
          options
        );
        const records = this.budgetService.getActionMessagesForAuditTrail(
          result.records,
          true
        );

        return {
          success: true,
          data: {
            recordCount: result.recordCount,
            records
          }
        };
      }
    );
  }
}
