import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { PolicyService } from '@core/services/policy.service';
import { FundingSource } from '@core/typings/budget.typing';
import { BudgetService } from '@features/budgets/budget.service';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { I18nService } from '@yourcause/common/i18n';


@Component({
  selector: 'gc-funding-source-detail-page',
  templateUrl: './funding-source-detail-page.component.html',
  styleUrls: ['./funding-source-detail-page.component.scss']
})
export class FundingSourceDetailPageComponent implements OnInit {
  id = +this.activatedRoute.snapshot.params.id;
  fundingSource: FundingSource;
  totalAllocatedText = this.i18n.translate(
    'GLOBAL:textTotalAllocated',
    {},
    'Total allocated'
  );
  totalPaymentsText = this.i18n.translate(
    'GLOBAL:textTotalPayments',
    {},
    'Total payments'
  );
  totalUnavailableText = this.i18n.translate(
    'GLOBAL:textTotalUnavailable',
    {},
    'Total unavailable'
  );
  totalUnallocatedText = this.i18n.translate(
    'BUDGET:textTotalUnallocated',
    {},
    'Total unallocated'
  );
  totalReservedText = this.i18n.translate(
    'BUDGET:textTotalReserved',
    {},
    'Total reserved'
  );


  hasInsightsPermission = this.policyService.insights.canViewFundingSources();

  constructor (
    private activatedRoute: ActivatedRoute,
    private budgetService: BudgetService,
    private i18n: I18nService,
    private clientSettingsService: ClientSettingsService,
    private policyService: PolicyService,
    private router: Router
  ) { }

  get canReserveFunds () {
    return this.clientSettingsService.get('clientSettings').reserveFunds;
  }

  async ngOnInit () {
    this.fundingSource = this.budgetService.allFundingSources.find((source) => {
      return source.id === this.id;
    });
  }

  routeToFundingInsights () {
    return this.router.navigateByUrl(
      `/management/insights/funding-sources/funding-source/${this.id}`
    );
  }
}
