import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { LocaleResolver } from '@core/resolvers/locale.resolver';
import { GCCoreModule } from '@core/_core.module';
import { InKindResolver } from '@features/in-kind/resolvers/in-kind.resolver';
import { BaseFundingSourcePageComponent } from './base-funding-source-page/base-funding-source-page.component';
import { FundingSourceAuditTrailTableComponent } from './funding-source-audit-trail-table/funding-source-audit-trail-table.component';
import { FundingSourceDetailPageComponent } from './funding-source-detail-page/funding-source-detail-page.component';
import { FundingSourceWrapperComponent } from './funding-source-wrapper/funding-source-wrapper.component';
import { FundingSourceModule } from './funding-source.module';
import { FundingSourceResolver } from './resolvers/funding-source.resolver';


@NgModule({
  imports: [
    GCCoreModule,
    FundingSourceModule,
    RouterModule.forChild([{
      path: '',
      component: FundingSourceWrapperComponent,
      resolve: {
        sources: FundingSourceResolver,
        locale: LocaleResolver,
        inKind: InKindResolver
      },
      data: {
        namespace: ['BUDGET'],
        breadcrumb: true,
        breadcrumbLabel: 'Funding Sources'
      },
      children: [{
        path: 'open',
        component: BaseFundingSourcePageComponent,
        data: {
          breadcrumbLabel: 'Open Sources',
          breadcrumb: true
        }
      }, {
        path: 'closed',
        component: BaseFundingSourcePageComponent,
        data: {
          breadcrumbLabel: 'Closed Sources',
          breadcrumb: true
        }
      }, {
        path: '',
        pathMatch: 'full',
        redirectTo: 'open'
      }]
    }, {
        path: ':id',
        component: FundingSourceDetailPageComponent,
        resolve: {
          sources: FundingSourceResolver
        },
        children: [{
          path: 'audit-trail',
          component: FundingSourceAuditTrailTableComponent,
          data: {
            breadcrumbLabel: 'Detail',
            breadcrumb: true
          }
        }],
        data: {
          breadcrumbLabel: 'Detail',
          breadcrumb: true
        }
    }, {
      path: '',
      pathMatch: 'full',
      redirectTo: 'open'
    }])
  ]
})
export class FundingSourceRoutingModule { }
