import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { ImpersonationBannerService } from './impersonation-banner.service';


@Injectable({ providedIn: 'root' })
@DescribeAngularService(ImpersonationBannerService, {
  imports: [
    GCMockModule
  ]
})
export class ImpersonationBannerServiceSpec implements Spec<ImpersonationBannerServiceSpec, ImpersonationBannerService> {

  @BeforeEach()
  mock (service: ImpersonationBannerService) {
    service['userService'].setUser({
      firstName: 'Tina',
      lastName: 'Conway',
      active: true,
      culture: 'en-US',
      email: '',
      id: 1,
      isNewUser: false,
      isRootUser: false,
      jobTitle: '',
      profileImageUrl: '',
      requirePasswordReset: false,
      roles: [],
      workflows: [],
      workFlowLevels: [],
      acceptedTermsOfService: true,
      clientHasNominations: true,
      isInNominationWorkFlow: true,
      isIntegratedWithCsrZone: true,
      isSso: false
    });
  }

  @TestCase('should be able to set banner info')
  async setBannerInfo (service: ImpersonationBannerService) {
    service['tokenService']['hasImpersonationToken'] = () => {
      return true;
    };
    const bannerInfo = service.getImpersonationBanner();
    expect(bannerInfo.visible).to.be.true;
    expect(bannerInfo.message).to.include('Tina Conway');
  }


}

