import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { DeepLinkingService } from '@core/services/deep-linking.service';
import { SpinnerService } from '@core/services/spinner.service';
import { AuthBehaviors } from '@core/services/token/token-behaviors';
import { TokenStorageService } from '@core/services/token/token-storage.service';
import { TokenService } from '@core/services/token/token.service';
import { TokenResponse } from '@core/typings/token.typing';
import { ClientUserService } from '@features/client-user/client-user.service';
import { ImpersonationService } from '../impersonation.service';

@Component({
  selector: 'gc-impersonation-landing-page',
  templateUrl: './impersonation-landing-page.component.html',
  styleUrls: ['./impersonation-landing-page.component.scss']
})
export class ImpersonationLandingPageComponent implements OnInit {

  constructor (
    private tokenService: TokenService,
    private activatedRoute: ActivatedRoute,
    private clientUserService: ClientUserService,
    private spinnerService: SpinnerService,
    private deepLinkingService: DeepLinkingService,
    private behavior: AuthBehaviors,
    private impersonationService: ImpersonationService,
    private tokenStorageService: TokenStorageService
  ) { }

  async ngOnInit () {
    this.spinnerService.startSpinner();
    if (!this.tokenService.getIsLoggedIn()) {
      await this.handleImpersonation();
    } else {
      await this.impersonationService.logoutExistingUserBeforeImpersonation();
      setTimeout(async () => {
        await this.handleImpersonation();
      }, 750);
    }
    this.spinnerService.stopSpinner();
  }

  async handleImpersonation () {
    const rawResponse = (
      this.activatedRoute.snapshot.queryParams as {
        response: string;
        clientIdentifier: string;
      }
    );
    const tokenResponse: TokenResponse = JSON.parse(atob(rawResponse.response));
    this.tokenService.tokenSignin(tokenResponse);
    this.tokenStorageService.overrideClientIdentifier(rawResponse.clientIdentifier);
    this.impersonationService.setImpersonationSessionId();
    await this.clientUserService.getUser(true, true);
    this.deepLinkingService.attemptDeepLink(this.behavior.current.postLoginRedirect);
  }
}
