import { Component, Input, OnInit } from '@angular/core';
import { Validators } from '@angular/forms';
import { CurrencyService } from '@core/services/currency.service';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { TypeSafeFormBuilder, TypeSafeFormGroup } from '@yourcause/common';
import { AnalyticsService, EventType } from '@yourcause/common/analytics';
import { YCModalComponent } from '@yourcause/common/modals';
import { AddEditItemModalResponse, InKindCategoryItemStat } from '../in-kind.typing';
interface AddEditCategoryItemFormGroup {
  itemName: string;
  units: number;
  value: number;
  identification: string;
}

@Component({
  selector: 'gc-add-edit-category-item-modal',
  templateUrl: './add-edit-category-item-modal.component.html',
  styleUrls: ['./add-edit-category-item-modal.component.scss']
})
export class AddEditCategoryItemModalComponent extends YCModalComponent<AddEditItemModalResponse> implements OnInit {
  @Input() item: InKindCategoryItemStat;

  formGroup: TypeSafeFormGroup<AddEditCategoryItemFormGroup>;
  currencyData = this.currencyService.formattingData;
  defaultCurrency = this.clientSettingsService.defaultCurrency;
  formattingData = this.currencyData[this.defaultCurrency];

  constructor (
    private formBuilder: TypeSafeFormBuilder,
    private currencyService: CurrencyService,
    private clientSettingsService: ClientSettingsService,
    private analyticsService: AnalyticsService
  ) {
    super();
  }

  ngOnInit () {
    const item = this.item || {} as InKindCategoryItemStat;
    this.formGroup = this.formBuilder.group<AddEditCategoryItemFormGroup>({
      itemName: [item.name, Validators.required],
      units: item.units,
      value: [item.value, Validators.required],
      identification: [item.identification, Validators.required]
    });
  }

  onCancel () {
    this.closeModal.emit();
  }

  onSubmit () {
    const formVal = this.formGroup.value;
    this.closeModal.emit({
      name: formVal.itemName,
      units: formVal.units,
      value: formVal.value,
      identification: formVal.identification
    });
    this.analyticsService.emitEvent({
      eventName: 'Add edit category item modal save',
      eventType: EventType.Click,
      extras: null
    });
  }
}
