import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { TableDataDownloadFormat, TopLevelFilter } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { ModalFactory } from '@yourcause/common/modals';
import { AddInKindCategoryModalComponent } from '../add-in-kind-category-modal/add-in-kind-category-modal.component';
import { InKindService } from '../in-kind.service';
import { InKindCategory } from '../in-kind.typing';

@Component({
  selector: 'gc-in-kind-categories-page',
  templateUrl: 'in-kind-categories-page.component.html',
  styleUrls: ['./in-kind-categories-page.component.scss']
})

export class InKindCategoriesPageComponent implements OnInit {
  hasInternational = this.clientSettingsService.clientSettings.hasInternational;
  hasLangs = !this.clientSettingsService.noSelectedLanguages;
  categories$ = this.inKindService.changesTo$('categories');
  topLevelFilters: TopLevelFilter[] = [];
  categoryTypeMap = this.inKindService.categoryTypeMap;
  TableDataDownloadFormat = TableDataDownloadFormat;
  constructor (
    private clientSettingsService: ClientSettingsService,
    private inKindService: InKindService,
    private modalFactory: ModalFactory,
    private i18n: I18nService,
    private router: Router
  ) { }

  ngOnInit () {
    this.setTopLevelFilters();
  }

  setTopLevelFilters () {
    this.topLevelFilters = [
      new TopLevelFilter(
        'text',
        'name',
        '',
        this.i18n.translate(
          'CONFIG:textSearchCategory',
          {},
          'Search category'
        )
      )
    ];
  }
  async editCategoryModal (category: InKindCategory) {
    const response = await this.modalFactory.open(
      AddInKindCategoryModalComponent,
      {
        id: category.id,
        context: 'edit',
        name: category.name
      }
    );
    if (response) {
      const payload = {
        categoryId: category.id,
        name: response as string
      };
      this.inKindService.editCategory(payload);
    }
  }
  async createNewCategory () {
    const id = await this.modalFactory.open(
      AddInKindCategoryModalComponent,
      {
        context: 'add'
       }
    );
    if (id) {
      this.router.navigate([
        '/management/configure/in-kind/detail/' + id
      ]);
    }
  }

  downloadRow (
    row: InKindCategory) {
    return this.inKindService.exportData(
      row.id,
      row.language || this.clientSettingsService.defaultLanguage
    );
  }
}
