import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { DebounceFactory, PaginationOptions, TableDataDownloadFormat, TableDataFactory, TopLevelFilter } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { ModalFactory } from '@yourcause/common/modals';
import { AddEditCategoryItemModalComponent } from '../add-edit-category-item-modal/add-edit-category-item-modal.component';
import { AddInKindCategoryModalComponent } from '../add-in-kind-category-modal/add-in-kind-category-modal.component';
import { InKindResources } from '../in-kind.resources';
import { InKindService } from '../in-kind.service';
import { InKindCategory, InKindCategoryItemStat } from '../in-kind.typing';

@Component({
  selector: 'gc-in-kind-category-detail-page',
  templateUrl: './in-kind-category-detail-page.component.html',
  styleUrls: ['./in-kind-category-detail-page.component.scss']
})
export class InKindCategoryDetailPageComponent implements OnInit {
  id: number;
  name: string;
  category: InKindCategory;
  defaultLang: string;
  topLevelFilters: TopLevelFilter[] = [];
  tableDataFactory: TableDataFactory<InKindCategoryItemStat>;
  hasItems = false;
  TableDataDownloadFormat = TableDataDownloadFormat;
  constructor (
    private inKindService: InKindService,
    private activatedRoute: ActivatedRoute,
    private modalFactory: ModalFactory,
    private i18n: I18nService,
    private inKindResources: InKindResources
  ) { }

  get categories () {
    return this.inKindService.categories;
  }

  ngOnInit () {
    this.id = this.activatedRoute.snapshot.params.id;
    this.category = this.categories.find((table) => {
      return +table.id === + this.id;
    });
    this.name = this.category.name;
    this.defaultLang = this.category.language;
    this.setTableDataFactory();
    this.topLevelFilters = [
      new TopLevelFilter(
        'text',
        'name',
        '',
        this.i18n.translate(
          'CONFIG:textSearchItemsandIdNumber',
          {},
          'Search by item name or identification'
        ),
        undefined,
        undefined,
        [{
          column: 'identification',
          filterType: 'cn'
        }, {
          column: 'name',
          filterType: 'cn'
        }]
      )
    ];
  }

  setTableDataFactory () {
    this.tableDataFactory = DebounceFactory.createSimple(
      async (options: PaginationOptions<InKindCategoryItemStat>) => {
        const data = await this.inKindResources.getCategoryItems(
          [this.id],
          this.defaultLang,
          options
        );
        this.hasItems = data.records.length > 0;

        return {
          success: true,
          data
        };
      }, 500);
  }

  export () {
    this.inKindService.exportData(
      this.category.id,
      this.defaultLang
    );
  }

  async toggleInactive (row: InKindCategoryItemStat) {
    await this.inKindService.toggleItemIsActive(
      this.category.id,
      row.identification,
      row.inUse
    );
  }

  async addOrEditItem (item?: InKindCategoryItemStat) {
    const response = await this.modalFactory.open(
      AddEditCategoryItemModalComponent,
      {
        item
      }
    );
    if (response) {
      await this.inKindService.addOrEditCategoryItem(
        this.category.id,
        response,
        !!item
      );
    }
  }
  async import () {
    const id = await this.modalFactory.open(
      AddInKindCategoryModalComponent,
      {
        importOnly: true,
        id: this.id,
        context: 'add'
      }
    );
    if (id) {
      this.inKindService.resetCategoryItemsRepo(id as number);
    }
  }
}
