import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { InKindService } from './in-kind.service';
import { InKindCategoryItemStat } from './in-kind.typing';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(InKindService, {
  imports: [
    GCMockModule
  ]
})
export class InKindServiceSpec implements Spec<
  InKindServiceSpec, InKindService
> {

  @TestCase('should be able to get value map')
  getValueMap (service: InKindService) {
    const items: InKindCategoryItemStat[] = [{
      inUse: true,
      identification: 'burgers',
      name: 'Burgers',
      units: 1500,
      value: 5.5,
      unitsPaid: 100,
      unitsRemaining: 1400,
      categoryId: 1
    }, {
      inUse: true,
      identification: 'hotdogs',
      name: 'Hotdogs',
      units: 1000,
      value: 2.5,
      unitsPaid: 50,
      unitsRemaining: 950,
      categoryId: 2
    }];
    const map = service.getValueMap(items);
    const expectedItemsInMap = 2;
    expect(Object.keys(map).length).to.be.equal(expectedItemsInMap);
    const hasCorrectBurgerValue = map['burgers'] === 5.5;
    const hasCorrectHotdogValue = map['hotdogs'] === 2.5;
    expect(hasCorrectBurgerValue).to.be.true;
    expect(hasCorrectHotdogValue).to.be.true;
  }
}
