import { Component, OnInit } from '@angular/core';
import { StringMap } from '@features/my-workspace/my-workspace.typing';
import { ArrayHelpersService, defaultCKEditorConfig, TopLevelFilter, TopLevelFilterOption } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { InKindService } from '../in-kind.service';

@Component({
  selector: 'gc-translate-in-kind-items-page',
  templateUrl: './translate-in-kind-items-page.component.html',
  styleUrls: ['./translate-in-kind-items-page.component.scss']
})

export class TranslateInKindItemsPageComponent implements OnInit {
  topLevelFilters: TopLevelFilter[];
  categoryOptions: TopLevelFilterOption[] = [];
  richTextMap: StringMap = {};
  ckEditorConfig = defaultCKEditorConfig;

  constructor (
    private i18n: I18nService,
    private inKindService: InKindService,
    private arrayHelper: ArrayHelpersService
  ) { }

  ngOnInit () {
    const categories = this.inKindService.categories;
    this.categoryOptions = this.arrayHelper.sort(
      categories.map((cat) => {
        return {
          label: cat.name,
          value: cat.id
        };
      }),
      'label'
    );
    categories.forEach((cat) => {
      this.richTextMap[cat.id] = cat.name;
    });
    const categoryFilter = new TopLevelFilter(
      'checkboxDropdown',
      'categoryId',
      [],
      this.i18n.translate(
        'GLOBAL:textSearchForACategory',
        {},
        'Search for a category'
      ),
      {
        selectOptions: this.categoryOptions,
        filterObjectName: this.i18n.translate(
          'GLOBAL:textCategory', {}
        ).toLowerCase(),
        filterObjectNamePlural: this.i18n.translate(
          'GLOBAL:textCategories', {}
        ).toLowerCase()
      }
    );
    this.topLevelFilters = [
      new TopLevelFilter(
        'text',
        'defaultTranslationText',
        '',
        this.i18n.translate(
          'FORMS:textSearchTranslations',
          {},
          'Search translations'
        )
      )
    ];
    if (this.categoryOptions.length > 0) {
      this.topLevelFilters.push(categoryFilter);
    }
  }

  getDefaultLangOnFilterChange = (ids: number[]) => {
    return this.inKindService.getMostCommonDefaultLangFromArray(ids);
  };
}
