import { Component, Input, OnInit } from '@angular/core';
import { StatusService } from '@core/services/status.service';
import { FundingSourceTypes } from '@core/typings/budget.typing';
import { Payment } from '@core/typings/payment.typing';
import { AwardForDash, AwardPaymentInfo } from '@features/awards/typings/award.typing';
import { BudgetService } from '@features/budgets/budget.service';

@Component({
  selector: 'gc-award-insights-drilldown',
  templateUrl: './award-insights-drilldown.component.html',
  styleUrls: ['./award-insights-drilldown.component.scss']
})
export class AwardInsightsDrilldownComponent implements OnInit {
  @Input() award: AwardForDash;
  @Input() payments: AwardPaymentInfo[];
  fundingSourceTypes = FundingSourceTypes;
  paymentStatusMap = this.statusService.paymentStatusMap;
  paymentsForSum: Partial<Payment>[];
  isInKind = false;

  constructor (
    private statusService: StatusService,
    private budgetService: BudgetService
  ) { }

  get fundingSources () {
    return this.budgetService.allFundingSources;
  }

  ngOnInit () {
    const items = this.award.award.inKindItems;
    this.isInKind = !!(items && items.length);
    this.paymentsForSum = this.payments.map((payment) => {
      return {
        typeId: payment.fundingSourceType,
        amount: payment.amount,
        fundingSource: payment.fundingSourceId,
        statusId: payment.status
      };
    });
  }

}
