import { Component, Input, OnChanges } from '@angular/core';
import { AwardResources } from '@features/awards/award.resources';
import { AwardTopLevelStats } from '@features/awards/typings/award.typing';
import { I18nService } from '@yourcause/common/i18n';


@Component({
  selector: 'gc-award-top-level-block',
  templateUrl: './award-top-level-block.component.html',
  styleUrls: ['./award-top-level-block.component.scss']
})

export class AwardTopLevelBlockComponent implements OnChanges {
  @Input() cycleIds: number[] = [];
  @Input() showPies = true;
  totalApplicationsString = '';
  totalPaymentsString = '';
  awardTopLevelStats: AwardTopLevelStats;
  graphOneData: number[];
  graphOneLabels = [
    this.i18n.translate(
      'AWARDS:textAwardsToIndividuals',
      {},
      'Awards to individuals'
    ),
    this.i18n.translate(
      'AWARDS:textAwardsToOrganizations',
      {},
      'Awards to organizations'
    )
  ];
  graphTwoData: number[];
  graphTwoLabels = [
    this.i18n.translate(
      'GLOBAL:lblAwardBalance',
      {},
      'Award balance'
    ),
    this.i18n.translate(
      'common:textPaymentTotal',
      {},
      'Payment total'
    )
  ];


  constructor (
    private i18n: I18nService,
    private awardResources: AwardResources
  ) { }

  async ngOnChanges () {
    this.awardTopLevelStats = await this.awardResources.getAwardStats(
      this.cycleIds
    );
    this.totalApplicationsString = this.i18n.translate(
      'common:textNumberOfApplications',
      {
        number: this.awardTopLevelStats.numberOfApplications
      },
      '__number__ applications'
    );
    this.totalPaymentsString = this.i18n.translate(
      'common:textNumberOfPayments',
      {
        number: this.awardTopLevelStats.numberOfPayments
      },
      '__number__ payments'
    );
    this.graphOneData = [
      this.awardTopLevelStats.totalAwardsToIndividuals,
      this.awardTopLevelStats.totalAwardsToOrganizations
    ];
    this.graphTwoData = [
      this.awardTopLevelStats.totalRemaining,
      this.awardTopLevelStats.totalSpent
    ];
  }
}
