import { Component, ElementRef, Input, OnChanges, ViewChild } from '@angular/core';
import { CurrencyService } from '@core/services/currency.service';
import { AwardTopLevelStats } from '@features/awards/typings/award.typing';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { ChartService } from '@yourcause/common';
import { ChartData, ChartOptions, ChartType, TooltipItem } from 'chart.js';

@Component({
  selector: 'gc-award-top-level-pie-chart',
  templateUrl: './award-top-level-pie-chart.component.html',
  styleUrls: ['./award-top-level-pie-chart.component.scss']
})

export class AwardTopLevelPieChartComponent implements OnChanges {
  @Input() awardTopLevelStats: AwardTopLevelStats;
  @Input() labels: string[];
  @Input() legendLabels: string[];
  @Input() data: number[];
  @ViewChild('pieChart') pieChart: ElementRef;

  branding = this.clientSettingsService.get('clientBranding');
  backgroundColors: string[] = [];
  legendData: {
    label: string;
    color: string;
  }[];
  chartOptions: ChartOptions = {
    responsive: true,
    plugins: {
      legend: {
        display: false
      }
    },
    tooltips: {
      callbacks: {
        label: (tooltipItem: TooltipItem<ChartType>, data: ChartData) => {
          return this.chartService.getLabelForTooltip(
            tooltipItem,
            data
          );
        },
        afterLabel: (tooltipItem: TooltipItem<ChartType>, data: ChartData) => {
          const amount = this.chartService.getAmountForTooltip(
            tooltipItem,
            data
          );

          return this.currencyService.formatMoney(
            amount
          );
        }
      }
    }
  } as ChartOptions;

  constructor (
    private clientSettingsService: ClientSettingsService,
    private chartService: ChartService,
    private currencyService: CurrencyService
  ) { }

  ngOnChanges () {
    this.pieChartSetup();
  }

  pieChartSetup () {
    this.backgroundColors = [
      this.chartService.convertHex(
        this.branding.brandPrimary,
        70
      ),
      this.chartService.convertHex(
        this.chartService.shadeColor(this.branding.brandPrimary, 0.5),
        70
      )
    ];
    this.legendData = this.chartService.generateLegendData(
      this.data,
      this.legendLabels,
      this.backgroundColors
    );
  }
}
