import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { LocaleResolver } from '@core/resolvers/locale.resolver';
import { GCCoreModule } from '@core/_core.module';
import { InsightsModule } from '../insights.module';
import { BudgetDashboardResolver } from './budget-dashboard.resolver';
import { BudgetDashboardComponent } from './budget-dashboard/budget-dashboard.component';
import { BudgetsInsightsPageComponent } from './budgets-insights-page/budgets-insights-page.component';

@NgModule({
  imports: [
    GCCoreModule,
    InsightsModule,
    RouterModule.forChild([{
      path: '',
      component: BudgetsInsightsPageComponent,
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Budgets',
        namespace: ['BUDGET'],
        pageName: 'Insights - Budgets'
      },
      resolve: {
        locale: LocaleResolver
      }
    }, {
        path: 'nonprofit',
        loadChildren: () => import('@features/nonprofit/nonprofit-routing.module').then(m => m.NonprofitRoutingModule)
    }, {
        path: 'applicant-profile',
        loadChildren: () => import('@features/applicant/applicant-routing.module').then(m => m.ApplicantRoutingModule)
    }, {
      path: '',
      children: [{
        path: 'budget/:id',
        component: BudgetDashboardComponent,
        data: {
          breadcrumb: true,
          breadcrumbLabel: 'Budget Dashboard',
          namespace: ['BUDGET'],
          pageName: 'Budget Dashboard'
        },
        resolve: {
          budget: BudgetDashboardResolver
        }
      }]
    }])
  ]
})
export class BudgetInsightsRoutingModule { }
