import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { LocaleResolver } from '@core/resolvers/locale.resolver';
import { FundingSourceResolver } from '@features/funding-sources/resolvers/funding-source.resolver';
import { BudgetInsightsResolver } from './budgets/budget-insights.resolver';
import { FundingSourceInsightsResolver } from './funding-sources/funding-source-insights.resolver';
import { InsightsPageComponent } from './insights-page/insights-page.component';
import { InsightsModule } from './insights.module';
import { ProgramInsightsResolver } from './program-manager/resolvers/program-insights.resolver';

@NgModule({
  imports: [
    InsightsModule,
    RouterModule.forChild([{
      path: '',
      component: InsightsPageComponent,
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Insights',
        pageName: 'Insights'
      }
    }, {
      path: 'program-manager',
      loadChildren: () => import('@features/insights/program-manager/program-manager-routing.module').then(m => m.ProgramManagerRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Programs',
        namespace: ['PROGRAM'],
        pageName: 'Insights - Programs'
      },
      resolve: {
        locale: LocaleResolver,
        programs: ProgramInsightsResolver,
        sources: FundingSourceResolver
      }
    }, {
      path: 'budgets',
      loadChildren: () => import('@features/insights/budgets/budget-insights-routing.module').then(m => m.BudgetInsightsRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Budgets',
        namespace: ['BUDGET'],
        pageName: 'Insights - Budgets'
      },
      resolve: {
        locale: LocaleResolver,
        budgets: BudgetInsightsResolver
      }
    }, {
      path: 'applicants',
      loadChildren: () => import('@features/insights/applicants/applicant-insights-routing.module').then(m => m.ApplicantInsightsRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Applicants',
        namespace: ['APPLICANT'],
        pageName: 'Insights - Applicants'
      },
      resolve: {
        programs: ProgramInsightsResolver
      }
    }, {
      path: 'funding-sources',
      loadChildren: () => import('@features/insights/funding-sources/funding-sources-insights-routing.module').then(m => m.FundingSourcesInsightsRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Funding Sources',
        namespace: ['BUDGET'],
        pageName: 'Insights - Funding Sources'
      },
      resolve: {
        fundingSources: FundingSourceInsightsResolver,
        locale: LocaleResolver
      }
    }, {
      path: 'organizations',
      loadChildren: () => import('@features/insights/organizations/organization-insights-routing.module').then(m => m.OrganizationInsightsRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Organizations',
        namespace: ['nonprofit'],
        pageName: 'Insights - Organizations'
      },
      resolve: {
        programs: ProgramInsightsResolver
      }
    }, {
      path: 'awards',
      loadChildren: () => import('@features/insights/awards/award-insights-routing.module').then(m => m.AwardInsightsRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Awards',
        namespace: ['AWARDS'],
        pageName: 'Insights - Awards'
      },
      resolve: {
        programs: ProgramInsightsResolver,
        sources: FundingSourceResolver
      }
    }]
  )]
})
export class InsightsRoutingModule { }
