import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { GCCoreModule } from '@core/_core.module';
import { ApplicationManagerResolver } from '@features/application-manager/resolvers/application-manager.resolver';
import { InsightsModule } from '../insights.module';
import { ApplicantsTabComponent } from './program-dashboard/applicants-tab/applicants-tab.component';
import { ApplicationsTabComponent } from './program-dashboard/applications-tab/applications-tab.component';
import { AwardsTabComponent } from './program-dashboard/awards-tab/awards-tab.component';
import { OrganizationsTabComponent } from './program-dashboard/organizations-tab/organizations-tab.component';
import { PaymentsTabComponent } from './program-dashboard/payments-tab/payments-tab.component';
import { ProgramDashboardPageComponent } from './program-dashboard/program-dashboard-page/program-dashboard-page.component';
import { ProgramInsightsPageComponent } from './program-insights-page/program-insights-page.component';
import { ProgramDashboardResolver } from './resolvers/program-dashboard.resolver';

const data = {
  breadcrumbLabel: 'Program Dashboard',
  breadcrumb: true
};

const children = [{
  path: 'applications',
  component: ApplicationsTabComponent,
  data
}, {
  path: 'applicants',
  component: ApplicantsTabComponent,
  data
}, {
  path: 'organizations',
  component: OrganizationsTabComponent,
  data
}, {
  path: 'awards',
  component: AwardsTabComponent,
  data
}, {
  path: 'payments',
  component: PaymentsTabComponent,
  data
}];

@NgModule({
  imports: [
    GCCoreModule,
    InsightsModule,
    RouterModule.forChild([{
      path: '',
      component: ProgramInsightsPageComponent,
      data: {
        breadcrumbLabel: 'Programs',
        breadcrumb: true,
        pageName: 'Programs Insights'
      }
    }, {
        path: 'nonprofit',
        loadChildren: () => import('@features/nonprofit/nonprofit-routing.module').then(m => m.NonprofitRoutingModule)
    }, {
        path: 'applicant-profile',
        loadChildren: () => import('@features/applicant/applicant-routing.module').then(m => m.ApplicantRoutingModule)
    }, {
      path: '',
      children: [{
        path: 'program/:id',
        component: ProgramDashboardPageComponent,
        resolve: {
          appManager: ApplicationManagerResolver,
          dashboard: ProgramDashboardResolver
        },
        data,
        children
      }, {
        path: 'nomination-program/:id',
        component: ProgramDashboardPageComponent,
        resolve: {
          appManager: ApplicationManagerResolver,
          dashboard: ProgramDashboardResolver
        },
        data,
        children
      }]
    }])
  ]
})
export class ProgramManagerRoutingModule { }
