import { Component, OnInit } from '@angular/core';
import { Validators } from '@angular/forms';
import { PortalDeterminationService } from '@core/services/portal-determination.service';
import { SpinnerService } from '@core/services/spinner.service';
import { TokenRevocationResources } from '@core/services/token/token-revocation.resources';
import { TokenService } from '@core/services/token/token.service';
import { VerificationCodeService } from '@core/services/token/verification-code.service';
import { UserService } from '@features/users/user.service';
import { TypeSafeFormBuilder, TypeSafeFormGroup } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { YCModalComponent } from '@yourcause/common/modals';
import { NotifierService } from '@yourcause/common/notifier';
interface ExpireAllSessionsFormGroup {
  password: string;
}
@Component({
  selector: 'gc-expire-all-sessions-modal',
  templateUrl: './expire-all-sessions-modal.component.html',
  styleUrls: ['./expire-all-sessions-modal.component.scss']
})
export class ExpireAllSessionsModalComponent extends YCModalComponent<boolean> implements OnInit {
  formGroup: TypeSafeFormGroup<ExpireAllSessionsFormGroup>;
  passwordVisible = false;
  userType = this.portal.getUserType();
  email = this.userService.currentUser.email;

  constructor (
    private formBuilder: TypeSafeFormBuilder,
    private spinnerService: SpinnerService,
    private userService: UserService,
    private verificationCodeService: VerificationCodeService,
    private portal: PortalDeterminationService,
    private tokenService: TokenService,
    private notifier: NotifierService,
    private i18n: I18nService,
    private tokenRevocationService: TokenRevocationResources
  ) {
    super();
  }

  get tokenCall () {
    return this.portal.isManager ? 'getTokenForManager' : 'getTokenForPortal';
  }

  ngOnInit () {
    this.formGroup = this.formBuilder.group({
      password: ['', Validators.required]
    });
  }

  togglePasswordVisible = () => {
    this.passwordVisible = !this.passwordVisible;
  };

  async endAllSessions () {
    this.spinnerService.startSpinner();
    const password = this.formGroup.value.password;
    const passes = await this.tokenService.isPasswordCorrect(this.email, password);
    this.spinnerService.stopSpinner();
    if (passes) {
      await this.proceedWithEndingSession();
    } else if (passes === false) {
      this.notifier.error(this.i18n.translate(
        'ACCOUNT:textPasswordIncorrect',
        {},
        'Incorrect password'
      ));
    } else {
      this.notifier.error(this.i18n.translate(
        'common:textErrorEndingAllSessions',
        {},
        'There was an error ending all sessions'
      ));
    }
  }

  async proceedWithEndingSession () {
    this.spinnerService.startSpinner();
    const passed = await this.verificationCodeService.expireAllSessions(
      this.email,
      this.formGroup.value.password,
      this.userType
    );
    if (passed) {
      await this.tokenRevocationService.revokeAll();
      await this.tokenService.logout(true, false);
    }
    this.spinnerService.stopSpinner();
    this.closeModal.emit(passed);
  }
}

