import { ChangeDetectorRef, Component, OnDestroy, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { StatusService } from '@core/services/status.service';
import { MyApplicationFromApi } from '@core/typings/application.typing';
import { ApplicationApplicantService } from '@features/apply/application-applicant.service';
import { MyApplicationFormUI } from '@features/configure-forms/form.typing';
import { AutoTableRepository, AutoTableRepositoryFactory, PanelTypes, TopLevelFilter, TypeSafeFormBuilder, TypeSafeFormGroup, ValueComparisonService } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { NotifierService } from '@yourcause/common/notifier';
import { Subscription } from 'rxjs';
interface MyAppsGroup {
  [x: string]: any;
}

@Component({
  selector: 'gc-my-applications-page',
  templateUrl: './my-applications-page.component.html',
  styleUrls: ['./my-applications-page.component.scss']
})
export class MyApplicationsPageComponent implements OnInit, OnDestroy {
  PanelTypes = PanelTypes;
  showApplicationForm = false;
  revisionAlerts: MyApplicationFromApi[];
  formsPastDue: MyApplicationFormUI[];
  repository: AutoTableRepository<MyApplicationFromApi>;
  formGroup: TypeSafeFormGroup<MyAppsGroup>;
  topLevelFilters: TopLevelFilter[] = [];
  sub = new Subscription();

  constructor (
    private i18n: I18nService,
    private autoTableFactory: AutoTableRepositoryFactory,
    private notifierService: NotifierService,
    private valueComparisonService: ValueComparisonService,
    private formBuilder: TypeSafeFormBuilder,
    private cdr: ChangeDetectorRef,
    private applicationApplicantService: ApplicationApplicantService,
    private router: Router,
    private statusService: StatusService
  ) {
    this.sub.add(this.applicationApplicantService.changesTo$('myApplications')
      .subscribe(() => {
        if (this.repository) {
          this.repository.rows = this.applicationApplicantService.myApplications;
        }
      }));
    this.sub.add(this.applicationApplicantService.changesTo$('revisionAlerts')
      .subscribe(() => {
        this.revisionAlerts = this.applicationApplicantService.revisionAlerts;
      }));
    this.sub.add(this.applicationApplicantService.changesTo$('formsPastDue')
      .subscribe(() => {
        this.formsPastDue = this.applicationApplicantService.formsPastDue;
      }));
  }

  get hasNominations () {
    return this.applicationApplicantService.hasNominations;
  }

  async ngOnInit () {
    const statusOptions = this.statusService.getAppFilterStatusSelects(false, true).statusOptions;
    this.topLevelFilters = [
      new TopLevelFilter(
        'text',
        'applicationId',
        '',
        this.i18n.translate(
          'APPLY:textSearchByApplicationId', {}, 'Search by application ID'
        )
      ),
      new TopLevelFilter(
        'checkboxDropdown',
        'status',
        status,
        this.i18n.translate(
          'GLOBAL:textSearchByStatus',
          {},
          'Search by status'
        ),
        statusOptions,
        undefined,
        undefined,
        true
      )
    ];
    const repo = this.autoTableFactory.getRepository('MY_APPLICATIONS');
    this.formGroup = this.formBuilder.group(
      this.topLevelFilters.reduce((formGroup, filter) => ({
        ...formGroup,
        [filter.column]: [filter.value]
      }), {})
    );
    if (
      repo &&
      repo.topLevelFormGroup &&
      (this.formGroup !== repo.topLevelFormGroup)
    ) {
      this.formGroup = repo.topLevelFormGroup;
    }
    this.repository = this.autoTableFactory.create({
      key: 'MY_APPLICATIONS',
      columns: [],
      notifier: this.notifierService,
      valueComparisonService: this.valueComparisonService,
      rows: this.applicationApplicantService.myApplications,
      topLevelFilters: this.topLevelFilters,
      formGroup: this.formGroup,
      cdr: this.cdr,
      skipAddToState: true
    });
  }

  selectApp (application: MyApplicationFromApi) {
    const draftForms = application.applicationForms
      .filter((form) => !!form.isDraft)
      .map((form) => form.formId);
    this.router.navigateByUrl(
      `/apply/application/${application.applicationId}/forms/${draftForms[0]}`
    );
  }

  ngOnDestroy () {
    this.sub.unsubscribe();
  }
}

