import { Component, Injector } from '@angular/core';
import { Router } from '@angular/router';
import { SpinnerService } from '@core/services/spinner.service';
import { TopLevelFilter } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { ConfirmationModalComponent, ModalFactory } from '@yourcause/common/modals';
import { ManageFilterModalComponent } from '../manage-filter-modal/manage-filter-modal.component';
import { MyFilterService } from '../my-filters.service';
import { MyFilterForUi } from '../my-filters.typing';

@Component({
  selector: 'gc-manage-filters-page',
  templateUrl: './manage-filters-page.component.html',
  styleUrls: ['./manage-filters-page.component.scss']
})
export class ManageFitlersPageComponent {
  topLevelFilters: TopLevelFilter[] = [
    new TopLevelFilter(
      'text',
      'name',
      '',
      this.i18n.translate(
        'common:textSearchByName',
        {},
        'Search by name'
      ),
      undefined,
      this.i18n.translate(
        'common:lblName',
        {},
        'Name'
      )
    )
  ];

  constructor (
    private myFilterService: MyFilterService,
    private i18n: I18nService,
    private modalFactory: ModalFactory,
    private spinnerService: SpinnerService,
    private router: Router,
    private injector: Injector
  ) { }

  get myFilters () {
    return this.myFilterService.myFilters;
  }

  async addNewFilter () {
    const response = await this.modalFactory.open(
      ManageFilterModalComponent,
      { }
    );
    if (response) {
      this.spinnerService.startSpinner();
      const id = await this.myFilterService.handleAddOrUpdateFilterConfig(
        null,
        response.filterName
      );
      this.spinnerService.stopSpinner();

      if (id) {
        this.router.navigate([
          `/management/manage-filters/${id}`
        ]);
      }
    }
  }

  async copyFilter (row: MyFilterForUi) {
    const response = await this.modalFactory.open(
      ConfirmationModalComponent,
      {
        modalHeader: this.i18n.translate(
          'common:hdrCopyFilter',
          {},
          'Copy Filter'
        ),
        modalSubHeader: row.name,
        confirmButtonText: this.i18n.translate(
          'common:textCopy',
          {},
          'Copy'
        ),
        confirmText: this.i18n.translate(
          'common:textCopyFilterConfirm',
          {},
          'Are you sure you want to copy this filter?'
        )
      }
    );
    if (response) {
      this.spinnerService.startSpinner();
      const id = await this.myFilterService.handleCopyFilter(row, this.injector);
      if (id) {
        this.router.navigate([`/management/manage-filters/${id}`]);
      }
      this.spinnerService.stopSpinner();
    }
  }

  async deleteFilter (row: any) {
    const response = await this.modalFactory.open(
      ConfirmationModalComponent,
      {
        modalHeader: this.i18n.translate(
          'common:hdrDeleteFilter',
          {},
          'Delete Filter'
        ),
        modalSubHeader: row.name,
        confirmButtonText: this.i18n.translate(
          'common:btnDelete',
          {},
          'Delete'
        ),
        confirmText: this.i18n.translate(
          'common:textDeleteFilterConfirm',
          {},
          'Are you sure you want to delete this filter?'
        )
      }
    );
    if (response) {
      this.spinnerService.startSpinner();
      await this.myFilterService.handleDeleteFilter(row.id);
      this.spinnerService.stopSpinner();
    }
  }
}
