import { Injectable } from '@angular/core';
import { HttpRestService } from '@core/services/http-rest.service';
import { FilterColumn } from '@yourcause/common';
import { FilterPages, MyFilterDetailFromApi, MyFilterFromApi } from './my-filters.typing';

@Injectable({ providedIn: 'root' })
export class MyFilterResources {

  constructor (
    private httpRestService: HttpRestService
  ) { }

  getMyFilters (): Promise<MyFilterFromApi[]> {
    const endpoint = 'api/manager/applications/FiltersForUser';

    return this.httpRestService.get(endpoint);
  }

  getFilterDetail (filterId: number): Promise<MyFilterDetailFromApi> {
    const endpoint = `api/manager/applications/${filterId}/FilterDetail`;

    return this.httpRestService.get(endpoint);
  }

  addOrUpdateFilterConfig (
    id: number,
    name: string,
    pageId = FilterPages.APPLICATIONS
  ): Promise<number> {
    const endpoint = 'api/manager/applications/SaveFilter';

    return this.httpRestService.post(endpoint, {
      id,
      name,
      pageId
    });
  }

  deleteFilter (filterId: number) {
    const endpoint = `api/manager/applications/${filterId}/RemoveFilter`;

    return this.httpRestService.delete(endpoint);
  }

  saveFilterDetails (
    filterId: number,
    filters: FilterColumn<any>[]
  ) {
    const endpoint = 'api/manager/applications/SaveFilterDetails';

    return this.httpRestService.post(endpoint, {
      filterId,
      filters
    });
  }

}


