import { Component, OnDestroy, OnInit, QueryList, ViewChildren } from '@angular/core';
import { SpinnerService } from '@core/services/spinner.service';
import { StatusService } from '@core/services/status.service';
import { TranslationService } from '@core/services/translation.service';
import { ProcessingTypes } from '@core/typings/payment.typing';
import { ApplicationStatuses } from '@core/typings/status.typing';
import { ApplicationActivityForUi, GrantActivityTypes } from '@features/application-activity/activity.typing';
import { ApplicationActivityService } from '@features/application-activity/application-activity.service';
import { RecentActivityModalComponent } from '@features/application-activity/recent-activity-modal/recent-activity-modal.component';
import { ApplicationManagerService, DefaultSearchByDropdownValue } from '@features/application-manager/services/application-manager/application-manager.service';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { CompletionRequirementType, FormAudience } from '@features/configure-forms/form.typing';
import { CyclesService } from '@features/cycles/cycles.service';
import { UserService } from '@features/users/user.service';
import { APISortColumn, AutoTableRepository, AutoTableRepositoryFactory, DebounceFactory, FilterColumn, FilterHelpersService, FilterModalTypes, OrganizationEligibleForGivingStatus, PaginationOptions, TableColumnDirective, TableDataFactory, TopLevelCustomLink, TopLevelFilter, TypeaheadSelectOption, TypeSafeFormBuilder, TypeSafeFormGroup, TypeToken, ValueComparisonService } from '@yourcause/common';
import { CachedAttr, CACHE_TYPES } from '@yourcause/common/cache';
import { I18nService } from '@yourcause/common/i18n';
import { ModalFactory } from '@yourcause/common/modals';
import { NotifierService } from '@yourcause/common/notifier';
import moment from 'moment';
import { filter, Subscription, take } from 'rxjs';
import { MyWorkspaceResources } from '../my-workspace.resources';
import { MyWorkspaceService } from '../my-workspace.service';
import { ActionOptions, MyWorkspaceCardView, MyWorkspacePayload, MyWorkspaceSummarySkeletonConfig, MyWorkspaceTableSkeletonConfig, SearchByDropdownValue } from '../my-workspace.typing';
import { ViewAllDeadlinesModalComponent } from '../view-all-deadlines-modal/view-all-deadlines-modal.component';
interface MyWorkspaceGroup {
  [x: string]: any;
}

@Component({
  selector: 'gc-my-workspace-page',
  templateUrl: './my-workspace-page.component.html',
  styleUrls: ['./my-workspace-page.component.scss']
})
export class MyWorkspacePageComponent implements OnInit, OnDestroy {
  @ViewChildren(TableColumnDirective) columns: QueryList<TableColumnDirective>;
  hideSideContainer = true;
  skeletonPromiseForTable: Promise<boolean>;
  skeletonPromiseForSummary: Promise<void>;
  ApplicationStatuses = ApplicationStatuses;
  GrantActivityTypes = GrantActivityTypes;
  formGroup: TypeSafeFormGroup<MyWorkspaceGroup>;
  notReviewedOptions: TypeaheadSelectOption[] = [{
    label: this.i18n.translate(
      'common:textAllApplications',
      {},
      'All applications'
    ),
    value: false
  }, {
    label: this.i18n.translate(
      'TODAY:textActionRequired',
      {},
      'Action required'
    ),
    value: true
  }];
  isTableView = false;
  subHeaderPrompt = '';
  hasMaskedAppsOnPage = false;
  showMaskedApplicantsIcon = 'user-shield';
  hideMaskedApplicantsIcon = 'user-times';
  hideMaskedApplicantsTooltip = this.i18n.translate(
    'MANAGE:textShowApplicantInformation',
    {},
    'Show applicant information'
  );
  showMaskedApplicantsTooltip = this.i18n.translate(
    'MANAGE:textHideApplicantInformation',
    {},
    'Hide applicant information'
  );
  showCardViewTooltip = this.i18n.translate(
    'GLOBAL:textShowCardView',
    {},
    'Show card view'
  );
  showTableViewTooltip = this.i18n.translate(
    'GLOBAL:textShowTableView',
    {},
    'Show table view'
  );
  topLevelCustomLinks: TopLevelCustomLink[] = [{
    name: this.showTableViewTooltip,
    tooltip: this.showTableViewTooltip,
    icon: 'th-large',
    isIcon: true,
    onClick: () => {
      this.toggleTableView();
    }
  }];
  loading = false;
  sub = new Subscription();
  tableDataFactory: TableDataFactory<MyWorkspaceCardView>;
  applicationRepository: AutoTableRepository<MyWorkspaceCardView>;
  statusClassMap: Record<ApplicationStatuses, string> = {
    [ApplicationStatuses.Draft]: 'dot-empty',
    [ApplicationStatuses.AwaitingReview]: 'dot-empty',
    [ApplicationStatuses.Approved]: 'dot-green',
    [ApplicationStatuses.Declined]: 'dot-red',
    [ApplicationStatuses.InProgress]: 'dot-blue',
    [ApplicationStatuses.Hold]: 'dot-yellow',
    [ApplicationStatuses.Canceled]: 'dot-yellow'
  };
  appStatusMap = this.statusService.applicationStatusMap;
  topLevelFilters: TopLevelFilter[];
  actionOptions: TypeaheadSelectOption[] = [{
    label: this.i18n.translate(
      'TODAY:textSubmissionsWithFormsINeedToComplete',
      {},
      'Submissions with forms I need to complete'
    ),
    value: ActionOptions.FORMS_TO_COMPLETE
  }, {
    label: this.i18n.translate(
      'TODAY:textSubmissionsWithActionsAvailable',
      {},
      'Submissions with actions available'
    ),
    value: ActionOptions.ACTIONS_AVAILABLE
  }, {
    label: this.i18n.translate(
      'TODAY:textAllSubmissions',
      {},
      'All submissions'
    ),
    value: ActionOptions.ALL_SUBMISSIONS
  }, {
    label: this.i18n.translate(
      'TODAY:textAllSubmissionInMyWFL',
      {},
      'All submissions in my workflow level'
    ),
    value: ActionOptions.ALL_SUBMISSION_IN_WFL
  }];
  viewTranslations = this.translationService.viewTranslations;
  programTranslationMap = this.viewTranslations.Grant_Program;
  cycleTranslationMap = this.viewTranslations.Grant_Program_Cycle;
  formTranslationMap = this.viewTranslations.FormTranslation;
  defaultCurrency = this.clientSettingsService.defaultCurrency;
  cycleOptions = this.cyclesService.allMyNonFutureCycleSelectOptions;
  organizationEligibleForGivingStatus = OrganizationEligibleForGivingStatus;
  ProcessingTypes = ProcessingTypes;
  MyWorkspaceSummarySkeletonConfig = MyWorkspaceSummarySkeletonConfig;
  MyWorkspaceTableSkeletonConfig = MyWorkspaceTableSkeletonConfig;
  $applicationStatus = new TypeToken<ApplicationStatuses>();

  @CachedAttr(CACHE_TYPES.LOCALSTORAGE, false)
  showMaskedApplicants: boolean;

  @CachedAttr(CACHE_TYPES.LOCALSTORAGE,  ActionOptions.FORMS_TO_COMPLETE)
  actionFilter: number;

  @CachedAttr(CACHE_TYPES.LOCALSTORAGE, {
    columnName: 'dateAddedToWorkflow',
    sortAscending: false
  })
  sortColumn: APISortColumn<any>;

  @CachedAttr(CACHE_TYPES.LOCALSTORAGE, DefaultSearchByDropdownValue)
  searchByDropdownValue: SearchByDropdownValue;

  @CachedAttr(
    CACHE_TYPES.LOCALSTORAGE,
    FilterModalTypes.Last6Months
  ) dateRangeValue: FilterModalTypes;

  constructor (
    private userService: UserService,
    private formBuilder: TypeSafeFormBuilder,
    private myWorkspaceService: MyWorkspaceService,
    private myWorkspaceResources: MyWorkspaceResources,
    private i18n: I18nService,
    private modalFactory: ModalFactory,
    private autoTableFactory: AutoTableRepositoryFactory,
    private notifierService: NotifierService,
    private valueComparisonService: ValueComparisonService,
    private statusService: StatusService,
    private applicationManagerService: ApplicationManagerService,
    private translationService: TranslationService,
    private clientSettingsService: ClientSettingsService,
    private cyclesService: CyclesService,
    private filterHelperService: FilterHelpersService,
    private spinnerService: SpinnerService,
    private applicationActivityService: ApplicationActivityService
  ) {
    this.sub.add(this.myWorkspaceService.changesTo$('totalActivePrograms').subscribe(() => {
      this.setPrompt();
    }));
  }

  get user () {
    return this.userService.user;
  }

  get activities () {
    return this.myWorkspaceService.activities;
  }

  get programOptions (): TypeaheadSelectOption[] {
    return this.myWorkspaceService.filterPrograms;
  }

  get levelOptions (): TypeaheadSelectOption[] {
    return this.myWorkspaceService.filterLevels;
  }

  get deadlines () {
    return this.myWorkspaceService.deadlines;
  }

  get statusOptions () {
    return this.statusService.appStatusOptions;
  }

  get totalApplications () {
    return this.myWorkspaceService.totalApplications;
  }

  get totalNotReviewed () {
    return this.myWorkspaceService.totalNotReviewed;
  }

  get totalActiveWorkflows () {
    return this.myWorkspaceService.totalActiveWorkflows;
  }

  get totalActivePrograms () {
    return this.myWorkspaceService.totalActivePrograms;
  }

  ngOnInit () {
    this.setSkeletonPromiseForSummary();
    this.prepareTableAndSkeletonPromise();
  }

  setSkeletonPromiseForTable () {
    this.skeletonPromiseForTable = this.applicationRepository.loading
      .pipe(filter(loading => !loading))
      .pipe(take(1))
      .toPromise();
  }


  setSkeletonPromiseForSummary () {
    this.skeletonPromiseForSummary = this.myWorkspaceService.setSummary();
  }

  async toggleSideContainer () {
    const hideSideContainer = !this.hideSideContainer;
    if (
      !hideSideContainer &&
      (!this.deadlines || !this.activities)
    ) {
      await this.setDeadlinesAndActivities();
    }
    this.hideSideContainer = hideSideContainer;
  }

  async setDeadlinesAndActivities () {
    this.spinnerService.startSpinner();
    await Promise.all([
      this.myWorkspaceService.setDeadlines(),
      this.myWorkspaceService.setActivities()
    ]);
    this.spinnerService.stopSpinner();
  }

  prepareTableAndSkeletonPromise () {
    this.tableDataFactory = DebounceFactory
      .createSimple((options: PaginationOptions<MyWorkspaceCardView>) => {
        return this.getApplications(options);
      }, 500);
    this.setTopLevelFilters();
    this.applicationRepository = this.autoTableFactory.create({
      key: 'MY_WORKSPACE',
      columns: [],
      notifier: this.notifierService,
      valueComparisonService: this.valueComparisonService,
      tableDataFactory: this.tableDataFactory,
      rowsPerPage: 5,
      topLevelFilters: this.topLevelFilters,
      formGroup: this.formGroup
    });
    this.setSkeletonPromiseForTable();
    this.sub.add(this.applicationRepository.loading.subscribe((val) => {
      this.loading = val;
      this.setSkeletonPromiseForTable();
    }));
  }

  setTopLevelFilters () {
    this.topLevelFilters = [
      new TopLevelFilter(
        'searchByDropdown',
        'applicationId',
        this.searchByDropdownValue,
        this.i18n.translate(
          'TODAY:textSeachByIdApplicantOrgAmount',
          {},
          'Search by id, applicant, organization, or amount'
        ),
        {
          selectOptions: [{
            label: this.i18n.translate(
              'common:textId',
              {},
              'ID'
            ),
            value: 'applicationId',
            isNumber: true,
            placeholder: this.i18n.translate(
              'APPLY:textSearchByApplicationId',
              {},
              'Search by application ID'
            )
          }, {
            label: this.i18n.translate(
              'common:textApplicantName',
              {},
              'Applicant name'
            ),
            value: 'applicantFullName',
            isNumber: false,
            placeholder: this.i18n.translate(
              'common:textSearchByApplicantName',
              {},
              'Search by applicant name'
            )
          }, {
            label: this.i18n.translate(
              'common:labelOrganizationName',
              {},
              'Organization name'
            ),
            value: 'organizationName',
            isNumber: false,
            placeholder: this.i18n.translate(
              'common:textSearchByOrgName',
              {},
              'Search by organization name'
            )
          }]
        }
      ),
      this.programOptions.length > 1 ?
        new TopLevelFilter(
          'checkboxDropdown',
          'grantProgramId',
          [],
          this.i18n.translate(
            'GLOBAL:textSearchForAProgram',
            {},
            'Search for a program'
          ),
          {
            selectOptions: this.programOptions,
            filterObjectName: this.i18n.translate(
              'common:lblProgram', {}
            ).toLowerCase(),
            filterObjectNamePlural: this.i18n.translate(
              'GLOBAL:textPrograms', {}
            ).toLowerCase()
          }
        ) : undefined,
      new TopLevelFilter(
        'typeaheadSingleEquals',
        'hasFormToComplete',
        this.actionFilter,
        undefined,
        {
          selectOptions: this.actionOptions
        },
        this.i18n.translate(
          'GLOBAL:textSubmissionType',
          {},
          'Submission type'
        ),
        undefined,
        false
      ),
      this.applicationManagerService.getRelativeDateFilterForApplications(
        this.dateRangeValue
      )
    ].filter((item) => !!item);
    const repo = this.autoTableFactory.getRepository('MY_WORKSPACE');
    this.formGroup = this.formBuilder.group(
      this.topLevelFilters.reduce((formGroup, topLevelFilter) => ({
        ...formGroup,
        [topLevelFilter.column]: [topLevelFilter.value]
      }), {})
    );

    if (
      repo &&
      repo.topLevelFormGroup &&
      (this.formGroup !== repo.topLevelFormGroup)
    ) {
      this.formGroup = repo.topLevelFormGroup;
    }
  }

  onSortColumnChange (column: APISortColumn<any>) {
    this.sortColumn = column;
  }

  setPrompt () {
    const workflows = this.totalActiveWorkflows;
    const programs = this.totalActivePrograms;
    if (workflows || programs) {
      if (workflows === 1 || programs === 1) {
        if (workflows === 1 && programs === 1) {
          this.subHeaderPrompt = this.i18n.translate(
            'TODAY:hdrYouHaveWorkflowApplicationsInProgram',
            {
              numberLevels: workflows,
              numberPrograms: programs
            },
            `You have applications in __numberLevels__ workflow level for __numberPrograms__ program.`
          );

        } else if (workflows === 1) {
          this.subHeaderPrompt = this.i18n.translate(
            'TODAY:hdrYouHaveWorkflowApplicationsInPrograms',
            {
              numberLevels: workflows,
              numberPrograms: programs
            },
            `You have applications in __numberLevels__ workflow level for __numberPrograms__ programs.`
          );
        } else if (programs === 1) {
          this.subHeaderPrompt = this.i18n.translate(
            'TODAY:hdrYouHaveWorkflowsApplicationsInProgram',
            {
              numberLevels: workflows,
              numberPrograms: programs
            },
            `You have applications in __numberLevels__ workflow levels for __numberPrograms__ program.`
          );
        }
      } else {
        this.subHeaderPrompt = this.i18n.translate(
          'TODAY:hdrYouHaveNumberApplicationsInPrograms',
          {
            numberLevels: workflows,
            numberPrograms: programs
          },
          'You have applications in __numberLevels__ workflow levels for __numberPrograms__ programs.'
        );
      }
    }
  }

  onTopLevelFilterChange (topLevelFilter: TopLevelFilter) {
    if (topLevelFilter.column === 'hasFormToComplete') {
      this.actionFilter = topLevelFilter.value || ActionOptions.FORMS_TO_COMPLETE;
    }
    if (topLevelFilter.column === 'updatedDate') {
      this.dateRangeValue = topLevelFilter.value || FilterModalTypes.Last6Months;
    }
    if (topLevelFilter.column === 'applicationId') {
      this.searchByDropdownValue = {
        ...topLevelFilter.value,
        filterValue: ''
      };
    }
  }

  formatOptionsAndPayload (
    options: PaginationOptions<MyWorkspaceCardView>
  ): {
    options: PaginationOptions<MyWorkspaceCardView>;
    data: MyWorkspacePayload;
  } {
    if (!this.applicationRepository.activeSortColumn) {
      options = {
        ...options,
        sortColumns: [
          this.sortColumn
        ]
      };
    }

    let hasActionsAvailable = false;
    let hasFormToComplete = true;
    let inMyWflFilter: FilterColumn<MyWorkspaceCardView>;
    switch (this.actionFilter) {
      case ActionOptions.FORMS_TO_COMPLETE:
      default:
        hasActionsAvailable = undefined;
        hasFormToComplete = true;
        break;
      case ActionOptions.ACTIONS_AVAILABLE:
        hasActionsAvailable = true;
        hasFormToComplete = undefined;
        break;
      case ActionOptions.ALL_SUBMISSIONS:
        hasActionsAvailable = undefined;
        hasFormToComplete = undefined;
        break;
      case ActionOptions.ALL_SUBMISSION_IN_WFL:
        hasActionsAvailable = undefined;
        hasFormToComplete = undefined;
        inMyWflFilter = {
          columnName: 'isInWorkflowLevel',
          filters: [{
            filterType: 'eq',
            filterValue: true
          }]
        };
        break;
    }

    let applicantName = '';
    let organizationName = '';
    let applicationId: number;
    let startDate: moment.Moment;
    let endDate: moment.Moment;
    options = {
      ...options,
      filterColumns: [
        ...options.filterColumns,
        inMyWflFilter
      ].filter((filterColumn) => {
        if (filterColumn) {
          let keepColumn = false;
          const filterVal = filterColumn.filters[0].filterValue;
          switch (filterColumn.columnName) {
            case 'applicationId':
              applicationId = filterVal as number;
              break;
            case 'applicantFullName':
              applicantName = filterVal as string;
              break;
            case 'organizationName':
              organizationName = filterVal as string;
              break;
            case 'hasFormToComplete':
              break;
            case 'updatedDate':
              const filterType = filterColumn.filters[0].filterType as FilterModalTypes;
              const dates = this.filterHelperService.getClientSideRelativeDates(
                filterType
              );
              if (dates) {
                startDate = dates[0];
                endDate = dates[1];
              }
              break;
            default:
              keepColumn = true;
              break;
          }

          return keepColumn;
        }

        return false;
      })
    };

    const data: MyWorkspacePayload = {
      programId: null,
      notReviewed: null,
      hasActionsAvailable,
      hasFormToComplete,
      applicantName,
      applicationId,
      organizationName,
      startDate,
      endDate
    };

    return {
      options,
      data
    };
  }

  async getApplications (options: PaginationOptions<MyWorkspaceCardView>) {
    const response = this.formatOptionsAndPayload(options);
    const results = await this.myWorkspaceResources.getTodaysView(
      response.options,
      response.data
    );
    this.handleMaskedApplicants(results.records);

    return {
      success: true,
      data: {
        records: results.records.map((record: MyWorkspaceCardView) => {
          const allFormsComplete = record.completionRequirementsMet &&
            !record.formsAvailableForCompletion;
          record.formsNeedingCompleted = record.forms.filter((form) => {
            return (
              !allFormsComplete &&
              (form.audience === FormAudience.MANAGER) &&
              (form.completionRequirementType !== CompletionRequirementType.NONE) &&
              (form.completionRequirementType !== CompletionRequirementType.VIEW_ONLY) &&
              record.formsAvailableForCompletion &&
              !form.userHasSubmittedResponse
            );
          });
          record.formsOptionalForCompletion = record.forms.filter((form) => {
            return (
              !allFormsComplete &&
              (form.audience === FormAudience.MANAGER) &&
              (form.completionRequirementType === CompletionRequirementType.NONE) &&
              record.formsAvailableForCompletion &&
              !form.userHasSubmittedResponse
            );
          });

          return record;
        }),
        recordCount: results.recordCount
      }
    };
  }

  handleMaskedApplicants (rows: MyWorkspaceCardView[]) {
    let hasMaskedApps = false;
    rows.forEach((row) => {
      if (row.isMasked && row.canViewMaskedApplicantInfo) {
        hasMaskedApps = true;
      }
    });

    this.hasMaskedAppsOnPage = hasMaskedApps;
    if (this.hasMaskedAppsOnPage) {
      this.setMaskedIconAndTooltip();
    }
  }

  toggleShowMaskedApplicants () {
    this.showMaskedApplicants = !this.showMaskedApplicants;
    this.setMaskedIconAndTooltip();
  }

  setMaskedIconAndTooltip () {
    const tooltip = this.showMaskedApplicants ?
      this.showMaskedApplicantsTooltip :
      this.hideMaskedApplicantsTooltip;
    const item = {
      name: tooltip,
      tooltip,
      icon: this.showMaskedApplicants ?
        this.hideMaskedApplicantsIcon :
        this.showMaskedApplicantsIcon,
      isIcon: true,
      onClick: () => {
        this.toggleShowMaskedApplicants();
      }
    };
    if (this.topLevelCustomLinks.length === 1) {
      this.topLevelCustomLinks.unshift(item);
    } else {
      this.topLevelCustomLinks[0] = item;
    }
  }

  isActionAvailable (
    app: MyWorkspaceCardView,
    action: 'approve'|'decline'|'route'|'award'|'any'
  ) {
    switch (action) {
      case 'approve':
        return this.canApprove(app);
      case 'decline':
        return this.canDecline(app);
      case 'route':
        return this.canRoute(app);
      case 'award':
        return this.canAward(app);
      case 'any':
        return this.canApprove(app) ||
          this.canDecline(app) ||
          this.canRoute(app) ||
          this.canAward(app);
      default:
        return false;
    }
  }

  canApprove (app: MyWorkspaceCardView) {
    return app.canApproveApplication &&
      (
        (app.applicationStatus === ApplicationStatuses.InProgress) ||
        (app.applicationStatus === ApplicationStatuses.AwaitingReview)
      ) && app.completionRequirementsMet;
  }

  canDecline (app: MyWorkspaceCardView) {
    return app.canDeclineApplication &&
      (
        (app.applicationStatus === ApplicationStatuses.InProgress) ||
        (app.applicationStatus === ApplicationStatuses.AwaitingReview)
      );
  }

  canRoute (app: MyWorkspaceCardView) {
    return app.canRouteApplication && app.completionRequirementsMet;
  }

  canAward (app: MyWorkspaceCardView) {
    return app.canAwardApplication &&
      (app.applicationStatus === ApplicationStatuses.Approved);
  }


  toggleTableView () {
    this.isTableView = !this.isTableView;
    const tooltip = this.isTableView ?
      this.showCardViewTooltip :
      this.showTableViewTooltip;
    const item = {
      name: tooltip,
      tooltip,
      icon: this.isTableView ?
        'columns' :
        'th-large',
      isIcon: true,
      onClick: () => {
        this.toggleTableView();
      }
    };
    if (this.topLevelCustomLinks.length === 1) {
      this.topLevelCustomLinks[0] = item;
    } else {
      this.topLevelCustomLinks[1] = item;
    }
  }

  async deadlinesModal () {
    await this.modalFactory.open(
      ViewAllDeadlinesModalComponent,
      { deadlines: this.deadlines }
    );
  }

  async activitiesModal (id?: number) {
    await this.modalFactory.open(
      RecentActivityModalComponent,
      {
        showAppId: !id,
        defaultCurrency: this.clientSettingsService.defaultCurrency,
        tableDataFactory: id ?
          DebounceFactory.createSimple(
            (options: PaginationOptions<ApplicationActivityForUi>) => {
              return this.applicationActivityService.getAppActivity(id, options);
            }, 0
          ) :
          this.myWorkspaceService.allActivitiesFactory
      }
    );
  }

  ngOnDestroy () {
    this.sub.unsubscribe();
  }
}
