import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { ApplicationActivityFromApi, GrantActivityTypes } from '@features/application-activity/activity.typing';
import { PaginatedResponse } from '@yourcause/common';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { MyWorkspaceService } from './my-workspace.service';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(MyWorkspaceService, {
  imports: [
    GCMockModule
  ]
})
export class MyWorkspaceServiceSpec implements Spec<MyWorkspaceServiceSpec, MyWorkspaceService> {
  response: PaginatedResponse<ApplicationActivityFromApi> = {
    records: [{
      id: 0,
      currencyRequested: 'USD',
      currencyRequestedAmountEquivalent: 0,
      recommendedFundingAmount: 0,
      applicationId: 1,
      applicantName: 'Lori Smith',
      createdByName: 'Lori Smith',
      createdImpersonatedByName: 'Tyson Collins',
      createdByProfileImageUrl: '',
      grantProgramName: 'LS_Masked_Program',
      organizationName: 'Hungry Kidzz',
      workFlowLevelName: '2nd',
      previousWorkFlowLevelName: '1st',
      grantProgramCycleName: '',
      previousGrantProgramCycleName: '',
      formName: 'Form name',
      activityTypeId: GrantActivityTypes.ApplicationAdvanced,
      activityTypeName: '',
      comment: '',
      createdDate: '',
      isMasked: false,
      canViewMaskedApplicantInfo: true,
      amountRequested: 100.00,
      isNomination: false,
      assignedBudgetName: '',
      assignedFundingSourceName: '',
      inKindAmountRequested: 0,
      applicantFullNameForActivity: 'Tyson Collins',
      grantManagerFullNameForActivity: 'Tyson Collins',
      formDueDate: '2020-01-01',
      applicantPermissions: {
        canManageApplicants: true,
        canReceiveEmails: true,
        isOwner: true
      },
      workflowLevelAutomationRuleName: 'test',
      fileName: 'log.jpg',
      communicationSubject: 'test subject',
      previousOrganizationName: 'Tyson org',
      amount: 0,
      paymentInKindItems: [],
      awardInKindItems: [],
      managerFormResponsesRequired: 4,
      managerFormResponsesCompleted: 2,
      applicationTagList: 'Global, Test',
      formId: 0,
      isAutomated: false,
      archiveReasonCode: null,
      previousGrantProgramId: null,
      newGrantProgramId: null,
      isInKindPaymentOrAward: false,
      grantActivityChangeJson: ''
    }],
    recordCount: 1
  };

  @BeforeEach()
  mock (service: MyWorkspaceService) {
    service['applicationActivityService']['applicationActivityResources'].getMyActivity = async () => {
      return this.response;
    };

    service['myWorkspaceResources'].getSummary = async () => {
      return {
        totalApplication: 1,
        totalActiveWorkflowLevels: 1,
        totalApplicationsNotReviewed: 1,
        totalActivePrograms: 2
      };
    };

    service['myWorkspaceResources'].getDeadlines = async () => {
      return [{
        date: '01/31/2019',
        name: 'Deadline',
        description: '',
        programName: 'Program',
        createdDate: '01/01/2019'
      }];
    };
  }

  @TestCase('should be able to convert program map to sorted filter options array')
   convertMapToFilterOptions (service: MyWorkspaceService) {
    const map = {
      68: 'QA Test Program',
      69: 'QA Employee Relief Fund',
      79: 'Lets Get Collaboration Going',
      85: 'Wednesday Program of the Week!',
      90: 'Eligibility Program',
      91: 'Save the Bumblebees',
      101: 'Whale Foundation',
      106: 'Save the Pandas',
      112: 'Golden Retriever Rescue',
      113: 'Dolphin Program',
      121: 'LSProgramTestWOrg-DO NOT MESS WITH THIS PROGRAM!',
      123: '1st Dog Rescue',
      127: 'LSPingPongProgram',
      133: 'Bobs Form Bonanza',
      135: 'PwC Education'
    };
    const filterOptions = service.convertMapToFilterOptions(map);
    expect(filterOptions.length).to.be.equal(15);
    expect(filterOptions[0].label).to.be.equal('1st Dog Rescue');
  }

  @TestCase('getActivity should return paginated response for tableDataFactory')
  async getActivity (service: MyWorkspaceService) {
    const activity = await service.getActivity({
      rowsPerPage: 5,
      pageNumber: 1,
      sortColumns: [],
      filterColumns: [],
      orFilterColumns: [],
      retrieveTotalRecordCount: true,
      returnAll: false
    });
    expect(activity.success).to.be.equal(true);
    expect(activity.data.recordCount).to.be.equal(1);
    expect(activity.data.records[0].applicationId).to.be.equal(1);
  }

  @TestCase('should be able to set my workspace data')
  async getMyWorkspaceData (service: MyWorkspaceService) {
    await service.setSummary();
    await service.setDeadlines();
    await service.setActivities();

    expect(service.totalNotReviewed).to.be.equal(1);
    expect(service.totalActiveWorkflows).to.be.equal(1);
    expect(service.deadlines[0].name).to.be.equal('Deadline');
    expect(service.activities[0].applicantName).to.be.equal('Lori Smith');
  }
}
