import { Injectable } from '@angular/core';
import { HttpRestService } from '@core/services/http-rest.service';
import { PaginatedResponse, PaginationOptions, SimpleStringMap } from '@yourcause/common';
import { MyWorkspaceCardView, MyWorkspaceDeadline, MyWorkspacePayload, MyWorkspaceSummary } from './my-workspace.typing';

@Injectable({ providedIn: 'root' })
export class MyWorkspaceResources {

  constructor (
    private httpRestService: HttpRestService
  ) { }

  getTodaysView (
    paginationOptions: PaginationOptions<MyWorkspaceCardView>,
    data: MyWorkspacePayload
  ): Promise<PaginatedResponse> {
    const endpoint = 'api/manager/today';

    return this.httpRestService.post(endpoint, {
      paginationOptions,
      ...data
    });
  }

  getSummary (): Promise<MyWorkspaceSummary> {
    return this.httpRestService.get('api/manager/today/summary');
  }

  getDeadlines (): Promise<MyWorkspaceDeadline[]> {
    return this.httpRestService.get('api/manager/today/programdeadlines');
  }

  getProgramsAndWorkflowsForMyWorkspace (): Promise<{
    programs: SimpleStringMap<string>;
    workflowLevels: SimpleStringMap<string>;
  }> {
    const endpoint = 'api/manager/today/summaryFacets';

    return this.httpRestService.get(endpoint);
  }
}

