import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { ApplicantForOrg } from '@core/typings/applicant.typing';
import { ApplicantManagerService } from '@features/applicant/applicant-manager.service';
import { DebounceFactory, PaginationOptions, TableDataFactory, TopLevelFilter } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { NonprofitService } from '../nonprofit.service';

@Component({
  selector: 'gc-nonprofit-applicants',
  templateUrl: './nonprofit-applicants.component.html',
  styleUrls: ['./nonprofit-applicants.component.scss']
})

export class NonprofitApplicantsComponent implements OnInit {
  tableDataFactory: TableDataFactory<ApplicantForOrg>;
  guid: string;
  tableKey: string;
  topLevelFilters: TopLevelFilter[] = [];
  fromRoute = this.applicantManagerService.getFromRouteForProfile();

  constructor (
    private nonprofitService: NonprofitService,
    private activatedRoute: ActivatedRoute,
    private i18n: I18nService,
    private applicantManagerService: ApplicantManagerService
  ) { }

  get applicantRouterLink () {
    return this.applicantManagerService.get('applicantProfileRouterLink');
  }

  ngOnInit () {
    this.topLevelFilters = [
      new TopLevelFilter(
        'text',
        'applicantInfo.fullName',
        '',
        this.i18n.translate(
          'GLOBAL:textSearchByApplicantNameOrEmail',
          {},
          'Search by applicant name or email'
        ),
        undefined,
        undefined,
        [{
          column: 'applicantInfo.fullName',
          filterType: 'cn'
        }, {
          column: 'applicantInfo.email',
          filterType: 'cn'
        }]
      )
    ];
    this.setTableDataFactory();
  }

  setTableDataFactory () {
    this.guid = this.activatedRoute.snapshot.parent.params.guid;
    const map = this.nonprofitService.get('nonprofitMap')[this.guid];
    this.tableKey = `ORG_APPLICANTS_${map.gcId}`;
    this.tableDataFactory = DebounceFactory.createSimple(
      async (options: PaginationOptions<ApplicantForOrg>) => {
        const result = await this.nonprofitService.getOrgApplicants(
          +map.gcId,
          options
        );

        return {
          success: true,
          data: {
            recordCount: result.recordCount,
            records: result.records.map((record) => {
              this.setApplicantRouterLinkMap(record.applicantInfo.id);

              return record;
            })
          }
        };
      }
    );
  }

  setApplicantRouterLinkMap (id: number) {
    const appId = this.activatedRoute.parent ?
      this.activatedRoute.parent.snapshot.params.id :
      null;
    this.applicantManagerService.setApplicantRouterLinkMap(
      id,
      appId
    );
  }
}
