import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { StatusService } from '@core/services/status.service';
import { FundingSourceTypes } from '@core/typings/budget.typing';
import { ApplicantManagerService } from '@features/applicant/applicant-manager.service';
import { AwardForOrg } from '@features/awards/typings/award.typing';
import { DebounceFactory, PaginationOptions, TableDataFactory, TopLevelFilter } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { NonprofitService } from '../nonprofit.service';

@Component({
  selector: 'gc-nonprofit-awards',
  templateUrl: './nonprofit-awards.component.html',
  styleUrls: ['./nonprofit-awards.component.scss']
})
export class NonprofitAwardsComponent implements OnInit {
  tableDataFactory: TableDataFactory<AwardForOrg>;
  guid: string;
  tableKey: string;
  topLevelFilters: TopLevelFilter[] = [];
  FundingSourceTypes = FundingSourceTypes;
  paymentStatusMap = this.statusService.paymentStatusMap;
  fromRoute = this.applicantManagerService.getFromRouteForProfile();
  popoverPlacement: 'right'|'left' = 'right';

  constructor (
    private nonprofitService: NonprofitService,
    private activatedRoute: ActivatedRoute,
    private statusService: StatusService,
    private i18n: I18nService,
    private applicantManagerService: ApplicantManagerService
  ) { }

  get applicantRouterLink () {
    return this.applicantManagerService.get('applicantProfileRouterLink');
  }

  async ngOnInit () {
    this.topLevelFilters = [
      new TopLevelFilter(
        'text',
        'applicantInfo.fullName',
        '',
        this.i18n.translate(
          'GLOBAL:textSearchByApplicantNameOrEmail',
          {},
          'Search by applicant, award amount, or application id'
        ),
        undefined,
        undefined,
        [{
          column: 'applicantInfo.fullName',
          filterType: 'cn'
        }, {
          column: 'applicantInfo.email',
          filterType: 'cn'
        }, {
          column: 'amount',
          filterType: 'eq'
        }, {
          column: 'applicationId',
          filterType: 'eq'
        }]
      )
    ];
    this.setTableDataFactory();
  }

  setTableDataFactory () {
    this.guid = this.activatedRoute.snapshot.parent.params.guid;
    const map = this.nonprofitService.get('nonprofitMap')[this.guid];
    this.tableKey = `ORG_AWARDS_${map.gcId}`;
    this.tableDataFactory = DebounceFactory.createSimple(
      async (options: PaginationOptions<AwardForOrg>) => {
        const result = await this.nonprofitService.getOrgAwards(
          +map.gcId,
          options
        );

        return {
          success: true,
          data: {
            recordCount: result.recordCount,
            records: result.records.map((record: AwardForOrg) => {
              this.setApplicantRouterLinkMap(record.applicantInfo.id);

              return record;
            })
          }
        };
      }
    );
  }

  setApplicantRouterLinkMap (id: number) {
    const appId = this.activatedRoute.parent ?
      this.activatedRoute.parent.snapshot.params.id :
      null;
    this.applicantManagerService.setApplicantRouterLinkMap(
      id,
      appId
    );
  }
}
