import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { SpinnerService } from '@core/services/spinner.service';
import { CommunicationsService } from '@features/communications/communications.service';
import { Communication } from '@features/communications/communications.typing';
import { NonprofitService } from '../nonprofit.service';

@Component({
  selector: 'gc-nonprofit-communications-page',
  templateUrl: './nonprofit-communications-page.component.html',
  styleUrls: ['./nonprofit-communications-page.component.scss']
})
export class NonprofitCommunicationsPageComponent implements OnInit {
  id: number;

  constructor (
    private nonprofitService: NonprofitService,
    private communicationsService: CommunicationsService,
    private activatedRoute: ActivatedRoute,
    private spinnerService: SpinnerService
  ) { }

  get allComms () {
    return this.communicationsService.nonprofitCommunicationsMap[this.nonprofitId];
  }

  get currentComm () {
    return this.allComms[0];
  }

  get nonprofit () {
    return this.nonprofitService.get('nonprofitMap')[this.id];
  }

  get nonprofitId () {
    return +this.nonprofit.gcId;
  }

  ngOnInit () {
    this.id = this.activatedRoute.snapshot.parent.params.guid;
  }

  async updatePanel () {
    await this.communicationsService.getCommunicationsForOrganization(this.nonprofitId);
  }

  async addCommunication (comm: Communication) {
    this.spinnerService.stopSpinner();
    await this.communicationsService.addOrganizationCommunication(comm, this.nonprofitId);
    await this.updatePanel();
    this.spinnerService.stopSpinner();
  }

  async updateCommunication (comm: Communication) {
    this.spinnerService.startSpinner();
    await this.communicationsService.updateOrganizationCommunication(comm, this.nonprofitId);
    await this.updatePanel();
    this.spinnerService.stopSpinner();
  }

  async removeCommunication (comm: Communication) {
    this.spinnerService.startSpinner();
    await this.communicationsService.deleteOrganizationCommunication(comm.joinCommunicationId, this.nonprofitId);
    await this.updatePanel();
    this.spinnerService.stopSpinner();
  }
}
