import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { LocationService } from '@core/services/location.service';
import { PolicyService } from '@core/services/policy.service';
import { CsrConnectStats, NonprofitDetail } from '@core/typings/organization.typing';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { SystemTagsResolver } from '@features/system-tags/resolvers/system-tags.resolver';
import { SystemTagsService } from '@features/system-tags/system-tags.service';
import { SystemTags } from '@features/system-tags/typings/system-tags.typing';
import { AddressFormatterService } from '@yourcause/common';
import { NonprofitService } from '../nonprofit.service';

@Component({
  selector: 'gc-nonprofit-profile-page',
  templateUrl: './nonprofit-profile-page.component.html',
  styleUrls: ['./nonprofit-profile-page.component.scss']
})

export class NonprofitProfilePageComponent implements OnInit {
  guid: string;
  csrStats: CsrConnectStats;
  companyName = this.clientSettingsService.get('clientBranding').name;
  tagType = SystemTags.Buckets.NonprofitProfile;
  canManageTags = this.policyService.system.canManageOrganizationCRM();
  nonprofitProfileTags: SystemTags.TagDefinition[] = [];
  nonprofitMap = this.nonprofitService.get('nonprofitMap');
  displayAddress: string;
  disbursementAddress: string;
  ready = false;

  constructor (
    private policyService: PolicyService,
    private clientSettingsService: ClientSettingsService,
    private activatedRoute: ActivatedRoute,
    private nonprofitService: NonprofitService,
    private addressFormatter: AddressFormatterService,
    private systemTagsResolver: SystemTagsResolver,
    private systemTagsService: SystemTagsService,
    private locationService: LocationService
  ) { }

  get nonprofitDetail (): NonprofitDetail {
    return this.nonprofitMap[this.guid] ?
      this.nonprofitMap[this.guid].nonprofitDetail :
      ({} as NonprofitDetail);
  }

  get isPrivateOrg () {
    return this.nonprofitMap[this.guid] ?
    this.nonprofitMap[this.guid].nonprofitDetail.isPrivateOrg :
      false;
  }

  get nonprofitId () {
    return this.nonprofitMap[this.guid] ?
      this.nonprofitMap[this.guid].gcId :
      (null);
  }

  get nonprofitTags () {
    if (this.nonprofitDetail.tags) {
      return this.nonprofitDetail.tags.split('|');
    }

    return [];
  }

  async ngOnInit () {
    await this.systemTagsResolver.resolve();
    this.nonprofitProfileTags = this.systemTagsService.getTagsForBucket(
      SystemTags.Buckets.NonprofitProfile
    );
    this.guid = this.activatedRoute.snapshot.parent.params.guid;
    const csrStats = this.nonprofitMap[this.guid] ?
      (this.nonprofitMap[this.guid].csrStats || null) :
      null;
    this.csrStats = this.nonprofitService.getCsrStatsWithData(csrStats);
    this.setAddresses();
    this.ready = true;
  }

  setAddresses () {
    if (
      this.nonprofitDetail.displayAddress ||
      this.nonprofitDetail.address
    ) {
      this.displayAddress = this.addressFormatter.format(
        this.nonprofitDetail.displayAddress || this.nonprofitDetail.address,
        true
      );
    }
    if (this.nonprofitDetail.remittanceInfo?.address) {
      this.disbursementAddress = this.addressFormatter.format(
        this.nonprofitDetail.remittanceInfo.address,
        true
      );
    }
  }

  openWebsite () {
    this.locationService.openExternalUrl(
      this.nonprofitDetail.displayUrl
    );
  }
}
