import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, Resolve } from '@angular/router';
import { PolicyService } from '@core/services/policy.service';
import { CommunicationsService } from '@features/communications/communications.service';
import { NonprofitService } from './nonprofit.service';

@Injectable({ providedIn: 'root' })
export class NonprofitProfileResolver implements Resolve<boolean> {

  constructor (
    private nonprofitService: NonprofitService,
    private communicationsService: CommunicationsService,
    private policyService: PolicyService
  ) { }

  async resolve (next: ActivatedRouteSnapshot) {
    // Will either be guid or id
    const guid = next.params.guid;
    if (isNaN(guid)) {
      await this.nonprofitService.setNonprofit(guid);
    } else {
      await this.nonprofitService.setCompanyOrgDetail(guid);
    }
    const map = this.nonprofitService.get('nonprofitMap')[guid];
    if (this.policyService.system.canManageOrganizationCRM()) {
      await this.communicationsService.getCommunicationsForOrganization(
        +map.gcId
      );
    }

    return true;
  }
}
