import { Component, Input, OnInit } from '@angular/core';
import { NonprofitDetail } from '@core/typings/organization.typing';
import { InflectService, NonprofitSDG, NonprofitSDGTarget, NonprofitSDGWithTargets, SDGMetadata } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { NonprofitService } from '../nonprofit.service';

@Component({
  selector: 'yc-nonprofit-sdg-summary',
  templateUrl: './nonprofit-sdg-summary.component.html',
  styleUrls: ['./nonprofit-sdg-summary.component.scss']
})
export class NonprofitSdgSummaryComponent implements OnInit {
  @Input() nonprofitDetail: NonprofitDetail;
  displaySdgs: NonprofitSDG[];
  sdgs: SDGMetadata[];

  constructor (
    private nonprofitService: NonprofitService,
    private i18n: I18nService,
    private inflect: InflectService
  ) { }

  ngOnInit () {
    this.sdgs = this.nonprofitService.get('sdgs');
    this.formatSdgs();
  }

  formatSdgs () {
    this.displaySdgs = this.nonprofitDetail.sdGs.map((sdg) => {
      return {
        ...sdg,
        iconUrl: this.getIconUrl(sdg),
        name: this.getSdgName(sdg.sdgId),
        givePercent: sdg.targets.length  ? sdg.targets.reduce((acc, target) => {
          return acc + +target.givePercent;
        }, 0) : +sdg.givePercent,
        volunteerPercent: sdg.targets.length ? sdg.targets.reduce((acc, target) => {
          return acc + +target.volunteerPercent;
        }, 0) : +sdg.volunteerPercent
      };
    });
  }

  getIconUrl (sdg: NonprofitSDGWithTargets) {
    const selected = this.sdgs.find(t => t.id === sdg.sdgId);

    return this.nonprofitService.getSdgIconUrl(selected);
  }

  getSdgName (id: number) {
    return this.sdgs.find(t => t.id === id).goal;
  }

  translateTarget (target: NonprofitSDGTarget) {
    return this.i18n.translate(
      `sdgs:text${this.inflect.pascalize(target.target)}`,
      {},
      target.target
    );
  }

  translateSdg (sdg: NonprofitSDG) {
    return this.nonprofitService.translateSdg(sdg.name);
  }

}
