import { Component, Input, OnInit } from '@angular/core';
import { SpinnerService } from '@core/services/spinner.service';
import { NonProfitStory } from '@core/typings/organization.typing';
import { YCModalComponent } from '@yourcause/common/modals';

@Component({
  selector: 'gc-nonprofit-stories-modal',
  templateUrl: './nonprofit-stories-modal.component.html',
  styleUrls: ['./nonprofit-stories-modal.component.scss']
})
export class NonprofitStoriesModalComponent extends YCModalComponent<void> implements OnInit {
  @Input() story: NonProfitStory;

  title: string;
  description: string;
  imageURL: string;
  link: string;
  linkText: string;

  constructor (
    private spinnerService: SpinnerService
  ) {
    super();
  }

  ngOnInit () {
    this.spinnerService.startSpinner();
    if (this.story) {
      this.title = this.story.storyTitle;
      this.imageURL = this.story.imageURL;
      this.description = this.parseDescription();
      this.linkText = this.story.externalLinkText ? this.story.externalLinkText : this.story.externalLink;
      this.link = this.story.externalLink;
    }

    this.spinnerService.stopSpinner();
  }

  parseDescription () {
    const tempEl = document.createElement('html');
    tempEl.innerHTML = this.story.storyDescription;
    const hasChildEl = tempEl.getElementsByTagName('body')[0].children.length > 0;
    const returnVal = hasChildEl ? tempEl.innerHTML : '';

    return returnVal;
  }

  onHide () {
    this.closeModal.emit();
  }

}
