
import { Component, Input, OnInit } from '@angular/core';
import { NonprofitDetail, NonProfitStory } from '@core/typings/organization.typing';
import { ArrayHelpersService, AutoTableRepository, AutoTableRepositoryFactory, ValueComparisonService } from '@yourcause/common';
import { ModalFactory } from '@yourcause/common/modals';
import { NotifierService } from '@yourcause/common/notifier';
import { NonprofitStoriesModalComponent } from '../nonprofit-stories-modal/nonprofit-stories-modal.component';

@Component({
  selector: 'yc-nonprofit-stories',
  templateUrl: './nonprofit-stories.component.html',
  styleUrls: ['./nonprofit-stories.component.scss']
})
export class NonprofitStoriesComponent implements OnInit {
  @Input() nonprofitDetail: NonprofitDetail;
  repository: AutoTableRepository<NonProfitStory>;

  constructor (
    private modalFactory: ModalFactory,
    private arrayHelper: ArrayHelpersService,
    private autoTableFactory: AutoTableRepositoryFactory,
    private notifierService: NotifierService,
    private valueComparisonService: ValueComparisonService
  ) { }

  ngOnInit () {
    const stories = this.nonprofitDetail && this.nonprofitDetail.nonprofitStoryModel ? this.nonprofitDetail.nonprofitStoryModel : [];
    const sortedStories = this.arrayHelper.sort(stories, 'lastModifiedDate', true);
    this.repository = this.autoTableFactory.create({
      key: 'STORIES_' + this.nonprofitDetail.id,
      columns: [],
      notifier: this.notifierService,
      valueComparisonService: this.valueComparisonService,
      rows: sortedStories,
      rowsPerPage: 2
    });
  }

  parseDescription (row: NonProfitStory) {
    const tempEl = document.createElement('html');
    tempEl.innerHTML = row.storyDescription;
    const hasChildEl = tempEl.getElementsByTagName('body')[0].children.length > 0;
    const returnVal = hasChildEl ? tempEl.innerHTML : '';

    return returnVal;
  }

  async openStoryModal (story?: NonProfitStory) {
    await this.modalFactory.open(
      NonprofitStoriesModalComponent,
      { story }
    );
  }

}
