import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { ClientSettings } from '@core/typings/api/admin-client.typing';
import { ApplicationForUi, ApplicationFromApi } from '@core/typings/application.typing';
import { ApplicantOrganization, CompanyOrgDetail, NonprofitDataApi, NonprofitDetail } from '@core/typings/organization.typing';
import { ProcessingTypes } from '@core/typings/payment.typing';
import { ApplicationStatuses } from '@core/typings/status.typing';
import { BucketSearchResultObj, OrganizationEligibleForGivingStatus, OrganizationSearchResponse, SearchResult } from '@yourcause/common';
import { AfterEach, BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect, spy } from 'chai';
import { NonprofitService } from './nonprofit.service';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(NonprofitService, {
  imports: [
    GCMockModule
  ]
})
export class NonprofitServiceSpec implements Spec<
  NonprofitServiceSpec, NonprofitService
> {
  companyDetail: CompanyOrgDetail = {
    address: '5516 Alexis Court',
    address2: '',
    charityId: 1235,
    city: 'North Charleston',
    country: 'US',
    id: 1,
    imageUrl: '',
    name: 'Tinas Org',
    nonprofitGuid: 'asdfasdf',
    orgIdentification: '05271989',
    phoneNumber: '',
    postalCode: '29406',
    state: 'SC',
    isPrivateOrg: true
  };
  publicResult = {
    text: 'Darkness to Light',
    document: {
      name: 'Darkness to Light',
      id: '4654',
      nonprofitGuid: '15135233131'
    }
  } as SearchResult;
  privateResult: OrganizationSearchResponse = {
    id: 464,
    name: 'Private org',
    address: '134 Main',
    address2: null,
    country: 'US',
    state: 'MD',
    city: 'Salisbury',
    postalCode: '21801',
    phoneNumber: null,
    imageUrl: null,
    orgIdentification: '135353',
    nonprofitGuid: '69600',
    isPrivateOrg: true
  };
  applicantOrg: ApplicantOrganization = {
    id: 5235,
    name: 'My Org',
    address: '134532 Main',
    address2: null,
    country: 'US',
    state: 'SC',
    city: 'Charleston',
    postalCode: '12422',
    phoneNumber: null,
    charityId: 2353,
    orgIdentification: '132543',
    imageUrl: null,
    isPrivateOrg: true,
    nonprofitGuid: '230r52033'
  };
  bucketResult: BucketSearchResultObj = {
    id: 1254,
    name: 'Bucket Org',
    imageUrl: null,
    location: {
      address: 'full address string',
      city: '',
      state: '',
      zip: '',
      country: '',
      countryCode: ''
    },
    misstionStatement: 'mission',
    contactPhone: null,
    siteUrl: null,
    registrationNumber: '12353',
    uniqueGuid: '235323'
  };
  nonprofitDetail = {
    address: {
      address1: '123 Main',
      address2: '',
      city: 'Charleston',
      stateProvRegCode: 'SC',
      country: 'US',
      postalCode: '13452'
    },
    remittanceInfo: {
      address: {
        address1: '123 Main',
        address2: '',
        city: 'Charleston',
        stateProvRegCode: 'SC',
        country: 'US',
        postalCode: '13452'
      }
    }
  } as NonprofitDetail;
  applicationFromApi: ApplicationFromApi = {
    reservedFunds: false,
    budgetId: null,
    fundingSourceId: null,
    applicationGuid: 'abc-123',
    applicationId: 23532,
    applicationStatus: ApplicationStatuses.Approved,
    submittedDate: '',
    applicantId: 12542,
    applicantFirstName: 'Lukie',
    applicantCanReceiveEmails: true,
    applicantLastName: 'Smith',
    applicantEmail: 'lukie@mailinator.com',
    applicantPhoneNumber: '',
    applicationProfileImageUrl: '',
    applicantAddress: '',
    programName: 'Program name',
    programId: 3456,
    grantProgramTimezone: 'UTC',
    organizationId: 56,
    charityId: 5,
    organizationName: 'Org name',
    orgnizationImageUrl: '',
    organizationIdentification: '12-3415',
    organizationAddress: '',
    isPrivateOrg: false,
    daysInReview: 0,
    isApplicationInClientUserWorkflowLevel: true,
    workflowLevelApplicationReview: null,
    reviewRuleType: null,
    totalWorkflowLevelReviews: 4,
    totalWorkflowLevelUsers: 4,
    majorityReviewed: false,
    currentWorkflowName: 'WFL name',
    currentWorkFlowLevelName: 'WF name',
    parentWorkFlowLevelName: 'Parent WFL name',
    parentWorkFlowLevelId: 1,
    canRouteApplication: true,
    canArchiveAndUnarchiveApplication: true,
    canApproveApplication: true,
    canAwardApplication: true,
    canDeclineApplication: true,
    canArchiveApplication: true,
    canExtendFormDueDate: true,
    canRecommendFunding: true,
    hasNonPaidPayments: true,
    workflowLevelUsersWithoutReviews: [],
    workflowLevelRoutes: [],
    managerWillComplete: true,
    isDraft: true,
    programAllowCollaboration: true,
    programSendEmailsToCollaborators: true,
    amountRequested: 500,
    paymentDesignation: 'payment designation',
    careOf: null,
    createdBy: null,
    lastUpdatedBy: null,
    updatedDate: '',
    createdDate: '',
    nonprofitGuid: 'guid',
    currentWorkFlowLevelId: 235325,
    currentWorkFlowId: 567,
    isArchived: true,
    nominee: null,
    nominationApplicationId: 6,
    canAccessNominationApplication: true,
    isMasked: true,
    canViewMaskedApplicantInfo: false,
    specialHandlingAddress1: 'Address 1',
    specialHandlingAddress2: 'Address 2',
    specialHandlingCity: 'City',
    specialHandlingCountry: 'US',
    specialHandlingName: 'Special Handling name',
    specialHandlingPostalCode: '32690',
    specialHandlingStateProvinceRegion: 'SC',
    specialHandlingNotes: '',
    specialHandlingFileUrl: null,
    specialHandlingReason: '',
    currencyRequestedAmountEquivalent: 0,
    currencyRequested: 'USD',
    inKindAmountRequested: 0,
    inKindItems: [],
    isProgramArchived: true,
    meetsVettingRequirement: true,
    grantProgramCycle: null,
    hasAwards: false,
    hasPayments: false,
    hasCashAward: false,
    organizationEligibleForGivingStatus: OrganizationEligibleForGivingStatus.ELIGIBLE,
    recommendedFundingAmount: 0,
    orgIsInternational: false,
    applicantIsSSO: false,
    currentWorkflowLevelShowBudgetSummaryInfo: true,
    currentWorkflowLevelAllowUserToViewAwardsAndPayments: true,
    currentWorkflowLevelAllowAward: true
  };
  applicationForUi: ApplicationForUi = {
    ...this.applicationFromApi,
    amountRequestedForEdit: this.applicationFromApi.currencyRequestedAmountEquivalent,
    designation: ''
  };

  @BeforeEach()
  prep (service: NonprofitService) {
    service['nonprofitResources'].getCompanyOrgDetail = async () => {
      return this.companyDetail;
    };
    service['nonprofitResources'].getNonprofitAdditionalDataByGuid = async () => {
      return {
        nonprofit: {} as NonprofitDataApi,
        nonprofitDetail: this.nonprofitDetail
      };
    };
    service['nonprofitResources'].getIdByNonprofitGuid = async () => {
      return '2353';
    };
    service['clientSettingsService']['set']('clientSettings', {
      hasInternational: false
    } as ClientSettings);
    service['clientSettingsService']['set']('domesticRegAuthFilters', []);
  }

  @AfterEach()
  reset () {
    spy.restore();
  }

  @TestCase('should be able to get applicant organization from org id')
  async getApplicantOrganizationFromOrgId (service: NonprofitService) {
    const detail = await service.getApplicantOrganizationFromOrgId(1);
    expect(detail.name).to.be.equal(this.companyDetail.name);
    expect(detail.nonprofitGuid).to.be.equal(this.companyDetail.nonprofitGuid);
    expect(detail.city).to.be.equal(this.companyDetail.city);
    expect(detail.orgIdentification).to.be.equal(this.companyDetail.orgIdentification);
  }

  @TestCase('should be able to search public orgs')
  async searchPublicNonprofits (service: NonprofitService) {
    spy.on(service['nonprofitResources'], 'searchPublicNonprofits', () => {});
    await service.searchPublicNonprofits('term', [], 1, 4);
    expect(service['nonprofitResources'].searchPublicNonprofits).to.have.been.called.once;
  }

  @TestCase('should be able to search bucket orgs')
  async searchNonprofitsWithinBucket (service: NonprofitService) {
    spy.on(service['nonprofitResources'], 'searchNonprofitsWithinBucket', () => {});
    await service.searchNonprofitsWithinBucket('term', 2, 2, 3, '4', null);
    expect(service['nonprofitResources'].searchNonprofitsWithinBucket).to.have.been.called.once;
  }

  @TestCase('should be able to search private orgs')
  async searchPrivateOrgs (service: NonprofitService) {
    spy.on(service['nonprofitResources'], 'searchPrivateOrgs', () => {});
    await service.searchPrivateOrgs(2, {} as any);
    expect(service['nonprofitResources'].searchPrivateOrgs).to.have.been.called.once;
  }

  @TestCase('should be able get vetting status')
  async getVettingStatus (service: NonprofitService) {
    spy.on(service['nonprofitResources'], 'getVettingStatus', () => {});
    await service.getVettingStatus('');
    expect(service['nonprofitResources'].getVettingStatus).to.have.been.called.once;
  }

  @TestCase('should be able to add or vet org')
  async addOrVetOrganization (service: NonprofitService) {
    spy.on(service['nonprofitResources'], 'addOrVetOrganization', () => {});
    await service.addOrVetOrganization({} as any);
    expect(service['nonprofitResources'].addOrVetOrganization).to.have.been.called.once;
  }

  @TestCase('should be able to submit vetting request by applicant')
  async submitVettingRequestByApplicant (service: NonprofitService) {
    spy.on(service['nonprofitResources'], 'submitVettingRequestByApplicant', () => {});
    await service.submitVettingRequestByApplicant({} as any);
    expect(service['nonprofitResources'].submitVettingRequestByApplicant).to.have.been.called.once;
  }

  @TestCase('should be able to sanitize search term')
  sanitizeSearchTerm (service: NonprofitService) {
    const sanitized = service.sanitizeSearchTerm(`'/term/#9$;`);
    expect(sanitized).to.be.equal('term9');
  }

  @TestCase('should be able to get org ID - public result')
  async getOrgIdPublic (service: NonprofitService) {
    spy.on(service, 'getIdByNonprofitGuid');
    spy.on(service, 'getNonprofitAdditionalDataByGuid');
    await service.getOrgId(this.publicResult);
    expect(service['getIdByNonprofitGuid']).to.have.been.called.once;
    expect(service['getNonprofitAdditionalDataByGuid']).to.have.been.called.once;
  }

  @TestCase('should be able to get org ID - private result')
  async getOrgIdPrivate (service: NonprofitService) {
    spy.on(service, 'getIdByNonprofitGuid');
    spy.on(service, 'getNonprofitAdditionalDataByGuid');
    await service.getOrgId(this.privateResult);
    expect(service['getIdByNonprofitGuid']).to.have.been.called.once;
    expect(service['getNonprofitAdditionalDataByGuid']).to.have.been.called.once;
  }

  @TestCase('should be able to get org ID - applicant result')
  async getOrgIdApplicant (service: NonprofitService) {
    spy.on(service, 'getIdByNonprofitGuid');
    spy.on(service, 'getNonprofitAdditionalDataByGuid');
    await service.getOrgId(this.applicantOrg);
    expect(service['getIdByNonprofitGuid']).to.have.been.called.once;
    expect(service['getNonprofitAdditionalDataByGuid']).to.have.been.called.once;
  }

  @TestCase('should be able to get org ID - bucket result')
  async getOrgIdBucket (service: NonprofitService) {
    spy.on(service, 'getIdByNonprofitGuid');
    spy.on(service, 'getNonprofitAdditionalDataByGuid');
    await service.getOrgId(this.bucketResult);
    expect(service['getIdByNonprofitGuid']).to.have.been.called.once;
    expect(service['getNonprofitAdditionalDataByGuid']).to.have.been.called.once;
  }

  @TestCase('should be able to get org ID - no guid')
  async getOrgIdNoGuid (service: NonprofitService) {
    spy.on(service, 'getIdByNonprofitGuid');
    spy.on(service, 'getNonprofitAdditionalDataByGuid');
    await service.getOrgId({
      ...this.applicantOrg,
      nonprofitGuid: null
    });
    expect(service['getIdByNonprofitGuid']).to.not.have.been.called;
    expect(service['getNonprofitAdditionalDataByGuid']).to.not.have.been.called;
  }


  @TestCase('should be able to get adapted org - public result')
  async getAdaptedOrgPublic (service: NonprofitService) {
    const adapted = await service.getAdaptedOrg(this.publicResult, false, false);
    expect(adapted.nonprofitDetail).to.deep.equal(this.nonprofitDetail);
    const adaptedOrg = adapted.adaptedOrg.document;
    expect(adaptedOrg.displayName).to.be.equal(this.publicResult.document.name);
    expect(adaptedOrg.nonprofitGuid).to.be.equal(this.publicResult.document.nonprofitGuid);
    expect(adaptedOrg.address1).to.be.equal(this.nonprofitDetail.address.address1);
  }

  @TestCase('should be able to get adapted org - bucket result')
  async getAdaptedOrgBucket (service: NonprofitService) {
    const adapted = await service.getAdaptedOrg(this.bucketResult, false, true);
    expect(adapted.nonprofitDetail).to.deep.equal(this.nonprofitDetail);
    const adaptedOrg = adapted.adaptedOrg.document;
    expect(adaptedOrg.name).to.be.equal(this.bucketResult.name);
    expect(adaptedOrg.nonprofitGuid).to.be.equal(this.bucketResult.uniqueGuid);
    expect(adaptedOrg.address1).to.be.equal(this.nonprofitDetail.address.address1);
  }

  @TestCase('should be able to get adapted org - private result')
  async getAdaptedOrgPrivate (service: NonprofitService) {
    const adapted = await service.getAdaptedOrg(this.privateResult, true, false);
    const adaptedOrg = adapted.adaptedOrg.document;
    expect(adaptedOrg.name).to.be.equal(this.privateResult.name);
    expect(adaptedOrg.nonprofitGuid).to.be.equal(this.privateResult.nonprofitGuid);
  }

  @TestCase('should be able to prep search filters - YC processed with state/country')
  prepSearchFilters (service: NonprofitService) {
    const filters = service.prepSearchFilters(
      'SC',
      'US',
      'United States',
      ProcessingTypes.YourCause
    );
    const hasEligibleFilter = filters.some((filter) => {
      return filter.filterValue === (OrganizationEligibleForGivingStatus.ELIGIBLE + '');
    });
    const hasNeedsInfoFilter = filters.some((filter) => {
      return filter.filterValue === (OrganizationEligibleForGivingStatus.INFO_REQUIRED + '');
    });
    const hasStateFilter = filters.some((filter) => {
      return filter.filterName === 'StateProvRegCode' &&
        filter.filterValue === 'SC';
    });
    const hasCountryFilter = filters.some((filter) => {
      return filter.filterName === 'Country' &&
        filter.filterValue === 'UNITED STATES';
    });
    expect(hasEligibleFilter).to.be.true;
    expect(hasNeedsInfoFilter).to.be.true;
    expect(hasStateFilter).to.be.true;
    expect(hasCountryFilter).to.be.true;
  }

  @TestCase('should be able to prep search filters - domestic reg auths')
  prepSearchFiltersDomesticRegAuths (service: NonprofitService) {
    service['clientSettingsService']['set']('domesticRegAuthFilters', [{
      filterName: 'reg auth name',
      filterValue: 'reg auth val'
    }]);
    const filters = service.prepSearchFilters(
      '',
      '',
      '',
      ProcessingTypes.Client
    );
    const hasEligibleFilter = filters.some((filter) => {
      return filter.filterValue === (OrganizationEligibleForGivingStatus.ELIGIBLE + '');
    });
    const hasNeedsInfoFilter = filters.some((filter) => {
      return filter.filterValue === (OrganizationEligibleForGivingStatus.INFO_REQUIRED + '');
    });
    const hasRegAuthFilter = filters.some((filter) => {
      return filter.filterValue === 'reg auth val';
    });
    const hasStateFilter = filters.some((filter) => {
      return filter.filterName === 'StateProvRegCode' &&
        filter.filterValue === 'SC';
    });
    const hasCountryFilter = filters.some((filter) => {
      return filter.filterName === 'Country' &&
        filter.filterValue === 'UNITED STATES';
    });
    expect(hasEligibleFilter).to.be.false;
    expect(hasNeedsInfoFilter).to.be.false;
    expect(hasRegAuthFilter).to.be.true;
    expect(hasStateFilter).to.be.false;
    expect(hasCountryFilter).to.be.false;
  }

  @TestCase('should be able to do public search - with term')
  async doSearchPublic (service: NonprofitService) {
    spy.on(service, 'searchPublicNonprofits', () => {
      return {
        results: [],
        count: 0
      };
    });
    await service.doSearchPublic(
      1,
      'darkness',
      4,
      'SC',
      'US',
      'United States',
      ProcessingTypes.YourCause
    );
    expect(service.searchPublicNonprofits).to.have.been.called.once;
  }

  @TestCase('should be able to do public search - no term')
  async doSearchPublicNoTerm (service: NonprofitService) {
    spy.on(service, 'searchPublicNonprofits', () => {
      return {
        results: [],
        count: 0
      };
    });
    await service.doSearchPublic(
      1,
      '',
      4,
      'SC',
      'US',
      'United States',
      ProcessingTypes.YourCause
    );
    expect(service.searchPublicNonprofits).to.not.have.been.called;
  }

  @TestCase('should be able to do bucket search - with term')
  async doBucketSearch (service: NonprofitService) {
    spy.on(service, 'searchNonprofitsWithinBucket', () => {
      return {
        totalCount: 0,
        pageCount: 4,
        page: 1,
        pageSize: 4,
        message: '',
        data: []
      };
    });
    await service.doBucketSearch(
      1,
      'term',
      'UK',
      'UK',
      4,
      3,
      '3'
    );
    expect(service.searchNonprofitsWithinBucket).to.have.been.called.once;
  }

  @TestCase('should be able to do bucket search - no term')
  async doBucketSearchNoTerm (service: NonprofitService) {
    spy.on(service, 'searchNonprofitsWithinBucket', () => {
      return {
        totalCount: 0,
        pageCount: 4,
        page: 1,
        pageSize: 4,
        message: '',
        data: []
      };
    });
    await service.doBucketSearch(
      1,
      '',
      'UK',
      'UK',
      4,
      3,
      '3'
    );
    expect(service.searchNonprofitsWithinBucket).to.not.have.been.called;
  }

  @TestCase('should be able to do private search - with term')
  async doPrivateSearch (service: NonprofitService) {
    spy.on(service, 'searchPrivateOrgs', () => {
      return {
        records: [],
        recordCount: 0
      };
    });
    await service.doSearchPrivate(
      1,
      'term',
      4,
      ProcessingTypes.YourCause,
      'SC',
      'US',
      1
    );
    expect(service.searchPrivateOrgs).to.have.been.called.once;
  }

  @TestCase('should be able to do private search - no term')
  async doPrivateSearchNoTerm (service: NonprofitService) {
    spy.on(service, 'searchPrivateOrgs', () => {
      return {
        records: [],
        recordCount: 0
      };
    });
    await service.doSearchPrivate(
      1,
      '',
      4,
      ProcessingTypes.Client,
      'SC',
      'US',
      2
    );
    expect(service.searchPrivateOrgs).to.not.have.been.called;
  }

  @TestCase('should be able to get org search tab text - public')
  getOrgSearchTabTextPublic (service: NonprofitService) {
    const numberOfResults = 4;
    // Non mobile Scenarios
    let isMobile = false;

    // With Text
    let hasSearchTerm = true;
    let text = service.getOrgSearchTabText(
      isMobile,
      numberOfResults,
      hasSearchTerm,
      false,
      false
    );
    expect(text).to.be.equal(`Organizations (${numberOfResults})`);
    // No Text
    hasSearchTerm = false;
    text = service.getOrgSearchTabText(
      isMobile,
      0,
      hasSearchTerm,
      false,
      false
    );
    expect(text).to.be.equal('Organizations');


    // Mobile Scenarios
    isMobile = true;

    // With Text
    hasSearchTerm = true;
    text = service.getOrgSearchTabText(
      isMobile,
      numberOfResults,
      hasSearchTerm,
      false,
      false
    );
    expect(text).to.be.equal(`Orgs (${numberOfResults})`);

    // No term
    hasSearchTerm = false;
    text = service.getOrgSearchTabText(
      isMobile,
      numberOfResults,
      hasSearchTerm,
      false,
      false
    );
    expect(text).to.be.equal(`Orgs`);
  }

  @TestCase('should be able to get org search tab text - client')
  getOrgSearchTabTextClient (service: NonprofitService) {
    const numberOfResults = 4;
    // Non mobile Scenarios
    let isMobile = false;

    // With Text
    let hasSearchTerm = true;
    let text = service.getOrgSearchTabText(
      isMobile,
      numberOfResults,
      hasSearchTerm,
      true,
      false
    );
    expect(text).to.be.equal(`Client Organizations (${numberOfResults})`);
    // No Text
    hasSearchTerm = false;
    text = service.getOrgSearchTabText(
      isMobile,
      0,
      hasSearchTerm,
      true,
      false
    );
    expect(text).to.be.equal('Client Organizations');


    // Mobile Scenarios
    isMobile = true;

    // With Text
    hasSearchTerm = true;
    text = service.getOrgSearchTabText(
      isMobile,
      numberOfResults,
      hasSearchTerm,
      true,
      false
    );
    expect(text).to.be.equal(`Client (${numberOfResults})`);

    // No term
    hasSearchTerm = false;
    text = service.getOrgSearchTabText(
      isMobile,
      numberOfResults,
      hasSearchTerm,
      true,
      false
    );
    expect(text).to.be.equal(`Client Orgs`);
  }

  @TestCase('should be able to get org search tab text - favorites')
  getOrgSearchTabTextFavorites (service: NonprofitService) {
    const numberOfResults = 4;
    // Non mobile Scenarios
    let isMobile = false;

    let text = service.getOrgSearchTabText(
      isMobile,
      numberOfResults,
      false,
      false,
      true
    );
    expect(text).to.be.equal(`My Organizations (${numberOfResults})`);

    // Mobile Scenarios
    isMobile = true;

    text = service.getOrgSearchTabText(
      isMobile,
      numberOfResults,
      false,
      false,
      true
    );
    expect(text).to.be.equal(`My Orgs (${numberOfResults})`);
  }

  @TestCase('should be able to construct search result')
  async constructSearchResult (service: NonprofitService) {
    const result = await service.constructSearchResult(this.applicationForUi);
    expect(result.document.id).to.be.equal('' + this.applicationFromApi.organizationId);
    expect(result.document.name).to.be.equal(this.applicationFromApi.organizationName);
  }
}

