import { Component, Input, OnInit } from '@angular/core';
import { APIAdminClient } from '@core/typings/api/admin-client.typing';
import { AdminClientService } from '@features/platform-admin/clients/admin-client.service';
import { TypeaheadSelectOption } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { YCModalComponent } from '@yourcause/common/modals';
import moment from 'moment';
export interface AtRiskModalResponse {
  isSnoozed: boolean;
  snoozedDateOption: APIAdminClient.ActivityAlertStatusOption;
}

@Component({
  selector: 'gc-at-risk-activity-tracking-email-modal',
  templateUrl: './at-risk-activity-tracking-email-modal.component.html',
  styleUrls: ['./at-risk-activity-tracking-email-modal.component.scss']
})
export class AtRiskActivityTrackingEmailModalComponent extends YCModalComponent<
  AtRiskModalResponse
> implements OnInit {
  @Input() clientName: string;
  @Input() clientActivityAlertsSnoozed: boolean;
  @Input() clientSnoozedTilDate: string;

  isSnoozed: boolean;
  snoozedDate: APIAdminClient.ActivityAlertStatusOption;
  confirmText = this.i18n.translate(
    'common:textAtRiskActivityTrackingHelp3',
    {},
    `By default, clients with no application activity in the last 30 days may be considered at-risk of leaving GrantsConnect and included in the monthly At Risk Notification email unless snoozed. If you want to temporarily remove this client from activity tracking and At Risk Notifications, select "Snooze activity tracking".`
  );
  snoozeOptions: TypeaheadSelectOption<boolean>[] = [{
    label: this.i18n.translate(
      'common:textTrackActivity',
      {},
      'Track activity'
    ),
    value: false
  }, {
    label: this.i18n.translate(
      'common:textSnoozeActivityTracking',
      {},
      'Snooze activity tracking'
    ),
    value: true
  }];
  snoozeDateOptions: TypeaheadSelectOption<APIAdminClient.ActivityAlertStatusOption>[] = [];

  constructor (
    private i18n: I18nService,
    private adminClientService: AdminClientService
  ) {
    super();
  }

  ngOnInit () {
    this.isSnoozed = this.clientActivityAlertsSnoozed;
    if (this.clientActivityAlertsSnoozed) {
      this.snoozedDate = APIAdminClient.ActivityAlertStatusOption.Snooze1Month;
      this.snoozeDateOptions = [{
        label: this.i18n.translate(
          'common:textSnoozedUntilDate',
          {
            date: moment(this.clientSnoozedTilDate).format('ll')
          },
          'Snoozed until __date__'
        ),
        value: this.snoozedDate
      }];
    } else {
      this.snoozeDateOptions = this.adminClientService.getSnoozeDateOptions();
      this.snoozedDate = this.snoozeDateOptions[0].value;
    }
  }

  onSave () {
    this.closeModal.emit({
      isSnoozed: this.isSnoozed,
      snoozedDateOption: this.snoozedDate
    });
  }
}
