import { Injectable } from '@angular/core';
import { HttpRestService } from '@core/services/http-rest.service';
import { APIDataImport } from '@core/typings/api/data-import.typing';
import { HistoricalDataImportModelInstance } from '@core/typings/data-import.typing';
import { PaginatedResponse, PaginationOptions, YcFile } from '@yourcause/common';

@Injectable({ providedIn: 'root' })
export class HistoricalImportResources {

  constructor (
    private httpRestService: HttpRestService
  ) { }

  createFileBatch (
    batchId: number,
    importFileType: APIDataImport.DataImportFileType,
    fileName: string
  ): Promise<{ dataImportBatchFileBatchId: number }> {
    return this.httpRestService.post(`api/admin/DataImport/Batches/${batchId}/ImportFileBatch`, {
      fileName,
      importFileType
    });
  }

  commitFileBatch (importFileBatchId: number, batchId: number) {
    return this.httpRestService.post(`api/admin/DataImport/Batches/${batchId}/ImportFileBatch/${importFileBatchId}/Complete`, {
      batchId,
      importFileBatchId
    });
  }

  validateBatch (batchId: number) {
    const endpoint = `api/admin/DataImport/Batches/${batchId}/Validate`;

    return this.httpRestService.post<APIDataImport.ValidationError[]>(endpoint, null);
  }

  getBatches (
    paginationOptions: PaginationOptions<APIDataImport.DataImportBatchModel>,
    clientId?: number
  ): Promise<PaginatedResponse<APIDataImport.DataImportBatchModel>> {
    const endpoint = 'api/admin/DataImport/GetBatches';

    return this.httpRestService.post(endpoint, {
      paginationOptions,
      clientId
    });
  }

  createBatch (batch: APIDataImport.CreateImportBatch) {
    return this.httpRestService.post<number>('/api/admin/DataImport/Batches', batch);
  }

  deleteBatch (batchId: number) {
    return this.httpRestService.delete(`/api/admin/DataImport/Batches/${batchId}`);
  }

  addFileToBatch (
    fileImportBatchId: number,
    fileType: APIDataImport.DataImportFileType,
    file: YcFile|string,
    timezone: string,
    fileBatchId?: number,
    paymentBatchName?: string
  ) {
    return this.httpRestService.postFile(
      `/api/admin/DataImport/UploadFile?fileImportBatchId=${fileImportBatchId}&fileType=${fileType}&timezone=${encodeURIComponent(timezone)}&paymentBatchName=${paymentBatchName ?? ''}` + (fileBatchId ? `&dataImportBatchFileBatchId=${fileBatchId}` : ''),
      (typeof file === 'string') ? file : file.file
    );
  }

  async getRowsInBatch (
    batchId: number,
    type: keyof typeof APIDataImport.DataImportFileType,
    paginationOptions: PaginationOptions<HistoricalDataImportModelInstance>
  ) {
    const result = await this.httpRestService.post(
      `/api/admin/DataImport/Batches/${batchId}/${type}`,
      { paginationOptions }
    );

    return ({
      success: true,
      data: result
    });
  }

  processBatch (batchId: number) {
    return this.httpRestService.post(
      `/api/admin/DataImport/Batches/${batchId}/Process`,
      null
    );
  }
}

