import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { ProductManagementPageComponent } from './product-management-page/product-management-page.component';
import { ProductManagementModule } from './product-management.module';

@NgModule({
  imports: [
    ProductManagementModule,
    RouterModule.forChild([{
      path: '',
      component: ProductManagementPageComponent
    }, {
      path: 'release-notes',
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Release Notes',
        pageName: 'Product Management - Release Notes'
      },
      loadChildren: () => import('@features/platform-admin/release-notes/release-notes-routing.module')
        .then(m => m.ReleaseNotesRoutingRoutingModule)
    }, {
      path: 'standard-product-configuration',
      loadChildren: () => import('@features/platform-admin/standard-product-configuration/standard-product-configuration-routing.module')
        .then(m => m.StandardProductConfigurationRoutingModule),
      data: {
        breadcrumbLabel: 'Standard Product Configuration',
        breadcrumb: true,
        pageName: 'Product Management - Standard Product Configuration'
      }
    }, {
      path: 'feature-flags',
      loadChildren: () => import('@features/platform-admin/feature-flags/feature-flags-routing.module')
        .then(m => m.FeatureFlagsRoutingModule),
      data: {
        breadcrumbLabel: 'Feature Flags',
        breadcrumb: true,
        pageName: 'Product Management - Feature Flags'
      }
    }, {
      path: 'help-content',
      loadChildren: () => import('@features/platform-admin/help-content/help-content-routing.module')
        .then(m => m.HelpContentRoutingModule),
      data: {
        breadcrumbLabel: 'Help Content',
        breadcrumb: true,
        pageName: 'Product Management - Help Content'
      }
    }])
  ]
})
export class ProductManagementRoutingModule { }
