import { Injectable } from '@angular/core';
import { CLIENT_SETTINGS_SERVICE_MOCK } from '@core/mocks/client-settings.service.mock';
import { CURRENCY_SERVICE_MOCK } from '@core/mocks/currency.service.mock';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { PlatformPermission, PlatformRoleFromApi } from '@core/typings/roles.typing';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { PLATFORM_PERMISSION_SET_TYPE, RolesAndPermissionsPlatformService } from './roles-and-permissions.service';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(RolesAndPermissionsPlatformService, {
  imports: [
    GCMockModule
  ],
  providers: [
    CURRENCY_SERVICE_MOCK,
    CLIENT_SETTINGS_SERVICE_MOCK
  ]
})
export class RolesAndPermissionsServiceSpec implements Spec<RolesAndPermissionsServiceSpec, RolesAndPermissionsPlatformService> {
  roleId = 1;
  roles: PlatformRoleFromApi[] = [{
    systemRole: {
      id: this.roleId,
      name: 'GC-Admin',
      description: '',
      permissions: [{
        permissionId: 1,
        permissionName: 'CreateOrEditClient'
      },
      {
        permissionId: 2,
        permissionName: 'ViewApplicants'
      }]
    },
    userAssignedToRole: true
  },
  {
    systemRole: {
      id: 2,
      name: 'GC-Account Manager',
      description: '',
      permissions: [{
        permissionId: 6,
        permissionName: 'CreatePasswordResetToken'
      }, {
        permissionId: 10,
        permissionName: 'CanImpersonateUsers'
      }, {
        permissionId: 12,
        permissionName: 'ViewClient'
      }]
    },
    userAssignedToRole: true
  },
  {
    systemRole: {
      id: 3,
      name: 'GC-Data Manager',
      description: '',
      permissions: [{
        permissionId: 5,
        permissionName: 'Import'
      }, {
        permissionId: 12,
        permissionName: 'ViewClient'
      }]
    },
    userAssignedToRole: true
  }];
  permissions: PlatformPermission[] = [{
    permissionId: 1,
    permissionName: 'CreateOrEditClient'
  }, {
    permissionId: 2,
    permissionName: 'ViewApplicants'
  }, {
    permissionId: 3,
    permissionName: 'ViewGrantManagers'
  }, {
    permissionId: 4,
    permissionName: 'ViewEmailLogs'
  }, {
    permissionId: 5,
    permissionName: 'Import'
  }, {
    permissionId: 6,
    permissionName: 'CreatePasswordResetToken'
  }, {
    permissionId: 7,
    permissionName: 'CanManageAdminUsers'
  }, {
    permissionId: 8,
    permissionName: 'EditGrantManager'
  }, {
    permissionId: 9,
    permissionName: 'EditApplicant'
  }, {
    permissionId: 10,
    permissionName: 'CanImpersonateUsers'
  },  {
    permissionId: 11,
    permissionName: 'CanAccessAllZones'
  }, {
    permissionId: 12,
    permissionName: 'ViewClient'
  }, {
    permissionId: 13,
    permissionName: 'CanManageSpecialHandling'
  }, {
    permissionId: 14,
    permissionName: 'ComplianceVetting'
  }, {
    permissionId: 15,
    permissionName: 'CanEditApplicantSsoId'
  }, {
    permissionId: 16,
    permissionName: 'ManageRoles'
  }, {
    permissionId: 18,
    permissionName: 'ResourceConfiguration'
  }];

  @BeforeEach()
  async mock (service: RolesAndPermissionsPlatformService) {
    service['roleAndPermissionResources'].getRoles = async () => {
      return this.roles;
    };
    service['roleAndPermissionResources'].getPermissions = async () => {
      return this.permissions;
    };
    await service.setAllRolesAndPermissions();
  }

  @TestCase('should be able to get permission sets')
  getPermissionSets (service: RolesAndPermissionsPlatformService) {
    const sets = service.getPermissionSets();
    expect(sets.length).to.be.equal(1);
    expect(sets[0].type).to.be.equal(PLATFORM_PERMISSION_SET_TYPE);
    expect(sets[0].permissions.length).to.be.equal(this.permissions.length);
  }

  @TestCase('should be able to get role by ID')
  getRoleById (service: RolesAndPermissionsPlatformService) {
    const role = service.getRoleById(this.roleId);
    expect(role).to.not.be.undefined;
    expect(role.roleName).to.be.equal(this.roles[0].systemRole.name);
  }

  @TestCase('should be able to get permission set options')
  getPermissionSetOptions (service: RolesAndPermissionsPlatformService) {
    const options = service.getPermissionSetOptions();
    expect(options.length).to.be.equal(1);
    expect(options[0].label).to.be.equal('System');
  }
}

