import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { AdHocReportingAPI } from '@core/typings/api/ad-hoc-reporting.typing';
import { APIAdminClient, ClientSettings } from '@core/typings/api/admin-client.typing';
import { ProcessingTypes } from '@core/typings/payment.typing';
import { BasicForm, BasicRevision, FormAudience, FormStates, FormTypes } from '@features/configure-forms/form.typing';
import { GCDashboards } from '@features/dashboards/dashboards.typing';
import { PaginationOptions } from '@yourcause/common';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect, spy } from 'chai';
import { StandardProductConfigurationService } from './standard-product-configuration.service';
import { StandardFormTemplate, StandardProductDashboard, StandardProductForm, StandardProductFormField, StandardProductTypes, StandardReportTemplate } from './standard-product-configuration.typing';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(StandardProductConfigurationService, {
  imports: [
    GCMockModule
  ]
})
export class StandardProductConfigurationServiceSpec implements Spec<
  StandardProductConfigurationServiceSpec, StandardProductConfigurationService
> {
  managerFormId = 1;
  applicantFormId = 2;
  fileIsStaged = true;
  standardFormTemplates: StandardFormTemplate[] = [{
    name: 'Request Form Template',
    description: '',
    createdDate: '',
    updatedDate: '',
    revisionVersion: 1,
    revisionId: 1,
    formId: this.applicantFormId,
    formTypeId: FormTypes.REQUEST,
    defaultLanguageId: 'en-US'
  }, {
    name: 'Review Form Template',
    description: '',
    createdDate: '',
    updatedDate: '',
    revisionVersion: 1,
    revisionId: 3,
    formId: this.managerFormId,
    formTypeId: FormTypes.REVIEW,
    defaultLanguageId: 'en-US'
  }];
  standardReportId = 3;
  standardReportTemplate: StandardReportTemplate = {
    id: this.standardReportId,
    name: 'Application Report',
    description: '',
    isUserOwnedReport: true,
    reportModelType: AdHocReportingAPI.AdHocReportModelType.Application,
    reportType: AdHocReportingAPI.AdHocReportType.AdHoc,
    userSavedReportColumns: [],
    createdDate: '',
    updatedDate: '',
    clientId: 5,
    forms: [],
    reportOwnerClientUserId: 3,
    accessType: AdHocReportingAPI.AdHocReportAccessType.MANAGE,
    primaryFormId: null,
    advancedFilterColumns: [],
    useLogicalOperatorAnd: true,
    referenceFieldTableId: null,
    referenceFieldTableKey: '',
    standardComponentIsPublished: true,
    isStandardReportTemplate: true,
    isMyReport: false
  };
  standardReportTemplates: StandardReportTemplate[] = [
    this.standardReportTemplate
  ];
  forms: BasicForm[]= [{
    revisions: [],
    name: '',
    description: '',
    createdDate: '',
    updatedDate: '',
    revisionId: 2,
    revisionVersion: 4,
    formId: this.applicantFormId,
    formType: FormTypes.REQUEST,
    canBeRemoved: false,
    canModify: true,
    availableForTranslation: true,
    defaultLanguageId: '',
    state: FormStates.PUBLISHED,
    programIds: [],
    standardComponentIsPublished: true,
    isDraft: false
  }, {
    revisions: [],
    name: '',
    description: '',
    createdDate: '',
    updatedDate: '',
    revisionId: 2,
    revisionVersion: 4,
    formId: this.applicantFormId,
    formType: FormTypes.REQUEST,
    canBeRemoved: false,
    canModify: true,
    availableForTranslation: true,
    defaultLanguageId: '',
    state: FormStates.DRAFT,
    programIds: [],
    standardComponentIsPublished: false,
    isDraft: false
  }];

  exampleStandardProductDB: StandardProductDashboard = {
    dashboardId: 1,
    name: 'string',
    order: 1,
    dashboardType: GCDashboards.DashboardTypes.CUSTOM,
    createdDate: 'string',
    updatedDate: 'string',
    createdBy: 'string',
    updatedBy: 'string',
    isDashboardOwner: true,
    numberOfUsersSharedWith: 1,
    isActive: true,
    createdByFirstName: 'string',
    createdByLastName: 'string',
    isHidden: true,
    isStandardDashboardPublished: true
  };
  clientSettings: ClientSettings = {
    clientId: 1,
    paymentDesignations: true,
    specialHandling: true,
    allowExpeditedPayments: true,
    hasInternational: true,
    hasSSO: true,
    country: 'US',
    defaultTimezone: null,
    allowBudgetOverages: true,
    apEnabled: true,
    canConfigureWebservices: true,
    isTestClient: true,
    reserveFunds: true,
    allowExternalApis: true,
    isRootClient: false,
    allowExternalEmails: false,
    clientProcessingType: ProcessingTypes.Client,
    applicantTypesSupported: APIAdminClient.SupportedApplicantTypes.EmployeesAndNonprofits
  };
  basicForm: BasicForm = {
    availableForTranslation: false,
    defaultLanguageId: 'es-MX',
    revisions: [],
    name: 'z2.10',
    description: 'Test Form',
    createdDate: '2018-12-14T18:06:44.6263801Z',
    updatedDate: '2018-12-14T18:22:30.3521994Z',
    revisionId: 2,
    revisionVersion: 1,
    formId: 4,
    formType: FormTypes.REQUEST,
    canModify: false,
    canBeRemoved: true,
    programIds: [],
    standardComponentIsPublished: false,
    isDraft: false
  };

  @BeforeEach()
  async mock (service: StandardProductConfigurationService) {
    service['clientSettingsService']['set']('clientSettings', this.clientSettings);
    service['standardProductResources']['getStandardFormTemplates'] = async () => {
      return this.standardFormTemplates;
    };
    service['standardProductResources']['getStandardReportTemplates'] = async () => {
      return this.standardReportTemplates;
    };
    service['standardProductResources']['retrieveStagedFileInfo'] = async () => {
      return {
        fileIsStaged: this.fileIsStaged
      };
    };
    service['standardProductResources']['getFieldsForPlatformTool'] = async () => {
      return {
        recordCount: 0,
        records: []
      };
    };
    service['standardProductResources']['getFormsForPlatformTool'] = async () => {
      return {
        recordCount: 0,
        records: []
      };
    };
    service['standardProductResources']['getReportsForPlatformTool'] = async () => {
      return {
        recordCount: 1,
        records: [{
          id: 2,
          name: '',
          description: '',
          formAudience: FormAudience.APPLICANT,
          lastSyncDate: '',
          updatedDate: '',
          reportType: AdHocReportingAPI.AdHocReportType.AdHoc,
          modalType: AdHocReportingAPI.AdHocReportModelType.Application,
          chartType: AdHocReportingAPI.ChartType.Table
        }]
      };
    };
    service['standardProductResources']['getDashboardsForPlatformTool'] = async () => {
      return {
        recordCount: 0,
        records: []
      };
    };
    service['policyService'].dataExport.canManageAdHoc = () => {
      return true;
    };
    service['formService'].setFormTypes();
    service['formService'].setForms([], this.forms);
    service['adHocService']['reportingResources'].getReports = async () => {
      return [{
        id: 5,
        name: 'Application Report',
        description: '',
        isUserOwnedReport: true,
        reportModelType: AdHocReportingAPI.AdHocReportModelType.Application,
        reportType: AdHocReportingAPI.AdHocReportType.AdHoc,
        userSavedReportColumns: [],
        createdDate: '',
        updatedDate: '',
        clientId: 5,
        forms: [],
        reportOwnerClientUserId: 3,
        accessType: AdHocReportingAPI.AdHocReportAccessType.MANAGE,
        primaryFormId: null,
        advancedFilterColumns: [],
        useLogicalOperatorAnd: true,
        referenceFieldTableId: null,
        referenceFieldTableKey: '',
        standardComponentIsPublished: false
      }];
    };
    await service['adHocService'].fetchReports();
  }

  @TestCase('should be able to adapt pagination options')
  adaptPaginationOptions (service: StandardProductConfigurationService) {
    const onlyReturnStaged = true;
    const options: PaginationOptions<StandardProductFormField|StandardProductForm> = {
      rowsPerPage: 10,
      pageNumber: 1,
      sortColumns: [],
      filterColumns: [{
        columnName: 'returnOnlyRecordsToBeStaged',
        filters: [{
          filterType: 'eq',
          filterValue: onlyReturnStaged
        }]
      }],
      retrieveTotalRecordCount: true,
      returnAll: false
    };
    const {
      paginationOptions,
      returnOnlyRecordsToBeStaged
    } = service.adaptPaginationOptions(options, false);
    expect(paginationOptions.filterColumns.length).to.be.equal(0);
    expect(returnOnlyRecordsToBeStaged).to.be.equal(onlyReturnStaged);
  }

  @TestCase('should be able to get toastr for publish / unpublish success')
  getToastrForPublishUnpublishPass (service: StandardProductConfigurationService) {
    let message = service.getToastrForPublishUnpublish(
      true,
      true,
      StandardProductTypes.FORMS
    );
    let expectedMessage = 'Successfully published the form(s)';
    expect(message).to.be.equal(expectedMessage);
    message = service.getToastrForPublishUnpublish(
      true,
      false,
      StandardProductTypes.FORM_FIELDS
    );
    expectedMessage = 'Successfully unpublished the form field';
    expect(message).to.be.equal(expectedMessage);
  }


  @TestCase('should be able to get toastr for publish / unpublish success fail')
  getToastrForPublishUnpublishFail (service: StandardProductConfigurationService) {
    let message = service.getToastrForPublishUnpublish(
      false,
      true,
      StandardProductTypes.FORMS
    );
    let expectedMessage = 'There was an error publishing the form(s)';
    expect(message).to.be.equal(expectedMessage);
    message = service.getToastrForPublishUnpublish(
      false,
      false,
      StandardProductTypes.FORM_FIELDS
    );
    expectedMessage = 'There was an error unpublishing the field';
    expect(message).to.be.equal(expectedMessage);
  }

  @TestCase('should be able to set standard form templates')
  async setStandardFormTemplates (service: StandardProductConfigurationService) {
    await service.setStandardFormTemplates();
    expect(service.standardFormTemplates).to.deep.equal(this.standardFormTemplates);
  }

  @TestCase('should be able to set standard report templates')
  async setStandardReportTemplates (service: StandardProductConfigurationService) {
    await service.setStandardReportTemplates();
    expect(service.standardReportTemplates).to.deep.equal(this.standardReportTemplates);
  }

  @TestCase('should be able to set form templates by audience for manager')
  async setFormOptionsByAudienceManager (service: StandardProductConfigurationService) {
    await service.setStandardFormTemplates();
    service.setFormOptionsByAudience(FormAudience.MANAGER);
    const managerOptions = service.standardManagerFormOptions;
    const hasManagerOption = managerOptions.some((opt) => {
      return opt.value === this.managerFormId;
    });
    expect(hasManagerOption).to.be.true;
  }

  @TestCase('should be able to set form templates by audience for applicant')
  async setFormOptionsByAudienceApplicant (service: StandardProductConfigurationService) {
    await service.setStandardFormTemplates();
    service.setFormOptionsByAudience(FormAudience.APPLICANT);
    const applicantOptions = service.standardApplicantFormOptions;
    const hasApplicantOption = applicantOptions.some((opt) => {
      return opt.value === this.applicantFormId;
    });
    expect(hasApplicantOption).to.be.true;
  }

  @TestCase('should be able to get forms and standard templates')
  async getFormsAndStandardTemplates (service: StandardProductConfigurationService) {
    await service.setStandardFormTemplates();
    const results = service.getFormsAndStandardTemplates();
    const resultsIncludesStandardTemplates = results.some((form) => {
      return form.isStandardFormTemplate;
    });
    expect(resultsIncludesStandardTemplates).to.be.true;
  }

  @TestCase('should be able to get reports and standard templates')
  async getReportsAndStandardTemplates (service: StandardProductConfigurationService) {
    await service.setStandardReportTemplates();
    const results = service.getReportsAndStandardTemplates();
    const resultsIncludesStandardTemplates = results.some((form) => {
      return form.isStandardReportTemplate;
    });
    expect(resultsIncludesStandardTemplates).to.be.true;
  }

  @TestCase('should be able to get form template options by audience')
  async getFormTemplateOptionsByAudience (service: StandardProductConfigurationService) {
    await service.setStandardFormTemplates();
    const applicantOptions = service.getFormTemplateOptionsByAudience(
      FormAudience.APPLICANT
    );
    const hasOnlyApplicantOptions = applicantOptions.every((opt) => {
      return opt.value.formId !== this.managerFormId;
    });
    expect(hasOnlyApplicantOptions).to.be.true;

    const managerOptions = service.getFormTemplateOptionsByAudience(
      FormAudience.MANAGER
    );
    const hasOnlyManagerOptions = managerOptions.every((opt) => {
      return opt.value.formId !== this.applicantFormId;
    });
    expect(hasOnlyManagerOptions).to.be.true;
  }


  @TestCase('should be able to get form type from form ID')
  async getFormTypeFromFormId (service: StandardProductConfigurationService) {
    await service.setStandardFormTemplates();
    let formType = service.getFormTypeFromFormId(this.applicantFormId);
    expect(formType).to.be.equal(FormTypes.REQUEST);
    formType = service.getFormTypeFromFormId(this.managerFormId);
    expect(formType).to.be.equal(FormTypes.REVIEW);
  }

  @TestCase('should be able to get do not allow update current revision')
  getDoNotAllowUpdateCurrentRevision (service: StandardProductConfigurationService) {
    let doNotAllowUpdate = service.getDoNotAllowUpdateCurrentRevision(
      this.applicantFormId,
      true
    );
    // This form is already published so no updates allowed
    expect(doNotAllowUpdate).to.be.true;
    doNotAllowUpdate = service.getDoNotAllowUpdateCurrentRevision(
      this.applicantFormId,
      false
    );
    // This form is not published so updates are allowed
    expect(doNotAllowUpdate).to.be.false;
  }

  @TestCase('should be able to retrieve staged file info')
  async retrieveStagedFileInfo (service: StandardProductConfigurationService) {
    await service.retrieveStagedFileInfo();
    expect(service.fileIsStaged).to.be.equal(this.fileIsStaged);
  }

  @TestCase('should be able to set records available to push')
  async setRecordsAvailableToPush (service: StandardProductConfigurationService) {
    await service.setRecordsAvailableToPush();
    expect(service.areRecordsAvailableToPush).to.be.equal(true);
    service['standardProductResources']['getReportsForPlatformTool'] = async () => {
      return {
        recordCount: 0,
        records: []
      };
    };
    await service.resetAreRecordsAvailableToPush();
    expect(service.areRecordsAvailableToPush).to.be.false;
  }

  @TestCase('should be able to get report template by id')
  getReportTemplateById (service: StandardProductConfigurationService) {
    const template = service.getReportTemplateById(this.standardReportId);
    expect(template.name).to.be.equal(this.standardReportTemplate.name);
    expect(template.userSavedReportColumns).to.deep.equal(
      this.standardReportTemplate.userSavedReportColumns
    );
  }

  @TestCase('should be able to get toastr text for publish/unpublish')
  getToastrTextForUnpublish (
    service: StandardProductConfigurationService
  ) {
    const formPublish = service.getToastrForPublishUnpublish(
      true,
      true,
      StandardProductTypes.FORMS
    );
    expect(formPublish).to.equal('Successfully published the form(s)');
    const formUnpublish = service.getToastrForPublishUnpublish(
      true,
      false,
      StandardProductTypes.FORMS
    );
    expect(formUnpublish).to.equal('Successfully unpublished the form');
    const formFieldPublish = service.getToastrForPublishUnpublish(
      true,
      true,
      StandardProductTypes.FORM_FIELDS
    );
    expect(formFieldPublish).to.equal('Successfully published the form field(s)');
    const formFieldUnpublish = service.getToastrForPublishUnpublish(
      true,
      false,
      StandardProductTypes.FORM_FIELDS
    );
    expect(formFieldUnpublish).to.equal('Successfully unpublished the form field');
    const reportPublish = service.getToastrForPublishUnpublish(
      true,
      true,
      StandardProductTypes.REPORTS
    );
    expect(reportPublish).to.equal('Successfully published the form report(s)');
    const reportUnpublish = service.getToastrForPublishUnpublish(
      true,
      false,
      StandardProductTypes.REPORTS
    );
    expect(reportUnpublish).to.equal('Successfully unpublished the form report');
    const dashboardPublish = service.getToastrForPublishUnpublish(
      true,
      true,
      StandardProductTypes.DASHBOARDS
    );
    expect(dashboardPublish).to.equal('Successfully published the dashboard');
    const dashboardUnpublish = service.getToastrForPublishUnpublish(
      true,
      false,
      StandardProductTypes.DASHBOARDS
    );
    expect(dashboardUnpublish).to.equal('Successfully unpublished the dashboard');
  }

  @TestCase('should be able to adapt standard product dashboard widgets')
  testAdaptStandardProductDashboardWidgets (
    service: StandardProductConfigurationService
  ) {
    const dashboardId = 123;
    const widgetsFromAPI: GCDashboards.WidgetConfigFromApi[] = [{
      id: 1,
      userSavedReportColumns: null,
      name: 'string',
      description: 'string',
      othersMayModify: true,
      othersMayView: true,
      reportModelType: AdHocReportingAPI.AdHocReportModelType.Application,
      reportType: AdHocReportingAPI.AdHocReportType.Chart,
      chartType: AdHocReportingAPI.ChartType.Bar,
      chartConfig: 'string',
      forms: [{
        id: 1,
        name: 'string',
        description: 'string',
        currentRevisionId: 1,
        formType: 1,
        formDefinition: 1
      }],
      chartMaxRows: 1,
      chartIncludeOtherAggregate: true,
      primaryFormId: 1,
      advancedFilterColumns: null,
      useLogicalOperatorAnd: true
    }];
    const comparisonWidgets = [{
      id: 1,
      userSavedReportColumns: null,
      name: 'string',
      description: 'string',
      othersMayModify: true,
      othersMayView: true,
      reportModelType: AdHocReportingAPI.AdHocReportModelType.Application,
      reportType: AdHocReportingAPI.AdHocReportType.Chart,
      chartType: AdHocReportingAPI.ChartType.Bar,
      chartConfig: 'string',
      formIds: [1],
      forms: [{
        id: 1,
        name: 'string',
        description: 'string',
        currentRevisionId: 1,
        formType: 1,
        formDefinition: 1
      }],
      reportUsers: [],
      referenceFieldTableId: null,
      chartMaxRows: 1,
      chartIncludeOtherAggregate: true,
      primaryFormId: 1,
      advancedFilterColumns: null,
      useLogicalOperatorAnd: true
    }] as any;
    const adaptedWidgets = service.adaptStandardProductDashboardWidgets(
      widgetsFromAPI,
      dashboardId
    );
    expect(adaptedWidgets).to.deep.equal([{
      widget: comparisonWidgets[0],
      dashboardId
    }]);
  }

  @TestCase('should be able to set standard product dashboard templates')
  async testSettingStandardProductDashboardTemplates (
    service: StandardProductConfigurationService
  ) {
    service['standardProductResources']['getStandardProductDashboards'] = async () => {
      return [
        this.exampleStandardProductDB
      ];
    };
    service['policyService']['dataExport']['canManageDashboards'] = () => {
      return true;
    };
    await service.setStandardProductDashboardTemplates();
    expect(service.standardProductDashboards).to.deep.equal([this.exampleStandardProductDB]);
    const everyStandardProductDashboardIsMarkedAsPublished = service.standardProductDashboards.every((dashboard) => {
      return dashboard.isStandardDashboardPublished;
    });
    expect(everyStandardProductDashboardIsMarkedAsPublished).to.be.true;
  }

  @TestCase('should be  able to get dashboards for platform admin tool')
  async testGetDashboardsForPlatformTool (
    service: StandardProductConfigurationService
  ) {
    service['standardProductResources']['getDashboardsForPlatformTool'] = async () => {
      return null;
    };
    const options: PaginationOptions<any> = {
      rowsPerPage: 5,
      pageNumber: 1,
      sortColumns: [{
        columnName: 'returnOnlyRecordsToBeStaged',
        sortAscending: true
      }],
      filterColumns: [],
      orFilterColumns: [],
      retrieveTotalRecordCount: true,
      returnAll: false
    };
    const resourceMethodSpy = spy.on(service['standardProductResources'], 'getDashboardsForPlatformTool', (a) => a);
    await service.getDashboardsForPlatformTool(
      options,
      true
    );
    expect(resourceMethodSpy).to.have.been.called();
  }

  @TestCase('should be able to get show remove form - root zone')
  showRemoveFormRoot (service: StandardProductConfigurationService) {
    service['clientSettingsService']['set']('clientSettings', {
      ...this.clientSettings,
      isRootClient: true
    });
    const form = {
      ...this.basicForm,
      standardComponentIsPublished: false
    };
    let showRemoveForm = service.showRemoveForm(form);
    // From the root zone, can only remove if not published
    expect(showRemoveForm).to.be.true;
    form.standardComponentIsPublished = true;
    showRemoveForm = service.showRemoveForm(form);
    expect(showRemoveForm).to.be.false;
  }

  @TestCase('should be able to get show remove form - not root zone')
  showRemoveFormNotRoot (service: StandardProductConfigurationService) {
    service['clientSettingsService']['set']('clientSettings', {
      ...this.clientSettings,
      isRootClient: false
    });
    const form = {
      ...this.basicForm,
      canBeRemoved: true
    };
    let showRemoveForm = service.showRemoveForm(form);
    // Can be removed if is draft or canBeRemoved is true
    expect(showRemoveForm).to.be.true;
    form.isDraft = true;
    showRemoveForm = service.showRemoveForm(form);
    expect(showRemoveForm).to.be.true;
    form.isDraft = false;
    form.canBeRemoved = false;
    showRemoveForm = service.showRemoveForm(form);
    expect(showRemoveForm).to.be.false;
  }

  @TestCase('should be able to get show remove revision - root zone')
  showRemoveRevisionRoot (service: StandardProductConfigurationService) {
    service['clientSettingsService']['set']('clientSettings', {
      ...this.clientSettings,
      isRootClient: true
    });
    const form = {
      ...this.basicForm,
      standardComponentIsPublished: true
    };
    const showRemoveRevision = service.showRemoveRevision(form);
    // From the root zone, if it's published it can't be removed
    expect(showRemoveRevision).to.be.false;
  }

  @TestCase('should be able to get show remove form - no revisions')
  showRemoveRevisionNoRevisions (service: StandardProductConfigurationService) {
    service['clientSettingsService']['set']('clientSettings', {
      ...this.clientSettings,
      isRootClient: false
    });
    const form = {
      ...this.basicForm,
      canBeRemoved: true
    };
    const showRemoveRevision = service.showRemoveRevision(form);
    // Don't show this option if no revisions - they can just remove the form
    expect(showRemoveRevision).to.be.false;
  }

  @TestCase('should be able to get show remove form - no revisions draft')
  showRemoveRevisionNoRevisionsDraft (service: StandardProductConfigurationService) {
    service['clientSettingsService']['set']('clientSettings', {
      ...this.clientSettings,
      isRootClient: false
    });
    const form: BasicForm = {
      ...this.basicForm,
      canBeRemoved: false,
      state: FormStates.DRAFT,
      revisions: []
    };
    // If can be removed is false, we check if it's a draft with no revisions and still allow this
    const showRemoveRevision = service.showRemoveRevision(form);
    expect(showRemoveRevision).to.be.true;
  }

  @TestCase('should be able to get show remove form - has revisions')
  showRemoveRevisionHasRevisions (service: StandardProductConfigurationService) {
    service['clientSettingsService']['set']('clientSettings', {
      ...this.clientSettings,
      isRootClient: false
    });
    const form: BasicForm = {
      ...this.basicForm,
      canBeRemoved: false,
      revisions: [{
        formId: this.basicForm.formId,
        formRevisionId: 2,
        revisionCanBeRemoved: false // this will be the result
      } as BasicRevision]
    };
    // If has revisions, we check if the revision can be removed
    let showRemoveRevision = service.showRemoveRevision(form);
    expect(showRemoveRevision).to.be.false;
    form.revisions[0].revisionCanBeRemoved = true;
    showRemoveRevision = service.showRemoveRevision(form);
    expect(showRemoveRevision).to.be.true;
  }
}
