import { Component } from '@angular/core';
import { SpinnerService } from '@core/services/spinner.service';
import { environment } from '@environment';
import { ModalFactory } from '@yourcause/common/modals';
import { FileAvailableToImportModalComponent } from '../file-available-to-import-modal/file-available-to-import-modal.component';
import { PushToProductionModalComponent } from '../push-to-production-modal/push-to-production-modal.component';
import { StandardProductConfigurationService } from '../standard-product-configuration.service';

@Component({
  selector: 'gc-standard-product-configuration-page',
  templateUrl: './standard-product-configuration-page.component.html',
  styleUrls: ['./standard-product-configuration-page.component.scss']
})
export class StandardProductConfigurationPageComponent {
  fileIsStaged = this.standardProductService.fileIsStaged;
  recordsAreAvailableToPush = this.standardProductService.areRecordsAvailableToPush;
  showPushToProduction = environment.supportsPushStandardFields;
  showFileAvailableToImport = environment.supportsImportStandardFields;

  constructor (
    private modalFactory: ModalFactory,
    private standardProductService: StandardProductConfigurationService,
    private spinnerService: SpinnerService
  ) { }

  async pushToProduction () {
    const doPush = await this.modalFactory.open(
      PushToProductionModalComponent,
      {}
    );
    if (doPush) {
      this.spinnerService.startSpinner();
      await this.standardProductService.handlePushToProduction();
      this.spinnerService.stopSpinner();
    }
  }

  async importFile () {
    const result = await this.modalFactory.open(
      FileAvailableToImportModalComponent,
      {}
    );
    if (result) {
      this.spinnerService.startSpinner();
      await this.standardProductService.handleImportFile();
      this.spinnerService.stopSpinner();
    }
  }
}
