import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { BudgetResolver } from '@features/budgets/resolvers/budget.resolver';
import { CycleMapResolver } from '@features/cycles/resolvers/cycle-map.resolver';
import { AllDistributionListsResolver } from '@features/invitations/resolvers/all-distribution-lists.resolver';
import { AllActiveManagerProgramsResolver } from '@features/programs/resolvers/all-active-manager-programs.resolver';
import { EmailResolver } from '@features/system-emails/resolvers/email.resolver';
import { WorkflowResolver } from '@features/workflow/resolvers/workflow.resolver';
import { ProgramSetupPageComponent } from './program-setup-page/program-setup-page.component';
import { ProgramSetupModule } from './program-setup.module';

@NgModule({
  imports: [
    ProgramSetupModule,
    RouterModule.forChild([{
      path: '',
      data: { pageName: 'Program Setup' },
      component: ProgramSetupPageComponent
    }, {
      path: 'forms',
      loadChildren: () => import('@features/configure-forms/configure-forms.module').then(m => m.ConfigureFormsModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Custom Forms'
      }
    }, {
      path: 'programs',
      loadChildren: () => import('@features/programs/programs-routing.module').then(m => m.ProgramsRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Grant Programs'
      }
    }, {
      path: 'nomination-programs',
      loadChildren: () => import('@features/nominations/nominations-routing.module').then(m => m.NominationsRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Nomination Programs'
      }
    }, {
      path: 'budgets',
      loadChildren: () => import('@features/budgets/budgets-routing.module').then(m => m.BudgetsRoutingModule),
      resolve: {
        budgets: BudgetResolver
      },
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Budgets'
      }
    }, {
      path: 'workflows',
      loadChildren: () => import('@features/workflow/workflow-routing.module').then(m => m.WorkflowRoutingModule),
      resolve: {
        workflows: WorkflowResolver
      },
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Workflows'
      }
    }, {
      path: 'workflow-automation',
      loadChildren: () => import('@features/workflow-automation/workflow-automation-routing.module').then(m => m.WorkflowAutomationRoutingModule),
      data: {
        breadcrumbLabel: 'Workflow Level Automation',
        breadcrumb: true
      }
    }, {
      path: 'funding-sources',
      loadChildren: () => import('@features/funding-sources/funding-source-routing.module')
        .then(m => m.FundingSourceRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Funding Sources'
      }
    }, {
      path: 'invitations',
      loadChildren: () => import('@features/invitations/invitation-routing.module').then(m => m.InvitationRoutingModule),
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Invitations'
      },
      resolve: {
        emails: EmailResolver,
        cycles: CycleMapResolver,
        distributionLists: AllDistributionListsResolver,
        progs: AllActiveManagerProgramsResolver
      }
    }])
  ]
})
export class ProgramSetupRoutingModule { }


