import { Component, OnInit } from '@angular/core';
import { ProgramAutomationService } from '@features/programs/program-automation/program-automation.service';
import { TopLevelFilter } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';

@Component({
  selector: 'gc-translate-automation-page',
  templateUrl: './translate-automation-page.component.html',
  styleUrls: ['./translate-automation-page.component.scss']
})
export class TranslateAutomationPageComponent implements OnInit {
  topLevelFilters: TopLevelFilter[];
  automationOptions = this.programAutomationService.activeProgramAutomationOptions;
  richTextMap: Record<string, string> = {};

  constructor (
    private i18n: I18nService,
    private programAutomationService: ProgramAutomationService
  ) { }

  get isNomination () {
    return location.pathname.includes('nomination');
  }

  ngOnInit () {
    this.automationOptions.forEach((option) => {
      this.richTextMap[option.value] = option.label;
    });
    const automationFilter = new TopLevelFilter(
      'checkboxDropdown',
      'grantProgramRoutingDetailsId',
      [],
      this.i18n.translate(
        'GLOBAL:textSearchForAnAutomationRuleSet',
        {},
        'Search for an automation rule set'
      ),
      {
        selectOptions: this.automationOptions,
        filterObjectName: this.i18n.translate(
          'common:lblAutomationRuleSet',
          {},
          'Automation rule set'
        ).toLowerCase(),
        filterObjectNamePlural: this.i18n.translate(
          'common:textAutomationRuleSets',
          {},
          'Automation rule sets'
        ).toLowerCase()
      }
    );
    this.topLevelFilters = [
      new TopLevelFilter(
        'text',
        'defaultTranslationText',
        '',
        this.i18n.translate(
          'FORMS:textSearchTranslations',
          {},
          'Search translations'
        )
      )
    ];
    if (this.automationOptions.length > 0) {
      this.topLevelFilters.push(automationFilter);
    }
  }
}
