import { Component, Input, OnInit } from '@angular/core';
import { Validators } from '@angular/forms';
import { TypeSafeFormBuilder, TypeSafeFormGroup } from '@yourcause/common';
import { AnalyticsService, EventType } from '@yourcause/common/analytics';
import { YCModalComponent } from '@yourcause/common/modals';

@Component({
  selector: 'gc-create-edit-category-modal',
  templateUrl: './create-edit-category-modal.component.html',
  styleUrls: ['./create-edit-category-modal.component.scss']
})
export class CreateEditCategoryModalComponent extends YCModalComponent<string> implements OnInit {
  @Input() categoryName: string;

  formGroup: TypeSafeFormGroup<{ categoryName: string }>;

  constructor (
    private formBuilder: TypeSafeFormBuilder,
    private analyticsService: AnalyticsService
  ) {
    super();
  }

  ngOnInit () {
    this.formGroup = this.formBuilder.group<{ categoryName: string }>({
      categoryName: [
        this.categoryName || '',
        Validators.required
      ]
    });
  }

  async onSave () {
    this.closeModal.emit(this.formGroup.value.categoryName);
    this.analyticsService.emitEvent({
      eventName: 'Save category',
      eventType: EventType.Click,
      extras: null
    });
  }
}
