import { Component, Input, OnInit } from '@angular/core';
import { SpinnerService } from '@core/services/spinner.service';
import { ReferenceFieldAPI } from '@core/typings/api/reference-fields.typing';
import { FormAudience } from '@features/configure-forms/form.typing';
import { CustomDataTablesService } from '@features/custom-data-tables/custom-data-table.service';
import { BulkAction, TopLevelFilter } from '@yourcause/common';
import { AnalyticsService, EventType } from '@yourcause/common/analytics';
import { I18nService } from '@yourcause/common/i18n';
import { YCModalComponent } from '@yourcause/common/modals';
import { ReferenceFieldsService } from '../services/reference-fields.service';

@Component({
  selector: 'gc-merge-with-standard-field-modal',
  templateUrl: './merge-with-standard-field-modal.component.html',
  styleUrls: ['./merge-with-standard-field-modal.component.scss']
})
export class MergeWithStandardFieldModalComponent extends YCModalComponent<number[]> implements OnInit {
  @Input() field: ReferenceFieldAPI.ReferenceFieldDisplayModel;

  rows: ReferenceFieldAPI.ReferenceFieldDisplayModel[] = [];
  alertMessage: string;
  conflictAlert: string;
  fieldTypeMap = this.referenceFieldService.getReferenceFieldTypeToLabelMap();
  formattingMap = this.referenceFieldService.getFormattingMap();
  FormAudiences = FormAudience;
  guidToNameMap = this.customDataTableService.guidToNameMap;
  categoryNameMap = this.referenceFieldService.categoryNameMap;
  topLevelFilters: TopLevelFilter[];
  bulkActions: BulkAction<ReferenceFieldAPI.ReferenceFieldDisplayModel>[] = [{
    label: '',
    disabled: () => false,
    exec: () => true
  }];
  bulkSelected: ReferenceFieldAPI.ReferenceFieldDisplayModel[] = [];

  constructor (
    private i18n: I18nService,
    private referenceFieldService: ReferenceFieldsService,
    private customDataTableService: CustomDataTablesService,
    private spinnerService: SpinnerService,
    private analyticsService: AnalyticsService
  ) {
    super();
  }

  ngOnInit () {
    this.setRows();
    this.setTopLevelFilters();
    this.alertMessage = this.i18n.translate(
      'common:textMergeWithStandardProductFieldAlert',
      {
        fieldName: this.field.name
      },
      `Select the form fields you want to merge with the Standard Product Field: __fieldName__. The available fields, below, are eligible to merge based on the Standard Product Field's configuration. After merging, all forms, ad hoc reports, and dashboards using the selected form fields will use the Standard Product Field.`
    );
  }

  setRows () {
    this.rows = this.referenceFieldService.getFieldsAvailableToMergeWithStandard(
      this.field.referenceFieldId
    );
  }

  setTopLevelFilters () {
    this.topLevelFilters = [
      this.referenceFieldService.getFilterForNameOrKey(),
      this.referenceFieldService.getCategoryDropdownFilter()
    ];
  }

  bulkSelectedChanged (fields: ReferenceFieldAPI.ReferenceFieldDisplayModel[]) {
    this.bulkSelected = fields;
  }

  async onMerge () {
    this.spinnerService.startSpinner();
    const response = await this.referenceFieldService.checkMergeForConflicts(
      this.field.referenceFieldId,
      this.bulkSelected.map((field) => field.referenceFieldId)
    );
    this.spinnerService.stopSpinner();
    this.conflictAlert = '';
    if (response.hasConflicts) {
      this.conflictAlert = this.i18n.translate(
        'common:textCantMergeWithStandardConflict',
        {},
        `These fields both contain information related to the same application(s) so can not be merged.`
      );
    } else if (response.hasOtherError) {
      this.closeModal.emit();
    } else {
      this.closeModal.emit(this.bulkSelected.map((field) => {
        return field.referenceFieldId;
      }));
    }
    this.analyticsService.emitEvent({
      eventName: 'Merge with standard field submit',
      eventType: EventType.Click,
      extras: null
    });
  }

}
