import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { FormsResolver } from '@features/configure-forms/resolvers/forms.resolver';
import { AllUsersResolver } from '@features/users/resolvers/all-users.resolver';
import { AdHocReportingModule } from './ad-hoc-reporting.module';
import { AdHocReportListViewPageComponent } from './pages/ad-hoc-report-list-view-page/ad-hoc-report-list-view-page.component';
import { DataFeedsListViewPageComponent } from './pages/data-feeds-list-view-page/data-feeds-list-view-page.component';
import { ManageAdHocReportPageComponent } from './pages/manage-ad-hoc-report-page/manage-ad-hoc-report-page.component';
import { ReportingWrapperPageComponent } from './pages/reporting-wrapper-page/reporting-wrapper-page.component';
import { AdHocReportingDetailResolver } from './resolvers/ad-hoc-reporting-detail.resolver';
import { AdHocReportingListResolver } from './resolvers/ad-hoc-reporting-list.resolver';
import { DataFeedRowsResolver } from './resolvers/data-feed-rows.resolver';

@NgModule({
  imports: [
    AdHocReportingModule,
    RouterModule.forChild([{
      path: '',
      component: ReportingWrapperPageComponent,
      resolve: {
        list: AdHocReportingListResolver,
        allUsers: AllUsersResolver,
        forms: FormsResolver
      },
      data: {
        breadcrumb: true,
        breadcrumbLabel: 'Ad Hoc and Data Feeds'
      },
      children: [{
        path: 'ad-hoc',
        data: {
          breadcrumb: true,
          breadcrumbLabel: 'Ad Hoc and Data Feeds',
          pageName: 'Ad Hoc Tab'
        },
        component: AdHocReportListViewPageComponent
      }, {
        path: 'data-feeds',
        data: {
          breadcrumb: true,
          breadcrumbLabel: 'Ad Hoc and Data Feeds',
          pageName: 'Data Feeds Tab'
        },
        resolve: {
          rows: DataFeedRowsResolver
        },
        component: DataFeedsListViewPageComponent
      }, {
        path: '',
        pathMatch: 'full',
        redirectTo: 'ad-hoc'
      }]
    }, {
      path: 'ad-hoc-edit/:id',
      component: ManageAdHocReportPageComponent,
      resolve: {
        detail: AdHocReportingDetailResolver
      },
      data: {
        breadcrumbLabel: 'Manage Report',
        breadcrumb: true,
        isReporting: true,
        isViewMode: false,
        pageName: 'Editing Ad Hoc Report'
      }
    }, {
      path: 'ad-hoc-view/:id',
      component: ManageAdHocReportPageComponent,
      resolve: {
        detail: AdHocReportingDetailResolver
      },
      data: {
        breadcrumbLabel: 'Manage Report',
        breadcrumb: true,
        isReporting: true,
        isViewMode: true,
        pageName: 'Viewing Ad Hoc Report'
      }
    }])
  ]
})
export class AdHocReportingRoutingModule { }
