import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { AdvancedFilterGroup, Columns, SwitchState } from '@yourcause/common';
import { AnalyticsService, EventType } from '@yourcause/common/analytics';
import { YCModalComponent } from '@yourcause/common/modals';

@Component({
  selector: 'gc-advanced-filters-modal',
  templateUrl: './advanced-filters-modal.component.html',
  styleUrls: ['./advanced-filters-modal.component.scss']
})
export class AdvancedFiltersModalComponent extends YCModalComponent<{
  groups: AdvancedFilterGroup[];
  useAnd: SwitchState;
}> implements OnInit {
  @Input() groups: AdvancedFilterGroup[] = [];
  @Input() globalAnd: SwitchState = SwitchState.Toggled;
  @Input() columns: Columns;
  @Output() globalAndChange = new EventEmitter<SwitchState>();
  @Output() groupsChange = new EventEmitter<AdvancedFilterGroup[]>();
  @Output() validityChange = new EventEmitter<boolean>();

  currentGroups: AdvancedFilterGroup[] = [];
  valid = true;

  constructor (
    private analyticsService: AnalyticsService
  ) {
    super();
  }

  ngOnInit () {
    this.currentGroups = this.groups;
  }

  submit () {
    this.closeModal.emit({
      groups: this.currentGroups,
      useAnd: this.globalAnd
    });
    this.analyticsService.emitEvent({
      eventName: 'Save advanced filters',
      eventType: EventType.Click,
      extras: null
    });
  }

  handleValidityChange (valid: boolean) {
    this.valid = valid;
  }

  handleGlobalAndChange (newState: SwitchState) {
    this.globalAnd = newState;
  }

  handleGroupChange (
    newGroups: AdvancedFilterGroup[]
  ) {
    this.currentGroups = newGroups;
  }

  clearFilters () {
    this.currentGroups = [];
    this.submit();
    this.analyticsService.emitEvent({
      eventName: 'Clear advanced filters',
      eventType: EventType.Click,
      extras: null
    });
  }
}
