import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, Resolve } from '@angular/router';
import { AdHocReportingAPI } from '@core/typings/api/ad-hoc-reporting.typing';
import { ClientSettingsService } from '@features/client-settings/client-settings.service';
import { StandardProductConfigurationService } from '@features/platform-admin/standard-product-configuration/standard-product-configuration.service';
import { AdHocReportingService } from '../services/ad-hoc-reporting.service';

@Injectable({ providedIn: 'root' })
export class AdHocReportingDetailResolver implements Resolve<void> {

  constructor (
    private adHocReportingService: AdHocReportingService,
    private standardProductService: StandardProductConfigurationService,
    private clientSettingsService: ClientSettingsService
  ) { }

  async resolve (next: ActivatedRouteSnapshot) {
    const reportId = next.params.id;
    const isRootZone = this.clientSettingsService.clientSettings.isRootClient;
    let template: AdHocReportingAPI.UserSavedReportForUi;
    if (!isRootZone) {
      template = this.standardProductService.getReportTemplateById(
        +reportId
      );
    }
    if (template) {
      await this.standardProductService.prepareCdtOptionsForReportTemplate(
        template
      );
    }

    await this.adHocReportingService.prepareAdHocReportDetail(
      reportId,
      template
    );
  }
}
