import { Component, EventEmitter, Input, OnChanges, Output } from '@angular/core';
import { FormArray } from '@angular/forms';
import { Automation } from '@core/typings/ui/automation.typing';
import { TypeaheadSelectOption, TypeSafeFormBuilder, TypeSafeFormGroup } from '@yourcause/common';
import { RuleGroup } from '../automation-rule/automation-rule.component';
import { RuleAutomationService } from '../rule-automation.service';

@Component({
  selector: 'gc-automation-criteria-value',
  templateUrl: './automation-criteria-value.component.html',
  styleUrls: ['./automation-criteria-value.component.scss']
})
export class AutomationCriteriaValueComponent implements OnChanges {
  private _valuesHidden: boolean;

  @Input()
  public get valuesHidden (): boolean {
    return this._valuesHidden;
  }
  public set valuesHidden (value: boolean) {
    this._valuesHidden = value;
    if (this.hasInitialized) {
      this.ngOnChanges();
    }
  }
  @Input() criteriaFormGroup: TypeSafeFormGroup<RuleGroup>;
  @Input() currentComparison: Automation.Comparisons;
  @Input() columnComparisonOptions: TypeaheadSelectOption<string>[] = [];
  @Input() rule: Automation.CriteriaFormState;
  @Input() isBetween: boolean;
  @Output() removeRule = new EventEmitter();

  formArray: FormArray;
  Comparisons = Automation.Comparisons;
  Majorities = Automation.MajorityOptions;
  negated: boolean;
  firstIndex: number;
  totalIndex: number;
  hasInitialized = false;
  fileUploadedOptions = this.ruleAutomationService.getFileUploadOptions();
  selectedOptions = this.ruleAutomationService.getSelectedOptions();
  numberIsOptions = this.ruleAutomationService.getNumberIsOptions();
  comparisonNegateOptions = this.ruleAutomationService.getComparisonNegateOptions();
  majorityOptions = this.ruleAutomationService.getMajorityOptions();
  dateOptions = this.ruleAutomationService.getDateOptions();

  constructor (
    private formBuilder: TypeSafeFormBuilder,
    private ruleAutomationService: RuleAutomationService
  ) { }

  get showOrButton () {
    return this.totalIndex > 0 &&
      (
        this.currentComparison === Automation.Comparisons.TextIs ||
        this.currentComparison === Automation.Comparisons.SelectIs
      );
  }

  ngOnChanges () {
    this.formArray = this.criteriaFormGroup.get('values') as FormArray;
    this.negated = this.formArray.at(0).value.comparisonNegated;
    if (!this.valuesHidden) {
      this.flushRemovedValues();
    }
    this.setIndices();
    this.hasInitialized = true;
  }

  negateChange (val: boolean) {
    this.negated = val;
  }

  addValue () {
    const newValue: Automation.CriteriaValueFormState = {
      comparisonGreaterThan: null,
      comparisonType: Automation.CriteriaComparisonTypes.Default,
      comparisonLessThan: null,
      comparisonNegated: false,
      comparisonValue: null,
      isRemoved: false,
      greaterThanId: null,
      lessThanId: null,
      valueId: null
    };
    this.rule.values = [
      ...(this.rule.values || []),
      newValue
    ];

    this.firstIndex = this.rule.values.findIndex(val => !val.isRemoved);

    this.formArray.push(this.formBuilder.group(newValue));

    this.flushRemovedValues();
    this.setIndices();
  }

  removeValue (index: number) {
    this.rule.values = this.rule.values.map((v, vIndex) => ({
      ...v,
      isRemoved: vIndex === index ? true : v.isRemoved
    }));
    this.formArray.at(index).get('isRemoved').setValue(true);
    this.flushRemovedValues();
    this.setIndices();

    if (this.totalIndex === 0) {
      this.removeRule.emit();
    }
  }

  flushRemovedValues () {
    this.rule.values = this.rule.values.filter((value, index) => {
      if (!value.isRemoved) {
        return true;
      }

      const shouldPurge = value.isRemoved && !(value.valueId || value.lessThanId || value.greaterThanId);

      if (shouldPurge) {
        this.formArray.removeAt(index);
      }

      return !shouldPurge;
    });
  }

  setIndices () {
    this.firstIndex = this.rule.values.findIndex(val => !val.isRemoved);
    this.totalIndex = this.rule.values.reduce((acc, value) => !value.isRemoved ? acc + 1 : acc, 0);
  }
}
