import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { AutomationAPI } from '@core/typings/api/automation.typing';
import { ReferenceFieldAPI } from '@core/typings/api/reference-fields.typing';
import { ApplicationStatuses } from '@core/typings/status.typing';
import { Automation } from '@core/typings/ui/automation.typing';
import { ReferenceFieldsUI } from '@core/typings/ui/reference-fields.typing';
import { FormAudience } from '@features/configure-forms/form.typing';
import { EmployeeSSOFieldFromApi, EmployeeSSOFields } from '@features/employee-sso-fields/employee-sso-fields.typing';
import { ArrayHelpersService } from '@yourcause/common';
import { I18nService } from '@yourcause/common/i18n';
import { MockI18nService } from '@yourcause/common/testing';
import { BeforeEach, It, Spec } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import moment from 'moment';
import { RuleAutomationService } from './rule-automation.service';
const aggregateKey = 'number';
const allReferenceFields: ReferenceFieldAPI.ReferenceFieldDisplayModel[] = [{
  aggregateType: null,
  referenceFieldId: 1,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: '',
  name: 'Text field',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.TextField,
  key: 'textfield',
  supportsMultiple: false,
  categoryId: 1,
  formAudience: FormAudience.APPLICANT,
  parentReferenceFieldId: null,
  isEncrypted: false,
  isSingleResponse: true,
  tableAllowsImport: false,
  isMasked: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}, {
  aggregateType: null,
  referenceFieldId: 1,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: 'customDataTableGuid',
  name: 'Select Boxes Alternate',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.SelectBoxes,
  key: 'selectBoxesAlternate',
  supportsMultiple: false,
  categoryId: 1,
  formAudience: FormAudience.APPLICANT,
  parentReferenceFieldId: null,
  isEncrypted: false,
  isSingleResponse: true,
  tableAllowsImport: false,
  isMasked: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}, {
  aggregateType: null,
  referenceFieldId: 2,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: 'customDataTableGuid',
  name: 'Select Boxes Alternate Alternate',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.SelectBoxes,
  key: 'selectBoxesAlternateAlternate',
  supportsMultiple: false,
  categoryId: 1,
  formAudience: FormAudience.APPLICANT,
  parentReferenceFieldId: null,
  isEncrypted: false,
  isSingleResponse: false,
  isMasked: false,
  tableAllowsImport: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}, {
  aggregateType: null,
  referenceFieldId: 3,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: '',
  name: 'Checkbox',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.Checkbox,
  key: 'checkbox',
  supportsMultiple: false,
  categoryId: 1,
  formAudience: FormAudience.APPLICANT,
  parentReferenceFieldId: null,
  isEncrypted: false,
  isSingleResponse: true,
  isMasked: false,
  tableAllowsImport: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}, {
  aggregateType: null,
  referenceFieldId: 4,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: 'customDataTableGuid',
  name: 'Picklist',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.CustomDataTable,
  key: 'picklist',
  supportsMultiple: false,
  categoryId: 1,
  formAudience: FormAudience.APPLICANT,
  parentReferenceFieldId: null,
  isEncrypted: false,
  tableAllowsImport: false,
  isSingleResponse: true,
  isMasked: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}, {
  aggregateType: ReferenceFieldAPI.ReferenceFieldAggregateType.Sum,
  referenceFieldId: 5,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: '',
  name: 'Number',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.Number,
  key: aggregateKey,
  supportsMultiple: false,
  categoryId: 3,
  formAudience: FormAudience.APPLICANT,
  parentReferenceFieldId: null,
  isEncrypted: false,
  isSingleResponse: true,
  tableAllowsImport: false,
  isMasked: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}, {
  aggregateType: null,
  referenceFieldId: 6,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: 'customDataTableGuid',
  name: 'Radio',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.Radio,
  key: 'radio',
  supportsMultiple: false,
  categoryId: 2,
  formAudience: FormAudience.MANAGER,
  parentReferenceFieldId: null,
  isEncrypted: false,
  isSingleResponse: true,
  tableAllowsImport: false,
  isMasked: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}, {
  aggregateType: null,
  referenceFieldId: 7,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: '',
  name: 'File upload',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.FileUpload,
  key: 'fileUpload',
  supportsMultiple: false,
  categoryId: 2,
  formAudience: FormAudience.MANAGER,
  parentReferenceFieldId: null,
  isEncrypted: false,
  isSingleResponse: true,
  isMasked: false,
  tableAllowsImport: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}, {
  aggregateType: null,
  referenceFieldId: 8,
  formCount: 0,
  usedOnReports: false,
  createdBy: '',
  createDate: '',
  updatedBy: '',
  updateDate: '',
  customDataTableGuid: 'customDataTableGuid',
  name: 'File upload',
  description: '',
  type: ReferenceFieldsUI.ReferenceFieldTypes.SelectBoxes,
  key: 'selectBoxes',
  supportsMultiple: false,
  categoryId: 2,
  formAudience: FormAudience.MANAGER,
  parentReferenceFieldId: null,
  isEncrypted: false,
  isSingleResponse: true,
  isMasked: false,
  tableAllowsImport: false,
  isTableField: false,
  aggregateTableReferenceFieldId: null,
  referenceFieldTableId: null,
  isStandardProductField: false,
  standardComponentIsPublished: false,
  subsetCollectionType: null
}];
const form: Automation.FormForRuleObject = {
  name: 'test',
  formDefinition: [{
    tabName: '',
    index: 0,
    logic: null,
    uniqueId: '',
    components: [
      {
        key: 'select_key',
        type: 'referenceFields-picklist',
        input: true,
        label: 'select_label'
      },
      {
        key: 'gmfile_key',
        type: 'referenceFields-fileUpload',
        input: true,
        label: 'gmfile_label'
      },
      {
        key: 'radio_key',
        type: 'referenceFields-radio',
        input: true,
        label: 'radio_label'
      },
      {
        key: 'selectboxes_key',
        type: 'referenceFields-selectBoxes',
        input: true,
        label: 'selectboxes_label'
      },
      {
        key: 'selectboxes_alternate_key',
        type: 'referenceFields-selectBoxesAlternate',
        input: true,
        label: 'selectboxes_alternate_label'
      },
      {
        key: 'selectboxes_alternate_alternate_key',
        type: 'referenceFields-selectBoxesAlternateAlternate',
        input: true,
        label: 'selectboxes_alternate_alternate_label'
      },
      {
        key: 'number_key',
        type: 'referenceFields-number',
        input: true,
        label: 'number_label'
      },
      {
        key: 'checkbox_key',
        type: 'referenceFields-checkbox',
        input: true,
        label: 'checkbox_label'
      },
      {
        key: 'text_key',
        type: 'referenceFields-textfield',
        input: true,
        label: 'text_label'
      },
      {
        key: 'should_not_exist_key',
        type: 'should_not_exist',
        input: false,
        label: 'should_not_exist_label'
      },
      {
        key: 'should_not_show_up',
        type: 'button',
        input: true,
        label: 'should_not_show_up_label'
      }
    ]
  }],
  id: 1
};
@Injectable({ providedIn: 'root' })
@DescribeAngularService(RuleAutomationService, {
  providers: [
    {
      provide: I18nService,
      useClass: MockI18nService
    }
  ],
  imports: [GCMockModule]
})
export class RuleAutomationServiceSpec
  implements Spec<RuleAutomationServiceSpec, RuleAutomationService> {
  private formState: Automation.CriteriaFormState;
  private apiRules: AutomationAPI.AutomationRuleSetExpressionBaseModel[];
  private objects: Automation.ObjectConfig[];
  employeeSsoFields: EmployeeSSOFieldFromApi[] = [{
    column: EmployeeSSOFields.Region,
    inUse: false,
    name: 'Region'
  }, {
    column: EmployeeSSOFields.Division,
    inUse: true,
    name: 'Division'
  }];

  private getFormStateForTextIs (): Automation.CriteriaFormState {
    const columns: Automation.BaseObjectColumnType[] = [
      {
        label: 'test_label',
        comparison: Automation.Comparisons.TextIs,
        key: 'test_key'
      }
    ];

    return {
      object: {
        label: 'test',
        key: 'test',
        columns
      },
      column: columns[0],
      isRemoved: false,
      values: [
        {
          comparisonGreaterThan: null,
          comparisonType: Automation.CriteriaComparisonTypes.Default,
          comparisonLessThan: null,
          comparisonValue: 'test',
          comparisonNegated: false,
          isRemoved: false,
          greaterThanId: null,
          lessThanId: null,
          valueId: 1
        }
      ]
    };
  }

  private getFormStateForNumberIs (): Automation.CriteriaFormState {
    const columns: Automation.BaseObjectColumnType[] = [
      {
        label: 'test_label',
        comparison: Automation.Comparisons.NumberIs,
        key: 'test_key'
      }
    ];

    return {
      object: {
        label: 'test',
        key: 'test',
        columns
      },
      column: columns[0],
      isRemoved: false,
      values: [
        {
          comparisonGreaterThan: null,
          comparisonType: Automation.CriteriaComparisonTypes.Default,
          comparisonLessThan: null,
          comparisonValue: '4',
          comparisonNegated: false,
          isRemoved: false,
          greaterThanId: null,
          lessThanId: null,
          valueId: 1
        }
      ]
    };
  }

  private getApiRules (): AutomationAPI.AutomationRuleSetExpressionBaseModel[] {
    return [{
      automationRuleSetExpressionId: null,
      columnName: 'test_object.test_text',
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
      comparisonValue: 'text_value',
      referenceFieldId: null
    }];
  }

  private getObjects (): Automation.ObjectConfig[] {
    return [{
      key: 'test_object',
      label: 'Test',
      columns: [{
        comparison: Automation.Comparisons.TextIs,
        key: 'test_text',
        label: 'text'
      }]
    }];
  }

  constructor (
    private arrayHelper: ArrayHelpersService
  ) { }

  @BeforeEach()
  async setUpValues (service: RuleAutomationService) {
    service['referenceFieldService'].setAllReferenceFields(
      allReferenceFields
    );
    service['referenceFieldService']['customDataTablesService']['set'](
      'customDataTableOptionsMap',
      {
        customDataTableGuid: []
      }
    );
    service['locationState']['set']('countries', []);
    service['employeeSSOFieldsService']['employeeSSOFieldsResources'].getEmployeeSSOFields = async () => {
      return this.employeeSsoFields;
    };
    await service['employeeSSOFieldsService'].setEmployeeSSOFields();
    this.formState = this.getFormStateForTextIs();
    this.apiRules = this.getApiRules();
    this.objects = this.getObjects();
  }

  @It async 'should be able to generate a rule object from a form' (
    ruleAutomationService: RuleAutomationService
  ) {
    const expectedResult: Automation.ObjectConfig = {
      label: form.name,
      key: 'referenceFields',
      columns: this.arrayHelper.sort([
        {
          label: 'select_label',
          comparison: Automation.Comparisons.SelectIs,
          key: 'picklist',
          options: []
        },
        {
          label: 'gmfile_label',
          comparison: Automation.Comparisons.IsUploaded,
          key: 'fileUpload'
        },
        {
          label: 'radio_label',
          comparison: Automation.Comparisons.SelectIs,
          key: 'radio',
          options: []
        },
        {
          label: 'selectboxes_label',
          comparison: Automation.Comparisons.SelectIs,
          key: 'selectBoxes',
          options: []
        },
        {
          label: 'selectboxes_alternate_label',
          comparison: Automation.Comparisons.SelectIs,
          key: 'selectBoxesAlternate',
          options: []
        },
        {
          label: 'number_label',
          comparison: Automation.Comparisons.NumberRange,
          key: 'number'
        },
        {
          label: 'checkbox_label',
          comparison: Automation.Comparisons.IsSelected,
          key: 'checkbox'
        },
        {
          label: 'text_label',
          comparison: Automation.Comparisons.TextIs,
          key: 'textfield'
        }
      ], 'label')
    };

    const formObject = await ruleAutomationService.formToRuleObject(form);

    expect(formObject).not.to.equal(undefined);
    expect(formObject).to.deep.equal(expectedResult);
  }

  @It async 'should not recreate the same formObject' (
    ruleAutomationService: RuleAutomationService
  ) {
    const formObject = await ruleAutomationService.formToRuleObject(form);
    const formObject2 = await ruleAutomationService.formToRuleObject(form);

    expect(formObject).to.equal(formObject2);
  }

  @It async 'should recreate the save formObject if the cache is cleared' (
    ruleAutomationService: RuleAutomationService
  ) {
    const formObject = await ruleAutomationService.formToRuleObject(form);
    ruleAutomationService.clearFormRuleObjectMap();
    const formObject2 = await ruleAutomationService.formToRuleObject(form);

    expect(formObject).not.to.equal(formObject2);
  }

  @It 'should be able to map to a string based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [apiRule] = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRule).to.deep.equal({
      automationRuleSetExpressionId: this.formState.values[0].valueId,
      isRemoved: false,
      columnName: this.formState.object.key + '.' + this.formState.column.key,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
      comparisonValue: this.formState.values[0].comparisonValue,
      referenceFieldId: null
    });
  }

  @It 'should be able to map to a NumberIs based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const formState = this.getFormStateForNumberIs();
    const [apiRule] = ruleAutomationService.getMappedCriteriaToAPIRule(
      formState
    );
    apiRule;
    expect(apiRule).to.deep.equal({
      automationRuleSetExpressionId: formState.values[0].valueId,
      isRemoved: false,
      columnName: formState.object.key + '.' + formState.column.key,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
      comparisonValue: formState.values[0].comparisonValue,
      referenceFieldId: null
    });
  }

  @It 'should be able to map to a NumberIs based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const objectsForNumber: Automation.ObjectConfig[] = [{
      ...this.objects[0],
      columns: [{
        ...this.objects[0].columns[0],
        comparison: Automation.Comparisons.NumberIs
      }]
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(
      this.apiRules,
      objectsForNumber
    );
    const expectedResult: Automation.CriteriaFormState = {
      column: objectsForNumber[0].columns[0],
      isRemoved: false,
      object: objectsForNumber[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: false,
        comparisonValue: this.apiRules[0].comparisonValue,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }

  @It 'should be able to map to a string based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: false,
        comparisonValue: this.apiRules[0].comparisonValue,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }

  @It 'should be able to map to anot string based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.values[0].comparisonNegated = true;
    const [apiRule] = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );
    expect(apiRule).to.deep.equal({
      automationRuleSetExpressionId: this.formState.values[0].valueId,
      isRemoved: false,
      columnName: this.formState.object.key + '.' + this.formState.column.key,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.NotEquals,
      comparisonValue: this.formState.values[0].comparisonValue,
      referenceFieldId: null
    });
  }

  @It 'should be able to map to a not string based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.apiRules[0].comparisonType = AutomationAPI.AutomationRuleComparisonTypes.NotEquals;
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: true,
        comparisonValue: this.apiRules[0].comparisonValue,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to a multiple string based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.values.push({
      comparisonGreaterThan: null,
      comparisonType: Automation.CriteriaComparisonTypes.Default,
      comparisonLessThan: null,
      comparisonNegated: false,
      comparisonValue: 'test2',
      isRemoved: false,
      greaterThanId: null,
      lessThanId: null,
      valueId: 2
    });
    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );
    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: this.formState.values[0].valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
        comparisonValue: this.formState.values[0].comparisonValue,
        referenceFieldId: null
      },
      {
        automationRuleSetExpressionId: this.formState.values[1].valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
        comparisonValue: this.formState.values[1].comparisonValue,
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to map to a multiple string based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.apiRules.push({ ...this.apiRules[0], comparisonValue: 'test_value_2', automationRuleSetExpressionId: 2 });
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: false,
        comparisonValue: this.apiRules[0].comparisonValue,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }, {
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: false,
        comparisonValue: this.apiRules[1].comparisonValue,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[1].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to a number range based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.NumberRange;
    const [value] = this.formState.values;
    value.comparisonGreaterThan = 1;
    value.comparisonLessThan = 2;
    value.greaterThanId = 1;
    value.lessThanId = 2;
    delete value.valueId;
    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );
    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.greaterThanId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType:
          AutomationAPI.AutomationRuleComparisonTypes.GreaterThanOrEquals,
        comparisonValue: '' + value.comparisonGreaterThan,
        referenceFieldId: null
      },
      {
        automationRuleSetExpressionId: value.lessThanId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType:
          AutomationAPI.AutomationRuleComparisonTypes.LessThanOrEquals,
        comparisonValue: '' + value.comparisonLessThan,
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to map to a number range based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.NumberRange;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.GreaterThan,
      comparisonValue: '0',
      referenceFieldId: null
    }, {
      ...baseRule,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.LessThan,
      comparisonValue: '1',
      automationRuleSetExpressionId: 2,
      referenceFieldId: null
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: +this.apiRules[0].comparisonValue,
        comparisonLessThan: +this.apiRules[1].comparisonValue,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonNegated: null,
        comparisonValue: null,
        greaterThanId: this.apiRules[0].automationRuleSetExpressionId,
        lessThanId: this.apiRules[1].automationRuleSetExpressionId,
        valueId: null,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to a greater than number based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.NumberRange;
    const [value] = this.formState.values;
    value.comparisonGreaterThan = 1;
    value.comparisonLessThan = null;
    value.greaterThanId = 1;
    value.lessThanId = null;
    delete value.valueId;
    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.greaterThanId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType:
          AutomationAPI.AutomationRuleComparisonTypes.GreaterThanOrEquals,
        comparisonValue: '' + value.comparisonGreaterThan,
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to map to a greater than number based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.NumberRange;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.GreaterThan,
      comparisonValue: '0',
      referenceFieldId: null
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: +this.apiRules[0].comparisonValue,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: null,
        comparisonValue: null,
        greaterThanId: this.apiRules[0].automationRuleSetExpressionId,
        lessThanId: null,
        valueId: null,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to a less than number based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.NumberRange;
    const [value] = this.formState.values;
    value.comparisonGreaterThan = null;
    value.comparisonLessThan = 1;
    value.greaterThanId = null;
    value.lessThanId = 1;
    delete value.valueId;
    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.lessThanId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType:
          AutomationAPI.AutomationRuleComparisonTypes.LessThanOrEquals,
        comparisonValue: '' + value.comparisonLessThan,
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to map to a less than number based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.NumberRange;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.LessThan,
      comparisonValue: '1',
      referenceFieldId: null
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: +this.apiRules[0].comparisonValue,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonNegated: null,
        comparisonValue: null,
        greaterThanId: null,
        lessThanId: this.apiRules[0].automationRuleSetExpressionId,
        valueId: null,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to an is uploaded based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.IsUploaded;
    const [value] = this.formState.values;

    value.comparisonValue = true;

    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: AutomationAPI.AutomationRuleComparisonTypes.HasAValue,
        comparisonValue: '' + value.comparisonValue,
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to map to an is uploaded based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.IsUploaded;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.HasAValue,
      comparisonValue: 'true',
      referenceFieldId: null
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: false,
        comparisonValue: true,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to an is not uploaded based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.IsUploaded;
    const [value] = this.formState.values;

    value.comparisonValue = false;

    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType:
          AutomationAPI.AutomationRuleComparisonTypes.DoesNotHaveAValue,
        comparisonValue: '' + value.comparisonValue,
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to map to an is not uploaded based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.IsUploaded;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.DoesNotHaveAValue,
      comparisonValue: 'false',
      referenceFieldId: null
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: false,
        comparisonValue: false,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to an is selected based API Rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.IsSelected;
    const [value] = this.formState.values;

    value.comparisonValue = true;

    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
        comparisonValue: '' + value.comparisonValue,
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to map to an is selected based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.IsSelected;
    (this.objects[0].columns[0] as Automation.ObjectColumnSelectType).options = [{
      label: '',
      value: true
    }];
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
      comparisonValue: 'true',
      referenceFieldId: null
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: false,
        comparisonValue: true,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };
    expect(uiRule).to.deep.equal(expectedResult);
  }

  @It 'should be able to map to a majority (one) based API Rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.Majority;
    const [value] = this.formState.values;

    value.comparisonValue = Automation.MajorityOptions.One;

    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: AutomationAPI.AutomationRuleComparisonTypes.GreaterThan,
        comparisonValue: '0',
        referenceFieldId: null
      }
    ]);
  }
  @It 'should be able to map to a majority (one) based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.Majority;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.GreaterThan,
      comparisonValue: '0.5',
      referenceFieldId: null
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: null,
        comparisonValue: Automation.MajorityOptions.Majority,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to a majority (majority) based API Rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.Majority;
    const [value] = this.formState.values;

    value.comparisonValue = Automation.MajorityOptions.Majority;

    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: AutomationAPI.AutomationRuleComparisonTypes.GreaterThan,
        comparisonValue: '0.5',
        referenceFieldId: null
      }
    ]);
  }
  @It 'should be able to map to a majority (majority) based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.Majority;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.GreaterThan,
      comparisonValue: '0',
      referenceFieldId: null
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: null,
        comparisonValue: Automation.MajorityOptions.One,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }
  @It 'should be able to map to a majority (all) based API Rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.Majority;
    const [value] = this.formState.values;

    value.comparisonValue = Automation.MajorityOptions.All;

    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
        comparisonValue: '1',
        referenceFieldId: null
      }
    ]);
  }
  @It 'should be able to map to a majority (all) based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.Majority;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
      comparisonValue: '1'
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: null,
        comparisonValue: Automation.MajorityOptions.All,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }

  @It 'should be able to map to an is not selected based API Rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.IsSelected;
    const [value] = this.formState.values;

    value.comparisonValue = false;

    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
        comparisonValue: '' + value.comparisonValue,
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to map to an is not selected based UI rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    const [baseRule] = this.apiRules;
    this.objects[0].columns[0].comparison = Automation.Comparisons.IsSelected;
    this.apiRules = [{
      ...baseRule,
      automationRuleSetExpressionId: 1,
      comparisonType: AutomationAPI.AutomationRuleComparisonTypes.Equals,
      comparisonValue: 'false'
    }];
    const [uiRule] = ruleAutomationService.mapAPIRulesToCriteria(this.apiRules, this.objects);
    const expectedResult: Automation.CriteriaFormState = {
      column: this.objects[0].columns[0],
      isRemoved: false,
      object: this.objects[0],
      values: [{
        comparisonGreaterThan: null,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonLessThan: null,
        comparisonNegated: false,
        comparisonValue: false,
        greaterThanId: null,
        lessThanId: null,
        valueId: this.apiRules[0].automationRuleSetExpressionId,
        isRemoved: false
      }],
      err: undefined
    };

    expect(uiRule).to.deep.equal(expectedResult);
  }

  @It 'should support removing values' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.values[0].isRemoved = true;
    const [apiRule] = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );

    expect(apiRule.isRemoved).to.equal(true);
  }

  @It 'should not return removed rules without an id' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.values[0].isRemoved = true;
    this.formState.values[0].valueId = null;
    const [apiRule] = ruleAutomationService.mapCriteriaToAPIRule(
      this.formState
    );

    expect(apiRule).to.equal(undefined);
  }

  @It 'should be able to update a relative date API rule to a date range based API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.DateRange;
    const [value] = this.formState.values;
    value.comparisonGreaterThan = 1;
    value.comparisonLessThan = 2;
    value.greaterThanId = 1;
    value.lessThanId = 2;
    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );
    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: true,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: value.comparisonType,
        comparisonValue: '',
        referenceFieldId: null
      },
      {
        automationRuleSetExpressionId: value.greaterThanId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType:
          AutomationAPI.AutomationRuleComparisonTypes.GreaterThanOrEquals,
        comparisonValue: moment(value.comparisonGreaterThan).utc().startOf('day'),
        referenceFieldId: null
      },
      {
        automationRuleSetExpressionId: value.lessThanId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType:
          AutomationAPI.AutomationRuleComparisonTypes.LessThanOrEquals,
        comparisonValue: moment(value.comparisonLessThan).utc().endOf('day'),
        referenceFieldId: null
      }
    ]);
  }
  @It 'should be able to update a date range based API rule to a relative date API rule' (
    ruleAutomationService: RuleAutomationService
  ) {
    this.formState.column.comparison = Automation.Comparisons.DateRange;
    const [value] = this.formState.values;
    value.comparisonType = Automation.CriteriaComparisonTypes.Last365Days;
    value.comparisonGreaterThan = 1;
    value.comparisonLessThan = 2;
    value.greaterThanId = 1;
    value.lessThanId = 2;
    const apiRules = ruleAutomationService.getMappedCriteriaToAPIRule(
      this.formState
    );
    expect(apiRules).to.deep.equal([
      {
        automationRuleSetExpressionId: value.valueId,
        isRemoved: false,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: value.comparisonType,
        comparisonValue: '',
        referenceFieldId: null
      },
      {
        automationRuleSetExpressionId: value.greaterThanId,
        isRemoved: true,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: value.comparisonType,
        comparisonValue: '',
        referenceFieldId: null
      },
      {
        automationRuleSetExpressionId: value.lessThanId,
        isRemoved: true,
        columnName: this.formState.object.key + '.' + this.formState.column.key,
        comparisonType: value.comparisonType,
        comparisonValue: '',
        referenceFieldId: null
      }
    ]);
  }

  @It 'should be able to get selected options' (
    service: RuleAutomationService
  ) {
    const options = service.getSelectedOptions();
    expect(options.length).to.be.equal(2);
    const hasIsSelected = options.some((opt) => {
      return opt.value === true &&
        opt.label === 'Is selected';
    });
    expect(hasIsSelected).to.be.true;
    const isNotSelected = options.some((opt) => {
      return opt.value === false &&
        opt.label === 'Is not selected';
    });
    expect(isNotSelected).to.be.true;
  }

  @It 'should be able to get file upload options' (
    service: RuleAutomationService
  ) {
    const options = service.getFileUploadOptions();
    expect(options.length).to.be.equal(2);
    const hasIsUploaded = options.some((opt) => {
      return opt.value === true &&
        opt.label === 'Is uploaded';
    });
    expect(hasIsUploaded).to.be.true;
    const isNotUploaded = options.some((opt) => {
      return opt.value === false &&
        opt.label === 'Is not uploaded';
    });
    expect(isNotUploaded).to.be.true;
  }

  @It 'should be able to get comparison negate options' (
    service: RuleAutomationService
  ) {
    const options = service.getComparisonNegateOptions();
    expect(options.length).to.be.equal(2);
    const hasIs = options.some((opt) => {
      return opt.value === false &&
        opt.label === 'is';
    });
    expect(hasIs).to.be.true;
    const hasIsNot = options.some((opt) => {
      return opt.value === true &&
        opt.label === 'is not';
    });
    expect(hasIsNot).to.be.true;
  }

  @It 'should be able to get majority options' (
    service: RuleAutomationService
  ) {
    const options = service.getMajorityOptions();
    expect(options.length).to.be.equal(3);
    const hasAll = options.some((opt) => {
      return opt.value === Automation.MajorityOptions.All &&
        opt.label === 'All';
    });
    expect(hasAll).to.be.true;
    const hasMajority = options.some((opt) => {
      return opt.value === Automation.MajorityOptions.Majority &&
        opt.label === 'Majority';
    });
    expect(hasMajority).to.be.true;
    const hasOne = options.some((opt) => {
      return opt.value === Automation.MajorityOptions.One &&
        opt.label === 'One';
    });
    expect(hasOne).to.be.true;
  }


  @It 'should be able to get date options' (
    service: RuleAutomationService
  ) {
    const options = service.getDateOptions();
    const hasIsBetween = options.some((opt) => {
      return opt.value === Automation.CriteriaComparisonTypes.Default &&
        opt.label === 'Is between';
    });
    expect(hasIsBetween).to.be.true;
    const hasToday = options.some((opt) => {
      return opt.value === Automation.CriteriaComparisonTypes.Today &&
        opt.label === 'Today';
    });
    expect(hasToday).to.be.true;
    const hasThisYear = options.some((opt) => {
      return opt.value === Automation.CriteriaComparisonTypes.ThisYear &&
        opt.label === 'This year';
    });
    expect(hasThisYear).to.be.true;
  }

  @It 'should be able to get number is options' (
    service: RuleAutomationService
  ) {
    const options = service.getNumberIsOptions();
    expect(options.length).to.be.equal(1);
    const hasEquals = options.some((opt) => {
      return opt.value === false &&
        opt.label === 'equals';
    });
    expect(hasEquals).to.be.true;
  }

  @It 'should be able to get application object - pre create non routing' (
    service: RuleAutomationService
  ) {
    const obj = service.getApplicationObject(false);
    expect(obj.key).to.be.equal('Application');
    expect(obj.columns.length).to.be.equal(3);
    const hasAmountRequested = obj.columns.some((col) => {
      return col.key === 'AmountRequested';
    });
    expect(hasAmountRequested).to.be.true;
    const hasInKind = obj.columns.some((col) => {
      return col.key === 'InKindAmountRequested';
    });
    expect(hasInKind).to.be.true;
    const hasTotalRequested = obj.columns.some((col) => {
      return col.key === 'TotalAmountRequested';
    });
    expect(hasTotalRequested).to.be.true;
  }

  @It 'should be able to get application object - pre create routing' (
    service: RuleAutomationService
  ) {
    const obj = service.getApplicationObject(false, true);
    expect(obj.key).to.be.equal('Application');
    expect(obj.columns.length).to.be.equal(1);
    const hasAmountRequested = obj.columns.some((col) => {
      return col.key === 'AmountRequested';
    });
    expect(hasAmountRequested).to.be.true;
  }

  @It 'should be able to get application object - post create non routing' (
    service: RuleAutomationService
  ) {
    const obj = service.getApplicationObject(true);
    expect(obj.key).to.be.equal('Application');
    expect(obj.columns.length).to.be.equal(5);
    const hasAmountRequested = obj.columns.some((col) => {
      return col.key === 'AmountRequested';
    });
    expect(hasAmountRequested).to.be.true;
    const hasInKind = obj.columns.some((col) => {
      return col.key === 'InKindAmountRequested';
    });
    expect(hasInKind).to.be.true;
    const hasTotalRequested = obj.columns.some((col) => {
      return col.key === 'TotalAmountRequested';
    });
    expect(hasTotalRequested).to.be.true;
    const hasRecommendedFunding = obj.columns.some((col) => {
      return col.key === 'RecommendedFundingAmount';
    });
    expect(hasRecommendedFunding).to.be.true;
    const hasStatus = obj.columns.some((col) => {
      return col.key === 'ApplicationStatus';
    });
    expect(hasStatus).to.be.true;
  }

  @It 'should be able to get applicant object' (
    service: RuleAutomationService
  ) {
    const obj = service.getApplicantObject();
    expect(obj.key).to.be.equal('ApplicantInfo');
    expect(obj.columns.length).to.be.equal(4);
    const hasCity = obj.columns.some((col) => {
      return col.key === 'City';
    });
    expect(hasCity).to.be.true;
    const hasState = obj.columns.some((col) => {
      return col.key === 'State';
    });
    expect(hasState).to.be.true;
    const hasCountry = obj.columns.some((col) => {
      return col.key === 'Country';
    });
    expect(hasCountry).to.be.true;
    const hasZip = obj.columns.some((col) => {
      return col.key === 'PostalCode';
    });
    expect(hasZip).to.be.true;
  }

  @It 'should be able to get budget object' (
    service: RuleAutomationService
  ) {
    const obj = service.getBudgetObject();
    expect(obj.key).to.be.equal('Budget');
    expect(obj.columns.length).to.be.equal(1);
    const hasTotalRemaining = obj.columns.some((col) => {
      return col.key === 'TotalRemaining';
    });
    expect(hasTotalRemaining).to.be.true;
  }

  @It 'should be able to get organization object' (
    service: RuleAutomationService
  ) {
    const obj = service.getOrganizationObject();
    expect(obj.key).to.be.equal('OrganizationInfo');
    const hasCity = obj.columns.some((col) => {
      return col.key === 'City';
    });
    expect(hasCity).to.be.true;
    const hasState = obj.columns.some((col) => {
      return col.key === 'State';
    });
    expect(hasState).to.be.true;
    const hasCountry = obj.columns.some((col) => {
      return col.key === 'Country';
    });
    expect(hasCountry).to.be.true;
    const hasZip = obj.columns.some((col) => {
      return col.key === 'PostalCode';
    });
    expect(hasZip).to.be.true;
    const hasTotalAwarded = obj.columns.some((col) => {
      return col.key === 'aggregatedTotalAwardedAmount';
    });
    expect(hasTotalAwarded).to.be.true;
    const hasTotalRequested = obj.columns.some((col) => {
      return col.key === 'aggregateTotalRequestedAmount';
    });
    expect(hasTotalRequested).to.be.true;
  }

  @It 'should be able to get employee sso object - no sso' (
    service: RuleAutomationService
  ) {
    service['clientSettingsService']['set']('clientSettings', {
      hasSSO: false
    } as any);
    const obj = service.getEmployeeSsoObject();
    expect(!obj).to.be.true;
  }

  @It 'should be able to get employee sso object - with sso' (
    service: RuleAutomationService
  ) {
    service['clientSettingsService']['set']('clientSettings', {
      hasSSO: true
    } as any);
    const obj = service.getEmployeeSsoObject();
    expect(obj.key).to.be.equal('EmployeeApplicantInfo');
    const inUseFields = this.employeeSsoFields.filter((field) => field.inUse);
    expect(obj.columns.length).to.be.equal(inUseFields.length);
    const hasDivision = obj.columns.some((col) => {
      return col.key === 'division';
    });
    expect(hasDivision).to.be.true;
    const hasRegion = obj.columns.some((col) => {
      return col.key === 'region';
    });
    expect(hasRegion).to.be.false;
  }

  @It 'should be able to get aggregate object' (
    service: RuleAutomationService
  ) {
    const obj = service.getAggregateObject();
    expect(obj.key).to.be.equal('referenceFields');
    const aggregateFields = allReferenceFields.filter((item) => !!item.aggregateType);
    expect(obj.columns.length).to.be.equal(aggregateFields.length);
    const hasAggregate = obj.columns.some((col) => {
      return col.key === aggregateKey;
    });
    expect(hasAggregate).to.be.true;
  }

  @It 'should be able to get award object' (
    service: RuleAutomationService
  ) {
    const obj = service.getAwardObject();
    expect(obj.key).to.be.equal('Award');
    expect(obj.columns.length).to.be.equal(2);
    const hasTotalAward = obj.columns.some((col) => {
      return col.key === 'TotalAwardAmount';
    });
    expect(hasTotalAward).to.be.true;
    const hasLastAwardDate = obj.columns.some((col) => {
      return col.key === 'LastAwardDateModel';
    });
    expect(hasLastAwardDate).to.be.true;
  }

  @It 'should be able to get decision object' (
    service: RuleAutomationService
  ) {
    const obj = service.getDecisionObject();
    expect(obj.key).to.be.equal('Decisions');
    expect(obj.columns.length).to.be.equal(4);
    const hasApprovalRatio = obj.columns.some((col) => {
      return col.key === 'DecisionsYesRatio';
    });
    expect(hasApprovalRatio).to.be.true;
    const hasDeclineRatio = obj.columns.some((col) => {
      return col.key === 'DecisionsNoRatio';
    });
    expect(hasDeclineRatio).to.be.true;
    const hasApprovalCount = obj.columns.some((col) => {
      return col.key === 'DecisionsYesCount';
    });
    expect(hasApprovalCount).to.be.true;
    const hasDeclineCount = obj.columns.some((col) => {
      return col.key === 'DecisionsNoCount';
    });
    expect(hasDeclineCount).to.be.true;
  }

  @It 'should be able to get payment object' (
    service: RuleAutomationService
  ) {
    const obj = service.getPaymentObject();
    expect(obj.key).to.be.equal('Payment');
    expect(obj.columns.length).to.be.equal(3);
    const hasPayment = obj.columns.some((col) => {
      return col.key === 'HasPayment';
    });
    expect(hasPayment).to.be.true;
    const hasRatio = obj.columns.some((col) => {
      return col.key === 'DisbursedPaymentRatio';
    });
    expect(hasRatio).to.be.true;
    const hasMatches = obj.columns.some((col) => {
      return col.key === 'PaymentAmountMatchesAwardAmount';
    });
    expect(hasMatches).to.be.true;
  }

  @It 'should be able to get column comparison options - from getSelectedOptions' (
    service: RuleAutomationService
  ) {
    const awardObj = service.getAwardObject();
    const totalAmountCol = awardObj.columns[0];
    const options = service.getColumnComparisonOptions(totalAmountCol);
    const sourcedOpts = service.getSelectedOptions();
    expect(options).to.deep.equal(sourcedOpts);
  }

  @It 'should be able to get column comparison options - from columns options' (
    service: RuleAutomationService
  ) {
    const paymentObj = service.getPaymentObject();
    const hasPaymentsCol = paymentObj.columns.find((col) => {
      return col.key === 'HasPayment';
    });
    const options = service.getColumnComparisonOptions(hasPaymentsCol);
    const sourcedOpts = 'options' in hasPaymentsCol ?
      hasPaymentsCol.options :
      [];
    expect(options).to.deep.equal(sourcedOpts);
  }

  @It 'should be able to get sentence connector - date' (
    service: RuleAutomationService
  ) {
    const dateRule = {
      column: {
        comparison: Automation.Comparisons.DateRange
      } as any,
      values: [{
        comparisonGreaterThan: moment('05/01/2022'),
        comparisonLessThan: moment('05/27/2022'),
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonValue: null,
        greaterThanId: 141454,
        lessThanId: 141455,
        valueId: 34643
      }]
    } as Automation.CriteriaFormState;
    let connector = service.getSentenceConnector(dateRule);
    // Is between doesn't have connector
    expect(!connector).to.be.true;
    // other date options use 'is'
    dateRule.values[0].comparisonType = Automation.CriteriaComparisonTypes.Last365Days;
    connector = service.getSentenceConnector(dateRule);
    expect(connector).to.be.equal('is');
  }

  @It 'should be able to get sentence connector - number is' (
    service: RuleAutomationService
  ) {
    const numberRule = {
      column: {
        comparison: Automation.Comparisons.NumberIs
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: 5,
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const connector = service.getSentenceConnector(numberRule);
    expect(connector).to.be.equals('equals');
  }

  @It 'should be able to get sentence connector - majority' (
    service: RuleAutomationService
  ) {
    const majorityRule = {
      column: {
        comparison: Automation.Comparisons.Majority
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: Automation.MajorityOptions.All,
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const connector = service.getSentenceConnector(majorityRule);
    expect(connector).to.be.equals('is');
  }

  @It 'should be able to get sentence connector - text is' (
    service: RuleAutomationService
  ) {
    const textIsRule = {
      column: {
        comparison: Automation.Comparisons.TextIs
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: 'test',
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    let connector = service.getSentenceConnector(textIsRule);
    expect(connector).to.be.equals('is');
    textIsRule.values[0].comparisonNegated = true;
    connector = service.getSentenceConnector(textIsRule);
    expect(connector).to.be.equal('is not');
  }

  @It 'should be able to get value connector - date range is between' (
    service: RuleAutomationService
  ) {
    const dateRule = {
      column: {
        comparison: Automation.Comparisons.DateRange
      } as any,
      values: [{
        comparisonGreaterThan: moment('05/01/2022'),
        comparisonLessThan: moment('05/27/2022'),
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonValue: null,
        greaterThanId: 141454,
        lessThanId: 141455,
        valueId: 34643
      }]
    } as Automation.CriteriaFormState;
    let valueConnector = service.getValueConnector(dateRule);
    expect(valueConnector).to.be.equal('is between May 1, 2022 and May 27, 2022');
    dateRule.values[0].comparisonGreaterThan = null;
    valueConnector = service.getValueConnector(dateRule);
    expect(valueConnector).to.be.equal('is before May 27, 2022');
    dateRule.values[0].comparisonLessThan = null;
    dateRule.values[0].comparisonGreaterThan = moment('05/01/2022');
    valueConnector = service.getValueConnector(dateRule);
    expect(valueConnector).to.be.equal('is after May 1, 2022');
  }

  @It 'should be able to get value connector - date range is not between' (
    service: RuleAutomationService
  ) {
    const dateRule = {
      column: {
        comparison: Automation.Comparisons.DateRange
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: Automation.CriteriaComparisonTypes.Today,
        comparisonValue: null,
        greaterThanId: 141454,
        lessThanId: 141455,
        valueId: 34643
      }]
    } as Automation.CriteriaFormState;
    const valueConnector = service.getValueConnector(dateRule);
    expect(valueConnector).to.be.equal('Today');
  }

  @It 'should be able to get value connector - is selected' (
    service: RuleAutomationService
  ) {
    const isSelectedRule = {
      column: {
        comparison: Automation.Comparisons.IsSelected
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: true,
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const connector = service.getValueConnector(isSelectedRule);
    expect(connector).to.be.equals('Is selected');
  }

  @It 'should be able to get value connector - is uploaded' (
    service: RuleAutomationService
  ) {
    const isUploadedRule = {
      column: {
        comparison: Automation.Comparisons.IsUploaded
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: true,
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const connector = service.getValueConnector(isUploadedRule);
    expect(connector).to.be.equals('Is uploaded');
  }

  @It 'should be able to get value connector - majority' (
    service: RuleAutomationService
  ) {
    const majorityRule = {
      column: {
        comparison: Automation.Comparisons.Majority
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: Automation.MajorityOptions.All,
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const connector = service.getValueConnector(majorityRule);
    expect(connector).to.be.equals('All');
  }

  @It 'should be able to get value connector - select is' (
    service: RuleAutomationService
  ) {
    const appObj = service.getApplicationObject(true);
    const statusCol = appObj.columns.find((col) => {
      return col.key === 'ApplicationStatus';
    });
    const selectIsRule = {
      column: {
        ...statusCol,
        options: [{
          label: 'Approved',
          value: ApplicationStatuses.Approved
        }]
      },
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: ApplicationStatuses.Approved,
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const connector = service.getValueConnector(selectIsRule);
    expect(connector).to.be.equals('Approved');
  }

  @It 'should be able to get value connector - number range' (
    service: RuleAutomationService
  ) {
    const numberRangeRule = {
      column: {
        comparison: Automation.Comparisons.NumberRange
      } as any,
      values: [{
        comparisonGreaterThan: 0,
        comparisonLessThan: 100,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonValue: null,
        greaterThanId: 141454,
        lessThanId: 141455,
        valueId: 34643
      }]
    } as Automation.CriteriaFormState;
    let valueConnector = service.getValueConnector(numberRangeRule);
    expect(valueConnector).to.be.equal('is between 0 and 100');
    numberRangeRule.values[0].comparisonGreaterThan = null;
    valueConnector = service.getValueConnector(numberRangeRule);
    expect(valueConnector).to.be.equal('is less than 100');
    numberRangeRule.values[0].comparisonLessThan = null;
    numberRangeRule.values[0].comparisonGreaterThan = 0;
    valueConnector = service.getValueConnector(numberRangeRule);
    expect(valueConnector).to.be.equal('is greater than 0');
  }

  @It 'should be able to get value connector - text is' (
    service: RuleAutomationService
  ) {
    const textIsRule = {
      column: {
        comparison: Automation.Comparisons.TextIs
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: 'test',
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const connector = service.getValueConnector(textIsRule);
    expect(connector).to.be.equals('test');
  }

  @It 'should be able to get value connector - number is' (
    service: RuleAutomationService
  ) {
    const numberIsRule = {
      column: {
        comparison: Automation.Comparisons.NumberIs
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: 5,
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState, {
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: 10,
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const connector = service.getValueConnector(numberIsRule);
    expect(connector).to.be.equals('5, 10');
  }

  @It 'should be able to create rule sentence - number range' (
    service: RuleAutomationService
  ) {
    const numberRangeRule = {
      column: {
        comparison: Automation.Comparisons.NumberRange,
        label: 'Cash amount requested'
      } as any,
      values: [{
        comparisonGreaterThan: 0,
        comparisonLessThan: 100,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: Automation.CriteriaComparisonTypes.Default,
        comparisonValue: null,
        greaterThanId: 141454,
        lessThanId: 141455,
        valueId: 34643
      }]
    } as Automation.CriteriaFormState;
    const sentence = service.createRuleSentence(numberRangeRule);
    expect (sentence).to.be.equal('Cash amount requested is between 0 and 100');
  }

  @It 'should be able to create rule sentence - text is' (
    service: RuleAutomationService
  ) {
     const textIsRule = {
      column: {
        comparison: Automation.Comparisons.TextIs,
        label: 'Name'
      } as any,
      values: [{
        comparisonGreaterThan: null,
        comparisonLessThan: null,
        isRemoved: false,
        comparisonNegated: false,
        comparisonType: null,
        comparisonValue: 'test',
        greaterThanId: null,
        lessThanId: null,
        valueId: 34643
      } as Automation.CriteriaValueFormState]
    } as Automation.CriteriaFormState;
    const sentence = service.createRuleSentence(textIsRule);
    expect (sentence).to.be.equal('Name is test');
  }
}
