import { Injectable } from '@angular/core';
import { GCMockModule } from '@core/mocks/gc-module.mock';
import { BeforeEach, Spec, TestCase } from '@yourcause/test-decorators';
import { DescribeAngularService } from '@yourcause/test-decorators/angular';
import { expect } from 'chai';
import { Email, EmailByApplication, EmailByProgramResponse, EmailNotificationType } from '../email.typing';
import { ExportEmailModalResponse } from './export-emails-modal/export-emails-modal.component';
import { ExportEmailService } from './export-emails.service';

@Injectable({ providedIn: 'root' })
@DescribeAngularService(ExportEmailService, {
  imports: [
    GCMockModule
  ]
})
export class ExportEmailServiceSpec implements Spec<ExportEmailServiceSpec, ExportEmailService> {
  appApprovedCopy = {
    id: 2883476,
    active: true,
    default: false,
    clientEmailTemplate: {
      id: 57173,
      name: 'Your Application is Approved - ID: {{APPLICATION_ID}}',
      body: '<tbody><tr>\r\n    <td bgcolor="#ffffff" style="padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555; text-align: left;"><p>&nbsp;</p>\n\n<p><br />\nDear {{APPLICANT_FIRST_NAME}} {{APPLICANT_LAST_NAME}},<br />\n<br />\nYour grant application to {{CLIENT_NAME}}/{{PROGRAM_NAME}} on behalf of {{ON_BEHALF_OF}} has been approved.<br />\n<br />\n(Application ID# {{APPLICATION_ID}})<br />\n<br />\n{{#if Custom.CustomMessage}} {{CUSTOM_MESSAGE}} {{#else GrantProgram.SenderDisplayName}} {{SENDER_DISPLAY_NAME}} {{/else}}<br />\n<br />\n{{/if}}</p>\n\n<p>Annual Operating Budget:&nbsp;{{ReferenceField.annualOperatingBudget2}}</p>\n\n<p>&nbsp;</p>\n\n<p>Sincerely,<br />\nGrantsConnect Support</p>\n\n<p>&nbsp;</p>\n</td>\r\n</tr>\r\n<tr>\r\n    <td bgcolor="#ffffff" style="padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555;">\r\n        <!-- Button : BEGIN -->\r\n        <table role="presentation" cellspacing="0" cellpadding="0" border="0" align="center" style="margin: auto">\r\n            <tbody><tr>\r\n                <td style="border-radius: 3px; background: [[CLIENT_BRAND_PRIMARY]];  text-align: center;" class="button-td">\r\n                    <a href="{{LINK_TO_PORTAL}}" style="background: [[CLIENT_BRAND_PRIMARY]]; border: 15px solid [[CLIENT_BRAND_PRIMARY]];  font-family: sans-serif; font-size: 13px; line-height: 110%; text-align: center; text-decoration: none; display: block; border-radius: 3px; font-weight: bold;" class="button-a">\r\n                              <span style="color:#ffffff;">Go to GrantsConnect</span>      \r\n                    </a>\r\n                </td>\r\n            </tr>\r\n        </tbody></table>\r\n        <!-- Button : END -->\r\n    </td>\r\n</tr></tbody>',
      subject: 'Your Application is Approved - ID: {{APPLICATION_ID}}',
      copy: true,
      description: 'Grant manager approves application',
      title: 'Application Approved'
    },
    emailNotificationTypeId: EmailNotificationType.ApplicationApproved
  };
  programEmails: EmailByProgramResponse[] = [{
    emailInfo: [{
      id: 2883465,
      active: true,
      default: false,
      clientEmailTemplate: {
        id: 51913,
        name: 'Your Application is Approved - ID: {{APPLICATION_ID}}  Copy',
        body: `<tbody><tr>\r\n    <td bgcolor=\"#ffffff\" style=\"padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555; text-align: left;\"><p>Copy</p>\n\n<p><br />\nDear {{APPLICANT_FIRST_NAME}} {{APPLICANT_LAST_NAME}},<br />\n<br />\nYour grant application to {{CLIENT_NAME}}/{{PROGRAM_NAME}} on behalf of {{ON_BEHALF_OF}} has been approved.<br />\n<br />\n(Application ID# {{APPLICATION_ID}})<br />\n<br />\n{{#if Custom.CustomMessage}} {{CUSTOM_MESSAGE}}<br />\n<br />\n{{/if}} Sincerely,<br />\nGrantsConnect Support</p>\n\n<p><a href=\"http://www.blackbaud.com\" target=\"_blank\"><img alt=\"\" src=\"https://grantsconnect-qa.imgix.net/management/160/images/deeb36cc-9087-42c5-9a1d-a2c88b6c3e8b.png\" style=\"height:43px; width:150px\" /></a></p>\n</td>\r\n</tr>\r\n<tr>\r\n    <td bgcolor=\"#ffffff\" style=\"padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555;\">\r\n        <!-- Button : BEGIN -->\r\n        <table role=\"presentation\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" align=\"center\" style=\"margin: auto\">\r\n            <tbody><tr>\r\n                <td style=\"border-radius: 3px; background: [[CLIENT_BRAND_PRIMARY]];  text-align: center;\" class=\"button-td\">\r\n                    <a href=\"{{LINK_TO_PORTAL}}\" style=\"background: [[CLIENT_BRAND_PRIMARY]]; border: 15px solid [[CLIENT_BRAND_PRIMARY]];  font-family: sans-serif; font-size: 13px; line-height: 110%; text-align: center; text-decoration: none; display: block; border-radius: 3px; font-weight: bold;\" class=\"button-a\">\r\n<span style=\"color:#ffffff;\">Go to GrantsConnect</span>\r\n</a>\r\n</td>\r\n</tr>\r\n</tbody></table>\r\n<!-- Button : END -->\r\n    </td>\r\n</tr></tbody>`,
        subject: 'Your Application is Approved - ID: {{APPLICATION_ID}}  Copy',
        copy: true,
        description: 'Grant manager approves application - has image',
        title: 'Application Approved'
      },
      emailNotificationTypeId: EmailNotificationType.ApplicationApproved
    },
    this.appApprovedCopy,
    {
      id: 0,
      active: false,
      default: true,
      clientEmailTemplate: {
        id: 0,
        name: 'Standard Email',
        body: '<tr>\r\n    <td bgcolor="#ffffff" style="padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555; text-align: left;">\r\n        <br>\r\n        <br>\r\n        Dear {{APPLICANT_FIRST_NAME}} {{APPLICANT_LAST_NAME}},\r\n        <br>\r\n        <br>\r\n        Your grant application to {{CLIENT_NAME}}/{{PROGRAM_NAME}} on behalf of {{ON_BEHALF_OF}} has been approved.\r\n        <br>\r\n        <br>\r\n        (Application ID# {{APPLICATION_ID}})\r\n        <br>\r\n        <br>\r\n        {{#if Custom.CustomMessage}}\r\n        {{CUSTOM_MESSAGE}}\r\n        {{#else GrantProgram.SenderDisplayName}}\r\n        {{SENDER_DISPLAY_NAME}}\r\n        {{/else}}\r\n        <br>\r\n        <br>\r\n        {{/if}}\r\n        Sincerely,<br>\r\n        GrantsConnect Support\r\n    </td>\r\n</tr>\r\n<tr>\r\n    <td bgcolor="#ffffff" style="padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555;">\r\n        <!-- Button : BEGIN -->\r\n        <table role="presentation" cellspacing="0" cellpadding="0" border="0" align="center" style="margin: auto">\r\n            <tr>\r\n                <td style="border-radius: 3px; background: [[CLIENT_BRAND_PRIMARY]];  text-align: center;" class="button-td">\r\n                    <a href="{{LINK_TO_PORTAL}}" style="background: [[CLIENT_BRAND_PRIMARY]]; border: 15px solid [[CLIENT_BRAND_PRIMARY]];  font-family: sans-serif; font-size: 13px; line-height: 110%; text-align: center; text-decoration: none; display: block; border-radius: 3px; font-weight: bold;" class="button-a">\r\n                        &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span style="color:#ffffff;">Go to GrantsConnect</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\r\n                    </a>\r\n                </td>\r\n            </tr>\r\n        </table>\r\n        <!-- Button : END -->\r\n    </td>\r\n</tr>',
        subject: 'Your Application is Approved - ID: {{APPLICATION_ID}}',
        copy: false,
        description: 'Standard Email',
        title: 'Application Approved'
      },
      emailNotificationTypeId: EmailNotificationType.ApplicationApproved
    }]
  },
  {
    emailInfo: [{
      id: 0,
      active: true,
      default: true,
      clientEmailTemplate: {
        id: 0,
        name: 'Standard Email',
        body: '<tr>\r\n    <td bgcolor="#ffffff" style="padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555; text-align: left;">\r\n        <br>\r\n        <br>\r\n        Dear {{APPLICANT_FIRST_NAME}} {{APPLICANT_LAST_NAME}},\r\n        <br>\r\n        <br>\r\n        Your grant application to {{CLIENT_NAME}}/{{PROGRAM_NAME}} on behalf of {{ON_BEHALF_OF}} has been declined.\r\n        <br>\r\n        <br>\r\n        (Application ID# {{APPLICATION_ID}})\r\n        <br>\r\n        <br>\r\n        {{#if Custom.CustomMessage}}\r\n        {{CUSTOM_MESSAGE}}\r\n        <br>\r\n        <br>\r\n        {{/if}}\r\n        Sincerely,<br>\r\n        GrantsConnect Support\r\n    </td>\r\n</tr>\r\n<tr>\r\n    <td bgcolor="#ffffff" style="padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555;">\r\n        <!-- Button : BEGIN -->\r\n        <table role="presentation" cellspacing="0" cellpadding="0" border="0" align="center" style="margin: auto">\r\n            <tr>\r\n                <td style="border-radius: 3px; background: [[CLIENT_BRAND_PRIMARY]];  text-align: center;" class="button-td">\r\n                    <a href="{{LINK_TO_PORTAL}}" style="background: [[CLIENT_BRAND_PRIMARY]];  border: 15px solid [[CLIENT_BRAND_PRIMARY]]; font-family: sans-serif; font-size: 13px; line-height: 110%; text-align: center; text-decoration: none; display: block; border-radius: 3px; font-weight: bold;" class="button-a">\r\n                        &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span style="color:#ffffff;">Go to GrantsConnect</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\r\n                    </a>\r\n                </td>\r\n            </tr>\r\n        </table>\r\n        <!-- Button : END -->\r\n    </td>\r\n</tr>',
        subject: 'Your Application is Declined - ID: {{APPLICATION_ID}}',
        copy: false,
        description: 'Standard Email',
        title: 'Application Declined'
      },
      emailNotificationTypeId: EmailNotificationType.ApplicationDeclined
    }]
  }];
  emails: Email[] = [{
    subject: 'Your Application is Approved - ID: {{APPLICATION_ID}}',
    name: 'ApplicationApproved',
    description: 'Grant manager approves application',
    title: 'Application Approved',
    emailNotificationType: EmailNotificationType.ApplicationApproved,
    canBeDisabledByClient: true,
    isDisabled: false,
    audienceType: 1,
    heroImage: 'https://grantsconnectui.azureedge.net/static/okCheck.png',
    customizable: true,
    maxCopies: -1,
    emailUsageType: 2,
    nomination: false,
    numberOfClientCopies: 6,
    mergeModelType: 1,
    supportsCarbonCopy: true,
    useClientDefaultCopy: true,
    allowDocumentTemplates: true
  },
  {
    subject: 'Your Application is Declined - ID: {{APPLICATION_ID}}',
    name: 'ApplicationDeclined',
    description: 'Grant manager declines an application',
    title: 'Application Declined',
    emailNotificationType: EmailNotificationType.ApplicationDeclined,
    canBeDisabledByClient: true,
    isDisabled: false,
    audienceType: 1,
    heroImage: 'https://grantsconnectui.azureedge.net/static/IconBell.png',
    customizable: true,
    maxCopies: -1,
    emailUsageType: 2,
    nomination: false,
    numberOfClientCopies: 1,
    mergeModelType: 1,
    supportsCarbonCopy: true,
    useClientDefaultCopy: false,
    allowDocumentTemplates: true
  }, {
    subject: 'Invitation to GrantsConnect',
    name: 'ApplicantAddedToApplication',
    description: 'Sent when an applicant or collaborator is added to an application',
    title: 'Invitation to GrantsConnect',
    emailNotificationType: EmailNotificationType.ApplicantAddedToApplication,
    canBeDisabledByClient: false,
    isDisabled: false,
    audienceType: 1,
    heroImage: 'https://grantsconnectui.azureedge.net/static/okCheck.png',
    customizable: false,
    maxCopies: -1,
    emailUsageType: 2,
    nomination: false,
    numberOfClientCopies: 0,
    mergeModelType: 5,
    supportsCarbonCopy: false,
    useClientDefaultCopy: false,
    allowDocumentTemplates: false
  }];
  buttonHtml = '<!-- Button : BEGIN -->\r\n        <table role=\"presentation\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" align=\"center\" style=\"margin: auto\">\r\n            <tbody><tr>\r\n                <td style=\"border-radius: 3px; background: [[CLIENT_BRAND_PRIMARY]];  text-align: center;\" class=\"button-td\">\r\n                    <a href=\"{{LINK_TO_PORTAL}}\" style=\"background: [[CLIENT_BRAND_PRIMARY]]; border: 15px solid [[CLIENT_BRAND_PRIMARY]]; font-family: sans-serif; font-size: 13px; line-height: 110%; text-align: center; text-decoration: none; display: block; border-radius: 3px; font-weight: bold;\" class=\"button-a\">\r\n                              <span style=\"color:#ffffff;\">Go to GrantsConnect</span>      \r\n                    </a>\r\n                </td>\r\n            </tr>\r\n        </tbody></table>\r\n        <!-- Button : END -->';
  bodyWithButton = `<tbody><tr>\r\n    <td bgcolor=\"#ffffff\" style=\"padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555; text-align: left;\"><p>&nbsp;</p>\n\n<p><br />\nDear {{APPLICANT_FIRST_NAME}} {{APPLICANT_LAST_NAME}},<br />\n<br />\nYou have successfully submitted your application for the following program:<br />\n<br />\n{{CLIENT_NAME}} / {{PROGRAM_NAME}}<br />\n<br />\n<strong>Application ID:</strong> {{APPLICATION_ID}}<br />\n<strong>Form Submitted:</strong> {{FORM_NAME}}<br />\n<br />\nSincerely,<br />\nGrantsConnect Support</p>\n\n<p>{{CLIENT_NAME}}</p>\n\n<p>&nbsp;</p>\n</td>\r\n</tr>\r\n<tr>\r\n    <td bgcolor=\"#ffffff\" style=\"padding: 0 40px 40px; font-family: sans-serif; font-size: 15px; line-height: 140%; color: #555555;\">\r\n        ${this.buttonHtml}\r\n    </td>\r\n</tr></tbody>`;
  applicationEmail: EmailByApplication = {
    emailNotificationTypeId: EmailNotificationType.ApplicantAddedToApplication,
    subject: 'Invitation to GrantsConnect',
    body: this.bodyWithButton,
    toAddress: 'kc54861+new@gmail.com',
    toName: 'Tina Conway',
    sentTime: '2022-02-10T20:50:26.058991Z',
    multipleToAddresses: [],
    carbonCopyRecipients: []
  };

  @BeforeEach()
  mock (service: ExportEmailService) {
    service['exportEmailResources'].getEmailsByProgramAndLang = async () => {
      return this.programEmails;
    };
  }

  @TestCase('should be able to handle export emails by program and lang')
  async handleExportEmailsByProgramAndLang (service: ExportEmailService) {
    let didDownload = false;
    let generatedPdf = false;
    service['pdfService'].generateEmailPdf = async () => {
      generatedPdf = true;

      return 'url';
    };
    service['fileService']['downloadUrlAs'] = async () => {
      didDownload = true;
    };
    const modalResponse: ExportEmailModalResponse = {
      programId: 123,
      emailLanguage: 'en-US'
    };

    await service.handleExportEmailsByProgramAndLang(modalResponse, this.emails);
    expect(didDownload).to.be.true;
    expect(generatedPdf).to.be.true;
  }

  @TestCase('should be able to handle export emails by program and lang - fail')
  async handleExportEmailsByProgramAndLangFail (service: ExportEmailService) {
    let errorNotified = false;
    let loggedError = false;
    service['notifier'].error = () => {
      errorNotified = true;
    };
    service['logger'].error = () => {
      loggedError = true;
    };
    service['exportEmailResources'].getEmailsByProgramAndLang= async () => {
      /* eslint-disable no-throw-literal */
      throw {
        error: {
          message: 'Error'
        }
      };
    };
    const modalResponse: ExportEmailModalResponse = {
      programId: 123,
      emailLanguage: 'en-US'
    };

    await service.handleExportEmailsByProgramAndLang(modalResponse, this.emails);
    expect(errorNotified).to.be.true;
    expect(loggedError).to.be.true;
  }

  @TestCase('should be able to adapt program email for UI')
  adaptProgramEmailForUi (service: ExportEmailService) {
    const foundEmail = this.emails.find((email) => {
      return email.emailNotificationType === this.appApprovedCopy.emailNotificationTypeId;
    });
    const adapted = service.adaptProgramEmailForUi(this.appApprovedCopy, foundEmail, 0);
    expect(adapted.emailNotificationTypeId).to.be.equal(this.appApprovedCopy.emailNotificationTypeId);
    expect(adapted.emailNotificationName).to.be.equal('GC-13.1');
    expect(adapted.description).to.be.equal(this.appApprovedCopy.clientEmailTemplate.description);
  }


  @TestCase('should be able to adapt program emails for UI')
  adaptProgramEmailsForUi (service: ExportEmailService) {
    const adapted = service.adaptProgramEmailsForUi(
      this.programEmails,
      this.emails
    );
    const numberOfActiveEmails = this.programEmails.reduce((acc, info) => {
      const activeEmails = info.emailInfo.filter((item) => {
        return item.active;
      });

      return acc + activeEmails.length;
    }, 0);

    expect(adapted.length).to.be.equal(numberOfActiveEmails);
    const hasAdaptedApprovedCopy = adapted.some((item) => {
      return item.emailNotificationTypeId === this.appApprovedCopy.emailNotificationTypeId;
    });
    expect(hasAdaptedApprovedCopy).to.be.true;
  }

  @TestCase('should be able to get application emails - fetch')
  async getApplicationEmails (service: ExportEmailService) {
    let fetchedData = false;
    service['exportEmailResources'].getApplicationEmails = async () => {
      fetchedData = true;

      return [];
    };
    await service.getApplicationEmails(
      232,
      this.emails
    );
    expect(fetchedData).to.be.true;
  }

  @TestCase('should be able to get application emails - adapt')
  async getApplicationEmailsAdapt (service: ExportEmailService) {
    service['exportEmailResources'].getApplicationEmails = async () => {
      return [this.applicationEmail];
    };
    const result = await service.getApplicationEmails(
      232,
      this.emails
    );
    const foundEmail = this.emails.find((email) => {
      return email.emailNotificationType === this.applicationEmail.emailNotificationTypeId;
    });
    const adaptedEmail = result[0];
    expect(adaptedEmail.emailNotificationName).to.be.equal('GC-16');
    expect(adaptedEmail.audienceText).to.be.equal('Applicant');
    expect(adaptedEmail.description).to.be.equal(foundEmail.description);
    expect(adaptedEmail.name).to.be.equal(foundEmail.name);
  }

  @TestCase('should be able to adapt application email for UI')
  adaptApplicationEmailForUi (service: ExportEmailService) {
    const foundEmail = this.emails.find((email) => {
      return email.emailNotificationType === this.applicationEmail.emailNotificationTypeId;
    });
    const adapted = service.adaptApplicationEmailForUi(this.applicationEmail, foundEmail);
    expect(adapted.audienceText).to.be.equal('Applicant');
    expect(adapted.name).to.be.equal(foundEmail.name);
    expect(adapted.title).to.be.equal(foundEmail.title);
  }

  @TestCase('should be able to get body without button')
  getBodyWithoutButton (service: ExportEmailService) {
    const {
      body,
      buttonHTML
    } = service.getBodyWithoutButton(this.bodyWithButton);
    expect(buttonHTML).to.be.equal(this.buttonHtml);
    const noButtonInBody = !body.includes(this.buttonHtml);
    expect(noButtonInBody).to.be.true;
  }

  @TestCase('should be able to get email notification name')
  getEmailNotificationName (service: ExportEmailService) {
    let name = service.getEmailNotificationName(
      EmailNotificationType.ApplicationApproved,
      2
    );
    let expectedName = 'GC-13.3';
    expect(name).to.be.equal(expectedName);
    name = service.getEmailNotificationName(
      EmailNotificationType.AddressRequestReadyForReview
    );
    expectedName = 'GC-71';
    expect(name).to.be.equal(expectedName);
  }

  @TestCase('should be able to get audience text')
  getAudienceText (service: ExportEmailService) {
    let text = service.getAudienceText(true);
    expect(text).to.be.equal('Applicant');
    text = service.getAudienceText(false);
    expect(text).to.be.equal('Grant manager');
  }
}

