import { Component, Input } from '@angular/core';
import { AnalyticsService, EventType } from '@yourcause/common/analytics';
import { I18nService } from '@yourcause/common/i18n';
import { YCModalComponent } from '@yourcause/common/modals';
import { ClientEmailTemplateFromAPI, ProgramEmailTemplateForUI } from '../email.typing';

@Component({
  selector: 'gc-toggle-activate-system-email-modal',
  templateUrl: './toggle-activate-system-email-modal.component.html',
  styleUrls: ['./toggle-activate-system-email-modal.component.scss']
})
export class ToggleActivateSystemEmailModalComponent extends YCModalComponent<boolean> {
  @Input() copyEmailRows: (ClientEmailTemplateFromAPI | ProgramEmailTemplateForUI)[] = [];
  @Input() modalHeader: string;
  @Input() modalSubHeader: string;
  @Input() primaryButtonText: string;
  @Input() confirmText: string;
  @Input() isActivate: boolean;

  cancelButtonText = this.i18n.translate(
    'common:btnCancel',
    {},
    'Cancel'
  );
  inUseTableDescription = this.i18n.translate(
    'common:textListedBelowAreEmailsUsedByPrograms',
    {},
    'This email is currently being used on the following programs listed below.'
  );

  constructor (
    private i18n: I18nService,
    private analyticsService: AnalyticsService
  ) {
    super();
  }

  handlePrimaryClick () {
    this.closeModal.emit(true);
    this.analyticsService.emitEvent({
      eventName: 'Toggle system email active',
      eventType: EventType.Click,
      extras: null
    });
  }
}
