import { Component } from '@angular/core';
import { SimpleStringMap } from '@yourcause/common';
import { AnalyticsService, EventType } from '@yourcause/common/analytics';
import { YCModalComponent } from '@yourcause/common/modals';
import { SystemTags } from '../typings/system-tags.typing';

@Component({
  selector: 'gc-inactive-tags-modal',
  templateUrl: './inactive-tags-modal.component.html',
  styleUrls: ['./inactive-tags-modal.component.scss']
})
export class InactiveTagsModalComponent extends YCModalComponent<SystemTags.InactiveModalResult[]> {
  tagsMarkedForRemove: SimpleStringMap<boolean> = {};
  bucket: SystemTags.TagBucketWithTags;
  results: SystemTags.InactiveModalResult[] = [];

  constructor (
    private analyticsService: AnalyticsService
  ) {
    super();
  }

  handleTag (tag: SystemTags.TagDefinition, action: 'delete'|'Enable'|'Disable') {
    this.results = this.results.filter(result => result.tag.id !== tag.id);

    if (action === 'Disable') {
      tag.enabled = false;
    } else {
      if (action === 'Enable') {
        tag.enabled = true;
        this.tagsMarkedForRemove[tag.id] = false;
      } else if (action === 'delete') {
        tag.enabled = false;
        this.tagsMarkedForRemove[tag.id] = true;
      }

      this.results.push({
        tag,
        change: action
      });
    }
  }

  rollback () {
    this.tagsMarkedForRemove = {};
    this.results
      .forEach(result => {
        if (result.change === 'Enable') {
          result.tag.enabled = false;
        }
      });

    this.closeModal.emit();
    this.analyticsService.emitEvent({
      eventName: 'Rollback changes to inactive tags',
      eventType: EventType.Click,
      extras: null
    });
  }

  submit () {
    this.closeModal.emit(this.results);
    this.analyticsService.emitEvent({
      eventName: 'Save changes to inactive tags',
      eventType: EventType.Click,
      extras: null
    });
  }
}
