import { Component, Input, OnInit } from '@angular/core';
import { SpinnerService } from '@core/services/spinner.service';
import { ModalFactory } from '@yourcause/common/modals';
import { ManageTagsModalComponent } from '../manage-tags-modal/manage-tags-modal.component';
import { SystemTagsResolver } from '../resolvers/system-tags.resolver';
import { SystemTagsService } from '../system-tags.service';
import { SystemTags } from '../typings/system-tags.typing';

@Component({
  selector: 'gc-manage-tags',
  templateUrl: './manage-tags.component.html',
  styleUrls: ['./manage-tags.component.scss']
})
export class ManageTagsComponent implements OnInit {
  @Input() tagType: SystemTags.Buckets;
  @Input() subTagType: SystemTags.Buckets;
  @Input() entityId: string|number;
  @Input() subEntityId: string|number;
  @Input() currentTagIds: number[];
  @Input() currentSubTagIds: number[];
  @Input() isNomination = false;
  @Input() showManageLink = true;

  manageLinkVisible: boolean;
  currentTags: SystemTags.TagDefinition[];

  constructor (
    private spinnerService: SpinnerService,
    private systemTagsService: SystemTagsService,
    private modalFactory: ModalFactory,
    private systemTagsResolver: SystemTagsResolver
  ) { }

  async ngOnInit () {
    await this.systemTagsResolver.resolve();
    this.currentTagIds = this.systemTagsService.getTagsForRecord(
      this.tagType,
      this.entityId
    );
    if (!this.currentTagIds) {
      await this.systemTagsService.fetchTagsForRecord(this.tagType, this.entityId);
      this.currentTagIds = this.systemTagsService.getTagsForRecord(
        this.tagType,
        this.entityId
      );
    }

    this.currentTags = this.currentTagIds.map((id) => {
      return this.systemTagsService.summaries.find(tag => tag.id === id);
    });

    if (this.subEntityId && this.subTagType) {
      this.currentSubTagIds = this.systemTagsService.getTagsForRecord(
        this.subTagType,
        this.subEntityId
      );
      if (!this.currentSubTagIds) {
        await this.systemTagsService.fetchTagsForRecord(this.subTagType, this.subEntityId);
        this.currentSubTagIds = this.systemTagsService.getTagsForRecord(
          this.subTagType,
          this.subEntityId
        );
      }

      this.currentTags = this.currentTags.concat(this.currentSubTagIds.map((id) => {
        return this.systemTagsService.summaries.find(tag => tag.id === id);
      }));
    }

    this.manageLinkVisible = this.showManageLink && (
      (this.systemTagsService.getTagsForBucket(this.tagType).length > 0) ||
      (this.subTagType ? this.systemTagsService.getTagsForBucket(this.subTagType).length > 0 : false)
    );
  }

  async manageTags () {
    const tagIds = await this.modalFactory.open(
      ManageTagsModalComponent,
      {
        bucket: this.tagType,
        subBucket: this.subTagType,
        currentTagIds: this.currentTagIds.concat(this.currentSubTagIds ?? []),
        isNomination: this.isNomination,
        ids: [+this.entityId]
      }
    );
    if (tagIds) {
      this.spinnerService.startSpinner();
      const {
        newTags,
        removedTags
      } = this.systemTagsService.getNewAndRemovedTags(
        this.currentTags,
        tagIds
      );
      await this.systemTagsService.setTagsOnRecord(
        this.tagType,
        this.entityId,
        newTags,
        removedTags,
        this.subTagType,
        this.subEntityId
      );
      this.ngOnInit();
      this.spinnerService.stopSpinner();

    }
  }
}
