import { Injectable } from '@angular/core';
import { HttpRestService } from '@core/services/http-rest.service';
import { SystemTags } from './typings/system-tags.typing';

@Injectable({ providedIn: 'root' })
export class SystemTagsResources {
  private apiBase = '/api/manager/tags';

  constructor (
    private httpRestService: HttpRestService
  ) { }

  private tagEndpointByTagType (
    tagType: SystemTags.Buckets,
    id?: string|number
  ) {
    let endpoint = this.apiBase;

    switch (tagType) {
      case SystemTags.Buckets.Application:
      case SystemTags.Buckets.Nomination:
        endpoint += '/Application';
        break;
      case SystemTags.Buckets.NonprofitProfile:
        endpoint += '/Organization';
        break;
      case SystemTags.Buckets.Payment:
        endpoint += '/Payment';
        break;
      case SystemTags.Buckets.Applicant:
        endpoint += '/Applicant';
        break;
    }

    return id ? `${endpoint}/${id}` : endpoint;
  }

  getTags (
    tagType: SystemTags.Buckets,
    entityId: string|number
  ) {
    return this.httpRestService.get<SystemTags.EntityTag[]>(
      this.tagEndpointByTagType(tagType, entityId)
    );
  }

  getTagsBulk (
    applicationIds: number[]
  ): Promise<SystemTags.BulkTagResponse[]> {
    const endpoint = 'api/manager/tags/GetBulkApplicationTags';

    return this.httpRestService.post(endpoint, {
      applicationIds
    });
  }

  addOrUpdateRecord (
    body: SystemTags.AddTagRequest|SystemTags.UpdateTagRequest
  ): Promise<number> {
    return this.httpRestService.post(this.apiBase, body);
  }

  setTags (
    tagType: SystemTags.Buckets,
    body: SystemTags.SetTags
  ) {
    return this.httpRestService.post(
      this.tagEndpointByTagType(tagType),
      body
    );
  }

  getAll (): Promise<SystemTags.TagDefinition[]> {
    return this.httpRestService.get(this.apiBase + '/all');
  }

  updateRecord<T extends 'Promote'|'Enable'|'Disable'|'ChangeColor'> (
    tag: SystemTags.TagDefinition,
    action: T,
    _color: T extends 'ChangeColor' ? SystemTags.TagColor : null
  ) {
    return this.httpRestService.get(this.apiBase + '/' + tag.id + '/' + action);
  }

  removeRecord (tag: SystemTags.TagDefinition) {
    return this.httpRestService.delete(this.apiBase + '/' + tag.id);
  }

  bulkAddTags (
    tagIds: number[],
    applicationIds: number[]
  ) {
    const endpoint = `${this.apiBase}/Applications`;

    return this.httpRestService.post(endpoint, {
      tagIds,
      applicationIds
    });
  }
}

