import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { FormsResolver } from '@features/configure-forms/resolvers/forms.resolver';
import { CustomDataTablesResolver } from '@features/custom-data-tables/resolvers/custom-data-tables.resolver';
import { EmployeeSSOFieldsConfigurationResolver } from '@features/employee-sso-fields/resolvers/employee-sso-fields-configuration.resolver';
import { AllActiveManagerProgramsResolver } from '@features/programs/resolvers/all-active-manager-programs.resolver';
import { AllProgramsResolver } from '@features/programs/resolvers/all-programs.resolver';
import { ReferenceFieldsResolver } from '@features/reference-fields/reference-fields.resolver';
import { WorkflowResolver } from '@features/workflow/resolvers/workflow.resolver';
import { CreateEditRulePageComponent } from './create-edit-rule-page/create-edit-rule-page.component';
import { WorkflowAutomationFactoriesResolver } from './resolvers/workflow-automation-factories.resolver';
import { WorkflowLevelAutomationDetailResolver } from './resolvers/workflow-level-automation-detail.resolver';
import { ViewHistoryPageComponent } from './view-history-page/view-history-page.component';
import { ViewRulesPageComponent } from './view-rules-page/view-rules-page.component';
import { WorkflowAutomationViewWrapperComponent } from './workflow-automation-view-wrapper/workflow-automation-view-wrapper.component';
import { WorkflowAutomationModule } from './workflow-automation.module';

@NgModule({
  imports: [
    WorkflowAutomationModule,
    RouterModule.forChild([{
      path: '',
      resolve: {
        workflow: WorkflowResolver,
        forms: FormsResolver,
        programs: AllProgramsResolver,
        myPrograms: AllActiveManagerProgramsResolver,
        customColumnConfig: EmployeeSSOFieldsConfigurationResolver,
        dataTables: CustomDataTablesResolver,
        referenceFields: ReferenceFieldsResolver,
        factories: WorkflowAutomationFactoriesResolver
      },
      data: {
        isWorkflowAutomation: true
      },
      children: [{
        path: '',
        component: WorkflowAutomationViewWrapperComponent,
        children: [{
          path: 'view-initial-rules',
          component: ViewRulesPageComponent,
          data: {
            breadcrumbLabel: 'Initial Assignments',
            breadcrumb: true,
            isAdvancement: false,
            noReuse: true,
            pageName: 'Workflow Level Automation - Initial Assignments'
          }
        }, {
          path: 'view-advancement-rules',
          component: ViewRulesPageComponent,
          data: {
            breadcrumbLabel: 'Routing',
            breadcrumb: true,
            isAdvancement: true,
            noReuse: true,
            pageName: 'Workflow Level Automation - Routing'
          }
        }, {
          path: 'assignment-history',
          component: ViewHistoryPageComponent,
          data: {
            breadcrumbLabel: 'Assignment History',
            breadcrumb: true,
            isAssignmentHistory: true,
            pageName: 'Workflow Level Automation - Assignment History'
          }
        }, {
          path: 'routing-history',
          component: ViewHistoryPageComponent,
          data: {
            breadcrumbLabel: 'Routing History',
            breadcrumb: true,
            isRoutingHistory: true,
            pageName: 'Workflow Level Automation - Routing History'
          }
        }, {
          path: '',
          pathMatch: 'full',
          redirectTo: 'view-initial-rules'
        }]
      }, {
        path: 'initial/new',
        component: CreateEditRulePageComponent,
        data: {
          breadcrumbLabel: 'Create Assignment Rules',
          breadcrumb: true,
          pageName: 'Workflow Level Automation - Create Assignment Rules'
        }
      }, {
        path: 'initial/:id',
        component: CreateEditRulePageComponent,
        resolve: {
          detail: WorkflowLevelAutomationDetailResolver
        },
        data: {
          breadcrumbLabel: 'Edit Assignment Rules',
          breadcrumb: true,
          pageName: 'Workflow Level Automation - Edit Assignment Rules'
        }
      }, {
        path: 'advancement/new',
        component: CreateEditRulePageComponent,
        data: {
          breadcrumbLabel: 'Create Routing Rules',
          breadcrumb: true,
          isAdvancement: true,
          pageName: 'Workflow Level Automation - Create Routing Rules'
        }
      }, {
        path: 'advancement/:id',
        component: CreateEditRulePageComponent,
        resolve: {
          detail: WorkflowLevelAutomationDetailResolver
        },
        data: {
          breadcrumbLabel: 'Edit Routing Rules',
          breadcrumb: true,
          isAdvancement: true,
          pageName: 'Workflow Level Automation - Edit Routing Rules'
        }
      }]
    }])
  ]
})
export class WorkflowAutomationRoutingModule {}
