import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { AllUsersResolver } from '@features/users/resolvers/all-users.resolver';
import { WorkflowDetailResolver } from './resolvers/workflow-detail.resolver';
import { UsersAndRoutesComponent } from './users-and-routes/users-and-routes.component';
import { WorkflowDetailPageComponent } from './workflow-detail-page/workflow-detail-page.component';
import { WorkflowManagersTableComponent } from './workflow-managers-table/workflow-managers-table.component';
import { WorkflowPageComponent } from './workflow-page/workflow-page.component';
import { WorkflowStructureComponent } from './workflow-structure/workflow-structure.component';
import { WorkflowModule } from './workflow.module';


@NgModule({
  imports: [
    WorkflowModule,
    RouterModule.forChild([{
      path: '',
      component: WorkflowPageComponent,
      resolve: {
        allUsers: AllUsersResolver
      },
      data: { pageName: 'Workflows' }
    }, {
      path: 'detail/:id',
      component: WorkflowDetailPageComponent,
      resolve: {
        workflow: WorkflowDetailResolver,
        allUsers: AllUsersResolver
      },
      children: [{
        path: 'structure',
        component: WorkflowStructureComponent,
        data: {
          breadcrumb: true,
          breadcrumbLabel: 'Structure',
          pageName: 'Edit Workflow - Structure'
        }
      }, {
        path: 'users-and-routes',
        component: UsersAndRoutesComponent,
        data: {
          breadcrumb: true,
          breadcrumbLabel: 'Users and Routes',
          pageName: 'Edit Workflow - Users and Routes'
        }
      }, {
        path: 'workflow-managers',
        component: WorkflowManagersTableComponent,
        data: {
          breadcrumb: true,
          breadcrumbLabel: 'Workflow Managers',
          pageName: 'Edit Workflow - Workflow Managers'
        }
      }]
    }])
  ]
})
export class WorkflowRoutingModule { }
