import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { PlatformWrapperComponent } from '@core/components/platform-wrapper/platform-wrapper.component';
import { CanAccessPortalGuard } from '@core/guards/can-access-portal.guard';
import { AdminClientsResolver } from '@core/resolvers/admin-clients.resolver';
import { FeatureFlagsResolver } from '@core/resolvers/feature-flags.resolver';
import { InitialRedirectResolver } from '@core/resolvers/initial-redirect.resolver';
import { LanguageResolver } from '@core/resolvers/language-resolver';
import { UnauthenticatedGuard } from '@features/auth/unauthenticated.guard';
import { HelpContentResolver } from '@features/platform-admin/help-content/help-content.resolver';
import { UserResolver } from '@features/users/resolvers/user.resolver';

@NgModule({
  imports: [
    RouterModule.forChild([{
      path: '',
      resolve: {
        redirect: InitialRedirectResolver,
        language: LanguageResolver
      },
      children: [{
        path: 'auth',
        canActivate: [
          UnauthenticatedGuard
        ],
        loadChildren: () => import('@features/auth/platform-auth.module')
          .then(m => m.PlatformAuthModule)
      }, {
        path: '',
        canActivate: [
          CanAccessPortalGuard
        ],
        resolve: {
          featureFlags: FeatureFlagsResolver,
          clients: AdminClientsResolver,
          user: UserResolver
        },
        children: [{
          path: '',
          component: PlatformWrapperComponent,
          resolve: {
            help: HelpContentResolver
          },
          loadChildren: () => import('@features/platform-admin/platform-admin-routing.module')
            .then(m => m.PlatformAdminRoutingModule)
        }]
      }]
    }])
  ]
})
export class PlatformPortalRoutingModule { }
